@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2019, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -16,13 +16,13 @@
 
 #if !defined(PETSC_USE_DEBUG)
 
-#define SlepcValidVecComp(y,arg) do {} while (0)
+#define SlepcValidVecComp(y,arg) do {(void)(y);} while (0)
 
 #else
 
 #define SlepcValidVecComp(y,arg) \
   do { \
-    if (((Vec_Comp*)(y)->data)->nx < ((Vec_Comp*)(y)->data)->n->n) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Invalid number of subvectors required: Parameter #%d",arg); \
+    PetscCheck(((Vec_Comp*)(y)->data)->nx>=((Vec_Comp*)(y)->data)->n->n,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Invalid number of subvectors required: Parameter #%d",arg); \
   } while (0)
 
 #endif
@@ -79,30 +79,6 @@ SLEPC_INTERN PetscErrorCode VecLog_Comp(Vec);
 SLEPC_INTERN PetscErrorCode VecShift_Comp(Vec,PetscScalar);
 SLEPC_EXTERN PetscErrorCode VecCreate_Comp(Vec);
 
-/* Definitions and structures for BLAS-type operations in Davidson solvers */
-
-typedef PetscInt MatType_t;
-#define DVD_MAT_HERMITIAN (1<<1)
-#define DVD_MAT_NEG_DEF (1<<2)
-#define DVD_MAT_POS_DEF (1<<3)
-#define DVD_MAT_SINGULAR (1<<4)
-#define DVD_MAT_COMPLEX (1<<5)
-#define DVD_MAT_IMPLICIT (1<<6)
-#define DVD_MAT_IDENTITY (1<<7)
-#define DVD_MAT_DIAG (1<<8)
-#define DVD_MAT_TRIANG (1<<9)
-#define DVD_MAT_UTRIANG (1<<9)
-#define DVD_MAT_LTRIANG (1<<10)
-#define DVD_MAT_UNITARY (1<<11)
-
-typedef PetscInt EPType_t;
-#define DVD_EP_STD (1<<1)
-#define DVD_EP_HERMITIAN (1<<2)
-#define DVD_EP_INDEFINITE (1<<3)
-
-#define DVD_IS(T,P) ((T) & (P))
-#define DVD_ISNOT(T,P) (((T) & (P)) ^ (P))
-
 /* VecPool */
 typedef struct VecPool_ {
   Vec      v;              /* template vector */
