@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -53,10 +53,14 @@ typedef const char* EPSType;
 #define EPSLYAPII      "lyapii"
 #define EPSLAPACK      "lapack"
 #define EPSARPACK      "arpack"
-#define EPSBLZPACK     "blzpack"
 #define EPSTRLAN       "trlan"
 #define EPSBLOPEX      "blopex"
 #define EPSPRIMME      "primme"
+#define EPSFEAST       "feast"
+#define EPSSCALAPACK   "scalapack"
+#define EPSELPA        "elpa"
+#define EPSELEMENTAL   "elemental"
+#define EPSEVSL        "evsl"
 
 /* Logging support */
 SLEPC_EXTERN PetscClassId EPS_CLASSID;
@@ -194,10 +198,12 @@ SLEPC_EXTERN PetscErrorCode EPSSolve(EPS);
 SLEPC_EXTERN PetscErrorCode EPSView(EPS,PetscViewer);
 SLEPC_EXTERN PetscErrorCode EPSViewFromOptions(EPS,PetscObject,const char[]);
 SLEPC_EXTERN PetscErrorCode EPSErrorView(EPS,EPSErrorType,PetscViewer);
-PETSC_DEPRECATED_FUNCTION("Use EPSErrorView()") PETSC_STATIC_INLINE PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer v) {return EPSErrorView(eps,EPS_ERROR_RELATIVE,v);}
+PETSC_DEPRECATED_FUNCTION("Use EPSErrorView()") static inline PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer v) {return EPSErrorView(eps,EPS_ERROR_RELATIVE,v);}
 SLEPC_EXTERN PetscErrorCode EPSErrorViewFromOptions(EPS);
-SLEPC_EXTERN PetscErrorCode EPSReasonView(EPS,PetscViewer);
-SLEPC_EXTERN PetscErrorCode EPSReasonViewFromOptions(EPS);
+SLEPC_EXTERN PetscErrorCode EPSConvergedReasonView(EPS,PetscViewer);
+SLEPC_EXTERN PetscErrorCode EPSConvergedReasonViewFromOptions(EPS);
+PETSC_DEPRECATED_FUNCTION("Use EPSConvergedReasonView() (since version 3.14)") static inline PetscErrorCode EPSReasonView(EPS eps,PetscViewer v) {return EPSConvergedReasonView(eps,v);}
+PETSC_DEPRECATED_FUNCTION("Use EPSConvergedReasonViewFromOptions() (since version 3.14)") static inline PetscErrorCode EPSReasonViewFromOptions(EPS eps) {return EPSConvergedReasonViewFromOptions(eps);}
 SLEPC_EXTERN PetscErrorCode EPSValuesView(EPS,PetscViewer);
 SLEPC_EXTERN PetscErrorCode EPSValuesViewFromOptions(EPS);
 SLEPC_EXTERN PetscErrorCode EPSVectorsView(EPS,PetscViewer);
@@ -239,17 +245,10 @@ SLEPC_EXTERN PetscErrorCode EPSGetEigenvector(EPS,PetscInt,Vec,Vec);
 SLEPC_EXTERN PetscErrorCode EPSGetLeftEigenvector(EPS,PetscInt,Vec,Vec);
 
 SLEPC_EXTERN PetscErrorCode EPSComputeError(EPS,PetscInt,EPSErrorType,PetscReal*);
-PETSC_DEPRECATED_FUNCTION("Use EPSComputeError()") PETSC_STATIC_INLINE PetscErrorCode EPSComputeRelativeError(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_RELATIVE,r);}
-PETSC_DEPRECATED_FUNCTION("Use EPSComputeError() with EPS_ERROR_ABSOLUTE") PETSC_STATIC_INLINE PetscErrorCode EPSComputeResidualNorm(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_ABSOLUTE,r);}
+PETSC_DEPRECATED_FUNCTION("Use EPSComputeError()") static inline PetscErrorCode EPSComputeRelativeError(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_RELATIVE,r);}
+PETSC_DEPRECATED_FUNCTION("Use EPSComputeError() with EPS_ERROR_ABSOLUTE") static inline PetscErrorCode EPSComputeResidualNorm(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_ABSOLUTE,r);}
 SLEPC_EXTERN PetscErrorCode EPSGetInvariantSubspace(EPS,Vec[]);
 SLEPC_EXTERN PetscErrorCode EPSGetErrorEstimate(EPS,PetscInt,PetscReal*);
-
-SLEPC_EXTERN PetscErrorCode EPSMonitor(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt);
-SLEPC_EXTERN PetscErrorCode EPSMonitorSet(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),void*,PetscErrorCode (*)(void**));
-SLEPC_EXTERN PetscErrorCode EPSMonitorSetFromOptions(EPS,const char*,const char*,const char*,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscBool);
-SLEPC_EXTERN PetscErrorCode EPSConvMonitorSetFromOptions(EPS,const char*,const char*,const char*,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,SlepcConvMonitor));
-SLEPC_EXTERN PetscErrorCode EPSMonitorCancel(EPS);
-SLEPC_EXTERN PetscErrorCode EPSGetMonitorContext(EPS,void**);
 SLEPC_EXTERN PetscErrorCode EPSGetIterationNumber(EPS,PetscInt*);
 
 SLEPC_EXTERN PetscErrorCode EPSSetWhichEigenpairs(EPS,EPSWhich);
@@ -266,13 +265,6 @@ SLEPC_EXTERN PetscErrorCode EPSIsGeneralized(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSIsHermitian(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSIsPositive(EPS,PetscBool*);
 
-SLEPC_EXTERN PetscErrorCode EPSMonitorFirst(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorConverged(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,SlepcConvMonitor);
-SLEPC_EXTERN PetscErrorCode EPSMonitorLGCreate(MPI_Comm,const char[],const char[],int,int,int,int,PetscDrawLG*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorLGAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*);
-
 SLEPC_EXTERN PetscErrorCode EPSSetTrackAll(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSGetTrackAll(EPS,PetscBool*);
 
@@ -280,12 +272,35 @@ SLEPC_EXTERN PetscErrorCode EPSSetDeflationSpace(EPS,PetscInt,Vec[]);
 SLEPC_EXTERN PetscErrorCode EPSSetInitialSpace(EPS,PetscInt,Vec[]);
 SLEPC_EXTERN PetscErrorCode EPSSetLeftInitialSpace(EPS,PetscInt,Vec[]);
 
+SLEPC_EXTERN PetscErrorCode EPSMonitor(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt);
+SLEPC_EXTERN PetscErrorCode EPSMonitorSet(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),void*,PetscErrorCode (*)(void**));
+SLEPC_EXTERN PetscErrorCode EPSMonitorCancel(EPS);
+SLEPC_EXTERN PetscErrorCode EPSGetMonitorContext(EPS,void*);
+
+SLEPC_EXTERN PetscErrorCode EPSMonitorSetFromOptions(EPS,const char[],const char[],void*,PetscBool);
+SLEPC_EXTERN PetscErrorCode EPSMonitorLGCreate(MPI_Comm,const char[],const char[],const char[],PetscInt,const char*[],int,int,int,int,PetscDrawLG*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorFirst(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorFirstDrawLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorFirstDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorAllDrawLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorAllDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConverged(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDrawLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDestroy(PetscViewerAndFormat**);
+
 SLEPC_EXTERN PetscErrorCode EPSSetOptionsPrefix(EPS,const char*);
 SLEPC_EXTERN PetscErrorCode EPSAppendOptionsPrefix(EPS,const char*);
 SLEPC_EXTERN PetscErrorCode EPSGetOptionsPrefix(EPS,const char*[]);
 
 SLEPC_EXTERN PetscFunctionList EPSList;
+SLEPC_EXTERN PetscFunctionList EPSMonitorList;
+SLEPC_EXTERN PetscFunctionList EPSMonitorCreateList;
+SLEPC_EXTERN PetscFunctionList EPSMonitorDestroyList;
 SLEPC_EXTERN PetscErrorCode EPSRegister(const char[],PetscErrorCode(*)(EPS));
+SLEPC_EXTERN PetscErrorCode EPSMonitorRegister(const char[],PetscViewerType,PetscViewerFormat,PetscErrorCode(*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscErrorCode(*)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**),PetscErrorCode(*)(PetscViewerAndFormat**));
 
 SLEPC_EXTERN PetscErrorCode EPSSetWorkVecs(EPS,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSAllocateSolution(EPS,PetscInt);
@@ -333,6 +348,7 @@ SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommInfo(EPS,PetscInt*,PetscInt*
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommPairs(EPS,PetscInt,PetscScalar*,Vec);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommMats(EPS,Mat*,Mat*);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurUpdateSubcommMats(EPS,PetscScalar,PetscScalar,Mat,PetscScalar,PetscScalar, Mat,MatStructure,PetscBool);
+SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetKSP(EPS,KSP*);
 
 /*E
     EPSLanczosReorthogType - determines the type of reorthogonalization
@@ -353,11 +369,6 @@ SLEPC_EXTERN const char *EPSLanczosReorthogTypes[];
 SLEPC_EXTERN PetscErrorCode EPSLanczosSetReorthog(EPS,EPSLanczosReorthogType);
 SLEPC_EXTERN PetscErrorCode EPSLanczosGetReorthog(EPS,EPSLanczosReorthogType*);
 
-SLEPC_EXTERN PetscErrorCode EPSBlzpackSetBlockSize(EPS,PetscInt);
-SLEPC_EXTERN PetscErrorCode EPSBlzpackGetBlockSize(EPS,PetscInt*);
-SLEPC_EXTERN PetscErrorCode EPSBlzpackSetNSteps(EPS,PetscInt);
-SLEPC_EXTERN PetscErrorCode EPSBlzpackGetNSteps(EPS,PetscInt*);
-
 /*E
     EPSPRIMMEMethod - determines the method selected in the PRIMME library
 
@@ -461,13 +472,50 @@ SLEPC_EXTERN PetscErrorCode EPSCISSSetUseST(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSCISSGetUseST(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSCISSGetKSPs(EPS,PetscInt*,KSP**);
 
-PETSC_EXTERN PetscErrorCode EPSLyapIISetLME(EPS,LME);
-PETSC_EXTERN PetscErrorCode EPSLyapIIGetLME(EPS,LME*);
-PETSC_EXTERN PetscErrorCode EPSLyapIISetRanks(EPS,PetscInt,PetscInt);
-PETSC_EXTERN PetscErrorCode EPSLyapIIGetRanks(EPS,PetscInt*,PetscInt*);
+SLEPC_EXTERN PetscErrorCode EPSLyapIISetLME(EPS,LME);
+SLEPC_EXTERN PetscErrorCode EPSLyapIIGetLME(EPS,LME*);
+SLEPC_EXTERN PetscErrorCode EPSLyapIISetRanks(EPS,PetscInt,PetscInt);
+SLEPC_EXTERN PetscErrorCode EPSLyapIIGetRanks(EPS,PetscInt*,PetscInt*);
 
 SLEPC_EXTERN PetscErrorCode EPSBLOPEXSetBlockSize(EPS,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSBLOPEXGetBlockSize(EPS,PetscInt*);
 
+/*E
+    EPSEVSLDOSMethod - the method to approximate the density of states (DOS) in the EVSL solver
+
+    Level: advanced
+
+.seealso: EPSEVSLSetDOSParameters(), EPSEVSLGetDOSParameters()
+E*/
+typedef enum { EPS_EVSL_DOS_KPM,
+               EPS_EVSL_DOS_LANCZOS } EPSEVSLDOSMethod;
+SLEPC_EXTERN const char *EPSEVSLDOSMethods[];
+
+/*E
+    EPSEVSLDamping - the damping type used in the EVSL solver
+
+    Level: advanced
+
+.seealso: EPSEVSLSetDOSParameters(), EPSEVSLGetDOSParameters()
+E*/
+typedef enum { EPS_EVSL_DAMPING_NONE,
+               EPS_EVSL_DAMPING_JACKSON,
+               EPS_EVSL_DAMPING_SIGMA } EPSEVSLDamping;
+SLEPC_EXTERN const char *EPSEVSLDampings[];
+
+SLEPC_EXTERN PetscErrorCode EPSEVSLSetRange(EPS,PetscReal,PetscReal);
+SLEPC_EXTERN PetscErrorCode EPSEVSLGetRange(EPS,PetscReal*,PetscReal*);
+SLEPC_EXTERN PetscErrorCode EPSEVSLSetSlices(EPS,PetscInt);
+SLEPC_EXTERN PetscErrorCode EPSEVSLGetSlices(EPS,PetscInt*);
+SLEPC_EXTERN PetscErrorCode EPSEVSLSetDOSParameters(EPS,EPSEVSLDOSMethod,PetscInt,PetscInt,PetscInt,PetscInt);
+SLEPC_EXTERN PetscErrorCode EPSEVSLGetDOSParameters(EPS,EPSEVSLDOSMethod*,PetscInt*,PetscInt*,PetscInt*,PetscInt*);
+SLEPC_EXTERN PetscErrorCode EPSEVSLSetPolParameters(EPS,PetscInt,PetscReal);
+SLEPC_EXTERN PetscErrorCode EPSEVSLGetPolParameters(EPS,PetscInt*,PetscReal*);
+SLEPC_EXTERN PetscErrorCode EPSEVSLSetDamping(EPS,EPSEVSLDamping);
+SLEPC_EXTERN PetscErrorCode EPSEVSLGetDamping(EPS,EPSEVSLDamping*);
+
+SLEPC_EXTERN PetscErrorCode EPSFEASTSetNumPoints(EPS,PetscInt);
+SLEPC_EXTERN PetscErrorCode EPSFEASTGetNumPoints(EPS,PetscInt*);
+
 #endif
 
