@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,21 +13,30 @@
 
 #include <slepc/private/epsimpl.h>      /*I "slepceps.h" I*/
 
+/* Logging support */
+PetscClassId      EPS_CLASSID = 0;
+PetscLogEvent     EPS_SetUp = 0,EPS_Solve = 0,EPS_CISS_SVD = 0;
+
+/* List of registered EPS routines */
 PetscFunctionList EPSList = 0;
 PetscBool         EPSRegisterAllCalled = PETSC_FALSE;
-PetscClassId      EPS_CLASSID = 0;
-PetscLogEvent     EPS_SetUp = 0,EPS_Solve = 0;
+
+/* List of registered EPS monitors */
+PetscFunctionList EPSMonitorList              = NULL;
+PetscFunctionList EPSMonitorCreateList        = NULL;
+PetscFunctionList EPSMonitorDestroyList       = NULL;
+PetscBool         EPSMonitorRegisterAllCalled = PETSC_FALSE;
 
 /*@
    EPSCreate - Creates the default EPS context.
 
-   Collective on MPI_Comm
+   Collective
 
    Input Parameter:
 .  comm - MPI communicator
 
    Output Parameter:
-.  eps - location to put the EPS context
+.  outeps - location to put the EPS context
 
    Note:
    The default EPS type is EPSKRYLOVSCHUR
@@ -38,19 +47,18 @@ PetscLogEvent     EPS_SetUp = 0,EPS_Solve = 0;
 @*/
 PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
 {
-  PetscErrorCode ierr;
   EPS            eps;
 
   PetscFunctionBegin;
   PetscValidPointer(outeps,2);
   *outeps = 0;
-  ierr = EPSInitializePackage();CHKERRQ(ierr);
-  ierr = SlepcHeaderCreate(eps,EPS_CLASSID,"EPS","Eigenvalue Problem Solver","EPS",comm,EPSDestroy,EPSView);CHKERRQ(ierr);
+  PetscCall(EPSInitializePackage());
+  PetscCall(SlepcHeaderCreate(eps,EPS_CLASSID,"EPS","Eigenvalue Problem Solver","EPS",comm,EPSDestroy,EPSView));
 
-  eps->max_it          = 0;
+  eps->max_it          = PETSC_DEFAULT;
   eps->nev             = 1;
-  eps->ncv             = 0;
-  eps->mpd             = 0;
+  eps->ncv             = PETSC_DEFAULT;
+  eps->mpd             = PETSC_DEFAULT;
   eps->nini            = 0;
   eps->nds             = 0;
   eps->target          = 0.0;
@@ -68,6 +76,7 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->trueres         = PETSC_FALSE;
   eps->trackall        = PETSC_FALSE;
   eps->purify          = PETSC_TRUE;
+  eps->twosided        = PETSC_FALSE;
 
   eps->converged       = EPSConvergedRelative;
   eps->convergeduser   = NULL;
@@ -84,9 +93,11 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->st              = NULL;
   eps->ds              = NULL;
   eps->V               = NULL;
+  eps->W               = NULL;
   eps->rg              = NULL;
   eps->D               = NULL;
   eps->IS              = NULL;
+  eps->ISL             = NULL;
   eps->defl            = NULL;
   eps->eigr            = NULL;
   eps->eigi            = NULL;
@@ -111,7 +122,7 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->ishermitian     = PETSC_FALSE;
   eps->reason          = EPS_CONVERGED_ITERATING;
 
-  ierr = PetscNewLog(eps,&eps->sc);CHKERRQ(ierr);
+  PetscCall(PetscNewLog(eps,&eps->sc));
   *outeps = eps;
   PetscFunctionReturn(0);
 }
@@ -119,7 +130,7 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
 /*@C
    EPSSetType - Selects the particular solver to be used in the EPS object.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - the eigensolver context
@@ -147,25 +158,25 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
 @*/
 PetscErrorCode EPSSetType(EPS eps,EPSType type)
 {
-  PetscErrorCode ierr,(*r)(EPS);
+  PetscErrorCode (*r)(EPS);
   PetscBool      match;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscObjectTypeCompare((PetscObject)eps,type,&match);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)eps,type,&match));
   if (match) PetscFunctionReturn(0);
 
-  ierr = PetscFunctionListFind(EPSList,type,&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown EPS type given: %s",type);
+  PetscCall(PetscFunctionListFind(EPSList,type,&r));
+  PetscCheck(r,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown EPS type given: %s",type);
 
-  if (eps->ops->destroy) { ierr = (*eps->ops->destroy)(eps);CHKERRQ(ierr); }
-  ierr = PetscMemzero(eps->ops,sizeof(struct _EPSOps));CHKERRQ(ierr);
+  if (eps->ops->destroy) PetscCall((*eps->ops->destroy)(eps));
+  PetscCall(PetscMemzero(eps->ops,sizeof(struct _EPSOps)));
 
   eps->state = EPS_STATE_INITIAL;
-  ierr = PetscObjectChangeTypeName((PetscObject)eps,type);CHKERRQ(ierr);
-  ierr = (*r)(eps);CHKERRQ(ierr);
+  PetscCall(PetscObjectChangeTypeName((PetscObject)eps,type));
+  PetscCall((*r)(eps));
   PetscFunctionReturn(0);
 }
 
@@ -178,7 +189,7 @@ PetscErrorCode EPSSetType(EPS eps,EPSType type)
 .  eps - the eigensolver context
 
    Output Parameter:
-.  name - name of EPS method
+.  type - name of EPS method
 
    Level: intermediate
 
@@ -221,10 +232,52 @@ $     -eps_type my_solver
 @*/
 PetscErrorCode EPSRegister(const char *name,PetscErrorCode (*function)(EPS))
 {
-  PetscErrorCode ierr;
+  PetscFunctionBegin;
+  PetscCall(EPSInitializePackage());
+  PetscCall(PetscFunctionListAdd(&EPSList,name,function));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   EPSMonitorRegister - Adds EPS monitor routine.
+
+   Not Collective
+
+   Input Parameters:
++  name    - name of a new monitor routine
+.  vtype   - a PetscViewerType for the output
+.  format  - a PetscViewerFormat for the output
+.  monitor - monitor routine
+.  create  - creation routine, or NULL
+-  destroy - destruction routine, or NULL
+
+   Notes:
+   EPSMonitorRegister() may be called multiple times to add several user-defined monitors.
+
+   Sample usage:
+.vb
+   EPSMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
+.ve
+
+   Then, your monitor can be chosen with the procedural interface via
+$      EPSMonitorSetFromOptions(eps,"-eps_monitor_my_monitor","my_monitor",NULL)
+   or at runtime via the option
+$      -eps_monitor_my_monitor
+
+   Level: advanced
+
+.seealso: EPSMonitorRegisterAll()
+@*/
+PetscErrorCode EPSMonitorRegister(const char name[],PetscViewerType vtype,PetscViewerFormat format,PetscErrorCode (*monitor)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscErrorCode (*create)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**),PetscErrorCode (*destroy)(PetscViewerAndFormat**))
+{
+  char           key[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
-  ierr = PetscFunctionListAdd(&EPSList,name,function);CHKERRQ(ierr);
+  PetscCall(EPSInitializePackage());
+  PetscCall(SlepcMonitorMakeKey_Internal(name,vtype,format,key));
+  PetscCall(PetscFunctionListAdd(&EPSMonitorList,key,monitor));
+  if (create)  PetscCall(PetscFunctionListAdd(&EPSMonitorCreateList,key,create));
+  if (destroy) PetscCall(PetscFunctionListAdd(&EPSMonitorDestroyList,key,destroy));
   PetscFunctionReturn(0);
 }
 
@@ -232,7 +285,7 @@ PetscErrorCode EPSRegister(const char *name,PetscErrorCode (*function)(EPS))
    EPSReset - Resets the EPS context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameter:
 .  eps - eigensolver context obtained from EPSCreate()
@@ -249,25 +302,24 @@ PetscErrorCode EPSRegister(const char *name,PetscErrorCode (*function)(EPS))
 @*/
 PetscErrorCode EPSReset(EPS eps)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   if (eps) PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (!eps) PetscFunctionReturn(0);
-  if (eps->ops->reset) { ierr = (eps->ops->reset)(eps);CHKERRQ(ierr); }
-  if (eps->st) { ierr = STReset(eps->st);CHKERRQ(ierr); }
-  ierr = VecDestroy(&eps->D);CHKERRQ(ierr);
-  ierr = BVDestroy(&eps->V);CHKERRQ(ierr);
-  ierr = VecDestroyVecs(eps->nwork,&eps->work);CHKERRQ(ierr);
+  if (eps->ops->reset) PetscCall((eps->ops->reset)(eps));
+  if (eps->st) PetscCall(STReset(eps->st));
+  PetscCall(VecDestroy(&eps->D));
+  PetscCall(BVDestroy(&eps->V));
+  PetscCall(BVDestroy(&eps->W));
+  PetscCall(VecDestroyVecs(eps->nwork,&eps->work));
   eps->nwork = 0;
   eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    EPSDestroy - Destroys the EPS context.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameter:
 .  eps - eigensolver context obtained from EPSCreate()
@@ -278,39 +330,32 @@ PetscErrorCode EPSReset(EPS eps)
 @*/
 PetscErrorCode EPSDestroy(EPS *eps)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   if (!*eps) PetscFunctionReturn(0);
   PetscValidHeaderSpecific(*eps,EPS_CLASSID,1);
   if (--((PetscObject)(*eps))->refct > 0) { *eps = 0; PetscFunctionReturn(0); }
-  ierr = EPSReset(*eps);CHKERRQ(ierr);
-  if ((*eps)->ops->destroy) { ierr = (*(*eps)->ops->destroy)(*eps);CHKERRQ(ierr); }
-  if ((*eps)->eigr) {
-    ierr = PetscFree4((*eps)->eigr,(*eps)->eigi,(*eps)->errest,(*eps)->perm);CHKERRQ(ierr);
-  }
-  if ((*eps)->rr) {
-    ierr = PetscFree2((*eps)->rr,(*eps)->ri);CHKERRQ(ierr);
-  }
-  ierr = STDestroy(&(*eps)->st);CHKERRQ(ierr);
-  ierr = RGDestroy(&(*eps)->rg);CHKERRQ(ierr);
-  ierr = DSDestroy(&(*eps)->ds);CHKERRQ(ierr);
-  ierr = PetscFree((*eps)->sc);CHKERRQ(ierr);
+  PetscCall(EPSReset(*eps));
+  if ((*eps)->ops->destroy) PetscCall((*(*eps)->ops->destroy)(*eps));
+  if ((*eps)->eigr) PetscCall(PetscFree4((*eps)->eigr,(*eps)->eigi,(*eps)->errest,(*eps)->perm));
+  if ((*eps)->rr) PetscCall(PetscFree2((*eps)->rr,(*eps)->ri));
+  PetscCall(STDestroy(&(*eps)->st));
+  PetscCall(RGDestroy(&(*eps)->rg));
+  PetscCall(DSDestroy(&(*eps)->ds));
+  PetscCall(PetscFree((*eps)->sc));
   /* just in case the initial vectors have not been used */
-  ierr = SlepcBasisDestroy_Private(&(*eps)->nds,&(*eps)->defl);CHKERRQ(ierr);
-  ierr = SlepcBasisDestroy_Private(&(*eps)->nini,&(*eps)->IS);CHKERRQ(ierr);
-  if ((*eps)->convergeddestroy) {
-    ierr = (*(*eps)->convergeddestroy)((*eps)->convergedctx);CHKERRQ(ierr);
-  }
-  ierr = EPSMonitorCancel(*eps);CHKERRQ(ierr);
-  ierr = PetscHeaderDestroy(eps);CHKERRQ(ierr);
+  PetscCall(SlepcBasisDestroy_Private(&(*eps)->nds,&(*eps)->defl));
+  PetscCall(SlepcBasisDestroy_Private(&(*eps)->nini,&(*eps)->IS));
+  PetscCall(SlepcBasisDestroy_Private(&(*eps)->ninil,&(*eps)->ISL));
+  if ((*eps)->convergeddestroy) PetscCall((*(*eps)->convergeddestroy)((*eps)->convergedctx));
+  PetscCall(EPSMonitorCancel(*eps));
+  PetscCall(PetscHeaderDestroy(eps));
   PetscFunctionReturn(0);
 }
 
 /*@
    EPSSetTarget - Sets the value of the target.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps    - eigensolver context
@@ -333,14 +378,12 @@ PetscErrorCode EPSDestroy(EPS *eps)
 @*/
 PetscErrorCode EPSSetTarget(EPS eps,PetscScalar target)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveScalar(eps,target,2);
   eps->target = target;
-  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
-  ierr = STSetDefaultShift(eps->st,target);CHKERRQ(ierr);
+  if (!eps->st) PetscCall(EPSGetST(eps,&eps->st));
+  PetscCall(STSetDefaultShift(eps->st,target));
   PetscFunctionReturn(0);
 }
 
@@ -374,7 +417,7 @@ PetscErrorCode EPSGetTarget(EPS eps,PetscScalar* target)
 /*@
    EPSSetInterval - Defines the computational interval for spectrum slicing.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps  - eigensolver context
@@ -405,7 +448,7 @@ PetscErrorCode EPSSetInterval(EPS eps,PetscReal inta,PetscReal intb)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveReal(eps,inta,2);
   PetscValidLogicalCollectiveReal(eps,intb,3);
-  if (inta>intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
+  PetscCheck(inta<intb,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Badly defined interval, must be inta<intb");
   if (eps->inta != inta || eps->intb != intb) {
     eps->inta = inta;
     eps->intb = intb;
@@ -445,7 +488,7 @@ PetscErrorCode EPSGetInterval(EPS eps,PetscReal* inta,PetscReal* intb)
 /*@
    EPSSetST - Associates a spectral transformation object to the eigensolver.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context obtained from EPSCreate()
@@ -461,16 +504,14 @@ PetscErrorCode EPSGetInterval(EPS eps,PetscReal* inta,PetscReal* intb)
 @*/
 PetscErrorCode EPSSetST(EPS eps,ST st)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidHeaderSpecific(st,ST_CLASSID,2);
   PetscCheckSameComm(eps,1,st,2);
-  ierr = PetscObjectReference((PetscObject)st);CHKERRQ(ierr);
-  ierr = STDestroy(&eps->st);CHKERRQ(ierr);
+  PetscCall(PetscObjectReference((PetscObject)st));
+  PetscCall(STDestroy(&eps->st));
   eps->st = st;
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->st);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->st));
   PetscFunctionReturn(0);
 }
 
@@ -492,14 +533,14 @@ PetscErrorCode EPSSetST(EPS eps,ST st)
 @*/
 PetscErrorCode EPSGetST(EPS eps,ST *st)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(st,2);
   if (!eps->st) {
-    ierr = STCreate(PetscObjectComm((PetscObject)eps),&eps->st);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->st);CHKERRQ(ierr);
+    PetscCall(STCreate(PetscObjectComm((PetscObject)eps),&eps->st));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)eps->st,(PetscObject)eps,0));
+    PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->st));
+    PetscCall(PetscObjectSetOptions((PetscObject)eps->st,((PetscObject)eps)->options));
   }
   *st = eps->st;
   PetscFunctionReturn(0);
@@ -508,32 +549,26 @@ PetscErrorCode EPSGetST(EPS eps,ST *st)
 /*@
    EPSSetBV - Associates a basis vectors object to the eigensolver.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context obtained from EPSCreate()
 -  V   - the basis vectors object
 
-   Note:
-   Use EPSGetBV() to retrieve the basis vectors context (for example,
-   to free them at the end of the computations).
-
    Level: advanced
 
 .seealso: EPSGetBV()
 @*/
 PetscErrorCode EPSSetBV(EPS eps,BV V)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidHeaderSpecific(V,BV_CLASSID,2);
   PetscCheckSameComm(eps,1,V,2);
-  ierr = PetscObjectReference((PetscObject)V);CHKERRQ(ierr);
-  ierr = BVDestroy(&eps->V);CHKERRQ(ierr);
+  PetscCall(PetscObjectReference((PetscObject)V));
+  PetscCall(BVDestroy(&eps->V));
   eps->V = V;
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->V);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->V));
   PetscFunctionReturn(0);
 }
 
@@ -554,14 +589,14 @@ PetscErrorCode EPSSetBV(EPS eps,BV V)
 @*/
 PetscErrorCode EPSGetBV(EPS eps,BV *V)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(V,2);
   if (!eps->V) {
-    ierr = BVCreate(PetscObjectComm((PetscObject)eps),&eps->V);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->V);CHKERRQ(ierr);
+    PetscCall(BVCreate(PetscObjectComm((PetscObject)eps),&eps->V));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)eps->V,(PetscObject)eps,0));
+    PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->V));
+    PetscCall(PetscObjectSetOptions((PetscObject)eps->V,((PetscObject)eps)->options));
   }
   *V = eps->V;
   PetscFunctionReturn(0);
@@ -570,7 +605,7 @@ PetscErrorCode EPSGetBV(EPS eps,BV *V)
 /*@
    EPSSetRG - Associates a region object to the eigensolver.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context obtained from EPSCreate()
@@ -586,16 +621,16 @@ PetscErrorCode EPSGetBV(EPS eps,BV *V)
 @*/
 PetscErrorCode EPSSetRG(EPS eps,RG rg)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidHeaderSpecific(rg,RG_CLASSID,2);
-  PetscCheckSameComm(eps,1,rg,2);
-  ierr = PetscObjectReference((PetscObject)rg);CHKERRQ(ierr);
-  ierr = RGDestroy(&eps->rg);CHKERRQ(ierr);
+  if (rg) {
+    PetscValidHeaderSpecific(rg,RG_CLASSID,2);
+    PetscCheckSameComm(eps,1,rg,2);
+  }
+  PetscCall(PetscObjectReference((PetscObject)rg));
+  PetscCall(RGDestroy(&eps->rg));
   eps->rg = rg;
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->rg);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->rg));
   PetscFunctionReturn(0);
 }
 
@@ -616,14 +651,14 @@ PetscErrorCode EPSSetRG(EPS eps,RG rg)
 @*/
 PetscErrorCode EPSGetRG(EPS eps,RG *rg)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(rg,2);
   if (!eps->rg) {
-    ierr = RGCreate(PetscObjectComm((PetscObject)eps),&eps->rg);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->rg);CHKERRQ(ierr);
+    PetscCall(RGCreate(PetscObjectComm((PetscObject)eps),&eps->rg));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)eps->rg,(PetscObject)eps,0));
+    PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->rg));
+    PetscCall(PetscObjectSetOptions((PetscObject)eps->rg,((PetscObject)eps)->options));
   }
   *rg = eps->rg;
   PetscFunctionReturn(0);
@@ -632,7 +667,7 @@ PetscErrorCode EPSGetRG(EPS eps,RG *rg)
 /*@
    EPSSetDS - Associates a direct solver object to the eigensolver.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps - eigensolver context obtained from EPSCreate()
@@ -648,16 +683,14 @@ PetscErrorCode EPSGetRG(EPS eps,RG *rg)
 @*/
 PetscErrorCode EPSSetDS(EPS eps,DS ds)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidHeaderSpecific(ds,DS_CLASSID,2);
   PetscCheckSameComm(eps,1,ds,2);
-  ierr = PetscObjectReference((PetscObject)ds);CHKERRQ(ierr);
-  ierr = DSDestroy(&eps->ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectReference((PetscObject)ds));
+  PetscCall(DSDestroy(&eps->ds));
   eps->ds = ds;
-  ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->ds);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->ds));
   PetscFunctionReturn(0);
 }
 
@@ -678,14 +711,14 @@ PetscErrorCode EPSSetDS(EPS eps,DS ds)
 @*/
 PetscErrorCode EPSGetDS(EPS eps,DS *ds)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidPointer(ds,2);
   if (!eps->ds) {
-    ierr = DSCreate(PetscObjectComm((PetscObject)eps),&eps->ds);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->ds);CHKERRQ(ierr);
+    PetscCall(DSCreate(PetscObjectComm((PetscObject)eps),&eps->ds));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)eps->ds,(PetscObject)eps,0));
+    PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->ds));
+    PetscCall(PetscObjectSetOptions((PetscObject)eps->ds,((PetscObject)eps)->options));
   }
   *ds = eps->ds;
   PetscFunctionReturn(0);
@@ -711,7 +744,7 @@ PetscErrorCode EPSIsGeneralized(EPS eps,PetscBool* is)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(is,2);
+  PetscValidBoolPointer(is,2);
   *is = eps->isgeneralized;
   PetscFunctionReturn(0);
 }
@@ -736,7 +769,7 @@ PetscErrorCode EPSIsHermitian(EPS eps,PetscBool* is)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(is,2);
+  PetscValidBoolPointer(is,2);
   *is = eps->ishermitian;
   PetscFunctionReturn(0);
 }
@@ -761,8 +794,7 @@ PetscErrorCode EPSIsPositive(EPS eps,PetscBool* is)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(is,2);
+  PetscValidBoolPointer(is,2);
   *is = eps->ispositive;
   PetscFunctionReturn(0);
 }
-
