@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -14,18 +14,34 @@
 #include <slepc/private/epsimpl.h>   /*I "slepceps.h" I*/
 #include <petscdraw.h>
 
+PetscErrorCode EPSMonitorLGCreate(MPI_Comm comm,const char host[],const char label[],const char metric[],PetscInt l,const char *names[],int x,int y,int m,int n,PetscDrawLG *lgctx)
+{
+  PetscDraw      draw;
+  PetscDrawAxis  axis;
+  PetscDrawLG    lg;
+
+  PetscFunctionBegin;
+  PetscCall(PetscDrawCreate(comm,host,label,x,y,m,n,&draw));
+  PetscCall(PetscDrawSetFromOptions(draw));
+  PetscCall(PetscDrawLGCreate(draw,l,&lg));
+  if (names) PetscCall(PetscDrawLGSetLegend(lg,names));
+  PetscCall(PetscDrawLGSetFromOptions(lg));
+  PetscCall(PetscDrawLGGetAxis(lg,&axis));
+  PetscCall(PetscDrawAxisSetLabels(axis,"Convergence","Iteration",metric));
+  PetscCall(PetscDrawDestroy(&draw));
+  *lgctx = lg;
+  PetscFunctionReturn(0);
+}
+
 /*
    Runs the user provided monitor routines, if any.
 */
 PetscErrorCode EPSMonitor(EPS eps,PetscInt it,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest)
 {
-  PetscErrorCode ierr;
   PetscInt       i,n = eps->numbermonitors;
 
   PetscFunctionBegin;
-  for (i=0;i<n;i++) {
-    ierr = (*eps->monitor[i])(eps,it,nconv,eigr,eigi,errest,nest,eps->monitorcontext[i]);CHKERRQ(ierr);
-  }
+  for (i=0;i<n;i++) PetscCall((*eps->monitor[i])(eps,it,nconv,eigr,eigi,errest,nest,eps->monitorcontext[i]));
   PetscFunctionReturn(0);
 }
 
@@ -33,7 +49,7 @@ PetscErrorCode EPSMonitor(EPS eps,PetscInt it,PetscInt nconv,PetscScalar *eigr,P
    EPSMonitorSet - Sets an ADDITIONAL function to be called at every
    iteration to monitor the error estimates for each requested eigenpair.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 +  eps     - eigensolver context obtained from EPSCreate()
@@ -59,10 +75,11 @@ $   monitor(EPS eps,int its,int nconv,PetscScalar *eigr,PetscScalar *eigi,PetscR
 .    -eps_monitor_all    - print error estimates at each iteration
 .    -eps_monitor_conv   - print the eigenvalue approximations only when
       convergence has been reached
-.    -eps_monitor_lg     - sets line graph monitor for the first unconverged
+.    -eps_monitor draw::draw_lg - sets line graph monitor for the first unconverged
       approximate eigenvalue
-.    -eps_monitor_lg_all - sets line graph monitor for all unconverged
+.    -eps_monitor_all draw::draw_lg - sets line graph monitor for all unconverged
       approximate eigenvalues
+.    -eps_monitor_conv draw::draw_lg - sets line graph monitor for convergence history
 -    -eps_monitor_cancel - cancels all monitors that have been hardwired into
       a code by calls to EPSMonitorSet(), but does not cancel those set via
       the options database.
@@ -80,7 +97,7 @@ PetscErrorCode EPSMonitorSet(EPS eps,PetscErrorCode (*monitor)(EPS,PetscInt,Pets
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (eps->numbermonitors >= MAXEPSMONITORS) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Too many EPS monitors set");
+  PetscCheck(eps->numbermonitors<MAXEPSMONITORS,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Too many EPS monitors set");
   eps->monitor[eps->numbermonitors]           = monitor;
   eps->monitorcontext[eps->numbermonitors]    = (void*)mctx;
   eps->monitordestroy[eps->numbermonitors++]  = monitordestroy;
@@ -90,7 +107,7 @@ PetscErrorCode EPSMonitorSet(EPS eps,PetscErrorCode (*monitor)(EPS,PetscInt,Pets
 /*@
    EPSMonitorCancel - Clears all monitors for an EPS object.
 
-   Logically Collective on EPS
+   Logically Collective on eps
 
    Input Parameters:
 .  eps - eigensolver context obtained from EPSCreate()
@@ -106,15 +123,12 @@ PetscErrorCode EPSMonitorSet(EPS eps,PetscErrorCode (*monitor)(EPS,PetscInt,Pets
 @*/
 PetscErrorCode EPSMonitorCancel(EPS eps)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   for (i=0; i<eps->numbermonitors; i++) {
-    if (eps->monitordestroy[i]) {
-      ierr = (*eps->monitordestroy[i])(&eps->monitorcontext[i]);CHKERRQ(ierr);
-    }
+    if (eps->monitordestroy[i]) PetscCall((*eps->monitordestroy[i])(&eps->monitorcontext[i]));
   }
   eps->numbermonitors = 0;
   PetscFunctionReturn(0);
@@ -136,76 +150,19 @@ PetscErrorCode EPSMonitorCancel(EPS eps)
 
 .seealso: EPSMonitorSet()
 @*/
-PetscErrorCode EPSGetMonitorContext(EPS eps,void **ctx)
+PetscErrorCode EPSGetMonitorContext(EPS eps,void *ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  *ctx = eps->monitorcontext[0];
+  *(void**)ctx = eps->monitorcontext[0];
   PetscFunctionReturn(0);
 }
 
 /*@C
-   EPSMonitorAll - Print the current approximate values and
-   error estimates at each iteration of the eigensolver.
-
-   Collective on EPS
-
-   Input Parameters:
-+  eps    - eigensolver context
-.  its    - iteration number
-.  nconv  - number of converged eigenpairs so far
-.  eigr   - real part of the eigenvalues
-.  eigi   - imaginary part of the eigenvalues
-.  errest - error estimates
-.  nest   - number of error estimates to display
--  vf     - viewer and format for monitoring
-
-   Level: intermediate
-
-.seealso: EPSMonitorSet(), EPSMonitorFirst(), EPSMonitorConverged()
-@*/
-PetscErrorCode EPSMonitorAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
-{
-  PetscErrorCode ierr;
-  PetscInt       i;
-  PetscScalar    er,ei;
-  PetscViewer    viewer;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(vf,8);
-  viewer = vf->viewer;
-  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
-  ierr = PetscViewerPushFormat(viewer,vf->format);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
-  if (its==1 && ((PetscObject)eps)->prefix) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Eigenvalue approximations and residual norms for %s solve.\n",((PetscObject)eps)->prefix);CHKERRQ(ierr);
-  }
-  ierr = PetscViewerASCIIPrintf(viewer,"%3D EPS nconv=%D Values (Errors)",its,nconv);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
-  for (i=0;i<nest;i++) {
-    er = eigr[i]; ei = eigi[i];
-    ierr = STBackTransform(eps->st,1,&er,&ei);CHKERRQ(ierr);
-#if defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerASCIIPrintf(viewer," %g%+gi",(double)PetscRealPart(er),(double)PetscImaginaryPart(er));CHKERRQ(ierr);
-#else
-    ierr = PetscViewerASCIIPrintf(viewer," %g",(double)er);CHKERRQ(ierr);
-    if (ei!=0.0) { ierr = PetscViewerASCIIPrintf(viewer,"%+gi",(double)ei);CHKERRQ(ierr); }
-#endif
-    ierr = PetscViewerASCIIPrintf(viewer," (%10.8e)",(double)errest[i]);CHKERRQ(ierr);
-  }
-  ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
-  ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
-  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-/*@C
-   EPSMonitorFirst - Print the first approximate value and
+   EPSMonitorFirst - Print the first unconverged approximate value and
    error estimate at each iteration of the eigensolver.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps    - eigensolver context
@@ -217,50 +174,48 @@ PetscErrorCode EPSMonitorAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *ei
 .  nest   - number of error estimates to display
 -  vf     - viewer and format for monitoring
 
+   Options Database Key:
+.  -eps_monitor - activates EPSMonitorFirst()
+
    Level: intermediate
 
 .seealso: EPSMonitorSet(), EPSMonitorAll(), EPSMonitorConverged()
 @*/
 PetscErrorCode EPSMonitorFirst(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
 {
-  PetscErrorCode ierr;
   PetscScalar    er,ei;
-  PetscViewer    viewer;
+  PetscViewer    viewer = vf->viewer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(vf,8);
-  viewer = vf->viewer;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
-  if (its==1 && ((PetscObject)eps)->prefix) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Eigenvalue approximations and residual norms for %s solve.\n",((PetscObject)eps)->prefix);CHKERRQ(ierr);
-  }
+  if (its==1 && ((PetscObject)eps)->prefix) PetscCall(PetscViewerASCIIPrintf(viewer,"  Eigenvalue approximations and residual norms for %s solve.\n",((PetscObject)eps)->prefix));
   if (nconv<nest) {
-    ierr = PetscViewerPushFormat(viewer,vf->format);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"%3D EPS nconv=%D first unconverged value (error)",its,nconv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
+    PetscCall(PetscViewerPushFormat(viewer,vf->format));
+    PetscCall(PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel));
+    PetscCall(PetscViewerASCIIPrintf(viewer,"%3" PetscInt_FMT " EPS nconv=%" PetscInt_FMT " first unconverged value (error)",its,nconv));
+    PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
     er = eigr[nconv]; ei = eigi[nconv];
-    ierr = STBackTransform(eps->st,1,&er,&ei);CHKERRQ(ierr);
+    PetscCall(STBackTransform(eps->st,1,&er,&ei));
 #if defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerASCIIPrintf(viewer," %g%+gi",(double)PetscRealPart(er),(double)PetscImaginaryPart(er));CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIIPrintf(viewer," %g%+gi",(double)PetscRealPart(er),(double)PetscImaginaryPart(er)));
 #else
-    ierr = PetscViewerASCIIPrintf(viewer," %g",(double)er);CHKERRQ(ierr);
-    if (ei!=0.0) { ierr = PetscViewerASCIIPrintf(viewer,"%+gi",(double)ei);CHKERRQ(ierr); }
+    PetscCall(PetscViewerASCIIPrintf(viewer," %g",(double)er));
+    if (ei!=0.0) PetscCall(PetscViewerASCIIPrintf(viewer,"%+gi",(double)ei));
 #endif
-    ierr = PetscViewerASCIIPrintf(viewer," (%10.8e)\n",(double)errest[nconv]);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
-    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIIPrintf(viewer," (%10.8e)\n",(double)errest[nconv]));
+    PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
+    PetscCall(PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel));
+    PetscCall(PetscViewerPopFormat(viewer));
   }
   PetscFunctionReturn(0);
 }
 
 /*@C
-   EPSMonitorConverged - Print the approximate values and
-   error estimates as they converge.
+   EPSMonitorAll - Print the current approximate values and
+   error estimates at each iteration of the eigensolver.
 
-   Collective on EPS
+   Collective on eps
 
    Input Parameters:
 +  eps    - eigensolver context
@@ -270,146 +225,370 @@ PetscErrorCode EPSMonitorFirst(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *
 .  eigi   - imaginary part of the eigenvalues
 .  errest - error estimates
 .  nest   - number of error estimates to display
--  ctx    - monitor context
+-  vf     - viewer and format for monitoring
+
+   Options Database Key:
+.  -eps_monitor_all - activates EPSMonitorAll()
+
+   Level: intermediate
+
+.seealso: EPSMonitorSet(), EPSMonitorFirst(), EPSMonitorConverged()
+@*/
+PetscErrorCode EPSMonitorAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
+{
+  PetscInt       i;
+  PetscScalar    er,ei;
+  PetscViewer    viewer = vf->viewer;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
+  PetscCall(PetscViewerPushFormat(viewer,vf->format));
+  PetscCall(PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel));
+  if (its==1 && ((PetscObject)eps)->prefix) PetscCall(PetscViewerASCIIPrintf(viewer,"  Eigenvalue approximations and residual norms for %s solve.\n",((PetscObject)eps)->prefix));
+  PetscCall(PetscViewerASCIIPrintf(viewer,"%3" PetscInt_FMT " EPS nconv=%" PetscInt_FMT " Values (Errors)",its,nconv));
+  PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
+  for (i=0;i<nest;i++) {
+    er = eigr[i]; ei = eigi[i];
+    PetscCall(STBackTransform(eps->st,1,&er,&ei));
+#if defined(PETSC_USE_COMPLEX)
+    PetscCall(PetscViewerASCIIPrintf(viewer," %g%+gi",(double)PetscRealPart(er),(double)PetscImaginaryPart(er)));
+#else
+    PetscCall(PetscViewerASCIIPrintf(viewer," %g",(double)er));
+    if (ei!=0.0) PetscCall(PetscViewerASCIIPrintf(viewer,"%+gi",(double)ei));
+#endif
+    PetscCall(PetscViewerASCIIPrintf(viewer," (%10.8e)",(double)errest[i]));
+  }
+  PetscCall(PetscViewerASCIIPrintf(viewer,"\n"));
+  PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
+  PetscCall(PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel));
+  PetscCall(PetscViewerPopFormat(viewer));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   EPSMonitorConverged - Print the approximate values and
+   error estimates as they converge.
+
+   Collective on eps
+
+   Input Parameters:
++  eps    - eigensolver context
+.  its    - iteration number
+.  nconv  - number of converged eigenpairs so far
+.  eigr   - real part of the eigenvalues
+.  eigi   - imaginary part of the eigenvalues
+.  errest - error estimates
+.  nest   - number of error estimates to display
+-  vf     - viewer and format for monitoring
+
+   Options Database Key:
+.  -eps_monitor_conv - activates EPSMonitorConverged()
 
    Level: intermediate
 
 .seealso: EPSMonitorSet(), EPSMonitorFirst(), EPSMonitorAll()
 @*/
-PetscErrorCode EPSMonitorConverged(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,SlepcConvMonitor ctx)
+PetscErrorCode EPSMonitorConverged(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
   PetscScalar    er,ei;
-  PetscViewer    viewer;
+  PetscViewer    viewer = vf->viewer;
+  SlepcConvMon   ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(ctx,8);
-  viewer = ctx->viewer;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
-  if (its==1 && ((PetscObject)eps)->prefix) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Convergence history for %s solve.\n",((PetscObject)eps)->prefix);CHKERRQ(ierr);
-  }
+  ctx = (SlepcConvMon)vf->data;
+  if (its==1 && ((PetscObject)eps)->prefix) PetscCall(PetscViewerASCIIPrintf(viewer,"  Convergence history for %s solve.\n",((PetscObject)eps)->prefix));
   if (its==1) ctx->oldnconv = 0;
   if (ctx->oldnconv!=nconv) {
-    ierr = PetscViewerPushFormat(viewer,ctx->format);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
+    PetscCall(PetscViewerPushFormat(viewer,vf->format));
+    PetscCall(PetscViewerASCIIAddTab(viewer,((PetscObject)eps)->tablevel));
     for (i=ctx->oldnconv;i<nconv;i++) {
-      ierr = PetscViewerASCIIPrintf(viewer,"%3D EPS converged value (error) #%D",its,i);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"%3" PetscInt_FMT " EPS converged value (error) #%" PetscInt_FMT,its,i));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
       er = eigr[i]; ei = eigi[i];
-      ierr = STBackTransform(eps->st,1,&er,&ei);CHKERRQ(ierr);
+      PetscCall(STBackTransform(eps->st,1,&er,&ei));
 #if defined(PETSC_USE_COMPLEX)
-      ierr = PetscViewerASCIIPrintf(viewer," %g%+gi",(double)PetscRealPart(er),(double)PetscImaginaryPart(er));CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer," %g%+gi",(double)PetscRealPart(er),(double)PetscImaginaryPart(er)));
 #else
-      ierr = PetscViewerASCIIPrintf(viewer," %g",(double)er);CHKERRQ(ierr);
-      if (ei!=0.0) { ierr = PetscViewerASCIIPrintf(viewer,"%+gi",(double)ei);CHKERRQ(ierr); }
+      PetscCall(PetscViewerASCIIPrintf(viewer," %g",(double)er));
+      if (ei!=0.0) PetscCall(PetscViewerASCIIPrintf(viewer,"%+gi",(double)ei));
 #endif
-      ierr = PetscViewerASCIIPrintf(viewer," (%10.8e)\n",(double)errest[i]);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer," (%10.8e)\n",(double)errest[i]));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
     }
-    ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel);CHKERRQ(ierr);
-    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIISubtractTab(viewer,((PetscObject)eps)->tablevel));
+    PetscCall(PetscViewerPopFormat(viewer));
     ctx->oldnconv = nconv;
   }
   PetscFunctionReturn(0);
 }
 
-/*@C
-   EPSMonitorLGCreate - Creates a line graph context for use with
-   EPS to monitor convergence.
+PetscErrorCode EPSMonitorConvergedCreate(PetscViewer viewer,PetscViewerFormat format,void *ctx,PetscViewerAndFormat **vf)
+{
+  SlepcConvMon   mctx;
 
-   Collective on MPI_Comm
+  PetscFunctionBegin;
+  PetscCall(PetscViewerAndFormatCreate(viewer,format,vf));
+  PetscCall(PetscNew(&mctx));
+  mctx->ctx = ctx;
+  (*vf)->data = (void*)mctx;
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode EPSMonitorConvergedDestroy(PetscViewerAndFormat **vf)
+{
+  PetscFunctionBegin;
+  if (!*vf) PetscFunctionReturn(0);
+  PetscCall(PetscFree((*vf)->data));
+  PetscCall(PetscViewerDestroy(&(*vf)->viewer));
+  PetscCall(PetscDrawLGDestroy(&(*vf)->lg));
+  PetscCall(PetscFree(*vf));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   EPSMonitorFirstDrawLG - Plots the error estimate of the first unconverged
+   approximation at each iteration of the eigensolver.
+
+   Collective on eps
 
    Input Parameters:
-+  comm - communicator context
-.  host - the X display to open, or null for the local machine
-.  label - the title to put in the title bar
-.  x, y - the screen coordinates of the upper left coordinate of
-          the window
--  m, n - the screen width and height in pixels
++  eps    - eigensolver context
+.  its    - iteration number
+.  its    - iteration number
+.  nconv  - number of converged eigenpairs so far
+.  eigr   - real part of the eigenvalues
+.  eigi   - imaginary part of the eigenvalues
+.  errest - error estimates
+.  nest   - number of error estimates to display
+-  vf     - viewer and format for monitoring
 
-   Output Parameter:
-.  lgctx - the drawing context
-
-   Options Database Keys:
-+  -eps_monitor_lg - Sets line graph monitor for the first residual
--  -eps_monitor_lg_all - Sets line graph monitor for all residuals
-
-   Notes:
-   Use PetscDrawLGDestroy() to destroy this line graph.
+   Options Database Key:
+.  -eps_monitor draw::draw_lg - activates EPSMonitorFirstDrawLG()
 
    Level: intermediate
 
 .seealso: EPSMonitorSet()
 @*/
-PetscErrorCode EPSMonitorLGCreate(MPI_Comm comm,const char host[],const char label[],int x,int y,int m,int n,PetscDrawLG *lgctx)
+PetscErrorCode EPSMonitorFirstDrawLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
 {
-  PetscDraw      draw;
-  PetscDrawLG    lg;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscDrawCreate(comm,host,label,x,y,m,n,&draw);CHKERRQ(ierr);
-  ierr = PetscDrawSetFromOptions(draw);CHKERRQ(ierr);
-  ierr = PetscDrawLGCreate(draw,1,&lg);CHKERRQ(ierr);
-  ierr = PetscDrawLGSetFromOptions(lg);CHKERRQ(ierr);
-  ierr = PetscDrawDestroy(&draw);CHKERRQ(ierr);
-  *lgctx = lg;
-  PetscFunctionReturn(0);
-}
-
-PetscErrorCode EPSMonitorLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
-{
-  PetscDrawLG    lg = (PetscDrawLG)ctx;
+  PetscViewer    viewer = vf->viewer;
+  PetscDrawLG    lg = vf->lg;
   PetscReal      x,y;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
   PetscValidHeaderSpecific(lg,PETSC_DRAWLG_CLASSID,8);
+  PetscCall(PetscViewerPushFormat(viewer,vf->format));
   if (its==1) {
-    ierr = PetscDrawLGReset(lg);CHKERRQ(ierr);
-    ierr = PetscDrawLGSetDimension(lg,1);CHKERRQ(ierr);
-    ierr = PetscDrawLGSetLimits(lg,1,1.0,PetscLog10Real(eps->tol)-2,0.0);CHKERRQ(ierr);
+    PetscCall(PetscDrawLGReset(lg));
+    PetscCall(PetscDrawLGSetDimension(lg,1));
+    PetscCall(PetscDrawLGSetLimits(lg,1,1,PetscLog10Real(eps->tol)-2,0.0));
   }
-  x = (PetscReal)its;
-  if (errest[nconv] > 0.0) y = PetscLog10Real(errest[nconv]);
-  else y = 0.0;
-  ierr = PetscDrawLGAddPoint(lg,&x,&y);CHKERRQ(ierr);
-  if (its <= 20 || !(its % 5) || eps->reason) {
-    ierr = PetscDrawLGDraw(lg);CHKERRQ(ierr);
-    ierr = PetscDrawLGSave(lg);CHKERRQ(ierr);
+  if (nconv<nest) {
+    x = (PetscReal)its;
+    if (errest[nconv] > 0.0) y = PetscLog10Real(errest[nconv]);
+    else y = 0.0;
+    PetscCall(PetscDrawLGAddPoint(lg,&x,&y));
+    if (its <= 20 || !(its % 5) || eps->reason) {
+      PetscCall(PetscDrawLGDraw(lg));
+      PetscCall(PetscDrawLGSave(lg));
+    }
   }
+  PetscCall(PetscViewerPopFormat(viewer));
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode EPSMonitorLGAll(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
+/*@C
+   EPSMonitorFirstDrawLGCreate - Creates the plotter for the first error estimate.
+
+   Collective on viewer
+
+   Input Parameters:
++  viewer - the viewer
+.  format - the viewer format
+-  ctx    - an optional user context
+
+   Output Parameter:
+.  vf     - the viewer and format context
+
+   Level: intermediate
+
+.seealso: EPSMonitorSet()
+@*/
+PetscErrorCode EPSMonitorFirstDrawLGCreate(PetscViewer viewer,PetscViewerFormat format,void *ctx,PetscViewerAndFormat **vf)
 {
-  PetscDrawLG    lg = (PetscDrawLG)ctx;
+  PetscFunctionBegin;
+  PetscCall(PetscViewerAndFormatCreate(viewer,format,vf));
+  (*vf)->data = ctx;
+  PetscCall(EPSMonitorLGCreate(PetscObjectComm((PetscObject)viewer),NULL,"First Error Estimate","Log Error Estimate",1,NULL,PETSC_DECIDE,PETSC_DECIDE,400,300,&(*vf)->lg));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   EPSMonitorAllDrawLG - Plots the error estimate of all unconverged
+   approximations at each iteration of the eigensolver.
+
+   Collective on eps
+
+   Input Parameters:
++  eps    - eigensolver context
+.  its    - iteration number
+.  its    - iteration number
+.  nconv  - number of converged eigenpairs so far
+.  eigr   - real part of the eigenvalues
+.  eigi   - imaginary part of the eigenvalues
+.  errest - error estimates
+.  nest   - number of error estimates to display
+-  vf     - viewer and format for monitoring
+
+   Options Database Key:
+.  -eps_monitor_all draw::draw_lg - activates EPSMonitorAllDrawLG()
+
+   Level: intermediate
+
+.seealso: EPSMonitorSet()
+@*/
+PetscErrorCode EPSMonitorAllDrawLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
+{
+  PetscViewer    viewer = vf->viewer;
+  PetscDrawLG    lg = vf->lg;
   PetscInt       i,n = PetscMin(eps->nev,255);
   PetscReal      *x,*y;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
   PetscValidHeaderSpecific(lg,PETSC_DRAWLG_CLASSID,8);
+  PetscCall(PetscViewerPushFormat(viewer,vf->format));
   if (its==1) {
-    ierr = PetscDrawLGReset(lg);CHKERRQ(ierr);
-    ierr = PetscDrawLGSetDimension(lg,n);CHKERRQ(ierr);
-    ierr = PetscDrawLGSetLimits(lg,1,1.0,PetscLog10Real(eps->tol)-2,0.0);CHKERRQ(ierr);
+    PetscCall(PetscDrawLGReset(lg));
+    PetscCall(PetscDrawLGSetDimension(lg,n));
+    PetscCall(PetscDrawLGSetLimits(lg,1,1,PetscLog10Real(eps->tol)-2,0.0));
   }
-  ierr = PetscMalloc2(n,&x,n,&y);CHKERRQ(ierr);
+  PetscCall(PetscMalloc2(n,&x,n,&y));
   for (i=0;i<n;i++) {
     x[i] = (PetscReal)its;
     if (i < nest && errest[i] > 0.0) y[i] = PetscLog10Real(errest[i]);
     else y[i] = 0.0;
   }
-  ierr = PetscDrawLGAddPoint(lg,x,y);CHKERRQ(ierr);
+  PetscCall(PetscDrawLGAddPoint(lg,x,y));
   if (its <= 20 || !(its % 5) || eps->reason) {
-    ierr = PetscDrawLGDraw(lg);CHKERRQ(ierr);
-    ierr = PetscDrawLGSave(lg);CHKERRQ(ierr);
+    PetscCall(PetscDrawLGDraw(lg));
+    PetscCall(PetscDrawLGSave(lg));
   }
-  ierr = PetscFree2(x,y);CHKERRQ(ierr);
+  PetscCall(PetscFree2(x,y));
+  PetscCall(PetscViewerPopFormat(viewer));
   PetscFunctionReturn(0);
 }
 
+/*@C
+   EPSMonitorAllDrawLGCreate - Creates the plotter for all the error estimates.
+
+   Collective on viewer
+
+   Input Parameters:
++  viewer - the viewer
+.  format - the viewer format
+-  ctx    - an optional user context
+
+   Output Parameter:
+.  vf     - the viewer and format context
+
+   Level: intermediate
+
+.seealso: EPSMonitorSet()
+@*/
+PetscErrorCode EPSMonitorAllDrawLGCreate(PetscViewer viewer,PetscViewerFormat format,void *ctx,PetscViewerAndFormat **vf)
+{
+  PetscFunctionBegin;
+  PetscCall(PetscViewerAndFormatCreate(viewer,format,vf));
+  (*vf)->data = ctx;
+  PetscCall(EPSMonitorLGCreate(PetscObjectComm((PetscObject)viewer),NULL,"All Error Estimates","Log Error Estimate",1,NULL,PETSC_DECIDE,PETSC_DECIDE,400,300,&(*vf)->lg));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   EPSMonitorConvergedDrawLG - Plots the number of converged eigenvalues
+   at each iteration of the eigensolver.
+
+   Collective on eps
+
+   Input Parameters:
++  eps    - eigensolver context
+.  its    - iteration number
+.  its    - iteration number
+.  nconv  - number of converged eigenpairs so far
+.  eigr   - real part of the eigenvalues
+.  eigi   - imaginary part of the eigenvalues
+.  errest - error estimates
+.  nest   - number of error estimates to display
+-  vf     - viewer and format for monitoring
+
+   Options Database Key:
+.  -eps_monitor_conv draw::draw_lg - activates EPSMonitorConvergedDrawLG()
+
+   Level: intermediate
+
+.seealso: EPSMonitorSet()
+@*/
+PetscErrorCode EPSMonitorConvergedDrawLG(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,PetscViewerAndFormat *vf)
+{
+  PetscViewer      viewer = vf->viewer;
+  PetscDrawLG      lg = vf->lg;
+  PetscReal        x,y;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,8);
+  PetscValidHeaderSpecific(lg,PETSC_DRAWLG_CLASSID,8);
+  PetscCall(PetscViewerPushFormat(viewer,vf->format));
+  if (its==1) {
+    PetscCall(PetscDrawLGReset(lg));
+    PetscCall(PetscDrawLGSetDimension(lg,1));
+    PetscCall(PetscDrawLGSetLimits(lg,1,1,0,eps->nev));
+  }
+  x = (PetscReal)its;
+  y = (PetscReal)eps->nconv;
+  PetscCall(PetscDrawLGAddPoint(lg,&x,&y));
+  if (its <= 20 || !(its % 5) || eps->reason) {
+    PetscCall(PetscDrawLGDraw(lg));
+    PetscCall(PetscDrawLGSave(lg));
+  }
+  PetscCall(PetscViewerPopFormat(viewer));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   EPSMonitorConvergedDrawLGCreate - Creates the plotter for the convergence history.
+
+   Collective on viewer
+
+   Input Parameters:
++  viewer - the viewer
+.  format - the viewer format
+-  ctx    - an optional user context
+
+   Output Parameter:
+.  vf     - the viewer and format context
+
+   Level: intermediate
+
+.seealso: EPSMonitorSet()
+@*/
+PetscErrorCode EPSMonitorConvergedDrawLGCreate(PetscViewer viewer,PetscViewerFormat format,void *ctx,PetscViewerAndFormat **vf)
+{
+  SlepcConvMon   mctx;
+
+  PetscFunctionBegin;
+  PetscCall(PetscViewerAndFormatCreate(viewer,format,vf));
+  PetscCall(PetscNew(&mctx));
+  mctx->ctx = ctx;
+  (*vf)->data = (void*)mctx;
+  PetscCall(EPSMonitorLGCreate(PetscObjectComm((PetscObject)viewer),NULL,"Convergence History","Number Converged",1,NULL,PETSC_DECIDE,PETSC_DECIDE,400,300,&(*vf)->lg));
+  PetscFunctionReturn(0);
+}
