@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -21,13 +21,9 @@
 */
 PetscErrorCode EPSSetDefaultST(EPS eps)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  if (eps->ops->setdefaultst) { ierr = (*eps->ops->setdefaultst)(eps);CHKERRQ(ierr); }
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STSHIFT);CHKERRQ(ierr);
-  }
+  if (eps->ops->setdefaultst) PetscCall((*eps->ops->setdefaultst)(eps));
+  if (!((PetscObject)eps->st)->type_name) PetscCall(STSetType(eps->st,STSHIFT));
   PetscFunctionReturn(0);
 }
 
@@ -37,17 +33,12 @@ PetscErrorCode EPSSetDefaultST(EPS eps)
 */
 PetscErrorCode EPSSetDefaultST_Precond(EPS eps)
 {
-  PetscErrorCode ierr;
   KSP            ksp;
 
   PetscFunctionBegin;
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  }
-  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
-  if (!((PetscObject)ksp)->type_name) {
-    ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
-  }
+  if (!((PetscObject)eps->st)->type_name) PetscCall(STSetType(eps->st,STPRECOND));
+  PetscCall(STGetKSP(eps->st,&ksp));
+  if (!((PetscObject)ksp)->type_name) PetscCall(KSPSetType(ksp,KSPPREONLY));
   PetscFunctionReturn(0);
 }
 
@@ -57,171 +48,124 @@ PetscErrorCode EPSSetDefaultST_Precond(EPS eps)
 */
 PetscErrorCode EPSSetDefaultST_GMRES(EPS eps)
 {
-  PetscErrorCode ierr;
   KSP            ksp;
 
   PetscFunctionBegin;
-  if (!((PetscObject)eps->st)->type_name) {
-    ierr = STSetType(eps->st,STPRECOND);CHKERRQ(ierr);
-  }
-  ierr = STPrecondSetKSPHasMat(eps->st,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
+  if (!((PetscObject)eps->st)->type_name) PetscCall(STSetType(eps->st,STPRECOND));
+  PetscCall(STPrecondSetKSPHasMat(eps->st,PETSC_TRUE));
+  PetscCall(STGetKSP(eps->st,&ksp));
   if (!((PetscObject)ksp)->type_name) {
-    ierr = KSPSetType(ksp,KSPGMRES);CHKERRQ(ierr);
-    ierr = KSPSetTolerances(ksp,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT,5);CHKERRQ(ierr);
+    PetscCall(KSPSetType(ksp,KSPGMRES));
+    PetscCall(KSPSetTolerances(ksp,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT,5));
   }
   PetscFunctionReturn(0);
 }
 
+#if defined(SLEPC_HAVE_SCALAPACK) || defined(SLEPC_HAVE_ELPA) || defined(SLEPC_HAVE_ELEMENTAL) || defined(SLEPC_HAVE_EVSL)
+/*
+   This is for direct eigensolvers that work with A and B directly, so
+   no need to factorize B.
+*/
+PetscErrorCode EPSSetDefaultST_NoFactor(EPS eps)
+{
+  KSP            ksp;
+  PC             pc;
+
+  PetscFunctionBegin;
+  if (!((PetscObject)eps->st)->type_name) PetscCall(STSetType(eps->st,STSHIFT));
+  PetscCall(STGetKSP(eps->st,&ksp));
+  if (!((PetscObject)ksp)->type_name) PetscCall(KSPSetType(ksp,KSPPREONLY));
+  PetscCall(KSPGetPC(ksp,&pc));
+  if (!((PetscObject)pc)->type_name) PetscCall(PCSetType(pc,PCNONE));
+  PetscFunctionReturn(0);
+}
+#endif
+
 /*
    Check that the ST selected by the user is compatible with the EPS solver and options
 */
 PetscErrorCode EPSCheckCompatibleST(EPS eps)
 {
-  PetscErrorCode ierr;
   PetscBool      precond,shift,sinvert,cayley,lyapii;
 #if defined(PETSC_USE_COMPLEX)
   PetscScalar    sigma;
 #endif
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&shift);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&sinvert);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&cayley);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STPRECOND,&precond));
+  PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STSHIFT,&shift));
+  PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&sinvert));
+  PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&cayley));
 
   /* preconditioned eigensolvers */
-  if (eps->categ==EPS_CATEGORY_PRECOND && !precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires ST=PRECOND");
-  if (eps->categ!=EPS_CATEGORY_PRECOND && precond) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"STPRECOND is intended for preconditioned eigensolvers only");
+  PetscCheck(eps->categ!=EPS_CATEGORY_PRECOND || precond,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver requires ST=PRECOND");
+  PetscCheck(eps->categ==EPS_CATEGORY_PRECOND || !precond,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"STPRECOND is intended for preconditioned eigensolvers only");
 
   /* harmonic extraction */
-  if (!(precond || shift) && eps->extraction && eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cannot use a spectral transformation combined with harmonic extraction");
+  PetscCheck(precond || shift || !eps->extraction || eps->extraction==EPS_RITZ,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cannot use a spectral transformation combined with harmonic extraction");
 
   /* real shifts in Hermitian problems */
 #if defined(PETSC_USE_COMPLEX)
-  ierr = STGetShift(eps->st,&sigma);CHKERRQ(ierr);
-  if (eps->ishermitian && PetscImaginaryPart(sigma) != 0.0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Hermitian problems are not compatible with complex shifts");
+  PetscCall(STGetShift(eps->st,&sigma));
+  PetscCheck(!eps->ishermitian || PetscImaginaryPart(sigma)==0.0,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Hermitian problems are not compatible with complex shifts");
 #endif
 
   /* Cayley with PGNHEP */
-  if (cayley && eps->problem_type == EPS_PGNHEP) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
+  PetscCheck(!cayley || eps->problem_type!=EPS_PGNHEP,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Cayley spectral transformation is not compatible with PGNHEP");
 
   /* make sure that the user does not specify smallest magnitude with shift-and-invert */
   if ((cayley || sinvert) && (eps->categ==EPS_CATEGORY_KRYLOV || eps->categ==EPS_CATEGORY_OTHER)) {
-    ierr = PetscObjectTypeCompare((PetscObject)eps,EPSLYAPII,&lyapii);CHKERRQ(ierr);
-    if (!lyapii && eps->which!=EPS_TARGET_MAGNITUDE && eps->which!=EPS_TARGET_REAL && eps->which!=EPS_TARGET_IMAGINARY && eps->which!=EPS_ALL && eps->which!=EPS_WHICH_USER) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Shift-and-invert requires a target 'which' (see EPSSetWhichEigenpairs), for instance -st_type sinvert -eps_target 0 -eps_target_magnitude");
+    PetscCall(PetscObjectTypeCompare((PetscObject)eps,EPSLYAPII,&lyapii));
+    PetscCheck(lyapii || eps->which==EPS_TARGET_MAGNITUDE || eps->which==EPS_TARGET_REAL || eps->which==EPS_TARGET_IMAGINARY || eps->which==EPS_ALL || eps->which==EPS_WHICH_USER,PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"Shift-and-invert requires a target 'which' (see EPSSetWhichEigenpairs), for instance -st_type sinvert -eps_target 0 -eps_target_magnitude");
   }
   PetscFunctionReturn(0);
 }
 
-/*@
-   EPSSetUp - Sets up all the internal data structures necessary for the
-   execution of the eigensolver. Then calls STSetUp() for any set-up
-   operations associated to the ST object.
-
-   Collective on eps
-
-   Input Parameter:
-.  eps   - eigenproblem solver context
-
-   Notes:
-   This function need not be called explicitly in most cases, since EPSSolve()
-   calls it. It can be useful when one wants to measure the set-up time
-   separately from the solve time.
-
-   Level: developer
-
-.seealso: EPSCreate(), EPSSolve(), EPSDestroy(), STSetUp(), EPSSetInitialSpace()
-@*/
-PetscErrorCode EPSSetUp(EPS eps)
+/*
+   MatEstimateSpectralRange_EPS: estimate the spectral range [left,right] of a
+   symmetric/Hermitian matrix A using an auxiliary EPS object
+*/
+PetscErrorCode MatEstimateSpectralRange_EPS(Mat A,PetscReal *left,PetscReal *right)
 {
-  PetscErrorCode ierr;
-  Mat            A,B;
-  SlepcSC        sc;
-  PetscInt       k,nmat;
-  PetscBool      flg,istrivial;
+  PetscInt       nconv;
+  PetscScalar    eig0;
+  PetscReal      tol=1e-3,errest=tol;
+  EPS            eps;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  if (eps->state) PetscFunctionReturn(0);
-  ierr = PetscLogEventBegin(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
+  *left = 0.0; *right = 0.0;
+  PetscCall(EPSCreate(PetscObjectComm((PetscObject)A),&eps));
+  PetscCall(EPSSetOptionsPrefix(eps,"eps_filter_"));
+  PetscCall(EPSSetOperators(eps,A,NULL));
+  PetscCall(EPSSetProblemType(eps,EPS_HEP));
+  PetscCall(EPSSetTolerances(eps,tol,50));
+  PetscCall(EPSSetConvergenceTest(eps,EPS_CONV_ABS));
+  PetscCall(EPSSetWhichEigenpairs(eps,EPS_SMALLEST_REAL));
+  PetscCall(EPSSolve(eps));
+  PetscCall(EPSGetConverged(eps,&nconv));
+  if (nconv>0) {
+    PetscCall(EPSGetEigenvalue(eps,0,&eig0,NULL));
+    PetscCall(EPSGetErrorEstimate(eps,0,&errest));
+  } else eig0 = eps->eigr[0];
+  *left = PetscRealPart(eig0)-errest;
+  PetscCall(EPSSetWhichEigenpairs(eps,EPS_LARGEST_REAL));
+  PetscCall(EPSSolve(eps));
+  PetscCall(EPSGetConverged(eps,&nconv));
+  if (nconv>0) {
+    PetscCall(EPSGetEigenvalue(eps,0,&eig0,NULL));
+    PetscCall(EPSGetErrorEstimate(eps,0,&errest));
+  } else eig0 = eps->eigr[0];
+  *right = PetscRealPart(eig0)+errest;
+  PetscCall(EPSDestroy(&eps));
+  PetscFunctionReturn(0);
+}
 
-  /* reset the convergence flag from the previous solves */
-  eps->reason = EPS_CONVERGED_ITERATING;
-
-  /* Set default solver type (EPSSetFromOptions was not called) */
-  if (!((PetscObject)eps)->type_name) {
-    ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
-  }
-  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
-  ierr = EPSSetDefaultST(eps);CHKERRQ(ierr);
-
-  ierr = STSetTransform(eps->st,PETSC_TRUE);CHKERRQ(ierr);
-  if (eps->useds && !eps->ds) { ierr = EPSGetDS(eps,&eps->ds);CHKERRQ(ierr); }
-  if (eps->twosided) {
-    if (!eps->hasts) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support computing left eigenvectors (no two-sided variant)");
-    if (eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Two-sided methods are not intended for symmetric problems");
-    if (!eps->dsts) { ierr = DSDuplicate(eps->ds,&eps->dsts);CHKERRQ(ierr); }
-  }
-  if (!eps->rg) { ierr = EPSGetRG(eps,&eps->rg);CHKERRQ(ierr); }
-  if (!((PetscObject)eps->rg)->type_name) {
-    ierr = RGSetType(eps->rg,RGINTERVAL);CHKERRQ(ierr);
-  }
-
-  /* Set problem dimensions */
-  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  if (!nmat) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first");
-  ierr = STMatGetSize(eps->st,&eps->n,NULL);CHKERRQ(ierr);
-  ierr = STMatGetLocalSize(eps->st,&eps->nloc,NULL);CHKERRQ(ierr);
-
-  /* Set default problem type */
-  if (!eps->problem_type) {
-    if (nmat==1) {
-      ierr = EPSSetProblemType(eps,EPS_NHEP);CHKERRQ(ierr);
-    } else {
-      ierr = EPSSetProblemType(eps,EPS_GNHEP);CHKERRQ(ierr);
-    }
-  } else if (nmat==1 && eps->isgeneralized) {
-    ierr = PetscInfo(eps,"Eigenproblem set as generalized but no matrix B was provided; reverting to a standard eigenproblem\n");CHKERRQ(ierr);
-    eps->isgeneralized = PETSC_FALSE;
-    eps->problem_type = eps->ishermitian? EPS_HEP: EPS_NHEP;
-  } else if (nmat>1 && !eps->isgeneralized) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state");
-
-  if (eps->nev > eps->n) eps->nev = eps->n;
-  if (eps->ncv > eps->n) eps->ncv = eps->n;
-
-  /* initialization of matrix norms */
-  if (eps->conv==EPS_CONV_NORM) {
-    if (!eps->nrma) {
-      ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
-      ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
-    }
-    if (nmat>1 && !eps->nrmb) {
-      ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
-      ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
-    }
-  }
-
-  /* call specific solver setup */
-  ierr = (*eps->ops->setup)(eps);CHKERRQ(ierr);
-
-  /* if purification is set, check that it really makes sense */
-  if (eps->purify) {
-    if (eps->categ==EPS_CATEGORY_PRECOND || eps->categ==EPS_CATEGORY_CONTOUR) eps->purify = PETSC_FALSE;
-    else {
-      if (!eps->isgeneralized) eps->purify = PETSC_FALSE;
-      else if (!eps->ishermitian && !eps->ispositive) eps->purify = PETSC_FALSE;
-      else {
-        ierr = PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg);CHKERRQ(ierr);
-        if (flg) eps->purify = PETSC_FALSE;
-      }
-    }
-  }
-
-  /* set tolerance if not yet set */
-  if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
-
-  /* fill sorting criterion context */
+/*
+   EPSSetUpSort_Basic: configure the EPS sorting criterion according to 'which'
+*/
+PetscErrorCode EPSSetUpSort_Basic(EPS eps)
+{
+  PetscFunctionBegin;
   switch (eps->which) {
     case EPS_LARGEST_MAGNITUDE:
       eps->sc->comparison    = SlepcCompareLargestMagnitude;
@@ -270,62 +214,174 @@ PetscErrorCode EPSSetUp(EPS eps)
   }
   eps->sc->map    = NULL;
   eps->sc->mapobj = NULL;
+  PetscFunctionReturn(0);
+}
 
+/*
+   EPSSetUpSort_Default: configure both EPS and DS sorting criterion
+*/
+PetscErrorCode EPSSetUpSort_Default(EPS eps)
+{
+  SlepcSC        sc;
+  PetscBool      istrivial;
+
+  PetscFunctionBegin;
+  /* fill sorting criterion context */
+  PetscCall(EPSSetUpSort_Basic(eps));
   /* fill sorting criterion for DS */
-  if (eps->useds && eps->which!=EPS_ALL) {
-    ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
-    ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
-    sc->rg            = istrivial? NULL: eps->rg;
-    sc->comparison    = eps->sc->comparison;
-    sc->comparisonctx = eps->sc->comparisonctx;
-    sc->map           = SlepcMap_ST;
-    sc->mapobj        = (PetscObject)eps->st;
-    if (eps->twosided) {
-      ierr = DSSetSlepcSC(eps->dsts,sc);CHKERRQ(ierr);
+  PetscCall(DSGetSlepcSC(eps->ds,&sc));
+  PetscCall(RGIsTrivial(eps->rg,&istrivial));
+  sc->rg            = istrivial? NULL: eps->rg;
+  sc->comparison    = eps->sc->comparison;
+  sc->comparisonctx = eps->sc->comparisonctx;
+  sc->map           = SlepcMap_ST;
+  sc->mapobj        = (PetscObject)eps->st;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   EPSSetUp - Sets up all the internal data structures necessary for the
+   execution of the eigensolver. Then calls STSetUp() for any set-up
+   operations associated to the ST object.
+
+   Collective on eps
+
+   Input Parameter:
+.  eps   - eigenproblem solver context
+
+   Notes:
+   This function need not be called explicitly in most cases, since EPSSolve()
+   calls it. It can be useful when one wants to measure the set-up time
+   separately from the solve time.
+
+   Level: developer
+
+.seealso: EPSCreate(), EPSSolve(), EPSDestroy(), STSetUp(), EPSSetInitialSpace()
+@*/
+PetscErrorCode EPSSetUp(EPS eps)
+{
+  Mat            A,B;
+  PetscInt       k,nmat;
+  PetscBool      flg;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  if (eps->state) PetscFunctionReturn(0);
+  PetscCall(PetscLogEventBegin(EPS_SetUp,eps,0,0,0));
+
+  /* reset the convergence flag from the previous solves */
+  eps->reason = EPS_CONVERGED_ITERATING;
+
+  /* Set default solver type (EPSSetFromOptions was not called) */
+  if (!((PetscObject)eps)->type_name) PetscCall(EPSSetType(eps,EPSKRYLOVSCHUR));
+  if (!eps->st) PetscCall(EPSGetST(eps,&eps->st));
+  PetscCall(EPSSetDefaultST(eps));
+
+  PetscCall(STSetTransform(eps->st,PETSC_TRUE));
+  if (eps->useds && !eps->ds) PetscCall(EPSGetDS(eps,&eps->ds));
+  if (eps->twosided) {
+    PetscCheck(!eps->ishermitian || (eps->isgeneralized && !eps->ispositive),PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Two-sided methods are not intended for %s problems",SLEPC_STRING_HERMITIAN);
+  }
+  if (!eps->rg) PetscCall(EPSGetRG(eps,&eps->rg));
+  if (!((PetscObject)eps->rg)->type_name) PetscCall(RGSetType(eps->rg,RGINTERVAL));
+
+  /* Set problem dimensions */
+  PetscCall(STGetNumMatrices(eps->st,&nmat));
+  PetscCheck(nmat,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSSetOperators must be called first");
+  PetscCall(STMatGetSize(eps->st,&eps->n,NULL));
+  PetscCall(STMatGetLocalSize(eps->st,&eps->nloc,NULL));
+
+  /* Set default problem type */
+  if (!eps->problem_type) {
+    if (nmat==1) PetscCall(EPSSetProblemType(eps,EPS_NHEP));
+    else PetscCall(EPSSetProblemType(eps,EPS_GNHEP));
+  } else if (nmat==1 && eps->isgeneralized) {
+    PetscCall(PetscInfo(eps,"Eigenproblem set as generalized but no matrix B was provided; reverting to a standard eigenproblem\n"));
+    eps->isgeneralized = PETSC_FALSE;
+    eps->problem_type = eps->ishermitian? EPS_HEP: EPS_NHEP;
+  } else PetscCheck(nmat==1 || eps->isgeneralized,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Inconsistent EPS state: the problem type does not match the number of matrices");
+
+  if (eps->nev > eps->n) eps->nev = eps->n;
+  if (eps->ncv > eps->n) eps->ncv = eps->n;
+
+  /* check some combinations of eps->which */
+  PetscCheck(!eps->ishermitian || (eps->isgeneralized && !eps->ispositive) || (eps->which!=EPS_LARGEST_IMAGINARY && eps->which!=EPS_SMALLEST_IMAGINARY && eps->which!=EPS_TARGET_IMAGINARY),PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Sorting the eigenvalues along the imaginary axis is not allowed when all eigenvalues are real");
+
+  /* initialization of matrix norms */
+  if (eps->conv==EPS_CONV_NORM) {
+    if (!eps->nrma) {
+      PetscCall(STGetMatrix(eps->st,0,&A));
+      PetscCall(MatNorm(A,NORM_INFINITY,&eps->nrma));
+    }
+    if (nmat>1 && !eps->nrmb) {
+      PetscCall(STGetMatrix(eps->st,1,&B));
+      PetscCall(MatNorm(B,NORM_INFINITY,&eps->nrmb));
     }
   }
 
+  /* call specific solver setup */
+  PetscCall((*eps->ops->setup)(eps));
+
+  /* if purification is set, check that it really makes sense */
+  if (eps->purify) {
+    if (eps->categ==EPS_CATEGORY_PRECOND || eps->categ==EPS_CATEGORY_CONTOUR) eps->purify = PETSC_FALSE;
+    else {
+      if (!eps->isgeneralized) eps->purify = PETSC_FALSE;
+      else if (!eps->ishermitian && !eps->ispositive) eps->purify = PETSC_FALSE;
+      else {
+        PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STCAYLEY,&flg));
+        if (flg) eps->purify = PETSC_FALSE;
+      }
+    }
+  }
+
+  /* set tolerance if not yet set */
+  if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL;
+
+  /* set up sorting criterion */
+  if (eps->ops->setupsort) PetscCall((*eps->ops->setupsort)(eps));
+
   /* Build balancing matrix if required */
   if (eps->balance!=EPS_BALANCE_USER) {
-    ierr = STSetBalanceMatrix(eps->st,NULL);CHKERRQ(ierr);
+    PetscCall(STSetBalanceMatrix(eps->st,NULL));
     if (!eps->ishermitian && (eps->balance==EPS_BALANCE_ONESIDE || eps->balance==EPS_BALANCE_TWOSIDE)) {
       if (!eps->D) {
-        ierr = BVCreateVec(eps->V,&eps->D);CHKERRQ(ierr);
-        ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->D);CHKERRQ(ierr);
+        PetscCall(BVCreateVec(eps->V,&eps->D));
+        PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)eps->D));
       }
-      ierr = EPSBuildBalance_Krylov(eps);CHKERRQ(ierr);
-      ierr = STSetBalanceMatrix(eps->st,eps->D);CHKERRQ(ierr);
+      PetscCall(EPSBuildBalance_Krylov(eps));
+      PetscCall(STSetBalanceMatrix(eps->st,eps->D));
     }
   }
 
   /* Setup ST */
-  ierr = STSetUp(eps->st);CHKERRQ(ierr);
-  ierr = EPSCheckCompatibleST(eps);CHKERRQ(ierr);
+  PetscCall(STSetUp(eps->st));
+  PetscCall(EPSCheckCompatibleST(eps));
 
   /* process deflation and initial vectors */
   if (eps->nds<0) {
     k = -eps->nds;
-    ierr = BVInsertConstraints(eps->V,&k,eps->defl);CHKERRQ(ierr);
-    ierr = SlepcBasisDestroy_Private(&eps->nds,&eps->defl);CHKERRQ(ierr);
+    PetscCall(BVInsertConstraints(eps->V,&k,eps->defl));
+    PetscCall(SlepcBasisDestroy_Private(&eps->nds,&eps->defl));
     eps->nds = k;
-    ierr = STCheckNullSpace(eps->st,eps->V);CHKERRQ(ierr);
+    PetscCall(STCheckNullSpace(eps->st,eps->V));
   }
   if (eps->nini<0) {
     k = -eps->nini;
-    if (k>eps->ncv) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The number of initial vectors is larger than ncv");
-    ierr = BVInsertVecs(eps->V,0,&k,eps->IS,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = SlepcBasisDestroy_Private(&eps->nini,&eps->IS);CHKERRQ(ierr);
+    PetscCheck(k<=eps->ncv,PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"The number of initial vectors is larger than ncv");
+    PetscCall(BVInsertVecs(eps->V,0,&k,eps->IS,PETSC_TRUE));
+    PetscCall(SlepcBasisDestroy_Private(&eps->nini,&eps->IS));
     eps->nini = k;
   }
   if (eps->twosided && eps->ninil<0) {
     k = -eps->ninil;
-    if (k>eps->ncv) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The number of left initial vectors is larger than ncv");
-    ierr = BVInsertVecs(eps->W,0,&k,eps->ISL,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = SlepcBasisDestroy_Private(&eps->ninil,&eps->ISL);CHKERRQ(ierr);
+    PetscCheck(k<=eps->ncv,PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"The number of left initial vectors is larger than ncv");
+    PetscCall(BVInsertVecs(eps->W,0,&k,eps->ISL,PETSC_TRUE));
+    PetscCall(SlepcBasisDestroy_Private(&eps->ninil,&eps->ISL));
     eps->ninil = k;
   }
 
-  ierr = PetscLogEventEnd(EPS_SetUp,eps,0,0,0);CHKERRQ(ierr);
+  PetscCall(PetscLogEventEnd(EPS_SetUp,eps,0,0,0));
   eps->state = EPS_STATE_SETUP;
   PetscFunctionReturn(0);
 }
@@ -343,8 +399,8 @@ PetscErrorCode EPSSetUp(EPS eps)
    Notes:
    To specify a standard eigenproblem, use NULL for parameter B.
 
-   It must be called before EPSSetUp(). If it is called again after EPSSetUp() then
-   the EPS object is reset.
+   It must be called before EPSSetUp(). If it is called again after EPSSetUp() and
+   the matrix sizes have changed then the EPS object is reset.
 
    Level: beginner
 
@@ -352,8 +408,7 @@ PetscErrorCode EPSSetUp(EPS eps)
 @*/
 PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 {
-  PetscErrorCode ierr;
-  PetscInt       m,n,m0,nmat;
+  PetscInt       m,n,m0,mloc,nloc,mloc0,nmat;
   Mat            mat[2];
 
   PetscFunctionBegin;
@@ -363,24 +418,29 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
   PetscCheckSameComm(eps,1,A,2);
   if (B) PetscCheckSameComm(eps,1,B,3);
 
-  /* Check for square matrices */
-  ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
-  if (m!=n) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"A is a non-square matrix");
+  /* Check matrix sizes */
+  PetscCall(MatGetSize(A,&m,&n));
+  PetscCall(MatGetLocalSize(A,&mloc,&nloc));
+  PetscCheck(m==n,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"A is a non-square matrix (%" PetscInt_FMT " rows, %" PetscInt_FMT " cols)",m,n);
+  PetscCheck(mloc==nloc,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"A does not have equal row and column sizes (%" PetscInt_FMT ", %" PetscInt_FMT ")",mloc,nloc);
   if (B) {
-    ierr = MatGetSize(B,&m0,&n);CHKERRQ(ierr);
-    if (m0!=n) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"B is a non-square matrix");
-    if (m!=m0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Dimensions of A and B do not match");
+    PetscCall(MatGetSize(B,&m0,&n));
+    PetscCall(MatGetLocalSize(B,&mloc0,&nloc));
+    PetscCheck(m0==n,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"B is a non-square matrix (%" PetscInt_FMT " rows, %" PetscInt_FMT " cols)",m0,n);
+    PetscCheck(mloc0==nloc,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"B does not have equal row and column local sizes (%" PetscInt_FMT ", %" PetscInt_FMT ")",mloc0,nloc);
+    PetscCheck(m==m0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Dimensions of A and B do not match (%" PetscInt_FMT ", %" PetscInt_FMT ")",m,m0);
+    PetscCheck(mloc==mloc0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_INCOMP,"Local dimensions of A and B do not match (%" PetscInt_FMT ", %" PetscInt_FMT ")",mloc,mloc0);
   }
-  if (eps->state && n!=eps->n) { ierr = EPSReset(eps);CHKERRQ(ierr); }
+  if (eps->state && (n!=eps->n || nloc!=eps->nloc)) PetscCall(EPSReset(eps));
   eps->nrma = 0.0;
   eps->nrmb = 0.0;
-  if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+  if (!eps->st) PetscCall(EPSGetST(eps,&eps->st));
   mat[0] = A;
   if (B) {
     mat[1] = B;
     nmat = 2;
   } else nmat = 1;
-  ierr = STSetMatrices(eps->st,nmat,mat);CHKERRQ(ierr);
+  PetscCall(STSetMatrices(eps->st,nmat,mat));
   eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
@@ -397,26 +457,29 @@ PetscErrorCode EPSSetOperators(EPS eps,Mat A,Mat B)
 +  A  - the matrix associated with the eigensystem
 -  B  - the second matrix in the case of generalized eigenproblems
 
+   Note:
+   Does not increase the reference count of the matrices, so you should not destroy them.
+
    Level: intermediate
 
 .seealso: EPSSolve(), EPSGetST(), STGetMatrix(), STSetMatrices()
 @*/
 PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 {
-  PetscErrorCode ierr;
   ST             st;
   PetscInt       k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  ierr = EPSGetST(eps,&st);CHKERRQ(ierr);
-  if (A) { ierr = STGetMatrix(st,0,A);CHKERRQ(ierr); }
+  PetscCall(EPSGetST(eps,&st));
+  PetscCall(STGetNumMatrices(st,&k));
+  if (A) {
+    if (k<1) *A = NULL;
+    else PetscCall(STGetMatrix(st,0,A));
+  }
   if (B) {
-    ierr = STGetNumMatrices(st,&k);CHKERRQ(ierr);
-    if (k==1) B = NULL;
-    else {
-      ierr = STGetMatrix(st,1,B);CHKERRQ(ierr);
-    }
+    if (k<2) *B = NULL;
+    else PetscCall(STGetMatrix(st,1,B));
   }
   PetscFunctionReturn(0);
 }
@@ -427,7 +490,7 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 
    Collective on eps
 
-   Input Parameter:
+   Input Parameters:
 +  eps - the eigenproblem solver context
 .  n   - number of vectors
 -  v   - set of basis vectors of the deflation space
@@ -450,17 +513,15 @@ PetscErrorCode EPSGetOperators(EPS eps,Mat *A,Mat *B)
 @*/
 PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec v[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
-  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n out of range");
+  PetscCheck(n>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
   if (n>0) {
     PetscValidPointer(v,3);
     PetscValidHeaderSpecific(*v,VEC_CLASSID,3);
   }
-  ierr = SlepcBasisReference_Private(n,v,&eps->nds,&eps->defl);CHKERRQ(ierr);
+  PetscCall(SlepcBasisReference_Private(n,v,&eps->nds,&eps->defl));
   if (n>0) eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
@@ -471,7 +532,7 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec v[])
 
    Collective on eps
 
-   Input Parameter:
+   Input Parameters:
 +  eps - the eigenproblem solver context
 .  n   - number of vectors
 -  is  - set of basis vectors of the initial space
@@ -495,17 +556,15 @@ PetscErrorCode EPSSetDeflationSpace(EPS eps,PetscInt n,Vec v[])
 @*/
 PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec is[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
-  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  PetscCheck(n>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
   if (n>0) {
     PetscValidPointer(is,3);
     PetscValidHeaderSpecific(*is,VEC_CLASSID,3);
   }
-  ierr = SlepcBasisReference_Private(n,is,&eps->nini,&eps->IS);CHKERRQ(ierr);
+  PetscCall(SlepcBasisReference_Private(n,is,&eps->nini,&eps->IS));
   if (n>0) eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
@@ -516,7 +575,7 @@ PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec is[])
 
    Collective on eps
 
-   Input Parameter:
+   Input Parameters:
 +  eps - the eigenproblem solver context
 .  n   - number of vectors
 -  isl - set of basis vectors of the left initial space
@@ -534,17 +593,15 @@ PetscErrorCode EPSSetInitialSpace(EPS eps,PetscInt n,Vec is[])
 @*/
 PetscErrorCode EPSSetLeftInitialSpace(EPS eps,PetscInt n,Vec isl[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,n,2);
-  if (n<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
+  PetscCheck(n>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument n cannot be negative");
   if (n>0) {
     PetscValidPointer(isl,3);
     PetscValidHeaderSpecific(*isl,VEC_CLASSID,3);
   }
-  ierr = SlepcBasisReference_Private(n,isl,&eps->ninil,&eps->ISL);CHKERRQ(ierr);
+  PetscCall(SlepcBasisReference_Private(n,isl,&eps->ninil,&eps->ISL));
   if (n>0) eps->state = EPS_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
@@ -555,18 +612,17 @@ PetscErrorCode EPSSetLeftInitialSpace(EPS eps,PetscInt n,Vec isl[])
  */
 PetscErrorCode EPSSetDimensions_Default(EPS eps,PetscInt nev,PetscInt *ncv,PetscInt *mpd)
 {
-  PetscErrorCode ierr;
   PetscBool      krylov;
 
   PetscFunctionBegin;
-  if (*ncv) { /* ncv set */
-    ierr = PetscObjectTypeCompareAny((PetscObject)eps,&krylov,EPSKRYLOVSCHUR,EPSARNOLDI,EPSLANCZOS,"");CHKERRQ(ierr);
+  if (*ncv!=PETSC_DEFAULT) { /* ncv set */
+    PetscCall(PetscObjectTypeCompareAny((PetscObject)eps,&krylov,EPSKRYLOVSCHUR,EPSARNOLDI,EPSLANCZOS,""));
     if (krylov) {
-      if (*ncv<nev+1 && !(*ncv==nev && *ncv==eps->n)) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must be at least nev+1");
+      PetscCheck(*ncv>=nev+1 || (*ncv==nev && *ncv==eps->n),PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"The value of ncv must be at least nev+1");
     } else {
-      if (*ncv<nev) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must be at least nev");
+      PetscCheck(*ncv>=nev,PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"The value of ncv must be at least nev");
     }
-  } else if (*mpd) { /* mpd set */
+  } else if (*mpd!=PETSC_DEFAULT) { /* mpd set */
     *ncv = PetscMin(eps->n,nev+(*mpd));
   } else { /* neither set: defaults depend on nev being small or large */
     if (nev<500) *ncv = PetscMin(eps->n,PetscMax(2*nev,nev+15));
@@ -575,7 +631,7 @@ PetscErrorCode EPSSetDimensions_Default(EPS eps,PetscInt nev,PetscInt *ncv,Petsc
       *ncv = PetscMin(eps->n,nev+(*mpd));
     }
   }
-  if (!*mpd) *mpd = *ncv;
+  if (*mpd==PETSC_DEFAULT) *mpd = *ncv;
   PetscFunctionReturn(0);
 }
 
@@ -590,61 +646,57 @@ PetscErrorCode EPSSetDimensions_Default(EPS eps,PetscInt nev,PetscInt *ncv,Petsc
 -  extra - number of additional positions, used for methods that require a
            working basis slightly larger than ncv
 
-   Developers Note:
+   Developer Notes:
    This is SLEPC_EXTERN because it may be required by user plugin EPS
    implementations.
 
    Level: developer
+
+.seealso: EPSSetUp()
 @*/
 PetscErrorCode EPSAllocateSolution(EPS eps,PetscInt extra)
 {
-  PetscErrorCode ierr;
   PetscInt       oldsize,newc,requested;
   PetscLogDouble cnt;
+  PetscRandom    rand;
   Vec            t;
 
   PetscFunctionBegin;
   requested = eps->ncv + extra;
 
   /* oldsize is zero if this is the first time setup is called */
-  ierr = BVGetSizes(eps->V,NULL,NULL,&oldsize);CHKERRQ(ierr);
+  PetscCall(BVGetSizes(eps->V,NULL,NULL,&oldsize));
   newc = PetscMax(0,requested-oldsize);
 
   /* allocate space for eigenvalues and friends */
   if (requested != oldsize || !eps->eigr) {
-    ierr = PetscFree4(eps->eigr,eps->eigi,eps->errest,eps->perm);CHKERRQ(ierr);
-    ierr = PetscMalloc4(requested,&eps->eigr,requested,&eps->eigi,requested,&eps->errest,requested,&eps->perm);CHKERRQ(ierr);
+    PetscCall(PetscFree4(eps->eigr,eps->eigi,eps->errest,eps->perm));
+    PetscCall(PetscMalloc4(requested,&eps->eigr,requested,&eps->eigi,requested,&eps->errest,requested,&eps->perm));
     cnt = 2*newc*sizeof(PetscScalar) + 2*newc*sizeof(PetscReal) + newc*sizeof(PetscInt);
-    ierr = PetscLogObjectMemory((PetscObject)eps,cnt);CHKERRQ(ierr);
+    PetscCall(PetscLogObjectMemory((PetscObject)eps,cnt));
   }
 
   /* workspace for the case of arbitrary selection */
   if (eps->arbitrary) {
-    if (eps->rr) {
-      ierr = PetscFree2(eps->rr,eps->ri);CHKERRQ(ierr);
-    }
-    ierr = PetscMalloc2(requested,&eps->rr,requested,&eps->ri);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)eps,2*newc*sizeof(PetscScalar));CHKERRQ(ierr);
+    if (eps->rr) PetscCall(PetscFree2(eps->rr,eps->ri));
+    PetscCall(PetscMalloc2(requested,&eps->rr,requested,&eps->ri));
+    PetscCall(PetscLogObjectMemory((PetscObject)eps,2*newc*sizeof(PetscScalar)));
   }
 
   /* allocate V */
-  if (!eps->V) { ierr = EPSGetBV(eps,&eps->V);CHKERRQ(ierr); }
+  if (!eps->V) PetscCall(EPSGetBV(eps,&eps->V));
   if (!oldsize) {
-    if (!((PetscObject)(eps->V))->type_name) {
-      ierr = BVSetType(eps->V,BVSVEC);CHKERRQ(ierr);
-    }
-    ierr = STMatCreateVecsEmpty(eps->st,&t,NULL);CHKERRQ(ierr);
-    ierr = BVSetSizesFromVec(eps->V,t,requested);CHKERRQ(ierr);
-    ierr = VecDestroy(&t);CHKERRQ(ierr);
-  } else {
-    ierr = BVResize(eps->V,requested,PETSC_FALSE);CHKERRQ(ierr);
-  }
+    if (!((PetscObject)(eps->V))->type_name) PetscCall(BVSetType(eps->V,BVSVEC));
+    PetscCall(STMatCreateVecsEmpty(eps->st,&t,NULL));
+    PetscCall(BVSetSizesFromVec(eps->V,t,requested));
+    PetscCall(VecDestroy(&t));
+  } else PetscCall(BVResize(eps->V,requested,PETSC_FALSE));
 
   /* allocate W */
   if (eps->twosided) {
-    ierr = BVDestroy(&eps->W);CHKERRQ(ierr);
-    ierr = BVDuplicate(eps->V,&eps->W);CHKERRQ(ierr);
+    PetscCall(BVGetRandomContext(eps->V,&rand));  /* make sure the random context is available when duplicating */
+    PetscCall(BVDestroy(&eps->W));
+    PetscCall(BVDuplicate(eps->V,&eps->W));
   }
   PetscFunctionReturn(0);
 }
-
