@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,18 +12,14 @@
 */
 
 #include <slepc/private/epsimpl.h>   /*I "slepceps.h" I*/
-#include <slepc/private/bvimpl.h>    /*I "slepcbv.h" I*/
+#include <slepc/private/bvimpl.h>
 #include <petscdraw.h>
 
 PetscErrorCode EPSComputeVectors(EPS eps)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   EPSCheckSolved(eps,1);
-  if (eps->state==EPS_STATE_SOLVED && eps->ops->computevectors) {
-    ierr = (*eps->ops->computevectors)(eps);CHKERRQ(ierr);
-  }
+  if (eps->state==EPS_STATE_SOLVED && eps->ops->computevectors) PetscCall((*eps->ops->computevectors)(eps));
   eps->state = EPS_STATE_EIGENVECTORS;
   PetscFunctionReturn(0);
 }
@@ -32,7 +28,6 @@ PetscErrorCode EPSComputeVectors(EPS eps)
 
 static PetscErrorCode EPSComputeValues(EPS eps)
 {
-  PetscErrorCode ierr;
   PetscBool      injective,iscomp,isfilter;
   PetscInt       i,n,aux,nconv0;
   Mat            A,B=NULL,G,Z;
@@ -41,40 +36,39 @@ static PetscErrorCode EPSComputeValues(EPS eps)
   switch (eps->categ) {
     case EPS_CATEGORY_KRYLOV:
     case EPS_CATEGORY_OTHER:
-      ierr = STIsInjective(eps->st,&injective);CHKERRQ(ierr);
+      PetscCall(STIsInjective(eps->st,&injective));
       if (injective) {
         /* one-to-one mapping: backtransform eigenvalues */
-        if (eps->ops->backtransform) {
-          ierr = (*eps->ops->backtransform)(eps);CHKERRQ(ierr);
-        } else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, spectral transform should have a backtransform operation");
+        PetscCheck(eps->ops->backtransform,PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, spectral transform should have a backtransform operation");
+        PetscCall((*eps->ops->backtransform)(eps));
       } else {
         /* compute eigenvalues from Rayleigh quotient */
-        ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL);CHKERRQ(ierr);
+        PetscCall(DSGetDimensions(eps->ds,&n,NULL,NULL,NULL));
         if (!n) break;
-        ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
-        ierr = BVSetActiveColumns(eps->V,0,n);CHKERRQ(ierr);
-        ierr = DSGetCompact(eps->ds,&iscomp);CHKERRQ(ierr);
-        ierr = DSSetCompact(eps->ds,PETSC_FALSE);CHKERRQ(ierr);
-        ierr = DSGetMat(eps->ds,DS_MAT_A,&G);CHKERRQ(ierr);
-        ierr = BVMatProject(eps->V,A,eps->V,G);CHKERRQ(ierr);
-        ierr = DSRestoreMat(eps->ds,DS_MAT_A,&G);CHKERRQ(ierr);
+        PetscCall(EPSGetOperators(eps,&A,&B));
+        PetscCall(BVSetActiveColumns(eps->V,0,n));
+        PetscCall(DSGetCompact(eps->ds,&iscomp));
+        PetscCall(DSSetCompact(eps->ds,PETSC_FALSE));
+        PetscCall(DSGetMat(eps->ds,DS_MAT_A,&G));
+        PetscCall(BVMatProject(eps->V,A,eps->V,G));
+        PetscCall(DSRestoreMat(eps->ds,DS_MAT_A,&G));
         if (B) {
-          ierr = DSGetMat(eps->ds,DS_MAT_B,&G);CHKERRQ(ierr);
-          ierr = BVMatProject(eps->V,B,eps->V,G);CHKERRQ(ierr);
-          ierr = DSRestoreMat(eps->ds,DS_MAT_A,&G);CHKERRQ(ierr);
+          PetscCall(DSGetMat(eps->ds,DS_MAT_B,&G));
+          PetscCall(BVMatProject(eps->V,B,eps->V,G));
+          PetscCall(DSRestoreMat(eps->ds,DS_MAT_A,&G));
         }
-        ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
-        ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
-        ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
-        ierr = DSSetCompact(eps->ds,iscomp);CHKERRQ(ierr);
+        PetscCall(DSSolve(eps->ds,eps->eigr,eps->eigi));
+        PetscCall(DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL));
+        PetscCall(DSSynchronize(eps->ds,eps->eigr,eps->eigi));
+        PetscCall(DSSetCompact(eps->ds,iscomp));
         if (eps->ishermitian && (!eps->isgeneralized || eps->ispositive)) { /* V = V * Z */
-          ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
-          ierr = DSGetMat(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
-          ierr = BVMultInPlace(eps->V,Z,0,n);CHKERRQ(ierr);
-          ierr = MatDestroy(&Z);CHKERRQ(ierr);
+          PetscCall(DSVectors(eps->ds,DS_MAT_X,NULL,NULL));
+          PetscCall(DSGetMat(eps->ds,DS_MAT_X,&Z));
+          PetscCall(BVMultInPlace(eps->V,Z,0,n));
+          PetscCall(MatDestroy(&Z));
         }
         /* in case of STFILTER discard computed eigenvalues that lie outside the wanted interval */
-        ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilter);CHKERRQ(ierr);
+        PetscCall(PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilter));
         if (isfilter) {
           nconv0 = eps->nconv;
           for (i=0;i<eps->nconv;i++) {
@@ -83,9 +77,7 @@ static PetscErrorCode EPSComputeValues(EPS eps)
               if (i<eps->nconv) { SWAP(eps->perm[i],eps->perm[eps->nconv],aux); i--; }
             }
           }
-          if (nconv0>eps->nconv) {
-            ierr = PetscInfo1(eps,"Discarded %D computed eigenvalues lying outside the interval\n",nconv0-eps->nconv);CHKERRQ(ierr);
-          }
+          if (nconv0>eps->nconv) PetscCall(PetscInfo(eps,"Discarded %" PetscInt_FMT " computed eigenvalues lying outside the interval\n",nconv0-eps->nconv));
         }
       }
       break;
@@ -107,22 +99,28 @@ static PetscErrorCode EPSComputeValues(EPS eps)
 
    Options Database Keys:
 +  -eps_view - print information about the solver used
-.  -eps_view_mat0 binary - save the first matrix (A) to the default binary viewer
-.  -eps_view_mat1 binary - save the second matrix (B) to the default binary viewer
-.  -eps_view_vectors binary - save the computed eigenvectors to the default binary viewer
-.  -eps_view_values - print computed eigenvalues
+.  -eps_view_mat0 - view the first matrix (A)
+.  -eps_view_mat1 - view the second matrix (B)
+.  -eps_view_vectors - view the computed eigenvectors
+.  -eps_view_values - view the computed eigenvalues
 .  -eps_converged_reason - print reason for convergence, and number of iterations
 .  -eps_error_absolute - print absolute errors of each eigenpair
 .  -eps_error_relative - print relative errors of each eigenpair
 -  -eps_error_backward - print backward errors of each eigenpair
 
+   Notes:
+   All the command-line options listed above admit an optional argument specifying
+   the viewer type and options. For instance, use '-eps_view_mat0 binary:amatrix.bin'
+   to save the A matrix to a binary file, '-eps_view_values draw' to draw the computed
+   eigenvalues graphically, or '-eps_error_relative :myerr.m:ascii_matlab' to save
+   the errors in a file that can be executed in Matlab.
+
    Level: beginner
 
 .seealso: EPSCreate(), EPSSetUp(), EPSDestroy(), EPSSetTolerances()
 @*/
 PetscErrorCode EPSSolve(EPS eps)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
   STMatMode      matmode;
   Mat            A,B;
@@ -130,10 +128,10 @@ PetscErrorCode EPSSolve(EPS eps)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   if (eps->state>=EPS_STATE_SOLVED) PetscFunctionReturn(0);
-  ierr = PetscLogEventBegin(EPS_Solve,eps,0,0,0);CHKERRQ(ierr);
+  PetscCall(PetscLogEventBegin(EPS_Solve,eps,0,0,0));
 
   /* Call setup */
-  ierr = EPSSetUp(eps);CHKERRQ(ierr);
+  PetscCall(EPSSetUp(eps));
   eps->nconv = 0;
   eps->its   = 0;
   for (i=0;i<eps->ncv;i++) {
@@ -142,27 +140,25 @@ PetscErrorCode EPSSolve(EPS eps)
     eps->errest[i] = 0.0;
     eps->perm[i]   = i;
   }
-  ierr = EPSViewFromOptions(eps,NULL,"-eps_view_pre");CHKERRQ(ierr);
-  ierr = RGViewFromOptions(eps->rg,NULL,"-rg_view");CHKERRQ(ierr);
+  PetscCall(EPSViewFromOptions(eps,NULL,"-eps_view_pre"));
+  PetscCall(RGViewFromOptions(eps->rg,NULL,"-rg_view"));
 
   /* Call solver */
-  ierr = (*eps->ops->solve)(eps);CHKERRQ(ierr);
-  if (!eps->reason) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
+  PetscCall((*eps->ops->solve)(eps));
+  PetscCheck(eps->reason,PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Internal error, solver returned without setting converged reason");
   eps->state = EPS_STATE_SOLVED;
 
   /* Only the first nconv columns contain useful information (except in CISS) */
-  ierr = BVSetActiveColumns(eps->V,0,eps->nconv);CHKERRQ(ierr);
-  if (eps->twosided) { ierr = BVSetActiveColumns(eps->W,0,eps->nconv);CHKERRQ(ierr); }
+  PetscCall(BVSetActiveColumns(eps->V,0,eps->nconv));
+  if (eps->twosided) PetscCall(BVSetActiveColumns(eps->W,0,eps->nconv));
 
   /* If inplace, purify eigenvectors before reverting operator */
-  ierr = STGetMatMode(eps->st,&matmode);CHKERRQ(ierr);
-  if (matmode == ST_MATMODE_INPLACE && eps->ispositive) {
-    ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
-  }
-  ierr = STPostSolve(eps->st);CHKERRQ(ierr);
+  PetscCall(STGetMatMode(eps->st,&matmode));
+  if (matmode == ST_MATMODE_INPLACE && eps->ispositive) PetscCall(EPSComputeVectors(eps));
+  PetscCall(STPostSolve(eps->st));
 
   /* Map eigenvalues back to the original problem if appropriate */
-  ierr = EPSComputeValues(eps);CHKERRQ(ierr);
+  PetscCall(EPSComputeValues(eps));
 
 #if !defined(PETSC_USE_COMPLEX)
   /* Reorder conjugate eigenvalues (positive imaginary first) */
@@ -172,8 +168,8 @@ PetscErrorCode EPSSolve(EPS eps)
         eps->eigi[i] = -eps->eigi[i];
         eps->eigi[i+1] = -eps->eigi[i+1];
         /* the next correction only works with eigenvectors */
-        ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
-        ierr = BVScaleColumn(eps->V,i+1,-1.0);CHKERRQ(ierr);
+        PetscCall(EPSComputeVectors(eps));
+        PetscCall(BVScaleColumn(eps->V,i+1,-1.0));
       }
       i++;
     }
@@ -181,24 +177,22 @@ PetscErrorCode EPSSolve(EPS eps)
 #endif
 
   /* Sort eigenvalues according to eps->which parameter */
-  ierr = SlepcSortEigenvalues(eps->sc,eps->nconv,eps->eigr,eps->eigi,eps->perm);CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(EPS_Solve,eps,0,0,0);CHKERRQ(ierr);
+  PetscCall(SlepcSortEigenvalues(eps->sc,eps->nconv,eps->eigr,eps->eigi,eps->perm));
+  PetscCall(PetscLogEventEnd(EPS_Solve,eps,0,0,0));
 
   /* Various viewers */
-  ierr = EPSViewFromOptions(eps,NULL,"-eps_view");CHKERRQ(ierr);
-  ierr = EPSConvergedReasonViewFromOptions(eps);CHKERRQ(ierr);
-  ierr = EPSErrorViewFromOptions(eps);CHKERRQ(ierr);
-  ierr = EPSValuesViewFromOptions(eps);CHKERRQ(ierr);
-  ierr = EPSVectorsViewFromOptions(eps);CHKERRQ(ierr);
-  ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
-  ierr = MatViewFromOptions(A,(PetscObject)eps,"-eps_view_mat0");CHKERRQ(ierr);
-  if (eps->isgeneralized) {
-    ierr = MatViewFromOptions(B,(PetscObject)eps,"-eps_view_mat1");CHKERRQ(ierr);
-  }
+  PetscCall(EPSViewFromOptions(eps,NULL,"-eps_view"));
+  PetscCall(EPSConvergedReasonViewFromOptions(eps));
+  PetscCall(EPSErrorViewFromOptions(eps));
+  PetscCall(EPSValuesViewFromOptions(eps));
+  PetscCall(EPSVectorsViewFromOptions(eps));
+  PetscCall(EPSGetOperators(eps,&A,&B));
+  PetscCall(MatViewFromOptions(A,(PetscObject)eps,"-eps_view_mat0"));
+  if (eps->isgeneralized) PetscCall(MatViewFromOptions(B,(PetscObject)eps,"-eps_view_mat1"));
 
   /* Remove deflation and initial subspaces */
   if (eps->nds) {
-    ierr = BVSetNumConstraints(eps->V,0);CHKERRQ(ierr);
+    PetscCall(BVSetNumConstraints(eps->V,0));
     eps->nds = 0;
   }
   eps->nini = 0;
@@ -254,7 +248,7 @@ PetscErrorCode EPSGetIterationNumber(EPS eps,PetscInt *its)
 
    Level: beginner
 
-.seealso: EPSSetDimensions(), EPSSolve()
+.seealso: EPSSetDimensions(), EPSSolve(), EPSGetEigenpair()
 @*/
 PetscErrorCode EPSGetConverged(EPS eps,PetscInt *nconv)
 {
@@ -278,6 +272,9 @@ PetscErrorCode EPSGetConverged(EPS eps,PetscInt *nconv)
    Output Parameter:
 .  reason - negative value indicates diverged, positive value converged
 
+   Options Database Key:
+.  -eps_converged_reason - print the reason to a viewer
+
    Notes:
    Possible values for reason are
 +  EPS_CONVERGED_TOL - converged up to tolerance
@@ -333,7 +330,6 @@ PetscErrorCode EPSGetConvergedReason(EPS eps,EPSConvergedReason *reason)
 @*/
 PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec v[])
 {
-  PetscErrorCode ierr;
   PetscInt       i;
   BV             V=eps->V;
   Vec            w;
@@ -343,24 +339,20 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec v[])
   PetscValidPointer(v,2);
   PetscValidHeaderSpecific(*v,VEC_CLASSID,2);
   EPSCheckSolved(eps,1);
-  if (!eps->ishermitian && eps->state==EPS_STATE_EIGENVECTORS) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSGetInvariantSubspace must be called before EPSGetEigenpair,EPSGetEigenvector or EPSComputeError");
+  PetscCheck(eps->ishermitian || eps->state!=EPS_STATE_EIGENVECTORS,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"EPSGetInvariantSubspace must be called before EPSGetEigenpair,EPSGetEigenvector or EPSComputeError");
   if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
-    ierr = BVDuplicateResize(eps->V,eps->nconv,&V);CHKERRQ(ierr);
-    ierr = BVSetActiveColumns(eps->V,0,eps->nconv);CHKERRQ(ierr);
-    ierr = BVCopy(eps->V,V);CHKERRQ(ierr);
+    PetscCall(BVDuplicateResize(eps->V,eps->nconv,&V));
+    PetscCall(BVSetActiveColumns(eps->V,0,eps->nconv));
+    PetscCall(BVCopy(eps->V,V));
     for (i=0;i<eps->nconv;i++) {
-      ierr = BVGetColumn(V,i,&w);CHKERRQ(ierr);
-      ierr = VecPointwiseDivide(w,w,eps->D);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(V,i,&w);CHKERRQ(ierr);
+      PetscCall(BVGetColumn(V,i,&w));
+      PetscCall(VecPointwiseDivide(w,w,eps->D));
+      PetscCall(BVRestoreColumn(V,i,&w));
     }
-    ierr = BVOrthogonalize(V,NULL);CHKERRQ(ierr);
-  }
-  for (i=0;i<eps->nconv;i++) {
-    ierr = BVCopyVec(V,i,v[i]);CHKERRQ(ierr);
-  }
-  if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
-    ierr = BVDestroy(&V);CHKERRQ(ierr);
+    PetscCall(BVOrthogonalize(V,NULL));
   }
+  for (i=0;i<eps->nconv;i++) PetscCall(BVCopyVec(V,i,v[i]));
+  if (eps->balance!=EPS_BALANCE_NONE && eps->D) PetscCall(BVDestroy(&V));
   PetscFunctionReturn(0);
 }
 
@@ -405,16 +397,14 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps,Vec v[])
 @*/
 PetscErrorCode EPSGetEigenpair(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar *eigi,Vec Vr,Vec Vi)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,i,2);
   EPSCheckSolved(eps,1);
-  if (i<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
-  if (i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
-  ierr = EPSGetEigenvalue(eps,i,eigr,eigi);CHKERRQ(ierr);
-  if (Vr || Vi) { ierr = EPSGetEigenvector(eps,i,Vr,Vi);CHKERRQ(ierr); }
+  PetscCheck(i>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
+  PetscCall(EPSGetEigenvalue(eps,i,eigr,eigi));
+  if (Vr || Vi) PetscCall(EPSGetEigenvector(eps,i,Vr,Vi));
   PetscFunctionReturn(0);
 }
 
@@ -451,8 +441,8 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   EPSCheckSolved(eps,1);
-  if (i<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
-  if (i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
+  PetscCheck(i>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
   k = eps->perm[i];
 #if defined(PETSC_USE_COMPLEX)
   if (eigr) *eigr = eps->eigr[k];
@@ -500,7 +490,6 @@ PetscErrorCode EPSGetEigenvalue(EPS eps,PetscInt i,PetscScalar *eigr,PetscScalar
 @*/
 PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
 {
-  PetscErrorCode ierr;
   PetscInt       k;
 
   PetscFunctionBegin;
@@ -509,11 +498,11 @@ PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
   if (Vr) { PetscValidHeaderSpecific(Vr,VEC_CLASSID,3); PetscCheckSameComm(eps,1,Vr,3); }
   if (Vi) { PetscValidHeaderSpecific(Vi,VEC_CLASSID,4); PetscCheckSameComm(eps,1,Vi,4); }
   EPSCheckSolved(eps,1);
-  if (i<0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
-  if (i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
-  ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
+  PetscCheck(i>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
+  PetscCall(EPSComputeVectors(eps));
   k = eps->perm[i];
-  ierr = BV_GetEigenvector(eps->V,k,eps->eigi[k],Vr,Vi);CHKERRQ(ierr);
+  PetscCall(BV_GetEigenvector(eps->V,k,eps->eigi[k],Vr,Vi));
   PetscFunctionReturn(0);
 }
 
@@ -552,7 +541,6 @@ PetscErrorCode EPSGetEigenvector(EPS eps,PetscInt i,Vec Vr,Vec Vi)
 @*/
 PetscErrorCode EPSGetLeftEigenvector(EPS eps,PetscInt i,Vec Wr,Vec Wi)
 {
-  PetscErrorCode ierr;
   PetscInt       k;
 
   PetscFunctionBegin;
@@ -561,11 +549,12 @@ PetscErrorCode EPSGetLeftEigenvector(EPS eps,PetscInt i,Vec Wr,Vec Wi)
   if (Wr) { PetscValidHeaderSpecific(Wr,VEC_CLASSID,3); PetscCheckSameComm(eps,1,Wr,3); }
   if (Wi) { PetscValidHeaderSpecific(Wi,VEC_CLASSID,4); PetscCheckSameComm(eps,1,Wi,4); }
   EPSCheckSolved(eps,1);
-  if (!eps->twosided) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"Must request left vectors with EPSSetTwoSided");
-  if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
-  ierr = EPSComputeVectors(eps);CHKERRQ(ierr);
+  PetscCheck(eps->twosided,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"Must request left vectors with EPSSetTwoSided");
+  PetscCheck(i>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
+  PetscCall(EPSComputeVectors(eps));
   k = eps->perm[i];
-  ierr = BV_GetEigenvector(eps->W,k,eps->eigi[k],Wr,Wi);CHKERRQ(ierr);
+  PetscCall(BV_GetEigenvector(eps->W,k,eps->eigi[k],Wr,Wi));
   PetscFunctionReturn(0);
 }
 
@@ -597,7 +586,8 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps,PetscInt i,PetscReal *errest)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidRealPointer(errest,3);
   EPSCheckSolved(eps,1);
-  if (i<0 || i>=eps->nconv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  PetscCheck(i>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index cannot be negative");
+  PetscCheck(i<eps->nconv,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The index can be nconv-1 at most, see EPSGetConverged()");
   *errest = eps->errest[eps->perm[i]];
   PetscFunctionReturn(0);
 }
@@ -614,7 +604,6 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps,PetscInt i,PetscReal *errest)
 */
 PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscBool trans,PetscScalar kr,PetscScalar ki,Vec xr,Vec xi,Vec *z,PetscReal *norm)
 {
-  PetscErrorCode ierr;
   PetscInt       nmat;
   Mat            A,B;
   Vec            u,w;
@@ -627,40 +616,40 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscBool trans,PetscScala
 
   PetscFunctionBegin;
   u = z[0]; w = z[2];
-  ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
-  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
-  if (nmat>1) { ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr); }
+  PetscCall(STGetNumMatrices(eps->st,&nmat));
+  PetscCall(STGetMatrix(eps->st,0,&A));
+  if (nmat>1) PetscCall(STGetMatrix(eps->st,1,&B));
 
 #if !defined(PETSC_USE_COMPLEX)
   v = z[1];
   if (ki == 0 || PetscAbsScalar(ki) < PetscAbsScalar(kr*PETSC_MACHINE_EPSILON)) {
 #endif
-    ierr = (*matmult)(A,xr,u);CHKERRQ(ierr);                          /* u=A*x */
+    PetscCall((*matmult)(A,xr,u));                          /* u=A*x */
     if (PetscAbsScalar(kr) > PETSC_MACHINE_EPSILON) {
-      if (nmat>1) { ierr = (*matmult)(B,xr,w);CHKERRQ(ierr); }
-      else { ierr = VecCopy(xr,w);CHKERRQ(ierr); }                    /* w=B*x */
+      if (nmat>1) PetscCall((*matmult)(B,xr,w));
+      else PetscCall(VecCopy(xr,w));                        /* w=B*x */
       alpha = trans? -PetscConj(kr): -kr;
-      ierr = VecAXPY(u,alpha,w);CHKERRQ(ierr);                        /* u=A*x-k*B*x */
+      PetscCall(VecAXPY(u,alpha,w));                        /* u=A*x-k*B*x */
     }
-    ierr = VecNorm(u,NORM_2,norm);CHKERRQ(ierr);
+    PetscCall(VecNorm(u,NORM_2,norm));
 #if !defined(PETSC_USE_COMPLEX)
   } else {
-    ierr = (*matmult)(A,xr,u);CHKERRQ(ierr);                          /* u=A*xr */
+    PetscCall((*matmult)(A,xr,u));                          /* u=A*xr */
     if (SlepcAbsEigenvalue(kr,ki) > PETSC_MACHINE_EPSILON) {
-      if (nmat>1) { ierr = (*matmult)(B,xr,v);CHKERRQ(ierr); }
-      else { ierr = VecCopy(xr,v);CHKERRQ(ierr); }                    /* v=B*xr */
-      ierr = VecAXPY(u,-kr,v);CHKERRQ(ierr);                          /* u=A*xr-kr*B*xr */
-      if (nmat>1) { ierr = (*matmult)(B,xi,w);CHKERRQ(ierr); }
-      else { ierr = VecCopy(xi,w);CHKERRQ(ierr); }                    /* w=B*xi */
-      ierr = VecAXPY(u,trans?-ki:ki,w);CHKERRQ(ierr);                 /* u=A*xr-kr*B*xr+ki*B*xi */
+      if (nmat>1) PetscCall((*matmult)(B,xr,v));
+      else PetscCall(VecCopy(xr,v));                        /* v=B*xr */
+      PetscCall(VecAXPY(u,-kr,v));                          /* u=A*xr-kr*B*xr */
+      if (nmat>1) PetscCall((*matmult)(B,xi,w));
+      else PetscCall(VecCopy(xi,w));                        /* w=B*xi */
+      PetscCall(VecAXPY(u,trans?-ki:ki,w));                 /* u=A*xr-kr*B*xr+ki*B*xi */
     }
-    ierr = VecNorm(u,NORM_2,&nr);CHKERRQ(ierr);
-    ierr = (*matmult)(A,xi,u);CHKERRQ(ierr);                          /* u=A*xi */
+    PetscCall(VecNorm(u,NORM_2,&nr));
+    PetscCall((*matmult)(A,xi,u));                          /* u=A*xi */
     if (SlepcAbsEigenvalue(kr,ki) > PETSC_MACHINE_EPSILON) {
-      ierr = VecAXPY(u,-kr,w);CHKERRQ(ierr);                          /* u=A*xi-kr*B*xi */
-      ierr = VecAXPY(u,trans?ki:-ki,v);CHKERRQ(ierr);                 /* u=A*xi-kr*B*xi-ki*B*xr */
+      PetscCall(VecAXPY(u,-kr,w));                          /* u=A*xi-kr*B*xi */
+      PetscCall(VecAXPY(u,trans?ki:-ki,v));                 /* u=A*xi-kr*B*xi-ki*B*xr */
     }
-    ierr = VecNorm(u,NORM_2,&ni);CHKERRQ(ierr);
+    PetscCall(VecNorm(u,NORM_2,&ni));
     *norm = SlepcAbsEigenvalue(nr,ni);
   }
 #endif
@@ -691,7 +680,6 @@ PetscErrorCode EPSComputeResidualNorm_Private(EPS eps,PetscBool trans,PetscScala
 @*/
 PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *error)
 {
-  PetscErrorCode ierr;
   Mat            A,B;
   Vec            xr,xi,w[3];
   PetscReal      t,vecnorm=1.0,errorl;
@@ -707,11 +695,11 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
 
   /* allocate work vectors */
 #if defined(PETSC_USE_COMPLEX)
-  ierr = EPSSetWorkVecs(eps,3);CHKERRQ(ierr);
+  PetscCall(EPSSetWorkVecs(eps,3));
   xi   = NULL;
   w[1] = NULL;
 #else
-  ierr = EPSSetWorkVecs(eps,5);CHKERRQ(ierr);
+  PetscCall(EPSSetWorkVecs(eps,5));
   xi   = eps->work[3];
   w[1] = eps->work[4];
 #endif
@@ -720,18 +708,16 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
   w[2] = eps->work[2];
 
   /* compute residual norm */
-  ierr = EPSGetEigenpair(eps,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
-  ierr = EPSComputeResidualNorm_Private(eps,PETSC_FALSE,kr,ki,xr,xi,w,error);CHKERRQ(ierr);
+  PetscCall(EPSGetEigenpair(eps,i,&kr,&ki,xr,xi));
+  PetscCall(EPSComputeResidualNorm_Private(eps,PETSC_FALSE,kr,ki,xr,xi,w,error));
 
   /* compute 2-norm of eigenvector */
-  if (eps->problem_type==EPS_GHEP) {
-    ierr = VecNorm(xr,NORM_2,&vecnorm);CHKERRQ(ierr);
-  }
+  if (eps->problem_type==EPS_GHEP) PetscCall(VecNorm(xr,NORM_2,&vecnorm));
 
   /* if two-sided, compute left residual norm and take the maximum */
   if (eps->twosided) {
-    ierr = EPSGetLeftEigenvector(eps,i,xr,xi);CHKERRQ(ierr);
-    ierr = EPSComputeResidualNorm_Private(eps,PETSC_TRUE,kr,ki,xr,xi,w,&errorl);CHKERRQ(ierr);
+    PetscCall(EPSGetLeftEigenvector(eps,i,xr,xi));
+    PetscCall(EPSComputeResidualNorm_Private(eps,PETSC_TRUE,kr,ki,xr,xi,w,&errorl));
     *error = PetscMax(*error,errorl);
   }
 
@@ -745,17 +731,17 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
     case EPS_ERROR_BACKWARD:
       /* initialization of matrix norms */
       if (!eps->nrma) {
-        ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
-        ierr = MatHasOperation(A,MATOP_NORM,&flg);CHKERRQ(ierr);
-        if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
-        ierr = MatNorm(A,NORM_INFINITY,&eps->nrma);CHKERRQ(ierr);
+        PetscCall(STGetMatrix(eps->st,0,&A));
+        PetscCall(MatHasOperation(A,MATOP_NORM,&flg));
+        PetscCheck(flg,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
+        PetscCall(MatNorm(A,NORM_INFINITY,&eps->nrma));
       }
       if (eps->isgeneralized) {
         if (!eps->nrmb) {
-          ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
-          ierr = MatHasOperation(B,MATOP_NORM,&flg);CHKERRQ(ierr);
-          if (!flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
-          ierr = MatNorm(B,NORM_INFINITY,&eps->nrmb);CHKERRQ(ierr);
+          PetscCall(STGetMatrix(eps->st,1,&B));
+          PetscCall(MatHasOperation(B,MATOP_NORM,&flg));
+          PetscCheck(flg,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
+          PetscCall(MatNorm(B,NORM_INFINITY,&eps->nrmb));
         }
       } else eps->nrmb = 1.0;
       t = SlepcAbsEigenvalue(kr,ki);
@@ -793,7 +779,6 @@ PetscErrorCode EPSComputeError(EPS eps,PetscInt i,EPSErrorType type,PetscReal *e
 */
 PetscErrorCode EPSGetStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
 {
-  PetscErrorCode ierr;
   PetscReal      norm;
   PetscBool      lindep;
   Vec            w,z;
@@ -803,28 +788,26 @@ PetscErrorCode EPSGetStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
   PetscValidLogicalCollectiveInt(eps,i,2);
 
   /* For the first step, use the first initial vector, otherwise a random one */
-  if (i>0 || eps->nini==0) {
-    ierr = BVSetRandomColumn(eps->V,i);CHKERRQ(ierr);
-  }
+  if (i>0 || eps->nini==0) PetscCall(BVSetRandomColumn(eps->V,i));
 
   /* Force the vector to be in the range of OP for definite generalized problems */
   if (eps->ispositive || (eps->isgeneralized && eps->ishermitian)) {
-    ierr = BVCreateVec(eps->V,&w);CHKERRQ(ierr);
-    ierr = BVCopyVec(eps->V,i,w);CHKERRQ(ierr);
-    ierr = BVGetColumn(eps->V,i,&z);CHKERRQ(ierr);
-    ierr = STApply(eps->st,w,z);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,i,&z);CHKERRQ(ierr);
-    ierr = VecDestroy(&w);CHKERRQ(ierr);
+    PetscCall(BVCreateVec(eps->V,&w));
+    PetscCall(BVCopyVec(eps->V,i,w));
+    PetscCall(BVGetColumn(eps->V,i,&z));
+    PetscCall(STApply(eps->st,w,z));
+    PetscCall(BVRestoreColumn(eps->V,i,&z));
+    PetscCall(VecDestroy(&w));
   }
 
   /* Orthonormalize the vector with respect to previous vectors */
-  ierr = BVOrthogonalizeColumn(eps->V,i,NULL,&norm,&lindep);CHKERRQ(ierr);
+  PetscCall(BVOrthogonalizeColumn(eps->V,i,NULL,&norm,&lindep));
   if (breakdown) *breakdown = lindep;
   else if (lindep || norm == 0.0) {
-    if (i==0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Initial vector is zero or belongs to the deflation space");
-    else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more start vectors");
+    PetscCheck(i,PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Initial vector is zero or belongs to the deflation space");
+    PetscCheck(!i,PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more start vectors");
   }
-  ierr = BVScaleColumn(eps->V,i,1.0/norm);CHKERRQ(ierr);
+  PetscCall(BVScaleColumn(eps->V,i,1.0/norm));
   PetscFunctionReturn(0);
 }
 
@@ -834,7 +817,6 @@ PetscErrorCode EPSGetStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
 */
 PetscErrorCode EPSGetLeftStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
 {
-  PetscErrorCode ierr;
   PetscReal      norm;
   PetscBool      lindep;
 
@@ -843,18 +825,15 @@ PetscErrorCode EPSGetLeftStartVector(EPS eps,PetscInt i,PetscBool *breakdown)
   PetscValidLogicalCollectiveInt(eps,i,2);
 
   /* For the first step, use the first initial vector, otherwise a random one */
-  if (i>0 || eps->ninil==0) {
-    ierr = BVSetRandomColumn(eps->W,i);CHKERRQ(ierr);
-  }
+  if (i>0 || eps->ninil==0) PetscCall(BVSetRandomColumn(eps->W,i));
 
   /* Orthonormalize the vector with respect to previous vectors */
-  ierr = BVOrthogonalizeColumn(eps->W,i,NULL,&norm,&lindep);CHKERRQ(ierr);
+  PetscCall(BVOrthogonalizeColumn(eps->W,i,NULL,&norm,&lindep));
   if (breakdown) *breakdown = lindep;
   else if (lindep || norm == 0.0) {
-    if (i==0) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Left initial vector is zero");
-    else SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more left start vectors");
+    PetscCheck(i,PetscObjectComm((PetscObject)eps),PETSC_ERR_PLIB,"Left initial vector is zero");
+    SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_CONV_FAILED,"Unable to generate more left start vectors");
   }
-  ierr = BVScaleColumn(eps->W,i,1.0/norm);CHKERRQ(ierr);
+  PetscCall(BVScaleColumn(eps->W,i,1.0/norm));
   PetscFunctionReturn(0);
 }
-
