@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -20,25 +20,24 @@
 */
 PetscErrorCode LMEDenseRankSVD(LME lme,PetscInt n,PetscScalar *A,PetscInt lda,PetscScalar *U,PetscInt ldu,PetscInt *rank)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,rk=0;
   PetscScalar    *work;
   PetscReal      tol,*sg,*rwork;
   PetscBLASInt   n_,lda_,ldu_,info,lw_;
 
   PetscFunctionBegin;
-  ierr = PetscCalloc3(n,&sg,10*n,&work,5*n,&rwork);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(n,&n_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(lda,&lda_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldu,&ldu_);CHKERRQ(ierr);
+  PetscCall(PetscCalloc3(n,&sg,10*n,&work,5*n,&rwork));
+  PetscCall(PetscBLASIntCast(n,&n_));
+  PetscCall(PetscBLASIntCast(lda,&lda_));
+  PetscCall(PetscBLASIntCast(ldu,&ldu_));
   lw_ = 10*n_;
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 #if !defined (PETSC_USE_COMPLEX)
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","O",&n_,&n_,A,&lda_,sg,U,&ldu_,NULL,&n_,work,&lw_,&info));
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","O",&n_,&n_,A,&lda_,sg,U,&ldu_,NULL,&n_,work,&lw_,rwork,&info));
 #endif
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
   SlepcCheckLapackInfo("gesvd",info);
   tol = 10*PETSC_MACHINE_EPSILON*n*sg[0];
   for (j=0;j<n;j++) {
@@ -48,7 +47,7 @@ PetscErrorCode LMEDenseRankSVD(LME lme,PetscInt n,PetscScalar *A,PetscInt lda,Pe
     } else break;
   }
   *rank = rk;
-  ierr = PetscFree3(sg,work,rwork);CHKERRQ(ierr);
+  PetscCall(PetscFree3(sg,work,rwork));
   PetscFunctionReturn(0);
 }
 
@@ -58,18 +57,17 @@ PetscErrorCode LMEDenseRankSVD(LME lme,PetscInt n,PetscScalar *A,PetscInt lda,Pe
 */
 static PetscErrorCode LyapunovCholResidual(PetscInt m,PetscScalar *A,PetscInt lda,PetscInt k,PetscScalar *B,PetscInt ldb,PetscScalar *U,PetscInt ldu,PetscReal *res)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   n,kk,la,lb,lu;
   PetscScalar    *M,*R,zero=0.0,done=1.0;
 
   PetscFunctionBegin;
   *res = 0;
-  ierr = PetscBLASIntCast(lda,&la);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldb,&lb);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldu,&lu);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(k,&kk);CHKERRQ(ierr);
-  ierr = PetscMalloc2(m*m,&M,m*m,&R);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(lda,&la));
+  PetscCall(PetscBLASIntCast(ldb,&lb));
+  PetscCall(PetscBLASIntCast(ldu,&lu));
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscBLASIntCast(k,&kk));
+  PetscCall(PetscMalloc2(m*m,&M,m*m,&R));
 
   /* R = B*B' */
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","C",&n,&n,&kk,&done,B,&lb,B,&lb,&zero,R,&n));
@@ -81,7 +79,7 @@ static PetscErrorCode LyapunovCholResidual(PetscInt m,PetscScalar *A,PetscInt ld
   PetscStackCallBLAS("BLASgemm",BLASgemm_("C","C",&n,&n,&n,&done,U,&lu,M,&n,&done,R,&n));
 
   *res = LAPACKlange_("F",&n,&n,R,&n,NULL);
-  ierr = PetscFree2(M,R);CHKERRQ(ierr);
+  PetscCall(PetscFree2(M,R));
   PetscFunctionReturn(0);
 }
 
@@ -90,29 +88,26 @@ static PetscErrorCode LyapunovCholResidual(PetscInt m,PetscScalar *A,PetscInt ld
 */
 static PetscErrorCode LyapunovResidual(PetscInt m,PetscScalar *A,PetscInt lda,PetscScalar *B,PetscInt ldb,PetscScalar *X,PetscInt ldx,PetscReal *res)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
   PetscBLASInt   n,la,lb,lx;
   PetscScalar    *R,done=1.0;
 
   PetscFunctionBegin;
   *res = 0;
-  ierr = PetscBLASIntCast(lda,&la);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldb,&lb);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldx,&lx);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscMalloc1(m*m,&R);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(lda,&la));
+  PetscCall(PetscBLASIntCast(ldb,&lb));
+  PetscCall(PetscBLASIntCast(ldx,&lx));
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscMalloc1(m*m,&R));
 
   /* R = B+A*X */
-  for (i=0;i<m;i++) {
-    ierr = PetscArraycpy(R+i*m,B+i*ldb,m);CHKERRQ(ierr);
-  }
+  for (i=0;i<m;i++) PetscCall(PetscArraycpy(R+i*m,B+i*ldb,m));
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&done,A,&la,X,&lx,&done,R,&n));
   /* R = R+X*A' */
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","C",&n,&n,&n,&done,X,&lx,A,&la,&done,R,&n));
 
   *res = LAPACKlange_("F",&n,&n,R,&n,NULL);
-  ierr = PetscFree(R);CHKERRQ(ierr);
+  PetscCall(PetscFree(R));
   PetscFunctionReturn(0);
 }
 #endif
@@ -123,21 +118,20 @@ static PetscErrorCode LyapunovResidual(PetscInt m,PetscScalar *A,PetscInt lda,Pe
 */
 static PetscErrorCode HessLyapunovChol_SLICOT(PetscInt m,PetscScalar *H,PetscInt ldh,PetscInt k,PetscScalar *B,PetscInt ldb,PetscScalar *U,PetscInt ldu,PetscReal *res)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   lwork,info,n,kk,lu,ione=1,sdim;
   PetscInt       i,j;
   PetscReal      scal;
   PetscScalar    *Q,*W,*wr,*wi,*work;
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ldu,&lu);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(k,&kk);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(6*m,&lwork);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ldu,&lu));
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscBLASIntCast(k,&kk));
+  PetscCall(PetscBLASIntCast(6*m,&lwork));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 
   /* transpose W = H' */
-  ierr = PetscMalloc5(m*m,&W,m*m,&Q,m,&wr,m,&wi,lwork,&work);CHKERRQ(ierr);
+  PetscCall(PetscMalloc5(m*m,&W,m*m,&Q,m,&wr,m,&wi,lwork,&work));
   for (j=0;j<m;j++) {
     for (i=0;i<m;i++) W[i+j*m] = H[j+i*ldh];
   }
@@ -146,7 +140,7 @@ static PetscErrorCode HessLyapunovChol_SLICOT(PetscInt m,PetscScalar *H,PetscInt
   PetscStackCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,W,&n,&sdim,wr,wi,Q,&n,work,&lwork,NULL,&info));
   SlepcCheckLapackInfo("gees",info);
 #if defined(PETSC_USE_DEBUG)
-  for (i=0;i<m;i++) if (PetscRealPart(wr[i])>=0.0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_USER_INPUT,"Eigenvalue with non-negative real part, the coefficient matrix is not stable");
+  for (i=0;i<m;i++) PetscCheck(PetscRealPart(wr[i])<0.0,PETSC_COMM_SELF,PETSC_ERR_USER_INPUT,"Eigenvalue with non-negative real part, the coefficient matrix is not stable");
 #endif
 
   /* copy B' into first rows of U */
@@ -156,19 +150,19 @@ static PetscErrorCode HessLyapunovChol_SLICOT(PetscInt m,PetscScalar *H,PetscInt
 
   /* solve Lyapunov equation (Hammarling) */
   PetscStackCallBLAS("SLICOTsb03od",SLICOTsb03od_("C","F","N",&n,&kk,W,&n,Q,&n,U,&lu,&scal,wr,wi,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in SLICOT subroutine SB03OD: info=%d",(int)info);
-  if (scal!=1.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
+  PetscCheck(!info,PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in SLICOT subroutine SB03OD: info=%" PetscBLASInt_FMT,info);
+  PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
 
   /* resnorm = norm(H(m+1,:)*U'*U), use Q(:,1) = U'*U(:,m) */
   if (res) {
     for (j=0;j<m;j++) Q[j] = U[j+(m-1)*ldu];
     PetscStackCallBLAS("BLAStrmv",BLAStrmv_("U","C","N",&n,U,&lu,Q,&ione));
-    if (k!=1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Residual error is intended for k=1 only, but you set k=%d",(int)k);
+    PetscCheck(k==1,PETSC_COMM_SELF,PETSC_ERR_LIB,"Residual error is intended for k=1 only, but you set k=%" PetscInt_FMT,k);
     *res *= BLASnrm2_(&n,Q,&ione);
   }
 
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscFree5(W,Q,wr,wi,work);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscFree5(W,Q,wr,wi,work));
   PetscFunctionReturn(0);
 }
 
@@ -179,42 +173,37 @@ static PetscErrorCode HessLyapunovChol_SLICOT(PetscInt m,PetscScalar *H,PetscInt
  */
 static PetscErrorCode CholeskyFactor(PetscInt m,PetscScalar *A,PetscInt lda)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
   PetscScalar    *S;
   PetscBLASInt   info,n,ld;
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(lda,&ld);CHKERRQ(ierr);
-  ierr = PetscMalloc1(m*m,&S);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscBLASIntCast(lda,&ld));
+  PetscCall(PetscMalloc1(m*m,&S));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 
   /* save a copy of matrix in S */
-  for (i=0;i<m;i++) {
-    ierr = PetscArraycpy(S+i*m,A+i*lda,m);CHKERRQ(ierr);
-  }
+  for (i=0;i<m;i++) PetscCall(PetscArraycpy(S+i*m,A+i*lda,m));
 
   /* compute upper Cholesky factor in R */
   PetscStackCallBLAS("LAPACKpotrf",LAPACKpotrf_("U",&n,A,&ld,&info));
-  ierr = PetscLogFlops((1.0*n*n*n)/3.0);CHKERRQ(ierr);
+  PetscCall(PetscLogFlops((1.0*n*n*n)/3.0));
 
   if (info) {
     for (i=0;i<m;i++) {
-      ierr = PetscArraycpy(A+i*lda,S+i*m,m);CHKERRQ(ierr);
+      PetscCall(PetscArraycpy(A+i*lda,S+i*m,m));
       A[i+i*lda] += 50.0*PETSC_MACHINE_EPSILON;
     }
     PetscStackCallBLAS("LAPACKpotrf",LAPACKpotrf_("U",&n,A,&ld,&info));
     SlepcCheckLapackInfo("potrf",info);
-    ierr = PetscLogFlops((1.0*n*n*n)/3.0);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops((1.0*n*n*n)/3.0));
   }
 
   /* Zero out entries below the diagonal */
-  for (i=0;i<m-1;i++) {
-    ierr = PetscArrayzero(A+i*lda+i+1,m-i-1);CHKERRQ(ierr);
-  }
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscFree(S);CHKERRQ(ierr);
+  for (i=0;i<m-1;i++) PetscCall(PetscArrayzero(A+i*lda+i+1,m-i-1));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscFree(S));
   PetscFunctionReturn(0);
 }
 
@@ -223,7 +212,6 @@ static PetscErrorCode CholeskyFactor(PetscInt m,PetscScalar *A,PetscInt lda)
 */
 static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt ldh,PetscInt k,PetscScalar *B,PetscInt ldb,PetscScalar *U,PetscInt ldu,PetscReal *res)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   ilo=1,lwork,info,n,kk,lu,lb,ione=1;
   PetscInt       i,j;
   PetscReal      scal;
@@ -233,18 +221,18 @@ static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt
 #endif
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ldb,&lb);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldu,&lu);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(k,&kk);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(6*m,&lwork);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ldb,&lb));
+  PetscCall(PetscBLASIntCast(ldu,&lu));
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscBLASIntCast(k,&kk));
+  PetscCall(PetscBLASIntCast(6*m,&lwork));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
   C = U;
 
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscMalloc6(m*m,&Q,m*m,&W,m*k,&Z,m,&wr,m,&wi,lwork,&work);CHKERRQ(ierr);
+  PetscCall(PetscMalloc6(m*m,&Q,m*m,&W,m*k,&Z,m,&wr,m,&wi,lwork,&work));
 #else
-  ierr = PetscMalloc5(m*m,&Q,m*m,&W,m*k,&Z,m,&wr,lwork,&work);CHKERRQ(ierr);
+  PetscCall(PetscMalloc5(m*m,&Q,m*m,&W,m*k,&Z,m,&wr,lwork,&work));
 #endif
 
   /* save a copy W = H */
@@ -260,7 +248,7 @@ static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt
 #endif
   SlepcCheckLapackInfo("hseqr",info);
 #if defined(PETSC_USE_DEBUG)
-  for (i=0;i<m;i++) if (PetscRealPart(wr[i])>=0.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER_INPUT,"Eigenvalue with non-negative real part %g, the coefficient matrix is not stable",PetscRealPart(wr[i]));
+  for (i=0;i<m;i++) PetscCheck(PetscRealPart(wr[i])<0.0,PETSC_COMM_SELF,PETSC_ERR_USER_INPUT,"Eigenvalue with non-negative real part %g, the coefficient matrix is not stable",PetscRealPart(wr[i]));
 #endif
 
   /* C = -Z*Z', Z = Q'*B */
@@ -270,7 +258,7 @@ static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt
   /* solve triangular Sylvester equation */
   PetscStackCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","C",&ione,&n,&n,W,&n,W,&n,C,&lu,&scal,&info));
   SlepcCheckLapackInfo("trsyl",info);
-  if (scal!=1.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
+  PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",(double)scal);
 
   /* back-transform C = Q*C*Q' */
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&done,Q,&n,C,&n,&zero,W,&n));
@@ -278,18 +266,18 @@ static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt
 
   /* resnorm = norm(H(m+1,:)*Y) */
   if (res) {
-    if (k!=1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Residual error is intended for k=1 only, but you set k=%d",(int)k);
+    PetscCheck(k==1,PETSC_COMM_SELF,PETSC_ERR_LIB,"Residual error is intended for k=1 only, but you set k=%" PetscInt_FMT,k);
     *res *= BLASnrm2_(&n,C+m-1,&n);
   }
 
   /* U = chol(C) */
-  ierr = CholeskyFactor(m,C,ldu);CHKERRQ(ierr);
+  PetscCall(CholeskyFactor(m,C,ldu));
 
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscFree6(Q,W,Z,wr,wi,work);CHKERRQ(ierr);
+  PetscCall(PetscFree6(Q,W,Z,wr,wi,work));
 #else
-  ierr = PetscFree5(Q,W,Z,wr,work);CHKERRQ(ierr);
+  PetscCall(PetscFree5(Q,W,Z,wr,work));
 #endif
   PetscFunctionReturn(0);
 }
@@ -312,11 +300,9 @@ static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt
 .  ldb - leading dimension of B
 -  ldu - leading dimension of U
 
-   Output Parameter:
-.  U   - Cholesky factor of the solution
-
-   Input/Output Parameter:
-.  res - (optional) residual norm, on input it should contain H(m+1,m)
+   Output Parameters:
++  U   - Cholesky factor of the solution
+-  res - (optional) residual norm, on input it should contain H(m+1,m)
 
    Note:
    The Lyapunov equation has the form H*X + X*H' = -B*B', where H is an mxm
@@ -332,7 +318,6 @@ static PetscErrorCode HessLyapunovChol_LAPACK(PetscInt m,PetscScalar *H,PetscInt
 @*/
 PetscErrorCode LMEDenseHessLyapunovChol(LME lme,PetscInt m,PetscScalar *H,PetscInt ldh,PetscInt k,PetscScalar *B,PetscInt ldb,PetscScalar *U,PetscInt ldu,PetscReal *res)
 {
-  PetscErrorCode ierr;
 #if defined(PETSC_USE_INFO)
   PetscReal      error;
 #endif
@@ -340,25 +325,25 @@ PetscErrorCode LMEDenseHessLyapunovChol(LME lme,PetscInt m,PetscScalar *H,PetscI
   PetscFunctionBegin;
   PetscValidHeaderSpecific(lme,LME_CLASSID,1);
   PetscValidLogicalCollectiveInt(lme,m,2);
-  PetscValidPointer(H,3);
+  PetscValidScalarPointer(H,3);
   PetscValidLogicalCollectiveInt(lme,ldh,4);
   PetscValidLogicalCollectiveInt(lme,k,5);
-  PetscValidPointer(B,6);
+  PetscValidScalarPointer(B,6);
   PetscValidLogicalCollectiveInt(lme,ldb,7);
-  PetscValidPointer(U,8);
+  PetscValidScalarPointer(U,8);
   PetscValidLogicalCollectiveInt(lme,ldu,9);
   if (res) PetscValidLogicalCollectiveReal(lme,*res,10);
 
 #if defined(SLEPC_HAVE_SLICOT)
-  ierr = HessLyapunovChol_SLICOT(m,H,ldh,k,B,ldb,U,ldu,res);CHKERRQ(ierr);
+  PetscCall(HessLyapunovChol_SLICOT(m,H,ldh,k,B,ldb,U,ldu,res));
 #else
-  ierr = HessLyapunovChol_LAPACK(m,H,ldh,k,B,ldb,U,ldu,res);CHKERRQ(ierr);
+  PetscCall(HessLyapunovChol_LAPACK(m,H,ldh,k,B,ldb,U,ldu,res));
 #endif
 
 #if defined(PETSC_USE_INFO)
   if (PetscLogPrintInfo) {
-    ierr = LyapunovCholResidual(m,H,ldh,k,B,ldb,U,ldu,&error);CHKERRQ(ierr);
-    ierr = PetscInfo1(lme,"Residual norm of dense Lyapunov equation = %g\n",error);CHKERRQ(ierr);
+    PetscCall(LyapunovCholResidual(m,H,ldh,k,B,ldb,U,ldu,&error));
+    PetscCall(PetscInfo(lme,"Residual norm of dense Lyapunov equation = %g\n",(double)error));
   }
 #endif
   PetscFunctionReturn(0);
@@ -370,20 +355,19 @@ PetscErrorCode LMEDenseHessLyapunovChol(LME lme,PetscInt m,PetscScalar *H,PetscI
 */
 static PetscErrorCode Lyapunov_SLICOT(PetscInt m,PetscScalar *H,PetscInt ldh,PetscScalar *B,PetscInt ldb,PetscScalar *X,PetscInt ldx)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   sdim,lwork,info,n,lx,*iwork;
   PetscInt       i,j;
   PetscReal      scal,sep,ferr,*work;
   PetscScalar    *Q,*W,*wr,*wi;
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ldx,&lx);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(PetscMax(20,m*m),&lwork);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ldx,&lx));
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscBLASIntCast(PetscMax(20,m*m),&lwork));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 
   /* transpose W = H' */
-  ierr = PetscMalloc6(m*m,&W,m*m,&Q,m,&wr,m,&wi,m*m,&iwork,lwork,&work);CHKERRQ(ierr);
+  PetscCall(PetscMalloc6(m*m,&W,m*m,&Q,m,&wr,m,&wi,m*m,&iwork,lwork,&work));
   for (j=0;j<m;j++) {
     for (i=0;i<m;i++) W[i+j*m] = H[j+i*ldh];
   }
@@ -399,11 +383,11 @@ static PetscErrorCode Lyapunov_SLICOT(PetscInt m,PetscScalar *H,PetscInt ldh,Pet
 
   /* solve Lyapunov equation (Hammarling) */
   PetscStackCallBLAS("SLICOTsb03md",SLICOTsb03md_("C","X","F","N",&n,W,&n,Q,&n,X,&lx,&scal,&sep,&ferr,wr,wi,iwork,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in SLICOT subroutine SB03OD: info=%d",(int)info);
-  if (scal!=1.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
+  PetscCheck(!info,PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in SLICOT subroutine SB03OD: info=%" PetscBLASInt_FMT,info);
+  PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
 
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscFree6(W,Q,wr,wi,iwork,work);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscFree6(W,Q,wr,wi,iwork,work));
   PetscFunctionReturn(0);
 }
 
@@ -414,7 +398,6 @@ static PetscErrorCode Lyapunov_SLICOT(PetscInt m,PetscScalar *H,PetscInt ldh,Pet
 */
 static PetscErrorCode Lyapunov_LAPACK(PetscInt m,PetscScalar *A,PetscInt lda,PetscScalar *B,PetscInt ldb,PetscScalar *X,PetscInt ldx)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   sdim,lwork,info,n,lx,lb,ione=1;
   PetscInt       i,j;
   PetscReal      scal;
@@ -426,16 +409,16 @@ static PetscErrorCode Lyapunov_LAPACK(PetscInt m,PetscScalar *A,PetscInt lda,Pet
 #endif
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ldb,&lb);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ldx,&lx);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(6*m,&lwork);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ldb,&lb));
+  PetscCall(PetscBLASIntCast(ldx,&lx));
+  PetscCall(PetscBLASIntCast(m,&n));
+  PetscCall(PetscBLASIntCast(6*m,&lwork));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscMalloc6(m*m,&Q,m*m,&W,m*m,&Z,m,&wr,m,&wi,lwork,&work);CHKERRQ(ierr);
+  PetscCall(PetscMalloc6(m*m,&Q,m*m,&W,m*m,&Z,m,&wr,m,&wi,lwork,&work));
 #else
-  ierr = PetscMalloc6(m*m,&Q,m*m,&W,m*m,&Z,m,&wr,lwork,&work,m,&rwork);CHKERRQ(ierr);
+  PetscCall(PetscMalloc6(m*m,&Q,m*m,&W,m*m,&Z,m,&wr,lwork,&work,m,&rwork));
 #endif
 
   /* save a copy W = A */
@@ -458,17 +441,17 @@ static PetscErrorCode Lyapunov_LAPACK(PetscInt m,PetscScalar *A,PetscInt lda,Pet
   /* solve triangular Sylvester equation */
   PetscStackCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","C",&ione,&n,&n,W,&n,W,&n,X,&lx,&scal,&info));
   SlepcCheckLapackInfo("trsyl",info);
-  if (scal!=1.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
+  PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",(double)scal);
 
   /* back-transform X = Q*X*Q' */
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&done,Q,&n,X,&n,&zero,W,&n));
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","C",&n,&n,&n,&done,W,&n,Q,&n,&zero,X,&lx));
 
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscFree6(Q,W,Z,wr,wi,work);CHKERRQ(ierr);
+  PetscCall(PetscFree6(Q,W,Z,wr,wi,work));
 #else
-  ierr = PetscFree6(Q,W,Z,wr,work,rwork);CHKERRQ(ierr);
+  PetscCall(PetscFree6(Q,W,Z,wr,work,rwork));
 #endif
   PetscFunctionReturn(0);
 }
@@ -503,7 +486,6 @@ static PetscErrorCode Lyapunov_LAPACK(PetscInt m,PetscScalar *A,PetscInt lda,Pet
 @*/
 PetscErrorCode LMEDenseLyapunov(LME lme,PetscInt m,PetscScalar *A,PetscInt lda,PetscScalar *B,PetscInt ldb,PetscScalar *X,PetscInt ldx)
 {
-  PetscErrorCode ierr;
 #if defined(PETSC_USE_INFO)
   PetscReal      error;
 #endif
@@ -511,25 +493,24 @@ PetscErrorCode LMEDenseLyapunov(LME lme,PetscInt m,PetscScalar *A,PetscInt lda,P
   PetscFunctionBegin;
   PetscValidHeaderSpecific(lme,LME_CLASSID,1);
   PetscValidLogicalCollectiveInt(lme,m,2);
-  PetscValidPointer(A,3);
+  PetscValidScalarPointer(A,3);
   PetscValidLogicalCollectiveInt(lme,lda,4);
-  PetscValidPointer(B,5);
+  PetscValidScalarPointer(B,5);
   PetscValidLogicalCollectiveInt(lme,ldb,6);
-  PetscValidPointer(X,7);
+  PetscValidScalarPointer(X,7);
   PetscValidLogicalCollectiveInt(lme,ldx,8);
 
 #if defined(SLEPC_HAVE_SLICOT)
-  ierr = Lyapunov_SLICOT(m,A,lda,B,ldb,X,ldx);CHKERRQ(ierr);
+  PetscCall(Lyapunov_SLICOT(m,A,lda,B,ldb,X,ldx));
 #else
-  ierr = Lyapunov_LAPACK(m,A,lda,B,ldb,X,ldx);CHKERRQ(ierr);
+  PetscCall(Lyapunov_LAPACK(m,A,lda,B,ldb,X,ldx));
 #endif
 
 #if defined(PETSC_USE_INFO)
   if (PetscLogPrintInfo) {
-    ierr = LyapunovResidual(m,A,lda,B,ldb,X,ldx,&error);CHKERRQ(ierr);
-    ierr = PetscInfo1(lme,"Residual norm of dense Lyapunov equation = %g\n",error);CHKERRQ(ierr);
+    PetscCall(LyapunovResidual(m,A,lda,B,ldb,X,ldx,&error));
+    PetscCall(PetscInfo(lme,"Residual norm of dense Lyapunov equation = %g\n",(double)error));
   }
 #endif
   PetscFunctionReturn(0);
 }
-
