@@ -1,13 +1,13 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !
-!  Include file for Fortran use of the MFN object in SLEPc
+!  Used by slepcmfnmod.F90 to create Fortran module file
 !
 #include "slepc/finclude/slepcsys.h"
 #include "slepc/finclude/slepcmfn.h"
@@ -28,8 +28,4 @@
 !   Possible arguments to MFNMonitorSet()
 !
       external MFNMONITORDEFAULT
-      external MFNMONITORLG
 
-!
-!  End of Fortran include file for the MFN package in SLEPc
-!
