@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -22,31 +22,43 @@
    Collective on mfn
 
    Input Parameters:
-+  mfn      - the eigensolver context
-.  name     - the monitor option name
-.  help     - message indicating what monitoring is done
-.  manual   - manual page for the monitor
--  monitor  - the monitor function, whose context is a PetscViewerAndFormat
++  mfn  - the matrix function context
+.  opt  - the command line option for this monitor
+.  name - the monitor type one is seeking
+-  ctx  - an optional user context for the monitor, or NULL
 
    Level: developer
 
 .seealso: MFNMonitorSet()
 @*/
-PetscErrorCode MFNMonitorSetFromOptions(MFN mfn,const char name[],const char help[],const char manual[],PetscErrorCode (*monitor)(MFN,PetscInt,PetscReal,PetscViewerAndFormat*))
+PetscErrorCode MFNMonitorSetFromOptions(MFN mfn,const char opt[],const char name[],void *ctx)
 {
-  PetscErrorCode       ierr;
-  PetscBool            flg;
+  PetscErrorCode       (*mfunc)(MFN,PetscInt,PetscReal,void*);
+  PetscErrorCode       (*cfunc)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
+  PetscErrorCode       (*dfunc)(PetscViewerAndFormat**);
+  PetscViewerAndFormat *vf;
   PetscViewer          viewer;
   PetscViewerFormat    format;
-  PetscViewerAndFormat *vf;
+  PetscViewerType      vtype;
+  char                 key[PETSC_MAX_PATH_LEN];
+  PetscBool            flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)mfn),((PetscObject)mfn)->options,((PetscObject)mfn)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscViewerAndFormatCreate(viewer,format,&vf);CHKERRQ(ierr);
-    ierr = PetscObjectDereference((PetscObject)viewer);CHKERRQ(ierr);
-    ierr = MFNMonitorSet(mfn,(PetscErrorCode (*)(MFN,PetscInt,PetscReal,void*))monitor,vf,(PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);CHKERRQ(ierr);
-  }
+  PetscCall(PetscOptionsGetViewer(PetscObjectComm((PetscObject)mfn),((PetscObject)mfn)->options,((PetscObject)mfn)->prefix,opt,&viewer,&format,&flg));
+  if (!flg) PetscFunctionReturn(0);
+
+  PetscCall(PetscViewerGetType(viewer,&vtype));
+  PetscCall(SlepcMonitorMakeKey_Internal(name,vtype,format,key));
+  PetscCall(PetscFunctionListFind(MFNMonitorList,key,&mfunc));
+  PetscCheck(mfunc,PetscObjectComm((PetscObject)mfn),PETSC_ERR_SUP,"Specified viewer and format not supported");
+  PetscCall(PetscFunctionListFind(MFNMonitorCreateList,key,&cfunc));
+  PetscCall(PetscFunctionListFind(MFNMonitorDestroyList,key,&dfunc));
+  if (!cfunc) cfunc = PetscViewerAndFormatCreate_Internal;
+  if (!dfunc) dfunc = PetscViewerAndFormatDestroy;
+
+  PetscCall((*cfunc)(viewer,format,ctx,&vf));
+  PetscCall(PetscObjectDereference((PetscObject)viewer));
+  PetscCall(MFNMonitorSet(mfn,mfunc,vf,(PetscErrorCode(*)(void **))dfunc));
   PetscFunctionReturn(0);
 }
 
@@ -64,6 +76,8 @@ PetscErrorCode MFNMonitorSetFromOptions(MFN mfn,const char name[],const char hel
    To see all options, run your program with the -help option.
 
    Level: beginner
+
+.seealso: MFNSetOptionsPrefix()
 @*/
 PetscErrorCode MFNSetFromOptions(MFN mfn)
 {
@@ -72,65 +86,46 @@ PetscErrorCode MFNSetFromOptions(MFN mfn)
   PetscBool      set,flg,flg1,flg2;
   PetscReal      r;
   PetscInt       i;
-  PetscDrawLG    lg;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
-  ierr = MFNRegisterAll();CHKERRQ(ierr);
-  ierr = PetscObjectOptionsBegin((PetscObject)mfn);CHKERRQ(ierr);
-    ierr = PetscOptionsFList("-mfn_type","Matrix Function method","MFNSetType",MFNList,(char*)(((PetscObject)mfn)->type_name?((PetscObject)mfn)->type_name:MFNKRYLOV),type,256,&flg);CHKERRQ(ierr);
-    if (flg) {
-      ierr = MFNSetType(mfn,type);CHKERRQ(ierr);
-    } else if (!((PetscObject)mfn)->type_name) {
-      ierr = MFNSetType(mfn,MFNKRYLOV);CHKERRQ(ierr);
-    }
+  PetscCall(MFNRegisterAll());
+  ierr = PetscObjectOptionsBegin((PetscObject)mfn);PetscCall(ierr);
+    PetscCall(PetscOptionsFList("-mfn_type","Matrix Function method","MFNSetType",MFNList,(char*)(((PetscObject)mfn)->type_name?((PetscObject)mfn)->type_name:MFNKRYLOV),type,sizeof(type),&flg));
+    if (flg) PetscCall(MFNSetType(mfn,type));
+    else if (!((PetscObject)mfn)->type_name) PetscCall(MFNSetType(mfn,MFNKRYLOV));
 
     i = mfn->max_it;
-    ierr = PetscOptionsInt("-mfn_max_it","Maximum number of iterations","MFNSetTolerances",mfn->max_it,&i,&flg1);CHKERRQ(ierr);
+    PetscCall(PetscOptionsInt("-mfn_max_it","Maximum number of iterations","MFNSetTolerances",mfn->max_it,&i,&flg1));
     if (!flg1) i = PETSC_DEFAULT;
     r = mfn->tol;
-    ierr = PetscOptionsReal("-mfn_tol","Tolerance","MFNSetTolerances",mfn->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:mfn->tol,&r,&flg2);CHKERRQ(ierr);
-    if (flg1 || flg2) { ierr = MFNSetTolerances(mfn,r,i);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsReal("-mfn_tol","Tolerance","MFNSetTolerances",SlepcDefaultTol(mfn->tol),&r,&flg2));
+    if (flg1 || flg2) PetscCall(MFNSetTolerances(mfn,r,i));
 
-    ierr = PetscOptionsInt("-mfn_ncv","Number of basis vectors","MFNSetDimensions",mfn->ncv,&i,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = MFNSetDimensions(mfn,i);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsInt("-mfn_ncv","Number of basis vectors","MFNSetDimensions",mfn->ncv,&i,&flg));
+    if (flg) PetscCall(MFNSetDimensions(mfn,i));
 
-    ierr = PetscOptionsBool("-mfn_error_if_not_converged","Generate error if solver does not converge","MFNSetErrorIfNotConverged",mfn->errorifnotconverged,&mfn->errorifnotconverged,NULL);CHKERRQ(ierr);
+    PetscCall(PetscOptionsBool("-mfn_error_if_not_converged","Generate error if solver does not converge","MFNSetErrorIfNotConverged",mfn->errorifnotconverged,&mfn->errorifnotconverged,NULL));
 
     /* -----------------------------------------------------------------------*/
     /*
       Cancels all monitors hardwired into code before call to MFNSetFromOptions()
     */
-    ierr = PetscOptionsBool("-mfn_monitor_cancel","Remove any hardwired monitor routines","MFNMonitorCancel",PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
-    if (set && flg) {
-      ierr = MFNMonitorCancel(mfn);CHKERRQ(ierr);
-    }
-    /*
-      Text monitors
-    */
-    ierr = MFNMonitorSetFromOptions(mfn,"-mfn_monitor","Monitor error estimate","MFNMonitorDefault",MFNMonitorDefault);CHKERRQ(ierr);
-    /*
-      Line graph monitors
-    */
-    ierr = PetscOptionsBool("-mfn_monitor_lg","Monitor error estimate graphically","MFNMonitorSet",PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
-    if (set && flg) {
-      ierr = MFNMonitorLGCreate(PetscObjectComm((PetscObject)mfn),NULL,"Error estimate",PETSC_DECIDE,PETSC_DECIDE,300,300,&lg);CHKERRQ(ierr);
-      ierr = MFNMonitorSet(mfn,MFNMonitorLG,lg,(PetscErrorCode (*)(void**))PetscDrawLGDestroy);CHKERRQ(ierr);
-    }
+    PetscCall(PetscOptionsBool("-mfn_monitor_cancel","Remove any hardwired monitor routines","MFNMonitorCancel",PETSC_FALSE,&flg,&set));
+    if (set && flg) PetscCall(MFNMonitorCancel(mfn));
+    PetscCall(MFNMonitorSetFromOptions(mfn,"-mfn_monitor","error_estimate",NULL));
 
     /* -----------------------------------------------------------------------*/
-    ierr = PetscOptionsName("-mfn_view","Print detailed information on solver used","MFNView",NULL);CHKERRQ(ierr);
+    PetscCall(PetscOptionsName("-mfn_view","Print detailed information on solver used","MFNView",NULL));
 
-    if (mfn->ops->setfromoptions) {
-      ierr = (*mfn->ops->setfromoptions)(PetscOptionsObject,mfn);CHKERRQ(ierr);
-    }
-    ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)mfn);CHKERRQ(ierr);
-  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+    if (mfn->ops->setfromoptions) PetscCall((*mfn->ops->setfromoptions)(PetscOptionsObject,mfn));
+    PetscCall(PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)mfn));
+  ierr = PetscOptionsEnd();PetscCall(ierr);
 
-  if (!mfn->V) { ierr = MFNGetBV(mfn,&mfn->V);CHKERRQ(ierr); }
-  ierr = BVSetFromOptions(mfn->V);CHKERRQ(ierr);
-  if (!mfn->fn) { ierr = MFNGetFN(mfn,&mfn->fn);CHKERRQ(ierr); }
-  ierr = FNSetFromOptions(mfn->fn);CHKERRQ(ierr);
+  if (!mfn->V) PetscCall(MFNGetBV(mfn,&mfn->V));
+  PetscCall(BVSetFromOptions(mfn->V));
+  if (!mfn->fn) PetscCall(MFNGetFN(mfn,&mfn->fn));
+  PetscCall(FNSetFromOptions(mfn->fn));
   PetscFunctionReturn(0);
 }
 
@@ -195,14 +190,14 @@ PetscErrorCode MFNSetTolerances(MFN mfn,PetscReal tol,PetscInt maxits)
     mfn->tol = PETSC_DEFAULT;
     mfn->setupcalled = 0;
   } else {
-    if (tol <= 0.0) SETERRQ(PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+    PetscCheck(tol>0.0,PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
     mfn->tol = tol;
   }
   if (maxits == PETSC_DEFAULT || maxits == PETSC_DECIDE) {
-    mfn->max_it = 0;
+    mfn->max_it = PETSC_DEFAULT;
     mfn->setupcalled = 0;
   } else {
-    if (maxits <= 0) SETERRQ(PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
+    PetscCheck(maxits>0,PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
     mfn->max_it = maxits;
   }
   PetscFunctionReturn(0);
@@ -258,9 +253,9 @@ PetscErrorCode MFNSetDimensions(MFN mfn,PetscInt ncv)
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
   PetscValidLogicalCollectiveInt(mfn,ncv,2);
   if (ncv == PETSC_DECIDE || ncv == PETSC_DEFAULT) {
-    mfn->ncv = 0;
+    mfn->ncv = PETSC_DEFAULT;
   } else {
-    if (ncv<1) SETERRQ(PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
+    PetscCheck(ncv>0,PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
     mfn->ncv = ncv;
   }
   mfn->setupcalled = 0;
@@ -311,7 +306,7 @@ PetscErrorCode MFNSetErrorIfNotConverged(MFN mfn,PetscBool flg)
 
    Level: intermediate
 
-.seealso:  MFNSetErrorIfNotConverged()
+.seealso: MFNSetErrorIfNotConverged()
 @*/
 PetscErrorCode MFNGetErrorIfNotConverged(MFN mfn,PetscBool *flag)
 {
@@ -350,15 +345,13 @@ PetscErrorCode MFNGetErrorIfNotConverged(MFN mfn,PetscBool *flag)
 @*/
 PetscErrorCode MFNSetOptionsPrefix(MFN mfn,const char *prefix)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
-  if (!mfn->V) { ierr = MFNGetBV(mfn,&mfn->V);CHKERRQ(ierr); }
-  ierr = BVSetOptionsPrefix(mfn->V,prefix);CHKERRQ(ierr);
-  if (!mfn->fn) { ierr = MFNGetFN(mfn,&mfn->fn);CHKERRQ(ierr); }
-  ierr = FNSetOptionsPrefix(mfn->fn,prefix);CHKERRQ(ierr);
-  ierr = PetscObjectSetOptionsPrefix((PetscObject)mfn,prefix);CHKERRQ(ierr);
+  if (!mfn->V) PetscCall(MFNGetBV(mfn,&mfn->V));
+  PetscCall(BVSetOptionsPrefix(mfn->V,prefix));
+  if (!mfn->fn) PetscCall(MFNGetFN(mfn,&mfn->fn));
+  PetscCall(FNSetOptionsPrefix(mfn->fn,prefix));
+  PetscCall(PetscObjectSetOptionsPrefix((PetscObject)mfn,prefix));
   PetscFunctionReturn(0);
 }
 
@@ -382,15 +375,13 @@ PetscErrorCode MFNSetOptionsPrefix(MFN mfn,const char *prefix)
 @*/
 PetscErrorCode MFNAppendOptionsPrefix(MFN mfn,const char *prefix)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
-  if (!mfn->V) { ierr = MFNGetBV(mfn,&mfn->V);CHKERRQ(ierr); }
-  ierr = BVAppendOptionsPrefix(mfn->V,prefix);CHKERRQ(ierr);
-  if (!mfn->fn) { ierr = MFNGetFN(mfn,&mfn->fn);CHKERRQ(ierr); }
-  ierr = FNAppendOptionsPrefix(mfn->fn,prefix);CHKERRQ(ierr);
-  ierr = PetscObjectAppendOptionsPrefix((PetscObject)mfn,prefix);CHKERRQ(ierr);
+  if (!mfn->V) PetscCall(MFNGetBV(mfn,&mfn->V));
+  PetscCall(BVAppendOptionsPrefix(mfn->V,prefix));
+  if (!mfn->fn) PetscCall(MFNGetFN(mfn,&mfn->fn));
+  PetscCall(FNAppendOptionsPrefix(mfn->fn,prefix));
+  PetscCall(PetscObjectAppendOptionsPrefix((PetscObject)mfn,prefix));
   PetscFunctionReturn(0);
 }
 
@@ -416,11 +407,9 @@ PetscErrorCode MFNAppendOptionsPrefix(MFN mfn,const char *prefix)
 @*/
 PetscErrorCode MFNGetOptionsPrefix(MFN mfn,const char *prefix[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
   PetscValidPointer(prefix,2);
-  ierr = PetscObjectGetOptionsPrefix((PetscObject)mfn,prefix);CHKERRQ(ierr);
+  PetscCall(PetscObjectGetOptionsPrefix((PetscObject)mfn,prefix));
   PetscFunctionReturn(0);
 }
