@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,10 +13,19 @@
 
 #include <slepc/private/nepimpl.h>      /*I "slepcnep.h" I*/
 
+/* Logging support */
+PetscClassId      NEP_CLASSID = 0;
+PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_Resolvent = 0,NEP_CISS_SVD = 0;
+
+/* List of registered NEP routines */
 PetscFunctionList NEPList = 0;
 PetscBool         NEPRegisterAllCalled = PETSC_FALSE;
-PetscClassId      NEP_CLASSID = 0;
-PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_Resolvent = 0;
+
+/* List of registered NEP monitors */
+PetscFunctionList NEPMonitorList              = NULL;
+PetscFunctionList NEPMonitorCreateList        = NULL;
+PetscFunctionList NEPMonitorDestroyList       = NULL;
+PetscBool         NEPMonitorRegisterAllCalled = PETSC_FALSE;
 
 /*@
    NEPCreate - Creates the default NEP context.
@@ -27,7 +36,7 @@ PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval =
 .  comm - MPI communicator
 
    Output Parameter:
-.  nep - location to put the NEP context
+.  outnep - location to put the NEP context
 
    Level: beginner
 
@@ -35,19 +44,18 @@ PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval =
 @*/
 PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
 {
-  PetscErrorCode ierr;
   NEP            nep;
 
   PetscFunctionBegin;
   PetscValidPointer(outnep,2);
   *outnep = 0;
-  ierr = NEPInitializePackage();CHKERRQ(ierr);
-  ierr = SlepcHeaderCreate(nep,NEP_CLASSID,"NEP","Nonlinear Eigenvalue Problem","NEP",comm,NEPDestroy,NEPView);CHKERRQ(ierr);
+  PetscCall(NEPInitializePackage());
+  PetscCall(SlepcHeaderCreate(nep,NEP_CLASSID,"NEP","Nonlinear Eigenvalue Problem","NEP",comm,NEPDestroy,NEPView));
 
-  nep->max_it          = 0;
+  nep->max_it          = PETSC_DEFAULT;
   nep->nev             = 1;
-  nep->ncv             = 0;
-  nep->mpd             = 0;
+  nep->ncv             = PETSC_DEFAULT;
+  nep->mpd             = PETSC_DEFAULT;
   nep->nini            = 0;
   nep->target          = 0.0;
   nep->tol             = PETSC_DEFAULT;
@@ -87,7 +95,9 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->A               = NULL;
   nep->f               = NULL;
   nep->nt              = 0;
-  nep->mstr            = DIFFERENT_NONZERO_PATTERN;
+  nep->mstr            = UNKNOWN_NONZERO_PATTERN;
+  nep->P               = NULL;
+  nep->mstrp           = UNKNOWN_NONZERO_PATTERN;
   nep->IS              = NULL;
   nep->eigr            = NULL;
   nep->eigi            = NULL;
@@ -105,11 +115,10 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->nrma            = NULL;
   nep->fui             = (NEPUserInterface)0;
   nep->useds           = PETSC_FALSE;
-  nep->hasts           = PETSC_FALSE;
   nep->resolvent       = NULL;
   nep->reason          = NEP_CONVERGED_ITERATING;
 
-  ierr = PetscNewLog(nep,&nep->sc);CHKERRQ(ierr);
+  PetscCall(PetscNewLog(nep,&nep->sc));
   *outnep = nep;
   PetscFunctionReturn(0);
 }
@@ -144,25 +153,25 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
 @*/
 PetscErrorCode NEPSetType(NEP nep,NEPType type)
 {
-  PetscErrorCode ierr,(*r)(NEP);
+  PetscErrorCode (*r)(NEP);
   PetscBool      match;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidCharPointer(type,2);
 
-  ierr = PetscObjectTypeCompare((PetscObject)nep,type,&match);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)nep,type,&match));
   if (match) PetscFunctionReturn(0);
 
-  ierr = PetscFunctionListFind(NEPList,type,&r);CHKERRQ(ierr);
-  if (!r) SETERRQ1(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown NEP type given: %s",type);
+  PetscCall(PetscFunctionListFind(NEPList,type,&r));
+  PetscCheck(r,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown NEP type given: %s",type);
 
-  if (nep->ops->destroy) { ierr = (*nep->ops->destroy)(nep);CHKERRQ(ierr); }
-  ierr = PetscMemzero(nep->ops,sizeof(struct _NEPOps));CHKERRQ(ierr);
+  if (nep->ops->destroy) PetscCall((*nep->ops->destroy)(nep));
+  PetscCall(PetscMemzero(nep->ops,sizeof(struct _NEPOps)));
 
   nep->state = NEP_STATE_INITIAL;
-  ierr = PetscObjectChangeTypeName((PetscObject)nep,type);CHKERRQ(ierr);
-  ierr = (*r)(nep);CHKERRQ(ierr);
+  PetscCall(PetscObjectChangeTypeName((PetscObject)nep,type));
+  PetscCall((*r)(nep));
   PetscFunctionReturn(0);
 }
 
@@ -175,7 +184,7 @@ PetscErrorCode NEPSetType(NEP nep,NEPType type)
 .  nep - the eigensolver context
 
    Output Parameter:
-.  name - name of NEP method
+.  type - name of NEP method
 
    Level: intermediate
 
@@ -218,34 +227,73 @@ $     -nep_type my_solver
 @*/
 PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
 {
-  PetscErrorCode ierr;
+  PetscFunctionBegin;
+  PetscCall(NEPInitializePackage());
+  PetscCall(PetscFunctionListAdd(&NEPList,name,function));
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   NEPMonitorRegister - Adds NEP monitor routine.
+
+   Not Collective
+
+   Input Parameters:
++  name    - name of a new monitor routine
+.  vtype   - a PetscViewerType for the output
+.  format  - a PetscViewerFormat for the output
+.  monitor - monitor routine
+.  create  - creation routine, or NULL
+-  destroy - destruction routine, or NULL
+
+   Notes:
+   NEPMonitorRegister() may be called multiple times to add several user-defined monitors.
+
+   Sample usage:
+.vb
+   NEPMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
+.ve
+
+   Then, your monitor can be chosen with the procedural interface via
+$      NEPMonitorSetFromOptions(nep,"-nep_monitor_my_monitor","my_monitor",NULL)
+   or at runtime via the option
+$      -nep_monitor_my_monitor
+
+   Level: advanced
+
+.seealso: NEPMonitorRegisterAll()
+@*/
+PetscErrorCode NEPMonitorRegister(const char name[],PetscViewerType vtype,PetscViewerFormat format,PetscErrorCode (*monitor)(NEP,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*),PetscErrorCode (*create)(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**),PetscErrorCode (*destroy)(PetscViewerAndFormat**))
+{
+  char           key[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
-  ierr = NEPInitializePackage();CHKERRQ(ierr);
-  ierr = PetscFunctionListAdd(&NEPList,name,function);CHKERRQ(ierr);
+  PetscCall(NEPInitializePackage());
+  PetscCall(SlepcMonitorMakeKey_Internal(name,vtype,format,key));
+  PetscCall(PetscFunctionListAdd(&NEPMonitorList,key,monitor));
+  if (create)  PetscCall(PetscFunctionListAdd(&NEPMonitorCreateList,key,create));
+  if (destroy) PetscCall(PetscFunctionListAdd(&NEPMonitorDestroyList,key,destroy));
   PetscFunctionReturn(0);
 }
 
 /*
    NEPReset_Problem - Destroys the problem matrices.
-@*/
+*/
 PetscErrorCode NEPReset_Problem(NEP nep)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  ierr = MatDestroy(&nep->function);CHKERRQ(ierr);
-  ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
-  ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
+  PetscCall(MatDestroy(&nep->function));
+  PetscCall(MatDestroy(&nep->function_pre));
+  PetscCall(MatDestroy(&nep->jacobian));
   if (nep->fui==NEP_USER_INTERFACE_SPLIT) {
-    ierr = MatDestroyMatrices(nep->nt,&nep->A);CHKERRQ(ierr);
-    for (i=0;i<nep->nt;i++) {
-      ierr = FNDestroy(&nep->f[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(nep->f);CHKERRQ(ierr);
-    ierr = PetscFree(nep->nrma);CHKERRQ(ierr);
+    PetscCall(MatDestroyMatrices(nep->nt,&nep->A));
+    for (i=0;i<nep->nt;i++) PetscCall(FNDestroy(&nep->f[i]));
+    PetscCall(PetscFree(nep->f));
+    PetscCall(PetscFree(nep->nrma));
+    if (nep->P) PetscCall(MatDestroyMatrices(nep->nt,&nep->P));
     nep->nt = 0;
   }
   PetscFunctionReturn(0);
@@ -265,24 +313,22 @@ PetscErrorCode NEPReset_Problem(NEP nep)
 @*/
 PetscErrorCode NEPReset(NEP nep)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   if (nep) PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (!nep) PetscFunctionReturn(0);
-  if (nep->ops->reset) { ierr = (nep->ops->reset)(nep);CHKERRQ(ierr); }
-  if (nep->refineksp) { ierr = KSPReset(nep->refineksp);CHKERRQ(ierr); }
-  ierr = NEPReset_Problem(nep);CHKERRQ(ierr);
-  ierr = BVDestroy(&nep->V);CHKERRQ(ierr);
-  ierr = BVDestroy(&nep->W);CHKERRQ(ierr);
-  ierr = VecDestroyVecs(nep->nwork,&nep->work);CHKERRQ(ierr);
-  ierr = MatDestroy(&nep->resolvent);CHKERRQ(ierr);
+  if (nep->ops->reset) PetscCall((nep->ops->reset)(nep));
+  if (nep->refineksp) PetscCall(KSPReset(nep->refineksp));
+  PetscCall(NEPReset_Problem(nep));
+  PetscCall(BVDestroy(&nep->V));
+  PetscCall(BVDestroy(&nep->W));
+  PetscCall(VecDestroyVecs(nep->nwork,&nep->work));
+  PetscCall(MatDestroy(&nep->resolvent));
   nep->nwork = 0;
   nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    NEPDestroy - Destroys the NEP context.
 
    Collective on nep
@@ -296,29 +342,23 @@ PetscErrorCode NEPReset(NEP nep)
 @*/
 PetscErrorCode NEPDestroy(NEP *nep)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   if (!*nep) PetscFunctionReturn(0);
   PetscValidHeaderSpecific(*nep,NEP_CLASSID,1);
   if (--((PetscObject)(*nep))->refct > 0) { *nep = 0; PetscFunctionReturn(0); }
-  ierr = NEPReset(*nep);CHKERRQ(ierr);
-  if ((*nep)->ops->destroy) { ierr = (*(*nep)->ops->destroy)(*nep);CHKERRQ(ierr); }
-  if ((*nep)->eigr) {
-    ierr = PetscFree4((*nep)->eigr,(*nep)->eigi,(*nep)->errest,(*nep)->perm);CHKERRQ(ierr);
-  }
-  ierr = RGDestroy(&(*nep)->rg);CHKERRQ(ierr);
-  ierr = DSDestroy(&(*nep)->ds);CHKERRQ(ierr);
-  ierr = KSPDestroy(&(*nep)->refineksp);CHKERRQ(ierr);
-  ierr = PetscSubcommDestroy(&(*nep)->refinesubc);CHKERRQ(ierr);
-  ierr = PetscFree((*nep)->sc);CHKERRQ(ierr);
+  PetscCall(NEPReset(*nep));
+  if ((*nep)->ops->destroy) PetscCall((*(*nep)->ops->destroy)(*nep));
+  if ((*nep)->eigr) PetscCall(PetscFree4((*nep)->eigr,(*nep)->eigi,(*nep)->errest,(*nep)->perm));
+  PetscCall(RGDestroy(&(*nep)->rg));
+  PetscCall(DSDestroy(&(*nep)->ds));
+  PetscCall(KSPDestroy(&(*nep)->refineksp));
+  PetscCall(PetscSubcommDestroy(&(*nep)->refinesubc));
+  PetscCall(PetscFree((*nep)->sc));
   /* just in case the initial vectors have not been used */
-  ierr = SlepcBasisDestroy_Private(&(*nep)->nini,&(*nep)->IS);CHKERRQ(ierr);
-  if ((*nep)->convergeddestroy) {
-    ierr = (*(*nep)->convergeddestroy)((*nep)->convergedctx);CHKERRQ(ierr);
-  }
-  ierr = NEPMonitorCancel(*nep);CHKERRQ(ierr);
-  ierr = PetscHeaderDestroy(nep);CHKERRQ(ierr);
+  PetscCall(SlepcBasisDestroy_Private(&(*nep)->nini,&(*nep)->IS));
+  if ((*nep)->convergeddestroy) PetscCall((*(*nep)->convergeddestroy)((*nep)->convergedctx));
+  PetscCall(NEPMonitorCancel(*nep));
+  PetscCall(PetscHeaderDestroy(nep));
   PetscFunctionReturn(0);
 }
 
@@ -341,16 +381,14 @@ PetscErrorCode NEPDestroy(NEP *nep)
 @*/
 PetscErrorCode NEPSetBV(NEP nep,BV bv)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidHeaderSpecific(bv,BV_CLASSID,2);
   PetscCheckSameComm(nep,1,bv,2);
-  ierr = PetscObjectReference((PetscObject)bv);CHKERRQ(ierr);
-  ierr = BVDestroy(&nep->V);CHKERRQ(ierr);
+  PetscCall(PetscObjectReference((PetscObject)bv));
+  PetscCall(BVDestroy(&nep->V));
   nep->V = bv;
-  ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V));
   PetscFunctionReturn(0);
 }
 
@@ -372,16 +410,14 @@ PetscErrorCode NEPSetBV(NEP nep,BV bv)
 @*/
 PetscErrorCode NEPGetBV(NEP nep,BV *bv)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(bv,2);
   if (!nep->V) {
-    ierr = BVCreate(PetscObjectComm((PetscObject)nep),&nep->V);CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->V,(PetscObject)nep,0);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V);CHKERRQ(ierr);
-    ierr = PetscObjectSetOptions((PetscObject)nep->V,((PetscObject)nep)->options);CHKERRQ(ierr);
+    PetscCall(BVCreate(PetscObjectComm((PetscObject)nep),&nep->V));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->V,(PetscObject)nep,0));
+    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V));
+    PetscCall(PetscObjectSetOptions((PetscObject)nep->V,((PetscObject)nep)->options));
   }
   *bv = nep->V;
   PetscFunctionReturn(0);
@@ -406,16 +442,16 @@ PetscErrorCode NEPGetBV(NEP nep,BV *bv)
 @*/
 PetscErrorCode NEPSetRG(NEP nep,RG rg)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidHeaderSpecific(rg,RG_CLASSID,2);
-  PetscCheckSameComm(nep,1,rg,2);
-  ierr = PetscObjectReference((PetscObject)rg);CHKERRQ(ierr);
-  ierr = RGDestroy(&nep->rg);CHKERRQ(ierr);
+  if (rg) {
+    PetscValidHeaderSpecific(rg,RG_CLASSID,2);
+    PetscCheckSameComm(nep,1,rg,2);
+  }
+  PetscCall(PetscObjectReference((PetscObject)rg));
+  PetscCall(RGDestroy(&nep->rg));
   nep->rg = rg;
-  ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg));
   PetscFunctionReturn(0);
 }
 
@@ -437,16 +473,14 @@ PetscErrorCode NEPSetRG(NEP nep,RG rg)
 @*/
 PetscErrorCode NEPGetRG(NEP nep,RG *rg)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(rg,2);
   if (!nep->rg) {
-    ierr = RGCreate(PetscObjectComm((PetscObject)nep),&nep->rg);CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->rg,(PetscObject)nep,0);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg);CHKERRQ(ierr);
-    ierr = PetscObjectSetOptions((PetscObject)nep->rg,((PetscObject)nep)->options);CHKERRQ(ierr);
+    PetscCall(RGCreate(PetscObjectComm((PetscObject)nep),&nep->rg));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->rg,(PetscObject)nep,0));
+    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg));
+    PetscCall(PetscObjectSetOptions((PetscObject)nep->rg,((PetscObject)nep)->options));
   }
   *rg = nep->rg;
   PetscFunctionReturn(0);
@@ -471,16 +505,14 @@ PetscErrorCode NEPGetRG(NEP nep,RG *rg)
 @*/
 PetscErrorCode NEPSetDS(NEP nep,DS ds)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidHeaderSpecific(ds,DS_CLASSID,2);
   PetscCheckSameComm(nep,1,ds,2);
-  ierr = PetscObjectReference((PetscObject)ds);CHKERRQ(ierr);
-  ierr = DSDestroy(&nep->ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectReference((PetscObject)ds));
+  PetscCall(DSDestroy(&nep->ds));
   nep->ds = ds;
-  ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds);CHKERRQ(ierr);
+  PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds));
   PetscFunctionReturn(0);
 }
 
@@ -502,16 +534,14 @@ PetscErrorCode NEPSetDS(NEP nep,DS ds)
 @*/
 PetscErrorCode NEPGetDS(NEP nep,DS *ds)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(ds,2);
   if (!nep->ds) {
-    ierr = DSCreate(PetscObjectComm((PetscObject)nep),&nep->ds);CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->ds,(PetscObject)nep,0);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds);CHKERRQ(ierr);
-    ierr = PetscObjectSetOptions((PetscObject)nep->ds,((PetscObject)nep)->options);CHKERRQ(ierr);
+    PetscCall(DSCreate(PetscObjectComm((PetscObject)nep),&nep->ds));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->ds,(PetscObject)nep,0));
+    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds));
+    PetscCall(PetscObjectSetOptions((PetscObject)nep->ds,((PetscObject)nep)->options));
   }
   *ds = nep->ds;
   PetscFunctionReturn(0);
@@ -535,7 +565,7 @@ PetscErrorCode NEPGetDS(NEP nep,DS *ds)
 @*/
 PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
 {
-  PetscErrorCode ierr;
+  MPI_Comm       comm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -543,18 +573,19 @@ PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
   if (!nep->refineksp) {
     if (nep->npart>1) {
       /* Split in subcomunicators */
-      ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&nep->refinesubc);CHKERRQ(ierr);
-      ierr = PetscSubcommSetNumber(nep->refinesubc,nep->npart);CHKERRQ(ierr);CHKERRQ(ierr);
-      ierr = PetscSubcommSetType(nep->refinesubc,PETSC_SUBCOMM_CONTIGUOUS);CHKERRQ(ierr);
-      ierr = PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm));CHKERRQ(ierr);
-    }
-    ierr = KSPCreate((nep->npart==1)?PetscObjectComm((PetscObject)nep):PetscSubcommChild(nep->refinesubc),&nep->refineksp);CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)nep->refineksp,(PetscObject)nep,0);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->refineksp);CHKERRQ(ierr);
-    ierr = PetscObjectSetOptions((PetscObject)nep->refineksp,((PetscObject)nep)->options);CHKERRQ(ierr);
-    ierr = KSPSetOptionsPrefix(*ksp,((PetscObject)nep)->prefix);CHKERRQ(ierr);
-    ierr = KSPAppendOptionsPrefix(*ksp,"nep_refine_");CHKERRQ(ierr);
-    ierr = KSPSetTolerances(nep->refineksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
+      PetscCall(PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&nep->refinesubc));
+      PetscCall(PetscSubcommSetNumber(nep->refinesubc,nep->npart));
+      PetscCall(PetscSubcommSetType(nep->refinesubc,PETSC_SUBCOMM_CONTIGUOUS));
+      PetscCall(PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm)));
+      PetscCall(PetscSubcommGetChild(nep->refinesubc,&comm));
+    } else PetscCall(PetscObjectGetComm((PetscObject)nep,&comm));
+    PetscCall(KSPCreate(comm,&nep->refineksp));
+    PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->refineksp,(PetscObject)nep,0));
+    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->refineksp));
+    PetscCall(PetscObjectSetOptions((PetscObject)nep->refineksp,((PetscObject)nep)->options));
+    PetscCall(KSPSetOptionsPrefix(*ksp,((PetscObject)nep)->prefix));
+    PetscCall(KSPAppendOptionsPrefix(*ksp,"nep_refine_"));
+    PetscCall(KSPSetTolerances(nep->refineksp,SlepcDefaultTol(nep->rtol),PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT));
   }
   *ksp = nep->refineksp;
   PetscFunctionReturn(0);
@@ -629,7 +660,7 @@ PetscErrorCode NEPGetTarget(NEP nep,PetscScalar* target)
    Input Parameters:
 +  nep - the NEP context
 .  A   - Function matrix
-.  B   - preconditioner matrix (usually same as the Function)
+.  B   - preconditioner matrix (usually same as A)
 .  fun - Function evaluation routine (if NULL then NEP retains any
          previously set value)
 -  ctx - [optional] user-defined context for private data for the Function
@@ -651,8 +682,6 @@ $   fun(NEP nep,PetscScalar lambda,Mat F,Mat P,void *ctx)
 @*/
 PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,PetscScalar,Mat,Mat,void*),void *ctx)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
@@ -660,19 +689,19 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
   if (A) PetscCheckSameComm(nep,1,A,2);
   if (B) PetscCheckSameComm(nep,1,B,3);
 
-  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
-  else if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
+  if (nep->state) PetscCall(NEPReset(nep));
+  else if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) PetscCall(NEPReset_Problem(nep));
 
   if (fun) nep->computefunction = fun;
   if (ctx) nep->functionctx     = ctx;
   if (A) {
-    ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-    ierr = MatDestroy(&nep->function);CHKERRQ(ierr);
+    PetscCall(PetscObjectReference((PetscObject)A));
+    PetscCall(MatDestroy(&nep->function));
     nep->function = A;
   }
   if (B) {
-    ierr = PetscObjectReference((PetscObject)B);CHKERRQ(ierr);
-    ierr = MatDestroy(&nep->function_pre);CHKERRQ(ierr);
+    PetscCall(PetscObjectReference((PetscObject)B));
+    PetscCall(MatDestroy(&nep->function_pre));
     nep->function_pre = B;
   }
   nep->fui   = NEP_USER_INTERFACE_CALLBACK;
@@ -712,7 +741,7 @@ PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,P
 }
 
 /*@C
-   NEPSetJacobian - Sets the function to compute Jacobian T'(lambda) as well
+   NEPSetJacobian - Sets the function to compute the Jacobian T'(lambda) as well
    as the location to store the matrix.
 
    Logically Collective on nep
@@ -740,21 +769,19 @@ $   jac(NEP nep,PetscScalar lambda,Mat J,void *ctx)
 @*/
 PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScalar,Mat,void*),void *ctx)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   if (A) PetscValidHeaderSpecific(A,MAT_CLASSID,2);
   if (A) PetscCheckSameComm(nep,1,A,2);
 
-  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
-  else if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
+  if (nep->state) PetscCall(NEPReset(nep));
+  else if (nep->fui && nep->fui!=NEP_USER_INTERFACE_CALLBACK) PetscCall(NEPReset_Problem(nep));
 
   if (jac) nep->computejacobian = jac;
   if (ctx) nep->jacobianctx     = ctx;
   if (A) {
-    ierr = PetscObjectReference((PetscObject)A);CHKERRQ(ierr);
-    ierr = MatDestroy(&nep->jacobian);CHKERRQ(ierr);
+    PetscCall(PetscObjectReference((PetscObject)A));
+    PetscCall(MatDestroy(&nep->jacobian));
     nep->jacobian = A;
   }
   nep->fui   = NEP_USER_INTERFACE_CALLBACK;
@@ -799,7 +826,7 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
-.  n   - number of terms in the split form
+.  nt  - number of terms in the split form
 .  A   - array of matrices
 .  f   - array of functions
 -  str - structure flag for matrices
@@ -812,50 +839,59 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
    The structure flag provides information about A_i's nonzero pattern
    (see MatStructure enum). If all matrices have the same pattern, then
    use SAME_NONZERO_PATTERN. If the patterns are different but contained
-   in the pattern of the first one, then use SUBSET_NONZERO_PATTERN.
-   Otherwise use DIFFERENT_NONZERO_PATTERN.
+   in the pattern of the first one, then use SUBSET_NONZERO_PATTERN. If
+   patterns are known to be different, use DIFFERENT_NONZERO_PATTERN.
+   If set to UNKNOWN_NONZERO_PATTERN, the patterns will be compared to
+   determine if they are equal.
 
    This function must be called before NEPSetUp(). If it is called again
    after NEPSetUp() then the NEP object is reset.
 
    Level: beginner
 
-.seealso: NEPGetSplitOperatorTerm(), NEPGetSplitOperatorInfo()
+.seealso: NEPGetSplitOperatorTerm(), NEPGetSplitOperatorInfo(), NEPSetSplitPreconditioner()
  @*/
-PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt n,Mat A[],FN f[],MatStructure str)
+PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt nt,Mat A[],FN f[],MatStructure str)
 {
-  PetscInt       i;
-  PetscErrorCode ierr;
+  PetscInt       i,n=0,m,m0=0,mloc,nloc,mloc0=0;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidLogicalCollectiveInt(nep,n,2);
-  if (n <= 0) SETERRQ1(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more terms, you have %D",n);
+  PetscValidLogicalCollectiveInt(nep,nt,2);
+  PetscCheck(nt>0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more terms, you have %" PetscInt_FMT,nt);
   PetscValidPointer(A,3);
-  PetscCheckSameComm(nep,1,*A,3);
   PetscValidPointer(f,4);
-  PetscCheckSameComm(nep,1,*f,4);
+  PetscValidLogicalCollectiveEnum(nep,str,5);
 
-  for (i=0;i<n;i++) {
+  for (i=0;i<nt;i++) {
     PetscValidHeaderSpecific(A[i],MAT_CLASSID,3);
+    PetscCheckSameComm(nep,1,A[i],3);
     PetscValidHeaderSpecific(f[i],FN_CLASSID,4);
-    ierr = PetscObjectReference((PetscObject)A[i]);CHKERRQ(ierr);
-    ierr = PetscObjectReference((PetscObject)f[i]);CHKERRQ(ierr);
+    PetscCheckSameComm(nep,1,f[i],4);
+    PetscCall(MatGetSize(A[i],&m,&n));
+    PetscCall(MatGetLocalSize(A[i],&mloc,&nloc));
+    PetscCheck(m==n,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"A[%" PetscInt_FMT "] is a non-square matrix (%" PetscInt_FMT " rows, %" PetscInt_FMT " cols)",i,m,n);
+    PetscCheck(mloc==nloc,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"A[%" PetscInt_FMT "] does not have equal row and column local sizes (%" PetscInt_FMT ", %" PetscInt_FMT ")",i,mloc,nloc);
+    if (!i) { m0 = m; mloc0 = mloc; }
+    PetscCheck(m==m0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_INCOMP,"Dimensions of A[%" PetscInt_FMT "] do not match with previous matrices (%" PetscInt_FMT ", %" PetscInt_FMT ")",i,m,m0);
+    PetscCheck(mloc==mloc0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_INCOMP,"Local dimensions of A[%" PetscInt_FMT "] do not match with previous matrices (%" PetscInt_FMT ", %" PetscInt_FMT ")",i,mloc,mloc0);
+    PetscCall(PetscObjectReference((PetscObject)A[i]));
+    PetscCall(PetscObjectReference((PetscObject)f[i]));
   }
 
-  if (nep->state) { ierr = NEPReset(nep);CHKERRQ(ierr); }
-  else { ierr = NEPReset_Problem(nep);CHKERRQ(ierr); }
+  if (nep->state && (n!=nep->n || nloc!=nep->nloc)) PetscCall(NEPReset(nep));
+  else PetscCall(NEPReset_Problem(nep));
 
   /* allocate space and copy matrices and functions */
-  ierr = PetscMalloc1(n,&nep->A);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(Mat));CHKERRQ(ierr);
-  for (i=0;i<n;i++) nep->A[i] = A[i];
-  ierr = PetscMalloc1(n,&nep->f);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(FN));CHKERRQ(ierr);
-  for (i=0;i<n;i++) nep->f[i] = f[i];
-  ierr = PetscCalloc1(n,&nep->nrma);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)nep,n*sizeof(PetscReal));CHKERRQ(ierr);
-  nep->nt    = n;
+  PetscCall(PetscMalloc1(nt,&nep->A));
+  PetscCall(PetscLogObjectMemory((PetscObject)nep,nt*sizeof(Mat)));
+  for (i=0;i<nt;i++) nep->A[i] = A[i];
+  PetscCall(PetscMalloc1(nt,&nep->f));
+  PetscCall(PetscLogObjectMemory((PetscObject)nep,nt*sizeof(FN)));
+  for (i=0;i<nt;i++) nep->f[i] = f[i];
+  PetscCall(PetscCalloc1(nt,&nep->nrma));
+  PetscCall(PetscLogObjectMemory((PetscObject)nep,nt*sizeof(PetscReal)));
+  nep->nt    = nt;
   nep->mstr  = str;
   nep->fui   = NEP_USER_INTERFACE_SPLIT;
   nep->state = NEP_STATE_INITIAL;
@@ -868,7 +904,7 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt n,Mat A[],FN f[],MatStructur
 
    Not collective, though parallel Mats and FNs are returned if the NEP is parallel
 
-   Input Parameter:
+   Input Parameters:
 +  nep - the nonlinear eigensolver context
 -  k   - the index of the requested term (starting in 0)
 
@@ -884,8 +920,9 @@ PetscErrorCode NEPGetSplitOperatorTerm(NEP nep,PetscInt k,Mat *A,FN *f)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(nep,k,2);
   NEPCheckSplit(nep,1);
-  if (k<0 || k>=nep->nt) SETERRQ1(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %D",nep->nt-1);
+  PetscCheck(k>=0 && k<nep->nt,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %" PetscInt_FMT,nep->nt-1);
   if (A) *A = nep->A[k];
   if (f) *f = nep->f[k];
   PetscFunctionReturn(0);
@@ -918,3 +955,133 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
   PetscFunctionReturn(0);
 }
 
+/*@
+   NEPSetSplitPreconditioner - Sets an operator in split form from which
+   to build the preconditioner to be used when solving the nonlinear
+   eigenvalue problem in split form.
+
+   Collective on nep
+
+   Input Parameters:
++  nep  - the nonlinear eigensolver context
+.  ntp  - number of terms in the split preconditioner
+.  P    - array of matrices
+-  strp - structure flag for matrices
+
+   Notes:
+   The matrix for the preconditioner is expressed as P(lambda) =
+   sum_i P_i*f_i(lambda), for i=1,...,n, where the f_i functions
+   are the same as in NEPSetSplitOperator(). It is not necessary to call
+   this function. If it is not invoked, then the preconditioner is
+   built from T(lambda), i.e., both matrices and functions passed in
+   NEPSetSplitOperator().
+
+   The structure flag provides information about P_i's nonzero pattern
+   in the same way as in NEPSetSplitOperator().
+
+   If the functions defining the preconditioner operator were different
+   from the ones given in NEPSetSplitOperator(), then the split form
+   cannot be used. Use the callback interface instead.
+
+   Use ntp=0 to reset a previously set split preconditioner.
+
+   Level: advanced
+
+.seealso: NEPGetSplitPreconditionerTerm(), NEPGetSplitPreconditionerInfo(), NEPSetSplitOperator()
+ @*/
+PetscErrorCode NEPSetSplitPreconditioner(NEP nep,PetscInt ntp,Mat P[],MatStructure strp)
+{
+  PetscInt       i,n=0,m,m0=0,mloc,nloc,mloc0=0;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(nep,ntp,2);
+  PetscCheck(ntp>=0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Negative value of ntp = %" PetscInt_FMT,ntp);
+  PetscCheck(nep->fui==NEP_USER_INTERFACE_SPLIT,PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"Must call NEPSetSplitOperator first");
+  PetscCheck(ntp==0 || nep->nt==ntp,PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"The number of terms must be the same as in NEPSetSplitOperator()");
+  if (ntp) PetscValidPointer(P,3);
+  PetscValidLogicalCollectiveEnum(nep,strp,4);
+
+  for (i=0;i<ntp;i++) {
+    PetscValidHeaderSpecific(P[i],MAT_CLASSID,3);
+    PetscCheckSameComm(nep,1,P[i],3);
+    PetscCall(MatGetSize(P[i],&m,&n));
+    PetscCall(MatGetLocalSize(P[i],&mloc,&nloc));
+    PetscCheck(m==n,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"P[%" PetscInt_FMT "] is a non-square matrix (%" PetscInt_FMT " rows, %" PetscInt_FMT " cols)",i,m,n);
+    PetscCheck(mloc==nloc,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"P[%" PetscInt_FMT "] does not have equal row and column local sizes (%" PetscInt_FMT ", %" PetscInt_FMT ")",i,mloc,nloc);
+    if (!i) { m0 = m; mloc0 = mloc; }
+    PetscCheck(m==m0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_INCOMP,"Dimensions of P[%" PetscInt_FMT "] do not match with previous matrices (%" PetscInt_FMT ", %" PetscInt_FMT ")",i,m,m0);
+    PetscCheck(mloc==mloc0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_INCOMP,"Local dimensions of P[%" PetscInt_FMT "] do not match with previous matrices (%" PetscInt_FMT ", %" PetscInt_FMT ")",i,mloc,mloc0);
+    PetscCall(PetscObjectReference((PetscObject)P[i]));
+  }
+
+  PetscCheck(!nep->state,PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"To call this function after NEPSetUp(), you must call NEPSetSplitOperator() again");
+  if (nep->P) PetscCall(MatDestroyMatrices(nep->nt,&nep->P));
+
+  /* allocate space and copy matrices */
+  if (ntp) {
+    PetscCall(PetscMalloc1(ntp,&nep->P));
+    PetscCall(PetscLogObjectMemory((PetscObject)nep,ntp*sizeof(Mat)));
+    for (i=0;i<ntp;i++) nep->P[i] = P[i];
+  }
+  nep->mstrp = strp;
+  nep->state = NEP_STATE_INITIAL;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   NEPGetSplitPreconditionerTerm - Gets the matrices associated with
+   the split preconditioner.
+
+   Not collective, though parallel Mats are returned if the NEP is parallel
+
+   Input Parameters:
++  nep - the nonlinear eigensolver context
+-  k   - the index of the requested term (starting in 0)
+
+   Output Parameter:
+.  P  - the matrix of the requested term
+
+   Level: advanced
+
+.seealso: NEPSetSplitPreconditioner(), NEPGetSplitPreconditionerInfo()
+@*/
+PetscErrorCode NEPGetSplitPreconditionerTerm(NEP nep,PetscInt k,Mat *P)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  PetscValidLogicalCollectiveInt(nep,k,2);
+  PetscValidPointer(P,3);
+  NEPCheckSplit(nep,1);
+  PetscCheck(k>=0 && k<nep->nt,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %" PetscInt_FMT,nep->nt-1);
+  PetscCheck(nep->P,PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"You have not called NEPSetSplitPreconditioner()");
+  *P = nep->P[k];
+  PetscFunctionReturn(0);
+}
+
+/*@
+   NEPGetSplitPreconditionerInfo - Returns the number of terms of the split
+   preconditioner, as well as the structure flag for matrices.
+
+   Not collective
+
+   Input Parameter:
+.  nep - the nonlinear eigensolver context
+
+   Output Parameters:
++  n    - the number of terms passed in NEPSetSplitPreconditioner()
+-  strp - the matrix structure flag passed in NEPSetSplitPreconditioner()
+
+   Level: advanced
+
+.seealso: NEPSetSplitPreconditioner(), NEPGetSplitPreconditionerTerm()
+@*/
+PetscErrorCode NEPGetSplitPreconditionerInfo(NEP nep,PetscInt *n,MatStructure *strp)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  NEPCheckSplit(nep,1);
+  if (n)    *n    = nep->P? nep->nt: 0;
+  if (strp) *strp = nep->mstrp;
+  PetscFunctionReturn(0);
+}
