@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -22,25 +22,26 @@
 +  nep - nonlinear eigensolver context
 -  nw  - number of work vectors to allocate
 
-   Developers Note:
+   Developer Notes:
    This is SLEPC_EXTERN because it may be required by user plugin NEP
    implementations.
 
    Level: developer
+
+.seealso: NEPSetUp()
 @*/
 PetscErrorCode NEPSetWorkVecs(NEP nep,PetscInt nw)
 {
-  PetscErrorCode ierr;
   Vec            t;
 
   PetscFunctionBegin;
   if (nep->nwork < nw) {
-    ierr = VecDestroyVecs(nep->nwork,&nep->work);CHKERRQ(ierr);
+    PetscCall(VecDestroyVecs(nep->nwork,&nep->work));
     nep->nwork = nw;
-    ierr = BVGetColumn(nep->V,0,&t);CHKERRQ(ierr);
-    ierr = VecDuplicateVecs(t,nw,&nep->work);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(nep->V,0,&t);CHKERRQ(ierr);
-    ierr = PetscLogObjectParents(nep,nw,nep->work);CHKERRQ(ierr);
+    PetscCall(BVGetColumn(nep->V,0,&t));
+    PetscCall(VecDuplicateVecs(t,nw,&nep->work));
+    PetscCall(BVRestoreColumn(nep->V,0,&t));
+    PetscCall(PetscLogObjectParents(nep,nw,nep->work));
   }
   PetscFunctionReturn(0);
 }
@@ -110,21 +111,26 @@ PetscErrorCode NEPConvergedNorm(NEP nep,PetscScalar eigr,PetscScalar eigi,PetscR
   PetscReal      w=0.0;
   PetscInt       j;
   PetscBool      flg;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (nep->fui!=NEP_USER_INTERFACE_SPLIT) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Backward error only available in split form");
-  /* initialization of matrix norms */
-  if (!nep->nrma[0]) {
+  if (nep->fui!=NEP_USER_INTERFACE_SPLIT) {
+    PetscCall(NEPComputeFunction(nep,eigr,nep->function,nep->function));
+    PetscCall(MatHasOperation(nep->function,MATOP_NORM,&flg));
+    PetscCheck(flg,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"The computation of backward errors requires a matrix norm operation");
+    PetscCall(MatNorm(nep->function,NORM_INFINITY,&w));
+  } else {
+    /* initialization of matrix norms */
+    if (!nep->nrma[0]) {
+      for (j=0;j<nep->nt;j++) {
+        PetscCall(MatHasOperation(nep->A[j],MATOP_NORM,&flg));
+        PetscCheck(flg,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"The convergence test related to the matrix norms requires a matrix norm operation");
+        PetscCall(MatNorm(nep->A[j],NORM_INFINITY,&nep->nrma[j]));
+      }
+    }
     for (j=0;j<nep->nt;j++) {
-      ierr = MatHasOperation(nep->A[j],MATOP_NORM,&flg);CHKERRQ(ierr);
-      if (!flg) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_WRONG,"The convergence test related to the matrix norms requires a matrix norm operation");
-      ierr = MatNorm(nep->A[j],NORM_INFINITY,&nep->nrma[j]);CHKERRQ(ierr);
+      PetscCall(FNEvaluateFunction(nep->f[j],eigr,&s));
+      w = w + nep->nrma[j]*PetscAbsScalar(s);
     }
-  }
-  for (j=0;j<nep->nt;j++) {
-    ierr = FNEvaluateFunction(nep->f[j],eigr,&s);CHKERRQ(ierr);
-    w = w + nep->nrma[j]*PetscAbsScalar(s);
   }
   *errest = res/w;
   PetscFunctionReturn(0);
@@ -164,41 +170,27 @@ PetscErrorCode NEPConvergedNorm(NEP nep,PetscScalar eigr,PetscScalar eigi,PetscR
 @*/
 PetscErrorCode NEPStoppingBasic(NEP nep,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,NEPConvergedReason *reason,void *ctx)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   *reason = NEP_CONVERGED_ITERATING;
   if (nconv >= nev) {
-    ierr = PetscInfo2(nep,"Nonlinear eigensolver finished successfully: %D eigenpairs converged at iteration %D\n",nconv,its);CHKERRQ(ierr);
+    PetscCall(PetscInfo(nep,"Nonlinear eigensolver finished successfully: %" PetscInt_FMT " eigenpairs converged at iteration %" PetscInt_FMT "\n",nconv,its));
     *reason = NEP_CONVERGED_TOL;
   } else if (its >= max_it) {
     *reason = NEP_DIVERGED_ITS;
-    ierr = PetscInfo1(nep,"Nonlinear eigensolver iteration reached maximum number of iterations (%D)\n",its);CHKERRQ(ierr);
+    PetscCall(PetscInfo(nep,"Nonlinear eigensolver iteration reached maximum number of iterations (%" PetscInt_FMT ")\n",its));
   }
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode NEPComputeVectors_Schur(NEP nep)
 {
-  PetscErrorCode ierr;
-  PetscInt       n,i;
   Mat            Z;
-  Vec            v;
 
   PetscFunctionBegin;
-  ierr = DSGetDimensions(nep->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
-  ierr = DSVectors(nep->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
-  ierr = DSGetMat(nep->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(nep->V,0,n);CHKERRQ(ierr);
-  ierr = BVMultInPlace(nep->V,Z,0,n);CHKERRQ(ierr);
-  ierr = MatDestroy(&Z);CHKERRQ(ierr);
-
-  /* normalization */
-  for (i=0;i<n;i++) {
-    ierr = BVGetColumn(nep->V,i,&v);CHKERRQ(ierr);
-    ierr = VecNormalize(v,NULL);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(nep->V,i,&v);CHKERRQ(ierr);
-  }
+  PetscCall(DSVectors(nep->ds,DS_MAT_X,NULL,NULL));
+  PetscCall(DSGetMat(nep->ds,DS_MAT_X,&Z));
+  PetscCall(BVMultInPlace(nep->V,Z,0,nep->nconv));
+  PetscCall(MatDestroy(&Z));
+  PetscCall(BVNormalize(nep->V,nep->eigi));
   PetscFunctionReturn(0);
 }
-
