@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -33,21 +33,20 @@ typedef struct {
 
 PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
 {
-  PetscErrorCode ierr;
   PEP_QARNOLDI   *ctx = (PEP_QARNOLDI*)pep->data;
   PetscBool      flg;
 
   PetscFunctionBegin;
   PEPCheckQuadratic(pep);
   PEPCheckShiftSinvert(pep);
-  ierr = PEPSetDimensions_Default(pep,pep->nev,&pep->ncv,&pep->mpd);CHKERRQ(ierr);
-  if (!ctx->lock && pep->mpd<pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Should not use mpd parameter in non-locking variant");
+  PetscCall(PEPSetDimensions_Default(pep,pep->nev,&pep->ncv,&pep->mpd));
+  PetscCheck(ctx->lock || pep->mpd>=pep->ncv,PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Should not use mpd parameter in non-locking variant");
   if (pep->max_it==PETSC_DEFAULT) pep->max_it = PetscMax(100,4*pep->n/pep->ncv);
-  if (!pep->which) { ierr = PEPSetWhichEigenpairs_Default(pep);CHKERRQ(ierr); }
-  if (pep->which==PEP_ALL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"This solver does not support computing all eigenvalues");
+  if (!pep->which) PetscCall(PEPSetWhichEigenpairs_Default(pep));
+  PetscCheck(pep->which!=PEP_ALL,PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"This solver does not support computing all eigenvalues");
 
-  ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
-  if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver requires the ST transformation flag set, see STSetTransform()");
+  PetscCall(STGetTransform(pep->st,&flg));
+  PetscCheck(flg,PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver requires the ST transformation flag set, see STSetTransform()");
 
   /* set default extraction */
   if (!pep->extract) pep->extract = PEP_EXTRACT_NONE;
@@ -55,39 +54,36 @@ PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
 
   if (!ctx->keep) ctx->keep = 0.5;
 
-  ierr = PEPAllocateSolution(pep,0);CHKERRQ(ierr);
-  ierr = PEPSetWorkVecs(pep,4);CHKERRQ(ierr);
+  PetscCall(PEPAllocateSolution(pep,0));
+  PetscCall(PEPSetWorkVecs(pep,4));
 
-  ierr = DSSetType(pep->ds,DSNHEP);CHKERRQ(ierr);
-  ierr = DSSetExtraRow(pep->ds,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = DSAllocate(pep->ds,pep->ncv+1);CHKERRQ(ierr);
+  PetscCall(DSSetType(pep->ds,DSNHEP));
+  PetscCall(DSSetExtraRow(pep->ds,PETSC_TRUE));
+  PetscCall(DSAllocate(pep->ds,pep->ncv+1));
 
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode PEPExtractVectors_QArnoldi(PEP pep)
 {
-  PetscErrorCode ierr;
   PetscInt       i,k=pep->nconv,ldds;
   PetscScalar    *X,*pX0;
   Mat            X0;
 
   PetscFunctionBegin;
   if (pep->nconv==0) PetscFunctionReturn(0);
-  ierr = DSGetLeadingDimension(pep->ds,&ldds);CHKERRQ(ierr);
-  ierr = DSVectors(pep->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
-  ierr = DSGetArray(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
+  PetscCall(DSGetLeadingDimension(pep->ds,&ldds));
+  PetscCall(DSVectors(pep->ds,DS_MAT_X,NULL,NULL));
+  PetscCall(DSGetArray(pep->ds,DS_MAT_X,&X));
 
   /* update vectors V = V*X */
-  ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&X0);CHKERRQ(ierr);
-  ierr = MatDenseGetArrayWrite(X0,&pX0);CHKERRQ(ierr);
-  for (i=0;i<k;i++) {
-    ierr = PetscArraycpy(pX0+i*k,X+i*ldds,k);CHKERRQ(ierr);
-  }
-  ierr = MatDenseRestoreArrayWrite(X0,&pX0);CHKERRQ(ierr);
-  ierr = BVMultInPlace(pep->V,X0,0,k);CHKERRQ(ierr);
-  ierr = MatDestroy(&X0);CHKERRQ(ierr);
-  ierr = DSRestoreArray(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
+  PetscCall(MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&X0));
+  PetscCall(MatDenseGetArrayWrite(X0,&pX0));
+  for (i=0;i<k;i++) PetscCall(PetscArraycpy(pX0+i*k,X+i*ldds,k));
+  PetscCall(MatDenseRestoreArrayWrite(X0,&pX0));
+  PetscCall(BVMultInPlace(pep->V,X0,0,k));
+  PetscCall(MatDestroy(&X0));
+  PetscCall(DSRestoreArray(pep->ds,DS_MAT_X,&X));
   PetscFunctionReturn(0);
 }
 
@@ -96,7 +92,6 @@ PetscErrorCode PEPExtractVectors_QArnoldi(PEP pep)
 */
 static PetscErrorCode PEPQArnoldiCGS(PEP pep,PetscScalar *H,PetscBLASInt ldh,PetscScalar *h,PetscBLASInt j,BV V,Vec t,Vec v,Vec w,PetscReal *onorm,PetscReal *norm,PetscScalar *work)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   ione = 1,j_1 = j+1;
   PetscReal      x,y;
   PetscScalar    dot,one = 1.0,zero = 0.0;
@@ -104,32 +99,32 @@ static PetscErrorCode PEPQArnoldiCGS(PEP pep,PetscScalar *H,PetscBLASInt ldh,Pet
   PetscFunctionBegin;
   /* compute norm of v and w */
   if (onorm) {
-    ierr = VecNorm(v,NORM_2,&x);CHKERRQ(ierr);
-    ierr = VecNorm(w,NORM_2,&y);CHKERRQ(ierr);
+    PetscCall(VecNorm(v,NORM_2,&x));
+    PetscCall(VecNorm(w,NORM_2,&y));
     *onorm = PetscSqrtReal(x*x+y*y);
   }
 
   /* orthogonalize: compute h */
-  ierr = BVDotVec(V,v,h);CHKERRQ(ierr);
-  ierr = BVDotVec(V,w,work);CHKERRQ(ierr);
+  PetscCall(BVDotVec(V,v,h));
+  PetscCall(BVDotVec(V,w,work));
   if (j>0)
     PetscStackCallBLAS("BLASgemv",BLASgemv_("C",&j_1,&j,&one,H,&ldh,work,&ione,&one,h,&ione));
-  ierr = VecDot(w,t,&dot);CHKERRQ(ierr);
+  PetscCall(VecDot(w,t,&dot));
   h[j] += dot;
 
   /* orthogonalize: update v and w */
-  ierr = BVMultVec(V,-1.0,1.0,v,h);CHKERRQ(ierr);
+  PetscCall(BVMultVec(V,-1.0,1.0,v,h));
   if (j>0) {
     PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&j_1,&j,&one,H,&ldh,h,&ione,&zero,work,&ione));
-    ierr = BVMultVec(V,-1.0,1.0,w,work);CHKERRQ(ierr);
+    PetscCall(BVMultVec(V,-1.0,1.0,w,work));
   }
-  ierr = VecAXPY(w,-h[j],t);CHKERRQ(ierr);
+  PetscCall(VecAXPY(w,-h[j],t));
 
   /* compute norm of v and w */
   if (norm) {
-    ierr = VecNorm(v,NORM_2,&x);CHKERRQ(ierr);
-    ierr = VecNorm(w,NORM_2,&y);CHKERRQ(ierr);
-    *norm = PetscSqrtReal(x*x+y*y);CHKERRQ(ierr);
+    PetscCall(VecNorm(v,NORM_2,&x));
+    PetscCall(VecNorm(w,NORM_2,&y));
+    *norm = PetscSqrtReal(x*x+y*y);
   }
   PetscFunctionReturn(0);
 }
@@ -139,7 +134,6 @@ static PetscErrorCode PEPQArnoldiCGS(PEP pep,PetscScalar *H,PetscBLASInt ldh,Pet
 */
 static PetscErrorCode PEPQArnoldi(PEP pep,PetscScalar *H,PetscInt ldh,PetscInt k,PetscInt *M,Vec v,Vec w,PetscReal *beta,PetscBool *breakdown,PetscScalar *work)
 {
-  PetscErrorCode     ierr;
   PetscInt           i,j,l,m = *M;
   Vec                t = pep->work[2],u = pep->work[3];
   BVOrthogRefineType refinement;
@@ -147,63 +141,55 @@ static PetscErrorCode PEPQArnoldi(PEP pep,PetscScalar *H,PetscInt ldh,PetscInt k
   PetscScalar        *c = work + m;
 
   PetscFunctionBegin;
-  ierr = BVGetOrthogonalization(pep->V,NULL,&refinement,&eta,NULL);CHKERRQ(ierr);
-  ierr = BVInsertVec(pep->V,k,v);CHKERRQ(ierr);
+  PetscCall(BVGetOrthogonalization(pep->V,NULL,&refinement,&eta,NULL));
+  PetscCall(BVInsertVec(pep->V,k,v));
   for (j=k;j<m;j++) {
     /* apply operator */
-    ierr = VecCopy(w,t);CHKERRQ(ierr);
-    if (pep->Dr) {
-      ierr = VecPointwiseMult(v,v,pep->Dr);CHKERRQ(ierr);
-    }
-    ierr = STMatMult(pep->st,0,v,u);CHKERRQ(ierr);
-    ierr = VecCopy(t,v);CHKERRQ(ierr);
-    if (pep->Dr) {
-      ierr = VecPointwiseMult(t,t,pep->Dr);CHKERRQ(ierr);
-    }
-    ierr = STMatMult(pep->st,1,t,w);CHKERRQ(ierr);
-    ierr = VecAXPY(u,pep->sfactor,w);CHKERRQ(ierr);
-    ierr = STMatSolve(pep->st,u,w);CHKERRQ(ierr);
-    ierr = VecScale(w,-1.0/(pep->sfactor*pep->sfactor));CHKERRQ(ierr);
-    if (pep->Dr) {
-      ierr = VecPointwiseDivide(w,w,pep->Dr);CHKERRQ(ierr);
-    }
-    ierr = VecCopy(v,t);CHKERRQ(ierr);
-    ierr = BVSetActiveColumns(pep->V,0,j+1);CHKERRQ(ierr);
+    PetscCall(VecCopy(w,t));
+    if (pep->Dr) PetscCall(VecPointwiseMult(v,v,pep->Dr));
+    PetscCall(STMatMult(pep->st,0,v,u));
+    PetscCall(VecCopy(t,v));
+    if (pep->Dr) PetscCall(VecPointwiseMult(t,t,pep->Dr));
+    PetscCall(STMatMult(pep->st,1,t,w));
+    PetscCall(VecAXPY(u,pep->sfactor,w));
+    PetscCall(STMatSolve(pep->st,u,w));
+    PetscCall(VecScale(w,-1.0/(pep->sfactor*pep->sfactor)));
+    if (pep->Dr) PetscCall(VecPointwiseDivide(w,w,pep->Dr));
+    PetscCall(VecCopy(v,t));
+    PetscCall(BVSetActiveColumns(pep->V,0,j+1));
 
     /* orthogonalize */
     switch (refinement) {
       case BV_ORTHOG_REFINE_NEVER:
-        ierr = PEPQArnoldiCGS(pep,H,ldh,H+ldh*j,j,pep->V,t,v,w,NULL,&norm,work);CHKERRQ(ierr);
+        PetscCall(PEPQArnoldiCGS(pep,H,ldh,H+ldh*j,j,pep->V,t,v,w,NULL,&norm,work));
         *breakdown = PETSC_FALSE;
         break;
       case BV_ORTHOG_REFINE_ALWAYS:
-        ierr = PEPQArnoldiCGS(pep,H,ldh,H+ldh*j,j,pep->V,t,v,w,NULL,NULL,work);CHKERRQ(ierr);
-        ierr = PEPQArnoldiCGS(pep,H,ldh,c,j,pep->V,t,v,w,&onorm,&norm,work);CHKERRQ(ierr);
+        PetscCall(PEPQArnoldiCGS(pep,H,ldh,H+ldh*j,j,pep->V,t,v,w,NULL,NULL,work));
+        PetscCall(PEPQArnoldiCGS(pep,H,ldh,c,j,pep->V,t,v,w,&onorm,&norm,work));
         for (i=0;i<=j;i++) H[ldh*j+i] += c[i];
         if (norm < eta * onorm) *breakdown = PETSC_TRUE;
         else *breakdown = PETSC_FALSE;
         break;
       case BV_ORTHOG_REFINE_IFNEEDED:
-        ierr = PEPQArnoldiCGS(pep,H,ldh,H+ldh*j,j,pep->V,t,v,w,&onorm,&norm,work);CHKERRQ(ierr);
+        PetscCall(PEPQArnoldiCGS(pep,H,ldh,H+ldh*j,j,pep->V,t,v,w,&onorm,&norm,work));
         /* ||q|| < eta ||h|| */
         l = 1;
         while (l<3 && norm < eta * onorm) {
           l++;
           onorm = norm;
-          ierr = PEPQArnoldiCGS(pep,H,ldh,c,j,pep->V,t,v,w,NULL,&norm,work);CHKERRQ(ierr);
+          PetscCall(PEPQArnoldiCGS(pep,H,ldh,c,j,pep->V,t,v,w,NULL,&norm,work));
           for (i=0;i<=j;i++) H[ldh*j+i] += c[i];
         }
         if (norm < eta * onorm) *breakdown = PETSC_TRUE;
         else *breakdown = PETSC_FALSE;
         break;
     }
-    ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
-    ierr = VecScale(w,1.0/norm);CHKERRQ(ierr);
+    PetscCall(VecScale(v,1.0/norm));
+    PetscCall(VecScale(w,1.0/norm));
 
     H[j+1+ldh*j] = norm;
-    if (j<m-1) {
-      ierr = BVInsertVec(pep->V,j+1,v);CHKERRQ(ierr);
-    }
+    if (j<m-1) PetscCall(BVInsertVec(pep->V,j+1,v));
   }
   *beta = norm;
   PetscFunctionReturn(0);
@@ -211,7 +197,6 @@ static PetscErrorCode PEPQArnoldi(PEP pep,PetscScalar *H,PetscInt ldh,PetscInt k
 
 PetscErrorCode PEPSolve_QArnoldi(PEP pep)
 {
-  PetscErrorCode ierr;
   PEP_QARNOLDI   *ctx = (PEP_QARNOLDI*)pep->data;
   PetscInt       j,k,l,lwork,nv,ld,nconv;
   Vec            v=pep->work[0],w=pep->work[1];
@@ -221,31 +206,29 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
   PetscBool      breakdown=PETSC_FALSE,sinv;
 
   PetscFunctionBegin;
-  ierr = DSGetLeadingDimension(pep->ds,&ld);CHKERRQ(ierr);
+  PetscCall(DSGetLeadingDimension(pep->ds,&ld));
   lwork = 7*pep->ncv;
-  ierr = PetscMalloc1(lwork,&work);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
-  ierr = RGPushScale(pep->rg,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
-  ierr = STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor);CHKERRQ(ierr);
+  PetscCall(PetscMalloc1(lwork,&work));
+  PetscCall(PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv));
+  PetscCall(RGPushScale(pep->rg,sinv?pep->sfactor:1.0/pep->sfactor));
+  PetscCall(STScaleShift(pep->st,sinv?pep->sfactor:1.0/pep->sfactor));
 
   /* Get the starting Arnoldi vector */
   for (j=0;j<2;j++) {
-    if (j>=pep->nini) {
-      ierr = BVSetRandomColumn(pep->V,j);CHKERRQ(ierr);
-    }
+    if (j>=pep->nini) PetscCall(BVSetRandomColumn(pep->V,j));
   }
-  ierr = BVCopyVec(pep->V,0,v);CHKERRQ(ierr);
-  ierr = BVCopyVec(pep->V,1,w);CHKERRQ(ierr);
-  ierr = VecNorm(v,NORM_2,&x);CHKERRQ(ierr);
-  ierr = VecNorm(w,NORM_2,&y);CHKERRQ(ierr);
-  norm = PetscSqrtReal(x*x+y*y);CHKERRQ(ierr);
-  ierr = VecScale(v,1.0/norm);CHKERRQ(ierr);
-  ierr = VecScale(w,1.0/norm);CHKERRQ(ierr);
+  PetscCall(BVCopyVec(pep->V,0,v));
+  PetscCall(BVCopyVec(pep->V,1,w));
+  PetscCall(VecNorm(v,NORM_2,&x));
+  PetscCall(VecNorm(w,NORM_2,&y));
+  norm = PetscSqrtReal(x*x+y*y);
+  PetscCall(VecScale(v,1.0/norm));
+  PetscCall(VecScale(w,1.0/norm));
 
   /* clean projected matrix (including the extra-arrow) */
-  ierr = DSGetArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-  ierr = PetscArrayzero(S,ld*ld);CHKERRQ(ierr);
-  ierr = DSRestoreArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
+  PetscCall(DSGetArray(pep->ds,DS_MAT_A,&S));
+  PetscCall(PetscArrayzero(S,ld*ld));
+  PetscCall(DSRestoreArray(pep->ds,DS_MAT_A,&S));
 
    /* Restart loop */
   l = 0;
@@ -254,66 +237,62 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
 
     /* Compute an nv-step Arnoldi factorization */
     nv = PetscMin(pep->nconv+pep->mpd,pep->ncv);
-    ierr = DSGetArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-    ierr = PEPQArnoldi(pep,S,ld,pep->nconv+l,&nv,v,w,&beta,&breakdown,work);CHKERRQ(ierr);
-    ierr = DSRestoreArray(pep->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-    ierr = DSSetDimensions(pep->ds,nv,0,pep->nconv,pep->nconv+l);CHKERRQ(ierr);
-    if (l==0) {
-      ierr = DSSetState(pep->ds,DS_STATE_INTERMEDIATE);CHKERRQ(ierr);
-    } else {
-      ierr = DSSetState(pep->ds,DS_STATE_RAW);CHKERRQ(ierr);
-    }
-    ierr = BVSetActiveColumns(pep->V,pep->nconv,nv);CHKERRQ(ierr);
+    PetscCall(DSGetArray(pep->ds,DS_MAT_A,&S));
+    PetscCall(PEPQArnoldi(pep,S,ld,pep->nconv+l,&nv,v,w,&beta,&breakdown,work));
+    PetscCall(DSRestoreArray(pep->ds,DS_MAT_A,&S));
+    PetscCall(DSSetDimensions(pep->ds,nv,pep->nconv,pep->nconv+l));
+    PetscCall(DSSetState(pep->ds,l?DS_STATE_RAW:DS_STATE_INTERMEDIATE));
+    PetscCall(BVSetActiveColumns(pep->V,pep->nconv,nv));
 
     /* Solve projected problem */
-    ierr = DSSolve(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
-    ierr = DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
-    ierr = DSUpdateExtraRow(pep->ds);CHKERRQ(ierr);
-    ierr = DSSynchronize(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
+    PetscCall(DSSolve(pep->ds,pep->eigr,pep->eigi));
+    PetscCall(DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL));
+    PetscCall(DSUpdateExtraRow(pep->ds));
+    PetscCall(DSSynchronize(pep->ds,pep->eigr,pep->eigi));
 
     /* Check convergence */
-    ierr = PEPKrylovConvergence(pep,PETSC_FALSE,pep->nconv,nv-pep->nconv,beta,&k);CHKERRQ(ierr);
-    ierr = (*pep->stopping)(pep,pep->its,pep->max_it,k,pep->nev,&pep->reason,pep->stoppingctx);CHKERRQ(ierr);
+    PetscCall(PEPKrylovConvergence(pep,PETSC_FALSE,pep->nconv,nv-pep->nconv,beta,&k));
+    PetscCall((*pep->stopping)(pep,pep->its,pep->max_it,k,pep->nev,&pep->reason,pep->stoppingctx));
     nconv = k;
 
     /* Update l */
     if (pep->reason != PEP_CONVERGED_ITERATING || breakdown) l = 0;
     else {
       l = PetscMax(1,(PetscInt)((nv-k)*ctx->keep));
-      ierr = DSGetTruncateSize(pep->ds,k,nv,&l);CHKERRQ(ierr);
+      PetscCall(DSGetTruncateSize(pep->ds,k,nv,&l));
     }
     if (!ctx->lock && l>0) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
-    if (l) { ierr = PetscInfo1(pep,"Preparing to restart keeping l=%D vectors\n",l);CHKERRQ(ierr); }
+    if (l) PetscCall(PetscInfo(pep,"Preparing to restart keeping l=%" PetscInt_FMT " vectors\n",l));
 
     if (pep->reason == PEP_CONVERGED_ITERATING) {
-      if (breakdown) {
+      if (PetscUnlikely(breakdown)) {
         /* Stop if breakdown */
-        ierr = PetscInfo2(pep,"Breakdown Quadratic Arnoldi method (it=%D norm=%g)\n",pep->its,(double)beta);CHKERRQ(ierr);
+        PetscCall(PetscInfo(pep,"Breakdown Quadratic Arnoldi method (it=%" PetscInt_FMT " norm=%g)\n",pep->its,(double)beta));
         pep->reason = PEP_DIVERGED_BREAKDOWN;
       } else {
         /* Prepare the Rayleigh quotient for restart */
-        ierr = DSTruncate(pep->ds,k+l,PETSC_FALSE);CHKERRQ(ierr);
+        PetscCall(DSTruncate(pep->ds,k+l,PETSC_FALSE));
       }
     }
     /* Update the corresponding vectors V(:,idx) = V*Q(:,idx) */
-    ierr = DSGetMat(pep->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-    ierr = BVMultInPlace(pep->V,Q,pep->nconv,k+l);CHKERRQ(ierr);
-    ierr = MatDestroy(&Q);CHKERRQ(ierr);
+    PetscCall(DSGetMat(pep->ds,DS_MAT_Q,&Q));
+    PetscCall(BVMultInPlace(pep->V,Q,pep->nconv,k+l));
+    PetscCall(MatDestroy(&Q));
 
     pep->nconv = k;
-    ierr = PEPMonitor(pep,pep->its,nconv,pep->eigr,pep->eigi,pep->errest,nv);CHKERRQ(ierr);
+    PetscCall(PEPMonitor(pep,pep->its,nconv,pep->eigr,pep->eigi,pep->errest,nv));
   }
-  ierr = BVSetActiveColumns(pep->V,0,pep->nconv);CHKERRQ(ierr);
+  PetscCall(BVSetActiveColumns(pep->V,0,pep->nconv));
   for (j=0;j<pep->nconv;j++) {
     pep->eigr[j] *= pep->sfactor;
     pep->eigi[j] *= pep->sfactor;
   }
 
-  ierr = STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor);CHKERRQ(ierr);
-  ierr = RGPopScale(pep->rg);CHKERRQ(ierr);
+  PetscCall(STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor));
+  PetscCall(RGPopScale(pep->rg));
 
-  ierr = DSTruncate(pep->ds,pep->nconv,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = PetscFree(work);CHKERRQ(ierr);
+  PetscCall(DSTruncate(pep->ds,pep->nconv,PETSC_TRUE));
+  PetscCall(PetscFree(work));
   PetscFunctionReturn(0);
 }
 
@@ -324,7 +303,7 @@ static PetscErrorCode PEPQArnoldiSetRestart_QArnoldi(PEP pep,PetscReal keep)
   PetscFunctionBegin;
   if (keep==PETSC_DEFAULT) ctx->keep = 0.5;
   else {
-    if (keep<0.1 || keep>0.9) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"The keep argument must be in the range [0.1,0.9]");
+    PetscCheck(keep>=0.1 && keep<=0.9,PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"The keep argument must be in the range [0.1,0.9]");
     ctx->keep = keep;
   }
   PetscFunctionReturn(0);
@@ -353,12 +332,10 @@ static PetscErrorCode PEPQArnoldiSetRestart_QArnoldi(PEP pep,PetscReal keep)
 @*/
 PetscErrorCode PEPQArnoldiSetRestart(PEP pep,PetscReal keep)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidLogicalCollectiveReal(pep,keep,2);
-  ierr = PetscTryMethod(pep,"PEPQArnoldiSetRestart_C",(PEP,PetscReal),(pep,keep));CHKERRQ(ierr);
+  PetscTryMethod(pep,"PEPQArnoldiSetRestart_C",(PEP,PetscReal),(pep,keep));
   PetscFunctionReturn(0);
 }
 
@@ -388,12 +365,10 @@ static PetscErrorCode PEPQArnoldiGetRestart_QArnoldi(PEP pep,PetscReal *keep)
 @*/
 PetscErrorCode PEPQArnoldiGetRestart(PEP pep,PetscReal *keep)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidRealPointer(keep,2);
-  ierr = PetscUseMethod(pep,"PEPQArnoldiGetRestart_C",(PEP,PetscReal*),(pep,keep));CHKERRQ(ierr);
+  PetscUseMethod(pep,"PEPQArnoldiGetRestart_C",(PEP,PetscReal*),(pep,keep));
   PetscFunctionReturn(0);
 }
 
@@ -420,12 +395,10 @@ static PetscErrorCode PEPQArnoldiSetLocking_QArnoldi(PEP pep,PetscBool lock)
 .  -pep_qarnoldi_locking - Sets the locking flag
 
    Notes:
-   The default is to keep all directions in the working subspace even if
-   already converged to working accuracy (the non-locking variant).
-   This behaviour can be changed so that converged eigenpairs are locked
-   when the method restarts.
-
-   Note that the default behaviour is the opposite to Krylov solvers in EPS.
+   The default is to lock converged eigenpairs when the method restarts.
+   This behaviour can be changed so that all directions are kept in the
+   working subspace even if already converged to working accuracy (the
+   non-locking variant).
 
    Level: advanced
 
@@ -433,12 +406,10 @@ static PetscErrorCode PEPQArnoldiSetLocking_QArnoldi(PEP pep,PetscBool lock)
 @*/
 PetscErrorCode PEPQArnoldiSetLocking(PEP pep,PetscBool lock)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidLogicalCollectiveBool(pep,lock,2);
-  ierr = PetscTryMethod(pep,"PEPQArnoldiSetLocking_C",(PEP,PetscBool),(pep,lock));CHKERRQ(ierr);
+  PetscTryMethod(pep,"PEPQArnoldiSetLocking_C",(PEP,PetscBool),(pep,lock));
   PetscFunctionReturn(0);
 }
 
@@ -468,69 +439,62 @@ static PetscErrorCode PEPQArnoldiGetLocking_QArnoldi(PEP pep,PetscBool *lock)
 @*/
 PetscErrorCode PEPQArnoldiGetLocking(PEP pep,PetscBool *lock)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidBoolPointer(lock,2);
-  ierr = PetscUseMethod(pep,"PEPQArnoldiGetLocking_C",(PEP,PetscBool*),(pep,lock));CHKERRQ(ierr);
+  PetscUseMethod(pep,"PEPQArnoldiGetLocking_C",(PEP,PetscBool*),(pep,lock));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode PEPSetFromOptions_QArnoldi(PetscOptionItems *PetscOptionsObject,PEP pep)
 {
-  PetscErrorCode ierr;
   PetscBool      flg,lock;
   PetscReal      keep;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead(PetscOptionsObject,"PEP Q-Arnoldi Options");CHKERRQ(ierr);
+  PetscCall(PetscOptionsHead(PetscOptionsObject,"PEP Q-Arnoldi Options"));
 
-    ierr = PetscOptionsReal("-pep_qarnoldi_restart","Proportion of vectors kept after restart","PEPQArnoldiSetRestart",0.5,&keep,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPQArnoldiSetRestart(pep,keep);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsReal("-pep_qarnoldi_restart","Proportion of vectors kept after restart","PEPQArnoldiSetRestart",0.5,&keep,&flg));
+    if (flg) PetscCall(PEPQArnoldiSetRestart(pep,keep));
 
-    ierr = PetscOptionsBool("-pep_qarnoldi_locking","Choose between locking and non-locking variants","PEPQArnoldiSetLocking",PETSC_FALSE,&lock,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = PEPQArnoldiSetLocking(pep,lock);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsBool("-pep_qarnoldi_locking","Choose between locking and non-locking variants","PEPQArnoldiSetLocking",PETSC_FALSE,&lock,&flg));
+    if (flg) PetscCall(PEPQArnoldiSetLocking(pep,lock));
 
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  PetscCall(PetscOptionsTail());
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode PEPView_QArnoldi(PEP pep,PetscViewer viewer)
 {
-  PetscErrorCode ierr;
   PEP_QARNOLDI   *ctx = (PEP_QARNOLDI*)pep->data;
   PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii));
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep));CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-");CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIIPrintf(viewer,"  %d%% of basis vectors kept after restart\n",(int)(100*ctx->keep)));
+    PetscCall(PetscViewerASCIIPrintf(viewer,"  using the %slocking variant\n",ctx->lock?"":"non-"));
   }
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode PEPDestroy_QArnoldi(PEP pep)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  ierr = PetscFree(pep->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetRestart_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetRestart_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetLocking_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetLocking_C",NULL);CHKERRQ(ierr);
+  PetscCall(PetscFree(pep->data));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetRestart_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetRestart_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetLocking_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetLocking_C",NULL));
   PetscFunctionReturn(0);
 }
 
 SLEPC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
 {
   PEP_QARNOLDI   *ctx;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(pep,&ctx);CHKERRQ(ierr);
+  PetscCall(PetscNewLog(pep,&ctx));
   pep->data = (void*)ctx;
 
   pep->lineariz = PETSC_TRUE;
@@ -546,10 +510,9 @@ SLEPC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
   pep->ops->extractvectors = PEPExtractVectors_QArnoldi;
   pep->ops->setdefaultst   = PEPSetDefaultST_Transform;
 
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetRestart_C",PEPQArnoldiSetRestart_QArnoldi);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetRestart_C",PEPQArnoldiGetRestart_QArnoldi);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetLocking_C",PEPQArnoldiSetLocking_QArnoldi);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetLocking_C",PEPQArnoldiGetLocking_QArnoldi);CHKERRQ(ierr);
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetRestart_C",PEPQArnoldiSetRestart_QArnoldi));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetRestart_C",PEPQArnoldiGetRestart_QArnoldi));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiSetLocking_C",PEPQArnoldiSetLocking_QArnoldi));
+  PetscCall(PetscObjectComposeFunction((PetscObject)pep,"PEPQArnoldiGetLocking_C",PEPQArnoldiGetLocking_QArnoldi));
   PetscFunctionReturn(0);
 }
-
