@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -33,68 +33,86 @@ int main(int argc,char **argv)
   Mat            A;               /* Grcar matrix */
   SVD            svd;             /* singular value solver context */
   Vec            v0,w0;           /* initial vectors */
-  PetscInt       N=35,M=30,Istart,Iend,i,col[5];
+  Vec            *U,*V;
+  PetscInt       N=35,M=30,Istart,Iend,i,col[5],nconv;
   PetscScalar    value[] = { -1, 1, 1, 1, 1 };
-  PetscErrorCode ierr;
+  PetscReal      lev1=0.0,lev2=0.0,tol=PETSC_SMALL;
+  PetscBool      skiporth=PETSC_FALSE;
 
-  ierr = SlepcInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
-  ierr = PetscOptionsGetInt(NULL,NULL,"-n",&N,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,NULL,"-m",&M,NULL);CHKERRQ(ierr);
-  ierr = PetscPrintf(PETSC_COMM_WORLD,"\nSVD of a rectangular Grcar matrix, %Dx%D\n\n",N,M);CHKERRQ(ierr);
+  PetscCall(SlepcInitialize(&argc,&argv,(char*)0,help));
+  PetscCall(PetscOptionsGetInt(NULL,NULL,"-n",&N,NULL));
+  PetscCall(PetscOptionsGetInt(NULL,NULL,"-m",&M,NULL));
+  PetscCall(PetscPrintf(PETSC_COMM_WORLD,"\nSVD of a rectangular Grcar matrix, %" PetscInt_FMT "x%" PetscInt_FMT "\n\n",N,M));
+  PetscCall(PetscOptionsGetBool(NULL,NULL,"-skiporth",&skiporth,NULL));
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         Generate the matrix
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
-  ierr = MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,N,M);CHKERRQ(ierr);
-  ierr = MatSetFromOptions(A);CHKERRQ(ierr);
-  ierr = MatSetUp(A);CHKERRQ(ierr);
+  PetscCall(MatCreate(PETSC_COMM_WORLD,&A));
+  PetscCall(MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,N,M));
+  PetscCall(MatSetFromOptions(A));
+  PetscCall(MatSetUp(A));
 
-  ierr = MatGetOwnershipRange(A,&Istart,&Iend);CHKERRQ(ierr);
+  PetscCall(MatGetOwnershipRange(A,&Istart,&Iend));
   for (i=Istart;i<Iend;i++) {
     col[0]=i-1; col[1]=i; col[2]=i+1; col[3]=i+2; col[4]=i+3;
-    if (i==0) {
-      ierr = MatSetValues(A,1,&i,PetscMin(4,M-i+1),col+1,value+1,INSERT_VALUES);CHKERRQ(ierr);
-    } else {
-      ierr = MatSetValues(A,1,&i,PetscMin(5,M-i+1),col,value,INSERT_VALUES);CHKERRQ(ierr);
-    }
+    if (i==0) PetscCall(MatSetValues(A,1,&i,PetscMin(4,M-i+1),col+1,value+1,INSERT_VALUES));
+    else PetscCall(MatSetValues(A,1,&i,PetscMin(5,M-i+1),col,value,INSERT_VALUES));
   }
-  ierr = MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  PetscCall(MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY));
+  PetscCall(MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY));
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
              Create the SVD context and solve the problem
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  ierr = SVDCreate(PETSC_COMM_WORLD,&svd);CHKERRQ(ierr);
-  ierr = SVDSetOperators(svd,A,NULL);CHKERRQ(ierr);
-  ierr = SVDSetFromOptions(svd);CHKERRQ(ierr);
+  PetscCall(SVDCreate(PETSC_COMM_WORLD,&svd));
+  PetscCall(SVDSetOperators(svd,A,NULL));
+  PetscCall(SVDSetFromOptions(svd));
 
   /*
      Set the initial vectors. This is optional, if not done the initial
      vectors are set to random values
   */
-  ierr = MatCreateVecs(A,&v0,&w0);CHKERRQ(ierr);
-  ierr = VecSet(v0,1.0);CHKERRQ(ierr);
-  ierr = VecSet(w0,1.0);CHKERRQ(ierr);
-  ierr = SVDSetInitialSpaces(svd,1,&v0,1,&w0);CHKERRQ(ierr);
+  PetscCall(MatCreateVecs(A,&v0,&w0));
+  PetscCall(VecSet(v0,1.0));
+  PetscCall(VecSet(w0,1.0));
+  PetscCall(SVDSetInitialSpaces(svd,1,&v0,1,&w0));
 
   /*
      Compute solution
   */
-  ierr = SVDSolve(svd);CHKERRQ(ierr);
-  ierr = SVDErrorView(svd,SVD_ERROR_RELATIVE,NULL);CHKERRQ(ierr);
+  PetscCall(SVDSolve(svd));
+  PetscCall(SVDErrorView(svd,SVD_ERROR_RELATIVE,NULL));
+
+  /*
+     Check orthonormality of computed singular vectors
+  */
+  PetscCall(SVDGetConverged(svd,&nconv));
+  if (nconv>1) {
+    PetscCall(VecDuplicateVecs(w0,nconv,&U));
+    PetscCall(VecDuplicateVecs(v0,nconv,&V));
+    for (i=0;i<nconv;i++) PetscCall(SVDGetSingularTriplet(svd,i,NULL,U[i],V[i]));
+    if (!skiporth) {
+      PetscCall(VecCheckOrthonormality(U,nconv,NULL,nconv,NULL,NULL,&lev1));
+      PetscCall(VecCheckOrthonormality(V,nconv,NULL,nconv,NULL,NULL,&lev2));
+    }
+    if (lev1+lev2<20*tol) PetscCall(PetscPrintf(PETSC_COMM_WORLD,"Level of orthogonality below the tolerance\n"));
+    else PetscCall(PetscPrintf(PETSC_COMM_WORLD,"Level of orthogonality: %g (U) %g (V)\n",(double)lev1,(double)lev2));
+    PetscCall(VecDestroyVecs(nconv,&U));
+    PetscCall(VecDestroyVecs(nconv,&V));
+  }
 
   /*
      Free work space
   */
-  ierr = VecDestroy(&v0);CHKERRQ(ierr);
-  ierr = VecDestroy(&w0);CHKERRQ(ierr);
-  ierr = SVDDestroy(&svd);CHKERRQ(ierr);
-  ierr = MatDestroy(&A);CHKERRQ(ierr);
-  ierr = SlepcFinalize();
-  return ierr;
+  PetscCall(VecDestroy(&v0));
+  PetscCall(VecDestroy(&w0));
+  PetscCall(SVDDestroy(&svd));
+  PetscCall(MatDestroy(&A));
+  PetscCall(SlepcFinalize());
+  return 0;
 }
 
 /*TEST
@@ -111,7 +129,7 @@ int main(int argc,char **argv)
          args: -svd_type lanczos -svd_lanczos_oneside
       test:
          suffix: 1_trlanczos
-         args: -svd_type trlanczos
+         args: -svd_type trlanczos -svd_trlanczos_locking {{0 1}}
       test:
          suffix: 1_trlanczos_one
          args: -svd_type trlanczos -svd_trlanczos_oneside
@@ -130,9 +148,11 @@ int main(int argc,char **argv)
       test:
          suffix: 1_cyclic
          args: -svd_type cyclic
+         requires: !__float128
       test:
          suffix: 1_cyclic_exp
          args: -svd_type cyclic -svd_cyclic_explicitmatrix
+         requires: !__float128
       test:
          suffix: 1_lapack
          args: -svd_type lapack
@@ -150,7 +170,7 @@ int main(int argc,char **argv)
       filter: sed -e "s/22176/22175/" | sed -e "s/21798/21797/" | sed -e "s/16826/16825/" | sed -e "s/15129/15128/"
       test:
          suffix: 2_lanczos
-         args: -svd_type lanczos
+         args: -svd_type lanczos -svd_conv_norm
       test:
          suffix: 2_lanczos_one
          args: -svd_type lanczos -svd_lanczos_oneside
@@ -175,7 +195,8 @@ int main(int argc,char **argv)
          requires: !complex
       test:
          suffix: 2_cyclic
-         args: -svd_type cyclic
+         args: -svd_type cyclic -svd_tol 1e-8
+         requires: double
       test:
          suffix: 2_lapack
          args: -svd_type lapack
