@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -42,7 +42,6 @@
 @*/
 PetscErrorCode BVScatter(BV Vin,BV Vout,VecScatter scat,Vec xdup)
 {
-  PetscErrorCode    ierr;
   PetscInt          i;
   Vec               v;
   const PetscScalar *array;
@@ -53,15 +52,15 @@ PetscErrorCode BVScatter(BV Vin,BV Vout,VecScatter scat,Vec xdup)
   PetscValidHeaderSpecific(scat,PETSCSF_CLASSID,3);
   PetscValidHeaderSpecific(xdup,VEC_CLASSID,4);
   for (i=Vin->l;i<Vin->k;i++) {
-    ierr = BVGetColumn(Vin,i,&v);CHKERRQ(ierr);
-    ierr = VecScatterBegin(scat,v,xdup,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-    ierr = VecScatterEnd(scat,v,xdup,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(Vin,i,&v);CHKERRQ(ierr);
-    ierr = VecGetArrayRead(xdup,&array);CHKERRQ(ierr);
-    ierr = VecPlaceArray(Vout->t,array);CHKERRQ(ierr);
-    ierr = BVInsertVec(Vout,i,Vout->t);CHKERRQ(ierr);
-    ierr = VecResetArray(Vout->t);CHKERRQ(ierr);
-    ierr = VecRestoreArrayRead(xdup,&array);CHKERRQ(ierr);
+    PetscCall(BVGetColumn(Vin,i,&v));
+    PetscCall(VecScatterBegin(scat,v,xdup,INSERT_VALUES,SCATTER_FORWARD));
+    PetscCall(VecScatterEnd(scat,v,xdup,INSERT_VALUES,SCATTER_FORWARD));
+    PetscCall(BVRestoreColumn(Vin,i,&v));
+    PetscCall(VecGetArrayRead(xdup,&array));
+    PetscCall(VecPlaceArray(Vout->t,array));
+    PetscCall(BVInsertVec(Vout,i,Vout->t));
+    PetscCall(VecResetArray(Vout->t));
+    PetscCall(VecRestoreArrayRead(xdup,&array));
   }
   PetscFunctionReturn(0);
 }
@@ -89,7 +88,7 @@ PetscErrorCode BVScatter(BV Vin,BV Vout,VecScatter scat,Vec xdup)
 
    Notes:
    This is a generalization of BVMult(). The resulting matrix S consists of M
-   panels of L columns, and the following formula is computed for each panel:
+   panels of L columns, and the following formula is computed for each panel
    S_k = sum_j w_j*zn_j^k*Y_j, where Y_j is the j-th panel of Y containing
    the result of solving T(z_j)^{-1}*X for each integration point j. L_max is
    the width of the panels in Y.
@@ -106,7 +105,6 @@ PetscErrorCode BVScatter(BV Vin,BV Vout,VecScatter scat,Vec xdup)
 @*/
 PetscErrorCode BVSumQuadrature(BV S,BV Y,PetscInt M,PetscInt L,PetscInt L_max,PetscScalar *w,PetscScalar *zn,VecScatter scat,PetscSubcomm subcomm,PetscInt npoints,PetscBool useconj)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,k,nloc;
   Vec            v,sj;
   PetscScalar    *ppk,*pv,one=1.0;
@@ -116,35 +114,33 @@ PetscErrorCode BVSumQuadrature(BV S,BV Y,PetscInt M,PetscInt L,PetscInt L_max,Pe
   PetscValidHeaderSpecific(Y,BV_CLASSID,2);
   if (scat) PetscValidHeaderSpecific(scat,PETSCSF_CLASSID,8);
 
-  ierr = BVGetSizes(Y,&nloc,NULL,NULL);CHKERRQ(ierr);
-  ierr = PetscMalloc1(npoints,&ppk);CHKERRQ(ierr);
+  PetscCall(BVGetSizes(Y,&nloc,NULL,NULL));
+  PetscCall(PetscMalloc1(npoints,&ppk));
   for (i=0;i<npoints;i++) ppk[i] = 1.0;
-  ierr = BVCreateVec(Y,&v);CHKERRQ(ierr);
+  PetscCall(BVCreateVec(Y,&v));
   for (k=0;k<M;k++) {
     for (j=0;j<L;j++) {
-      ierr = VecSet(v,0.0);CHKERRQ(ierr);
+      PetscCall(VecSet(v,0.0));
       for (i=0;i<npoints;i++) {
-        ierr = BVSetActiveColumns(Y,i*L_max+j,i*L_max+j+1);CHKERRQ(ierr);
-        ierr = BVMultVec(Y,ppk[i]*w[p_id(i)],1.0,v,&one);CHKERRQ(ierr);
+        PetscCall(BVSetActiveColumns(Y,i*L_max+j,i*L_max+j+1));
+        PetscCall(BVMultVec(Y,ppk[i]*w[p_id(i)],1.0,v,&one));
       }
-      if (useconj) {
-        ierr = VecGetArray(v,&pv);CHKERRQ(ierr);
+      if (PetscUnlikely(useconj)) {
+        PetscCall(VecGetArray(v,&pv));
         for (i=0;i<nloc;i++) pv[i] = 2.0*PetscRealPart(pv[i]);
-        ierr = VecRestoreArray(v,&pv);CHKERRQ(ierr);
+        PetscCall(VecRestoreArray(v,&pv));
       }
-      ierr = BVGetColumn(S,k*L+j,&sj);CHKERRQ(ierr);
-      if (scat) {
-        ierr = VecScatterBegin(scat,v,sj,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-        ierr = VecScatterEnd(scat,v,sj,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-      } else {
-        ierr = VecCopy(v,sj);CHKERRQ(ierr);
-      }
-      ierr = BVRestoreColumn(S,k*L+j,&sj);CHKERRQ(ierr);
+      PetscCall(BVGetColumn(S,k*L+j,&sj));
+      if (PetscUnlikely(scat)) {
+        PetscCall(VecScatterBegin(scat,v,sj,ADD_VALUES,SCATTER_REVERSE));
+        PetscCall(VecScatterEnd(scat,v,sj,ADD_VALUES,SCATTER_REVERSE));
+      } else PetscCall(VecCopy(v,sj));
+      PetscCall(BVRestoreColumn(S,k*L+j,&sj));
     }
     for (i=0;i<npoints;i++) ppk[i] *= zn[p_id(i)];
   }
-  ierr = PetscFree(ppk);CHKERRQ(ierr);
-  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  PetscCall(PetscFree(ppk));
+  PetscCall(VecDestroy(&v));
   PetscFunctionReturn(0);
 }
 
@@ -171,7 +167,7 @@ PetscErrorCode BVSumQuadrature(BV S,BV Y,PetscInt M,PetscInt L,PetscInt L_max,Pe
 
    Notes:
    This is a generalization of BVDot(). The resulting matrix Mu consists of M
-   blocks of size LxL (placed horizontally), each of them computed as:
+   blocks of size LxL (placed horizontally), each of them computed as
    Mu_k = sum_j w_j*zn_j^k*V'*Y_j, where Y_j is the j-th panel of Y containing
    the result of solving T(z_j)^{-1}*X for each integration point j. L_max is
    the width of the panels in Y.
@@ -188,25 +184,26 @@ PetscErrorCode BVSumQuadrature(BV S,BV Y,PetscInt M,PetscInt L,PetscInt L_max,Pe
 @*/
 PetscErrorCode BVDotQuadrature(BV Y,BV V,PetscScalar *Mu,PetscInt M,PetscInt L,PetscInt L_max,PetscScalar *w,PetscScalar *zn,PetscSubcomm subcomm,PetscInt npoints,PetscBool useconj)
 {
-  PetscErrorCode ierr;
   PetscMPIInt       sub_size,count;
   PetscInt          i,j,k,s;
   PetscScalar       *temp,*temp2,*ppk,alp;
   Mat               H;
   const PetscScalar *pH;
+  MPI_Comm          child,parent;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(Y,BV_CLASSID,1);
   PetscValidHeaderSpecific(V,BV_CLASSID,2);
 
-  ierr = MPI_Comm_size(PetscSubcommChild(subcomm),&sub_size);CHKERRMPI(ierr);
-  ierr = PetscMalloc3(npoints*L*(L+1),&temp,2*M*L*L,&temp2,npoints,&ppk);CHKERRQ(ierr);
-  ierr = MatCreateSeqDense(PETSC_COMM_SELF,L,L_max*npoints,NULL,&H);CHKERRQ(ierr);
-  ierr = PetscArrayzero(temp2,2*M*L*L);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(Y,0,L_max*npoints);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(V,0,L);CHKERRQ(ierr);
-  ierr = BVDot(Y,V,H);CHKERRQ(ierr);
-  ierr = MatDenseGetArrayRead(H,&pH);CHKERRQ(ierr);
+  PetscCall(PetscSubcommGetChild(subcomm,&child));
+  PetscCallMPI(MPI_Comm_size(child,&sub_size));
+  PetscCall(PetscMalloc3(npoints*L*(L+1),&temp,2*M*L*L,&temp2,npoints,&ppk));
+  PetscCall(MatCreateSeqDense(PETSC_COMM_SELF,L,L_max*npoints,NULL,&H));
+  PetscCall(PetscArrayzero(temp2,2*M*L*L));
+  PetscCall(BVSetActiveColumns(Y,0,L_max*npoints));
+  PetscCall(BVSetActiveColumns(V,0,L));
+  PetscCall(BVDot(Y,V,H));
+  PetscCall(MatDenseGetArrayRead(H,&pH));
   for (i=0;i<npoints;i++) {
     for (j=0;j<L;j++) {
       for (k=0;k<L;k++) {
@@ -214,25 +211,26 @@ PetscErrorCode BVDotQuadrature(BV Y,BV V,PetscScalar *Mu,PetscInt M,PetscInt L,P
       }
     }
   }
-  ierr = MatDenseRestoreArrayRead(H,&pH);CHKERRQ(ierr);
+  PetscCall(MatDenseRestoreArrayRead(H,&pH));
   for (i=0;i<npoints;i++) ppk[i] = 1;
   for (k=0;k<2*M;k++) {
     for (j=0;j<L;j++) {
       for (i=0;i<npoints;i++) {
         alp = ppk[i]*w[p_id(i)];
         for (s=0;s<L;s++) {
-          if (useconj) temp2[s+(j+k*L)*L] += 2.0*PetscRealPart(alp*temp[s+(j+i*L)*L]);
-          else temp2[s+(j+k*L)*L] += alp*temp[s+(j+i*L)*L];
+          if (!useconj) temp2[s+(j+k*L)*L] += alp*temp[s+(j+i*L)*L];
+          else temp2[s+(j+k*L)*L] += 2.0*PetscRealPart(alp*temp[s+(j+i*L)*L]);
         }
       }
     }
     for (i=0;i<npoints;i++) ppk[i] *= zn[p_id(i)];
   }
   for (i=0;i<2*M*L*L;i++) temp2[i] /= sub_size;
-  ierr = PetscMPIIntCast(2*M*L*L,&count);CHKERRQ(ierr);
-  ierr = MPIU_Allreduce(temp2,Mu,count,MPIU_SCALAR,MPIU_SUM,PetscSubcommParent(subcomm));CHKERRMPI(ierr);
-  ierr = PetscFree3(temp,temp2,ppk);CHKERRQ(ierr);
-  ierr = MatDestroy(&H);CHKERRQ(ierr);
+  PetscCall(PetscMPIIntCast(2*M*L*L,&count));
+  PetscCall(PetscSubcommGetParent(subcomm,&parent));
+  PetscCall(MPIU_Allreduce(temp2,Mu,count,MPIU_SCALAR,MPIU_SUM,parent));
+  PetscCall(PetscFree3(temp,temp2,ppk));
+  PetscCall(MatDestroy(&H));
   PetscFunctionReturn(0);
 }
 
@@ -274,7 +272,6 @@ PetscErrorCode BVDotQuadrature(BV Y,BV V,PetscScalar *Mu,PetscInt M,PetscInt L,P
 @*/
 PetscErrorCode BVTraceQuadrature(BV Y,BV V,PetscInt L,PetscInt L_max,PetscScalar *w,VecScatter scat,PetscSubcomm subcomm,PetscInt npoints,PetscBool useconj,PetscReal *est_eig)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j;
   Vec            y,yall,vj;
   PetscScalar    dot,sum=0.0,one=1.0;
@@ -284,35 +281,32 @@ PetscErrorCode BVTraceQuadrature(BV Y,BV V,PetscInt L,PetscInt L_max,PetscScalar
   PetscValidHeaderSpecific(V,BV_CLASSID,2);
   if (scat) PetscValidHeaderSpecific(scat,PETSCSF_CLASSID,6);
 
-  ierr = BVCreateVec(Y,&y);CHKERRQ(ierr);
-  ierr = BVCreateVec(V,&yall);CHKERRQ(ierr);
+  PetscCall(BVCreateVec(Y,&y));
+  PetscCall(BVCreateVec(V,&yall));
   for (j=0;j<L;j++) {
-    ierr = VecSet(y,0.0);CHKERRQ(ierr);
+    PetscCall(VecSet(y,0.0));
     for (i=0;i<npoints;i++) {
-      ierr = BVSetActiveColumns(Y,i*L_max+j,i*L_max+j+1);CHKERRQ(ierr);
-      ierr = BVMultVec(Y,w[p_id(i)],1.0,y,&one);CHKERRQ(ierr);
+      PetscCall(BVSetActiveColumns(Y,i*L_max+j,i*L_max+j+1));
+      PetscCall(BVMultVec(Y,w[p_id(i)],1.0,y,&one));
     }
-    ierr = BVGetColumn(V,j,&vj);CHKERRQ(ierr);
+    PetscCall(BVGetColumn(V,j,&vj));
     if (scat) {
-      ierr = VecScatterBegin(scat,y,yall,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-      ierr = VecScatterEnd(scat,y,yall,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-      ierr = VecDot(vj,yall,&dot);CHKERRQ(ierr);
-    } else {
-      ierr = VecDot(vj,y,&dot);CHKERRQ(ierr);
-    }
-    ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
+      PetscCall(VecScatterBegin(scat,y,yall,ADD_VALUES,SCATTER_REVERSE));
+      PetscCall(VecScatterEnd(scat,y,yall,ADD_VALUES,SCATTER_REVERSE));
+      PetscCall(VecDot(vj,yall,&dot));
+    } else PetscCall(VecDot(vj,y,&dot));
+    PetscCall(BVRestoreColumn(V,j,&vj));
     if (useconj) sum += 2.0*PetscRealPart(dot);
     else sum += dot;
   }
   *est_eig = PetscAbsScalar(sum)/(PetscReal)L;
-  ierr = VecDestroy(&y);CHKERRQ(ierr);
-  ierr = VecDestroy(&yall);CHKERRQ(ierr);
+  PetscCall(VecDestroy(&y));
+  PetscCall(VecDestroy(&yall));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscReal *sigma,PetscInt *rank)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,k,ml=S->k;
   PetscMPIInt    len;
   PetscScalar    *work,*B,*tempB,*sarray,*Q1,*Q2,*temp2,alpha=1.0,beta=0.0;
@@ -322,30 +316,30 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
 #endif
 
   PetscFunctionBegin;
-  ierr = BVGetArray(S,&sarray);CHKERRQ(ierr);
-  ierr = PetscMalloc6(ml*ml,&temp2,S->n*ml,&Q1,S->n*ml,&Q2,ml*ml,&B,ml*ml,&tempB,5*ml,&work);CHKERRQ(ierr);
+  PetscCall(BVGetArray(S,&sarray));
+  PetscCall(PetscMalloc6(ml*ml,&temp2,S->n*ml,&Q1,S->n*ml,&Q2,ml*ml,&B,ml*ml,&tempB,5*ml,&work));
 #if defined(PETSC_USE_COMPLEX)
-  ierr = PetscMalloc1(5*ml,&rwork);CHKERRQ(ierr);
+  PetscCall(PetscMalloc1(5*ml,&rwork));
 #endif
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 
-  ierr = PetscArrayzero(B,ml*ml);CHKERRQ(ierr);
+  PetscCall(PetscArrayzero(B,ml*ml));
   for (i=0;i<ml;i++) B[i*ml+i]=1;
 
   for (k=0;k<2;k++) {
-    ierr = PetscBLASIntCast(S->n,&m);CHKERRQ(ierr);
-    ierr = PetscBLASIntCast(ml,&l);CHKERRQ(ierr);
+    PetscCall(PetscBLASIntCast(S->n,&m));
+    PetscCall(PetscBLASIntCast(ml,&l));
     n = l; lda = m; ldb = m; ldc = l;
     if (!k) {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&l,&n,&m,&alpha,sarray,&lda,sarray,&ldb,&beta,pA,&ldc));
     } else {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&l,&n,&m,&alpha,Q1,&lda,Q1,&ldb,&beta,pA,&ldc));
     }
-    ierr = PetscArrayzero(temp2,ml*ml);CHKERRQ(ierr);
-    ierr = PetscMPIIntCast(ml*ml,&len);CHKERRQ(ierr);
-    ierr = MPIU_Allreduce(pA,temp2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)S));CHKERRMPI(ierr);
+    PetscCall(PetscArrayzero(temp2,ml*ml));
+    PetscCall(PetscMPIIntCast(ml*ml,&len));
+    PetscCall(MPIU_Allreduce(pA,temp2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)S)));
 
-    ierr = PetscBLASIntCast(ml,&m);CHKERRQ(ierr);
+    PetscCall(PetscBLASIntCast(ml,&m));
     n = m; lda = m; lwork = 5*m, ldu = 1; ldvt = 1;
 #if defined(PETSC_USE_COMPLEX)
     PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("O","N",&m,&n,temp2,&lda,sigma,NULL,&ldu,NULL,&ldvt,work,&lwork,rwork,&info));
@@ -354,8 +348,8 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
 #endif
     SlepcCheckLapackInfo("gesvd",info);
 
-    ierr = PetscBLASIntCast(S->n,&l);CHKERRQ(ierr);
-    ierr = PetscBLASIntCast(ml,&n);CHKERRQ(ierr);
+    PetscCall(PetscBLASIntCast(S->n,&l));
+    PetscCall(PetscBLASIntCast(ml,&n));
     m = n; lda = l; ldb = m; ldc = l;
     if (!k) {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&l,&n,&m,&alpha,sarray,&lda,temp2,&ldb,&beta,Q1,&ldc));
@@ -363,7 +357,7 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&l,&n,&m,&alpha,Q1,&lda,temp2,&ldb,&beta,Q2,&ldc));
     }
 
-    ierr = PetscBLASIntCast(ml,&l);CHKERRQ(ierr);
+    PetscCall(PetscBLASIntCast(ml,&l));
     m = l; n = l; lda = l; ldb = m; ldc = l;
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&l,&n,&m,&alpha,B,&lda,temp2,&ldb,&beta,tempB,&ldc));
     for (i=0;i<ml;i++) {
@@ -376,7 +370,7 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
     }
   }
 
-  ierr = PetscBLASIntCast(ml,&m);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ml,&m));
   n = m; lda = m; ldu=1; ldvt=1;
 #if defined(PETSC_USE_COMPLEX)
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("N","O",&m,&n,B,&lda,sigma,NULL,&ldu,NULL,&ldvt,work,&lwork,rwork,&info));
@@ -385,8 +379,8 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
 #endif
   SlepcCheckLapackInfo("gesvd",info);
 
-  ierr = PetscBLASIntCast(S->n,&l);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ml,&n);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(S->n,&l));
+  PetscCall(PetscBLASIntCast(ml,&n));
   m = n; lda = l; ldb = m; ldc = l;
   if (k%2) {
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&l,&n,&m,&alpha,Q1,&lda,B,&ldb,&beta,sarray,&ldc));
@@ -394,8 +388,8 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&l,&n,&m,&alpha,Q2,&lda,B,&ldb,&beta,sarray,&ldc));
   }
 
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = BVRestoreArray(S,&sarray);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
+  PetscCall(BVRestoreArray(S,&sarray));
 
   if (rank) {
     (*rank) = 0;
@@ -403,16 +397,15 @@ PetscErrorCode BVSVDAndRank_Refine(BV S,PetscReal delta,PetscScalar *pA,PetscRea
       if (sigma[i]/PetscMax(sigma[0],1.0)>delta) (*rank)++;
     }
   }
-  ierr = PetscFree6(temp2,Q1,Q2,B,tempB,work);CHKERRQ(ierr);
+  PetscCall(PetscFree6(temp2,Q1,Q2,B,tempB,work));
 #if defined(PETSC_USE_COMPLEX)
-  ierr = PetscFree(rwork);CHKERRQ(ierr);
+  PetscCall(PetscFree(rwork));
 #endif
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVSVDAndRank_QR(BV S,PetscReal delta,PetscScalar *pA,PetscReal *sigma,PetscInt *rank)
 {
-  PetscErrorCode ierr;
   PetscInt       i,n,ml=S->k;
   PetscBLASInt   m,lda,lwork,info;
   PetscScalar    *work;
@@ -422,46 +415,45 @@ PetscErrorCode BVSVDAndRank_QR(BV S,PetscReal delta,PetscScalar *pA,PetscReal *s
 
   PetscFunctionBegin;
   /* Compute QR factorizaton of S */
-  ierr = BVGetSizes(S,NULL,&n,NULL);CHKERRQ(ierr);
+  PetscCall(BVGetSizes(S,NULL,&n,NULL));
   n    = PetscMin(n,ml);
-  ierr = BVSetActiveColumns(S,0,n);CHKERRQ(ierr);
-  ierr = PetscArrayzero(pA,ml*n);CHKERRQ(ierr);
-  ierr = MatCreateDense(PETSC_COMM_SELF,n,n,PETSC_DECIDE,PETSC_DECIDE,pA,&A);CHKERRQ(ierr);
-  ierr = BVOrthogonalize(S,A);CHKERRQ(ierr);
+  PetscCall(BVSetActiveColumns(S,0,n));
+  PetscCall(PetscArrayzero(pA,ml*n));
+  PetscCall(MatCreateDense(PETSC_COMM_SELF,n,n,PETSC_DECIDE,PETSC_DECIDE,pA,&A));
+  PetscCall(BVOrthogonalize(S,A));
   if (n<ml) {
     /* the rest of the factorization */
     for (i=n;i<ml;i++) {
-      ierr = BVGetColumn(S,i,&v);CHKERRQ(ierr);
-      ierr = BVOrthogonalizeVec(S,v,pA+i*n,NULL,NULL);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(S,i,&v);CHKERRQ(ierr);
+      PetscCall(BVGetColumn(S,i,&v));
+      PetscCall(BVOrthogonalizeVec(S,v,pA+i*n,NULL,NULL));
+      PetscCall(BVRestoreColumn(S,i,&v));
     }
   }
-  ierr = PetscBLASIntCast(n,&lda);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ml,&m);CHKERRQ(ierr);
-  ierr = PetscMalloc2(5*ml,&work,5*ml,&rwork);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(n,&lda));
+  PetscCall(PetscBLASIntCast(ml,&m));
+  PetscCall(PetscMalloc2(5*ml,&work,5*ml,&rwork));
   lwork = 5*m;
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 #if !defined (PETSC_USE_COMPLEX)
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("O","N",&lda,&m,pA,&lda,sigma,NULL,&lda,NULL,&lda,work,&lwork,&info));
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("O","N",&lda,&m,pA,&lda,sigma,NULL,&lda,NULL,&lda,work,&lwork,rwork,&info));
 #endif
   SlepcCheckLapackInfo("gesvd",info);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
   *rank = 0;
   for (i=0;i<n;i++) {
     if (sigma[i]/PetscMax(sigma[0],1)>delta) (*rank)++;
   }
   /* n first columns of A have the left singular vectors */
-  ierr = BVMultInPlace(S,A,0,*rank);CHKERRQ(ierr);
-  ierr = PetscFree2(work,rwork);CHKERRQ(ierr);
-  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  PetscCall(BVMultInPlace(S,A,0,*rank));
+  PetscCall(PetscFree2(work,rwork));
+  PetscCall(MatDestroy(&A));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode BVSVDAndRank_QR_CAA(BV S,PetscInt M,PetscInt L,PetscReal delta,PetscScalar *pA,PetscReal *sigma,PetscInt *rank)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,n,ml=S->k;
   PetscBLASInt   m,k_,lda,lwork,info;
   PetscScalar    *work,*T,*U,*R,sone=1.0,zero=0.0;
@@ -470,46 +462,44 @@ PetscErrorCode BVSVDAndRank_QR_CAA(BV S,PetscInt M,PetscInt L,PetscReal delta,Pe
 
   PetscFunctionBegin;
   /* Compute QR factorizaton of S */
-  ierr = BVGetSizes(S,NULL,&n,NULL);CHKERRQ(ierr);
-  if (n<ml) SETERRQ(PetscObjectComm((PetscObject)S),PETSC_ERR_SUP,"The QR_CAA method does not support problem size n < m*L");
-  ierr = BVSetActiveColumns(S,0,ml);CHKERRQ(ierr);
-  ierr = PetscArrayzero(pA,ml*ml);CHKERRQ(ierr);
-  ierr = MatCreateDense(PETSC_COMM_SELF,ml,ml,PETSC_DECIDE,PETSC_DECIDE,pA,&A);CHKERRQ(ierr);
-  ierr = BVOrthogonalize(S,A);CHKERRQ(ierr);
-  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  PetscCall(BVGetSizes(S,NULL,&n,NULL));
+  PetscCheck(n>=ml,PetscObjectComm((PetscObject)S),PETSC_ERR_SUP,"The QR_CAA method does not support problem size n < m*L");
+  PetscCall(BVSetActiveColumns(S,0,ml));
+  PetscCall(PetscArrayzero(pA,ml*ml));
+  PetscCall(MatCreateDense(PETSC_COMM_SELF,ml,ml,PETSC_DECIDE,PETSC_DECIDE,pA,&A));
+  PetscCall(BVOrthogonalize(S,A));
+  PetscCall(MatDestroy(&A));
 
   /* SVD of first (M-1)*L diagonal block */
-  ierr = PetscBLASIntCast((M-1)*L,&m);CHKERRQ(ierr);
-  ierr = PetscMalloc5(m*m,&T,m*m,&R,m*m,&U,5*ml,&work,5*ml,&rwork);CHKERRQ(ierr);
-  for (j=0;j<m;j++) {
-    ierr = PetscArraycpy(R+j*m,pA+j*ml,m);CHKERRQ(ierr);
-  }
+  PetscCall(PetscBLASIntCast((M-1)*L,&m));
+  PetscCall(PetscMalloc5(m*m,&T,m*m,&R,m*m,&U,5*ml,&work,5*ml,&rwork));
+  for (j=0;j<m;j++) PetscCall(PetscArraycpy(R+j*m,pA+j*ml,m));
   lwork = 5*m;
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
 #if !defined (PETSC_USE_COMPLEX)
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","O",&m,&m,R,&m,sigma,U,&m,NULL,&m,work,&lwork,&info));
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","O",&m,&m,R,&m,sigma,U,&m,NULL,&m,work,&lwork,rwork,&info));
 #endif
   SlepcCheckLapackInfo("gesvd",info);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
+  PetscCall(PetscFPTrapPop());
   *rank = 0;
   for (i=0;i<m;i++) {
     if (sigma[i]/PetscMax(sigma[0],1)>delta) (*rank)++;
   }
-  ierr = MatCreateDense(PETSC_COMM_SELF,m,m,PETSC_DECIDE,PETSC_DECIDE,U,&A);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(S,0,m);CHKERRQ(ierr);
-  ierr = BVMultInPlace(S,A,0,*rank);CHKERRQ(ierr);
-  ierr = MatDestroy(&A);CHKERRQ(ierr);
+  PetscCall(MatCreateDense(PETSC_COMM_SELF,m,m,PETSC_DECIDE,PETSC_DECIDE,U,&A));
+  PetscCall(BVSetActiveColumns(S,0,m));
+  PetscCall(BVMultInPlace(S,A,0,*rank));
+  PetscCall(MatDestroy(&A));
   /* Projected linear system */
   /* m first columns of A have the right singular vectors */
-  ierr = PetscBLASIntCast(*rank,&k_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ml,&lda);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(*rank,&k_));
+  PetscCall(PetscBLASIntCast(ml,&lda));
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","C",&m,&k_,&m,&sone,pA+L*lda,&lda,R,&m,&zero,T,&m));
-  ierr = PetscArrayzero(pA,ml*ml);CHKERRQ(ierr);
+  PetscCall(PetscArrayzero(pA,ml*ml));
   PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&k_,&k_,&m,&sone,U,&m,T,&m,&zero,pA,&k_));
   for (j=0;j<k_;j++) for (i=0;i<k_;i++) pA[j*k_+i] /= sigma[j];
-  ierr = PetscFree5(T,R,U,work,rwork);CHKERRQ(ierr);
+  PetscCall(PetscFree5(T,R,U,work,rwork));
   PetscFunctionReturn(0);
 }
 
@@ -554,32 +544,70 @@ PetscErrorCode BVSVDAndRank_QR_CAA(BV S,PetscInt M,PetscInt L,PetscReal delta,Pe
 @*/
 PetscErrorCode BVSVDAndRank(BV S,PetscInt m,PetscInt l,PetscReal delta,BVSVDMethod meth,PetscScalar *A,PetscReal *sigma,PetscInt *rank)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(S,BV_CLASSID,1);
   PetscValidLogicalCollectiveInt(S,m,2);
   PetscValidLogicalCollectiveInt(S,l,3);
   PetscValidLogicalCollectiveReal(S,delta,4);
   PetscValidLogicalCollectiveEnum(S,meth,5);
-  PetscValidPointer(A,6);
-  PetscValidPointer(sigma,7);
-  PetscValidPointer(rank,8);
+  PetscValidScalarPointer(A,6);
+  PetscValidRealPointer(sigma,7);
+  PetscValidIntPointer(rank,8);
 
-  ierr = PetscLogEventBegin(BV_SVDAndRank,S,0,0,0);CHKERRQ(ierr);
-  ierr = BVSetActiveColumns(S,0,m*l);CHKERRQ(ierr);
+  PetscCall(PetscLogEventBegin(BV_SVDAndRank,S,0,0,0));
+  PetscCall(BVSetActiveColumns(S,0,m*l));
   switch (meth) {
     case BV_SVD_METHOD_REFINE:
-      ierr = BVSVDAndRank_Refine(S,delta,A,sigma,rank);CHKERRQ(ierr);
+      PetscCall(BVSVDAndRank_Refine(S,delta,A,sigma,rank));
       break;
     case BV_SVD_METHOD_QR:
-      ierr = BVSVDAndRank_QR(S,delta,A,sigma,rank);CHKERRQ(ierr);
+      PetscCall(BVSVDAndRank_QR(S,delta,A,sigma,rank));
       break;
     case BV_SVD_METHOD_QR_CAA:
-      ierr = BVSVDAndRank_QR_CAA(S,m,l,delta,A,sigma,rank);CHKERRQ(ierr);
+      PetscCall(BVSVDAndRank_QR_CAA(S,m,l,delta,A,sigma,rank));
       break;
   }
-  ierr = PetscLogEventEnd(BV_SVDAndRank,S,0,0,0);CHKERRQ(ierr);
+  PetscCall(PetscLogEventEnd(BV_SVDAndRank,S,0,0,0));
   PetscFunctionReturn(0);
 }
 
+/*@
+   BVCISSResizeBases - Resize the bases involved in CISS solvers when the L grows.
+
+   Collective on S
+
+   Input Parameters:
++  S      - basis of L*M columns
+.  V      - basis of L columns (may be associated to subcommunicators)
+.  Y      - basis of npoints*L columns
+.  Lold   - old value of L
+.  Lnew   - new value of L
+.  M      - the moment size
+-  npoints - number of integration points
+
+   Level: developer
+
+.seealso: BVResize()
+@*/
+PetscErrorCode BVCISSResizeBases(BV S,BV V,BV Y,PetscInt Lold,PetscInt Lnew,PetscInt M,PetscInt npoints)
+{
+  PetscInt       i,j;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(S,BV_CLASSID,1);
+  PetscValidHeaderSpecific(V,BV_CLASSID,2);
+  PetscValidHeaderSpecific(Y,BV_CLASSID,3);
+  PetscValidLogicalCollectiveInt(S,Lold,4);
+  PetscValidLogicalCollectiveInt(S,Lnew,5);
+  PetscValidLogicalCollectiveInt(S,M,6);
+  PetscValidLogicalCollectiveInt(S,npoints,7);
+
+  PetscCall(BVResize(S,Lnew*M,PETSC_FALSE));
+  PetscCall(BVResize(V,Lnew,PETSC_TRUE));
+  PetscCall(BVResize(Y,Lnew*npoints,PETSC_TRUE));
+  /* columns of Y are interleaved */
+  for (i=npoints-1;i>=0;i--) {
+    for (j=Lold-1;j>=0;j--) PetscCall(BVCopyColumn(Y,i*Lold+j,i*Lnew+j));
+  }
+  PetscFunctionReturn(0);
+}
