@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -13,23 +13,20 @@
 
 PetscErrorCode DSAllocate_GHIEP(DS ds,PetscInt ld)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  ierr = DSAllocateMat_Private(ds,DS_MAT_A);CHKERRQ(ierr);
-  ierr = DSAllocateMat_Private(ds,DS_MAT_B);CHKERRQ(ierr);
-  ierr = DSAllocateMat_Private(ds,DS_MAT_Q);CHKERRQ(ierr);
-  ierr = DSAllocateMatReal_Private(ds,DS_MAT_T);CHKERRQ(ierr);
-  ierr = DSAllocateMatReal_Private(ds,DS_MAT_D);CHKERRQ(ierr);
-  ierr = PetscFree(ds->perm);CHKERRQ(ierr);
-  ierr = PetscMalloc1(ld,&ds->perm);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)ds,ld*sizeof(PetscInt));CHKERRQ(ierr);
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_A));
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_B));
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_Q));
+  PetscCall(DSAllocateMatReal_Private(ds,DS_MAT_T));
+  PetscCall(DSAllocateMatReal_Private(ds,DS_MAT_D));
+  PetscCall(PetscFree(ds->perm));
+  PetscCall(PetscMalloc1(ld,&ds->perm));
+  PetscCall(PetscLogObjectMemory((PetscObject)ds,ld*sizeof(PetscInt)));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode DSSwitchFormat_GHIEP(DS ds,PetscBool tocompact)
 {
-  PetscErrorCode ierr;
   PetscReal      *T,*S;
   PetscScalar    *A,*B;
   PetscInt       i,n,ld;
@@ -42,8 +39,10 @@ PetscErrorCode DSSwitchFormat_GHIEP(DS ds,PetscBool tocompact)
   n = ds->n;
   ld = ds->ld;
   if (tocompact) { /* switch from dense (arrow) to compact storage */
-    ierr = PetscArrayzero(T,3*ld);CHKERRQ(ierr);
-    ierr = PetscArrayzero(S,ld);CHKERRQ(ierr);
+    PetscCall(PetscArrayzero(T,n));
+    PetscCall(PetscArrayzero(T+ld,n));
+    PetscCall(PetscArrayzero(T+2*ld,n));
+    PetscCall(PetscArrayzero(S,n));
     for (i=0;i<n-1;i++) {
       T[i]    = PetscRealPart(A[i+i*ld]);
       T[ld+i] = PetscRealPart(A[i+1+i*ld]);
@@ -53,8 +52,10 @@ PetscErrorCode DSSwitchFormat_GHIEP(DS ds,PetscBool tocompact)
     S[n-1] = PetscRealPart(B[n-1+(n-1)*ld]);
     for (i=ds->l;i<ds->k;i++) T[2*ld+i] = PetscRealPart(A[ds->k+i*ld]);
   } else { /* switch from compact (arrow) to dense storage */
-    ierr = PetscArrayzero(A,ld*ld);CHKERRQ(ierr);
-    ierr = PetscArrayzero(B,ld*ld);CHKERRQ(ierr);
+    for (i=0;i<n;i++) {
+      PetscCall(PetscArrayzero(A+i*ld,n));
+      PetscCall(PetscArrayzero(B+i*ld,n));
+    }
     for (i=0;i<n-1;i++) {
       A[i+i*ld]     = T[i];
       A[i+1+i*ld]   = T[ld+i];
@@ -73,7 +74,6 @@ PetscErrorCode DSSwitchFormat_GHIEP(DS ds,PetscBool tocompact)
 
 PetscErrorCode DSView_GHIEP(DS ds,PetscViewer viewer)
 {
-  PetscErrorCode    ierr;
   PetscViewerFormat format;
   PetscInt          i,j;
   PetscReal         value;
@@ -85,79 +85,72 @@ PetscErrorCode DSView_GHIEP(DS ds,PetscViewer viewer)
   const int         nmeth=sizeof(methodname)/sizeof(methodname[0]);
 
   PetscFunctionBegin;
-  ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+  PetscCall(PetscViewerGetFormat(viewer,&format));
   if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
-    if (ds->method<nmeth) {
-      ierr = PetscViewerASCIIPrintf(viewer,"solving the problem with: %s\n",methodname[ds->method]);CHKERRQ(ierr);
-    }
+    if (ds->method<nmeth) PetscCall(PetscViewerASCIIPrintf(viewer,"solving the problem with: %s\n",methodname[ds->method]));
     PetscFunctionReturn(0);
   }
   if (ds->compact) {
-    ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
     if (format == PETSC_VIEWER_ASCII_MATLAB) {
-      ierr = PetscViewerASCIIPrintf(viewer,"%% Size = %D %D\n",ds->n,ds->n);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"zzz = zeros(%D,3);\n",3*ds->n);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"zzz = [\n");CHKERRQ(ierr);
-      for (i=0;i<ds->n;i++) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+1,i+1,(double)*(ds->rmat[DS_MAT_T]+i));CHKERRQ(ierr);
-      }
+      PetscCall(PetscViewerASCIIPrintf(viewer,"%% Size = %" PetscInt_FMT " %" PetscInt_FMT "\n",ds->n,ds->n));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"zzz = zeros(%" PetscInt_FMT ",3);\n",3*ds->n));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"zzz = [\n"));
+      for (i=0;i<ds->n;i++) PetscCall(PetscViewerASCIIPrintf(viewer,"%" PetscInt_FMT " %" PetscInt_FMT "  %18.16e\n",i+1,i+1,(double)*(ds->rmat[DS_MAT_T]+i)));
       for (i=0;i<ds->n-1;i++) {
         if (*(ds->rmat[DS_MAT_T]+ds->ld+i) !=0 && i!=ds->k-1) {
-          ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+2,i+1,(double)*(ds->rmat[DS_MAT_T]+ds->ld+i));CHKERRQ(ierr);
-          ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+1,i+2,(double)*(ds->rmat[DS_MAT_T]+ds->ld+i));CHKERRQ(ierr);
+          PetscCall(PetscViewerASCIIPrintf(viewer,"%" PetscInt_FMT " %" PetscInt_FMT "  %18.16e\n",i+2,i+1,(double)*(ds->rmat[DS_MAT_T]+ds->ld+i)));
+          PetscCall(PetscViewerASCIIPrintf(viewer,"%" PetscInt_FMT " %" PetscInt_FMT "  %18.16e\n",i+1,i+2,(double)*(ds->rmat[DS_MAT_T]+ds->ld+i)));
         }
       }
       for (i = ds->l;i<ds->k;i++) {
         if (*(ds->rmat[DS_MAT_T]+2*ds->ld+i)) {
-          ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",ds->k+1,i+1,(double)*(ds->rmat[DS_MAT_T]+2*ds->ld+i));CHKERRQ(ierr);
-          ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+1,ds->k+1,(double)*(ds->rmat[DS_MAT_T]+2*ds->ld+i));CHKERRQ(ierr);
+          PetscCall(PetscViewerASCIIPrintf(viewer,"%" PetscInt_FMT " %" PetscInt_FMT "  %18.16e\n",ds->k+1,i+1,(double)*(ds->rmat[DS_MAT_T]+2*ds->ld+i)));
+          PetscCall(PetscViewerASCIIPrintf(viewer,"%" PetscInt_FMT " %" PetscInt_FMT "  %18.16e\n",i+1,ds->k+1,(double)*(ds->rmat[DS_MAT_T]+2*ds->ld+i)));
         }
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"];\n%s = spconvert(zzz);\n",DSMatName[DS_MAT_A]);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"];\n%s = spconvert(zzz);\n",DSMatName[DS_MAT_A]));
 
-      ierr = PetscViewerASCIIPrintf(viewer,"%% Size = %D %D\n",ds->n,ds->n);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"omega = zeros(%D,3);\n",3*ds->n);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"omega = [\n");CHKERRQ(ierr);
-      for (i=0;i<ds->n;i++) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%D %D  %18.16e\n",i+1,i+1,(double)*(ds->rmat[DS_MAT_D]+i));CHKERRQ(ierr);
-      }
-      ierr = PetscViewerASCIIPrintf(viewer,"];\n%s = spconvert(omega);\n",DSMatName[DS_MAT_B]);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"%% Size = %" PetscInt_FMT " %" PetscInt_FMT "\n",ds->n,ds->n));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"omega = zeros(%" PetscInt_FMT ",3);\n",3*ds->n));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"omega = [\n"));
+      for (i=0;i<ds->n;i++) PetscCall(PetscViewerASCIIPrintf(viewer,"%" PetscInt_FMT " %" PetscInt_FMT "  %18.16e\n",i+1,i+1,(double)*(ds->rmat[DS_MAT_D]+i)));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"];\n%s = spconvert(omega);\n",DSMatName[DS_MAT_B]));
 
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"T\n");CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"T\n"));
       for (i=0;i<ds->n;i++) {
         for (j=0;j<ds->n;j++) {
           if (i==j) value = *(ds->rmat[DS_MAT_T]+i);
           else if (i==j+1 || j==i+1) value = *(ds->rmat[DS_MAT_T]+ds->ld+PetscMin(i,j));
           else if ((i<ds->k && j==ds->k) || (i==ds->k && j<ds->k)) value = *(ds->rmat[DS_MAT_T]+2*ds->ld+PetscMin(i,j));
           else value = 0.0;
-          ierr = PetscViewerASCIIPrintf(viewer," %18.16e ",(double)value);CHKERRQ(ierr);
+          PetscCall(PetscViewerASCIIPrintf(viewer," %18.16e ",(double)value));
         }
-        ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+        PetscCall(PetscViewerASCIIPrintf(viewer,"\n"));
       }
-      ierr = PetscViewerASCIIPrintf(viewer,"omega\n");CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"omega\n"));
       for (i=0;i<ds->n;i++) {
         for (j=0;j<ds->n;j++) {
           if (i==j) value = *(ds->rmat[DS_MAT_D]+i);
           else value = 0.0;
-          ierr = PetscViewerASCIIPrintf(viewer," %18.16e ",(double)value);CHKERRQ(ierr);
+          PetscCall(PetscViewerASCIIPrintf(viewer," %18.16e ",(double)value));
         }
-        ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+        PetscCall(PetscViewerASCIIPrintf(viewer,"\n"));
       }
     }
-    ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
+    PetscCall(PetscViewerFlush(viewer));
   } else {
-    ierr = DSViewMat(ds,viewer,DS_MAT_A);CHKERRQ(ierr);
-    ierr = DSViewMat(ds,viewer,DS_MAT_B);CHKERRQ(ierr);
+    PetscCall(DSViewMat(ds,viewer,DS_MAT_A));
+    PetscCall(DSViewMat(ds,viewer,DS_MAT_B));
   }
-  if (ds->state>DS_STATE_INTERMEDIATE) { ierr = DSViewMat(ds,viewer,DS_MAT_Q);CHKERRQ(ierr); }
+  if (ds->state>DS_STATE_INTERMEDIATE) PetscCall(DSViewMat(ds,viewer,DS_MAT_Q));
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *rnorm)
 {
-  PetscErrorCode ierr;
   PetscReal      b[4],M[4],d1,d2,s1,s2,e;
   PetscReal      scal1,scal2,wr1,wr2,wi,ep,norm;
   PetscScalar    *Q,*X,Y[4],alpha,zeroS = 0.0;
@@ -171,15 +164,14 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
   X = ds->mat[DS_MAT_X];
   Q = ds->mat[DS_MAT_Q];
   k = *idx;
-  ierr = PetscBLASIntCast(ds->n,&n_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ds->n,&n_));
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
   if (k < ds->n-1) e = (ds->compact)?*(ds->rmat[DS_MAT_T]+ld+k):PetscRealPart(*(ds->mat[DS_MAT_A]+(k+1)+ld*k));
   else e = 0.0;
   if (e == 0.0) { /* Real */
-    if (ds->state>=DS_STATE_CONDENSED) {
-      ierr = PetscArraycpy(X+k*ld,Q+k*ld,ld);CHKERRQ(ierr);
-    } else {
-      ierr = PetscArrayzero(X+k*ds->ld,ds->ld);CHKERRQ(ierr);
+    if (ds->state>=DS_STATE_CONDENSED) PetscCall(PetscArraycpy(X+k*ld,Q+k*ld,ld));
+    else {
+      PetscCall(PetscArrayzero(X+k*ds->ld,ds->ld));
       X[k+k*ds->ld] = 1.0;
     }
     if (rnorm) *rnorm = PetscAbsScalar(X[ds->n-1+k*ld]);
@@ -200,9 +192,9 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
     ep = LAPACKlamch_("S");
     /* Compute eigenvalues of the block */
     PetscStackCallBLAS("LAPACKlag2",LAPACKlag2_(M,&two,b,&two,&ep,&scal1,&scal2,&wr1,&wr2,&wi));
-    if (wi==0.0) SETERRQ(PETSC_COMM_SELF,1,"Real block in DSVectors_GHIEP");
+    PetscCheck(wi!=0.0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"Real block in DSVectors_GHIEP");
     else { /* Complex eigenvalues */
-      if (scal1<ep) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
+      PetscCheck(scal1>=ep,PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
       wr1 /= scal1;
       wi  /= scal1;
 #if !defined(PETSC_USE_COMPLEX)
@@ -218,7 +210,7 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
         PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n_,&two,&two,&alpha,ds->mat[DS_MAT_Q]+k*ld,&ld,Y,&two,&zeroS,X+k*ld,&ld));
         if (rnorm) *rnorm = SlepcAbsEigenvalue(X[ds->n-1+k*ld],X[ds->n-1+(k+1)*ld]);
       } else {
-        ierr = PetscArrayzero(X+k*ld,2*ld);CHKERRQ(ierr);
+        PetscCall(PetscArrayzero(X+k*ld,2*ld));
         X[k*ld+k]       = Y[0]*norm;
         X[k*ld+k+1]     = Y[1]*norm;
         X[(k+1)*ld+k]   = Y[2]*norm;
@@ -239,7 +231,7 @@ static PetscErrorCode DSVectors_GHIEP_Eigen_Some(DS ds,PetscInt *idx,PetscReal *
         PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n_,&two,&alpha,ds->mat[DS_MAT_Q]+k*ld,&ld,Y,&one,&zeroS,X+k*ld,&one));
         if (rnorm) *rnorm = PetscAbsScalar(X[ds->n-1+k*ld]);
       } else {
-        ierr = PetscArrayzero(X+k*ld,2*ld);CHKERRQ(ierr);
+        PetscCall(PetscArrayzero(X+k*ld,2*ld));
         X[k*ld+k]   = Y[0]*norm;
         X[k*ld+k+1] = Y[1]*norm;
       }
@@ -256,34 +248,28 @@ PetscErrorCode DSVectors_GHIEP(DS ds,DSMatType mat,PetscInt *k,PetscReal *rnorm)
 {
   PetscInt       i;
   PetscReal      e;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   switch (mat) {
     case DS_MAT_X:
     case DS_MAT_Y:
-      if (k) {
-        ierr = DSVectors_GHIEP_Eigen_Some(ds,k,rnorm);CHKERRQ(ierr);
-      } else {
+      if (k) PetscCall(DSVectors_GHIEP_Eigen_Some(ds,k,rnorm));
+      else {
         for (i=0; i<ds->n; i++) {
           e = (ds->compact)?*(ds->rmat[DS_MAT_T]+ds->ld+i):PetscRealPart(*(ds->mat[DS_MAT_A]+(i+1)+ds->ld*i));
           if (e == 0.0) { /* real */
-            if (ds->state >= DS_STATE_CONDENSED) {
-              ierr = PetscArraycpy(ds->mat[mat]+i*ds->ld,ds->mat[DS_MAT_Q]+i*ds->ld,ds->ld);CHKERRQ(ierr);
-            } else {
-              ierr = PetscArrayzero(ds->mat[mat]+i*ds->ld,ds->ld);CHKERRQ(ierr);
+            if (ds->state >= DS_STATE_CONDENSED) PetscCall(PetscArraycpy(ds->mat[mat]+i*ds->ld,ds->mat[DS_MAT_Q]+i*ds->ld,ds->ld));
+            else {
+              PetscCall(PetscArrayzero(ds->mat[mat]+i*ds->ld,ds->ld));
               *(ds->mat[mat]+i+i*ds->ld) = 1.0;
             }
-          } else {
-            ierr = DSVectors_GHIEP_Eigen_Some(ds,&i,rnorm);CHKERRQ(ierr);
-          }
+          } else PetscCall(DSVectors_GHIEP_Eigen_Some(ds,&i,rnorm));
         }
       }
       break;
     case DS_MAT_U:
-    case DS_MAT_VT:
+    case DS_MAT_V:
       SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented yet");
-      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid mat parameter");
   }
@@ -334,9 +320,9 @@ PetscErrorCode DSGHIEPComplexEigs(DS ds,PetscInt n0,PetscInt n1,PetscScalar *wr,
       ep = LAPACKlamch_("S");
       /* Compute eigenvalues of the block */
       PetscStackCallBLAS("LAPACKlag2",LAPACKlag2_(M,&two,b,&two,&ep,&scal1,&scal2,&wr1,&wr2,&wi1));
-      if (scal1<ep) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
+      PetscCheck(scal1>=ep,PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
       if (wi1==0.0) { /* Real eigenvalues */
-        if (scal2<ep) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
+        PetscCheck(scal2>=ep,PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
         wr[k] = wr1/scal1; wr[k+1] = wr2/scal2;
 #if !defined(PETSC_USE_COMPLEX)
         wi[k] = wi[k+1] = 0.0;
@@ -365,7 +351,6 @@ PetscErrorCode DSGHIEPComplexEigs(DS ds,PetscInt n0,PetscInt n1,PetscScalar *wr,
 
 PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
-  PetscErrorCode ierr;
   PetscInt       n,i,*perm;
   PetscReal      *d,*e,*s;
 
@@ -377,31 +362,62 @@ PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *r
   d = ds->rmat[DS_MAT_T];
   e = d + ds->ld;
   s = ds->rmat[DS_MAT_D];
-  ierr = DSAllocateWork_Private(ds,ds->ld,ds->ld,0);CHKERRQ(ierr);
+  PetscCall(DSAllocateWork_Private(ds,ds->ld,ds->ld,0));
   perm = ds->perm;
   if (!rr) {
     rr = wr;
     ri = wi;
   }
-  ierr = DSSortEigenvalues_Private(ds,rr,ri,perm,PETSC_TRUE);CHKERRQ(ierr);
-  if (!ds->compact) { ierr = DSSwitchFormat_GHIEP(ds,PETSC_TRUE);CHKERRQ(ierr); }
-  ierr = PetscArraycpy(ds->work,wr,n);CHKERRQ(ierr);
+  PetscCall(DSSortEigenvalues_Private(ds,rr,ri,perm,PETSC_TRUE));
+  if (!ds->compact) PetscCall(DSSwitchFormat_GHIEP(ds,PETSC_TRUE));
+  PetscCall(PetscArraycpy(ds->work,wr,n));
   for (i=ds->l;i<n;i++) wr[i] = *(ds->work+perm[i]);
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscArraycpy(ds->work,wi,n);CHKERRQ(ierr);
+  PetscCall(PetscArraycpy(ds->work,wi,n));
   for (i=ds->l;i<n;i++) wi[i] = *(ds->work+perm[i]);
 #endif
-  ierr = PetscArraycpy(ds->rwork,s,n);CHKERRQ(ierr);
+  PetscCall(PetscArraycpy(ds->rwork,s,n));
   for (i=ds->l;i<n;i++) s[i] = *(ds->rwork+perm[i]);
-  ierr = PetscArraycpy(ds->rwork,d,n);CHKERRQ(ierr);
+  PetscCall(PetscArraycpy(ds->rwork,d,n));
   for (i=ds->l;i<n;i++) d[i] = *(ds->rwork+perm[i]);
-  ierr = PetscArraycpy(ds->rwork,e,n-1);CHKERRQ(ierr);
-  ierr = PetscArrayzero(e+ds->l,n-1-ds->l);CHKERRQ(ierr);
+  PetscCall(PetscArraycpy(ds->rwork,e,n-1));
+  PetscCall(PetscArrayzero(e+ds->l,n-1-ds->l));
   for (i=ds->l;i<n-1;i++) {
     if (perm[i]<n-1) e[i] = *(ds->rwork+perm[i]);
   }
-  if (!ds->compact) { ierr = DSSwitchFormat_GHIEP(ds,PETSC_FALSE);CHKERRQ(ierr); }
-  ierr = DSPermuteColumns_Private(ds,ds->l,n,DS_MAT_Q,perm);CHKERRQ(ierr);
+  if (!ds->compact) PetscCall(DSSwitchFormat_GHIEP(ds,PETSC_FALSE));
+  PetscCall(DSPermuteColumns_Private(ds,ds->l,n,n,DS_MAT_Q,perm));
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode DSUpdateExtraRow_GHIEP(DS ds)
+{
+  PetscInt       i;
+  PetscBLASInt   n,ld,incx=1;
+  PetscScalar    *A,*Q,*x,*y,one=1.0,zero=0.0;
+  PetscReal      *b,*r,beta;
+
+  PetscFunctionBegin;
+  PetscCall(PetscBLASIntCast(ds->n,&n));
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
+  A  = ds->mat[DS_MAT_A];
+  Q  = ds->mat[DS_MAT_Q];
+  b  = ds->rmat[DS_MAT_T]+ld;
+  r  = ds->rmat[DS_MAT_T]+2*ld;
+
+  if (ds->compact) {
+    beta = b[n-1];   /* in compact, we assume all entries are zero except the last one */
+    for (i=0;i<n;i++) r[i] = PetscRealPart(beta*Q[n-1+i*ld]);
+    ds->k = n;
+  } else {
+    PetscCall(DSAllocateWork_Private(ds,2*ld,0,0));
+    x = ds->work;
+    y = ds->work+ld;
+    for (i=0;i<n;i++) x[i] = PetscConj(A[n+i*ld]);
+    PetscStackCallBLAS("BLASgemv",BLASgemv_("C",&n,&n,&one,Q,&ld,x,&incx,&zero,y,&incx));
+    for (i=0;i<n;i++) A[n+i*ld] = PetscConj(y[i]);
+    ds->k = n;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -411,7 +427,6 @@ PetscErrorCode DSSort_GHIEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *r
 */
 PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-  PetscErrorCode ierr;
   PetscInt       i,off;
   PetscBLASInt   *select,*infoC,ld,n1,mout,info;
   PetscScalar    *A,*B,*H,*X;
@@ -421,10 +436,10 @@ PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->n-ds->l,&n1);CHKERRQ(ierr);
-  ierr = DSAllocateWork_Private(ds,ld*ld+2*ld,ld,2*ld);CHKERRQ(ierr);
-  ierr = DSAllocateMat_Private(ds,DS_MAT_W);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
+  PetscCall(PetscBLASIntCast(ds->n-ds->l,&n1));
+  PetscCall(DSAllocateWork_Private(ds,ld*ld+2*ld,ld,2*ld));
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_W));
   A = ds->mat[DS_MAT_A];
   B = ds->mat[DS_MAT_B];
   H = ds->mat[DS_MAT_W];
@@ -458,7 +473,7 @@ PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
     H[ds->n-1+(ds->n-2)*ld] = A[ds->n-1+(ds->n-2)*ld]*s[ds->n-1];
     H[ds->n-1+(ds->n-1)*ld] = A[ds->n-1+(ds->n-1)*ld]*s[ds->n-1];
   }
-  ierr = DSAllocateMat_Private(ds,DS_MAT_X);CHKERRQ(ierr);
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_X));
   X = ds->mat[DS_MAT_X];
   for (i=0;i<n1;i++) select[i] = 1;
 #if !defined(PETSC_USE_COMPLEX)
@@ -478,7 +493,7 @@ PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
   }
 #endif
   SlepcCheckLapackInfo("hsein",info);
-  ierr = DSGHIEPOrthogEigenv(ds,DS_MAT_X,wr,wi,PETSC_TRUE);CHKERRQ(ierr);
+  PetscCall(DSGHIEPOrthogEigenv(ds,DS_MAT_X,wr,wi,PETSC_TRUE));
   PetscFunctionReturn(0);
 }
 
@@ -487,7 +502,6 @@ PetscErrorCode DSGHIEPInverseIteration(DS ds,PetscScalar *wr,PetscScalar *wi)
 */
 PetscErrorCode DSGHIEPRealBlocks(DS ds)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
   PetscReal      e,d1,d2,s1,s2,ss1,ss2,t,dd,ss;
   PetscReal      maxy,ep,scal1,scal2,snorm;
@@ -497,13 +511,13 @@ PetscErrorCode DSGHIEPRealBlocks(DS ds)
   PetscBool      isreal;
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->n-ds->l,&m);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
+  PetscCall(PetscBLASIntCast(ds->n-ds->l,&m));
   A = ds->mat[DS_MAT_A];
   B = ds->mat[DS_MAT_B];
   T = ds->rmat[DS_MAT_T];
   D = ds->rmat[DS_MAT_D];
-  ierr = DSAllocateWork_Private(ds,2*m,0,0);CHKERRQ(ierr);
+  PetscCall(DSAllocateWork_Private(ds,2*m,0,0));
   for (i=ds->l;i<ds->n-1;i++) {
     e = (ds->compact)?T[ld+i]:PetscRealPart(A[(i+1)+ld*i]);
     if (e != 0.0) { /* 2x2 block */
@@ -544,7 +558,7 @@ PetscErrorCode DSGHIEPRealBlocks(DS ds)
         PetscStackCallBLAS("LAPACKlag2",LAPACKlag2_(M,&two,b,&two,&ep,&scal1,&scal2,&wr1,&wr2,&wi));
         if (wi==0.0) { /* Real eigenvalues */
           isreal = PETSC_TRUE;
-          if (scal1<ep||scal2<ep) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
+          PetscCheck(scal1>=ep && scal2>=ep,PETSC_COMM_SELF,PETSC_ERR_FP,"Nearly infinite eigenvalue");
           wr1 /= scal1;
           wr2 /= scal2;
           if (PetscAbsReal(s1*d1-wr1)<PetscAbsReal(s2*d2-wr1)) {
@@ -595,8 +609,8 @@ PetscErrorCode DSGHIEPRealBlocks(DS ds)
           A[i+ld*(i+1)]   = 0.0;
         }
         PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&m,&two,&two,&oneS,ds->mat[DS_MAT_Q]+ds->l+i*ld,&ld,Y,&two,&zeroS,ds->work,&m));
-        ierr = PetscArraycpy(ds->mat[DS_MAT_Q]+ds->l+i*ld,ds->work,m);CHKERRQ(ierr);
-        ierr = PetscArraycpy(ds->mat[DS_MAT_Q]+ds->l+(i+1)*ld,ds->work+m,m);CHKERRQ(ierr);
+        PetscCall(PetscArraycpy(ds->mat[DS_MAT_Q]+ds->l+i*ld,ds->work,m));
+        PetscCall(PetscArraycpy(ds->mat[DS_MAT_Q]+ds->l+(i+1)*ld,ds->work+m,m));
       }
       i++;
     }
@@ -606,7 +620,6 @@ PetscErrorCode DSGHIEPRealBlocks(DS ds)
 
 PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-  PetscErrorCode ierr;
   PetscInt       i,off;
   PetscBLASInt   n1,ld,one,info,lwork;
   PetscScalar    *H,*A,*B,*Q;
@@ -617,11 +630,11 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(wi,3);
+  PetscValidScalarPointer(wi,3);
 #endif
   one = 1;
-  ierr = PetscBLASIntCast(ds->n-ds->l,&n1);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ds->n-ds->l,&n1));
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
   off = ds->l + ds->l*ld;
   A = ds->mat[DS_MAT_A];
   B = ds->mat[DS_MAT_B];
@@ -629,13 +642,20 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
   d = ds->rmat[DS_MAT_T];
   e = ds->rmat[DS_MAT_T] + ld;
   s = ds->rmat[DS_MAT_D];
-  ierr = DSAllocateWork_Private(ds,ld*ld,2*ld,ld*2);CHKERRQ(ierr);
+#if defined(PETSC_USE_DEBUG)
+  /* Check signature */
+  for (i=0;i<ds->n;i++) {
+    PetscReal de = (ds->compact)?s[i]:PetscRealPart(B[i*ld+i]);
+    PetscCheck(de==1.0 || de==-1.0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"Diagonal elements of the signature matrix must be 1 or -1");
+  }
+#endif
+  PetscCall(DSAllocateWork_Private(ds,ld*ld,2*ld,ld*2));
   lwork = ld*ld;
 
   /* Quick return if possible */
   if (n1 == 1) {
     for (i=0;i<=ds->l;i++) Q[i+i*ld] = 1.0;
-    ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
+    PetscCall(DSGHIEPComplexEigs(ds,0,ds->l,wr,wi));
     if (!ds->compact) {
       d[ds->l] = PetscRealPart(A[off]);
       s[ds->l] = PetscRealPart(B[off]);
@@ -645,10 +665,10 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
     PetscFunctionReturn(0);
   }
   /* Reduce to pseudotriadiagonal form */
-  ierr = DSIntermediate_GHIEP(ds);CHKERRQ(ierr);
+  PetscCall(DSIntermediate_GHIEP(ds));
 
-  /* Compute Eigenvalues (QR)*/
-  ierr = DSAllocateMat_Private(ds,DS_MAT_W);CHKERRQ(ierr);
+  /* Compute Eigenvalues (QR) */
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_W));
   H = ds->mat[DS_MAT_W];
   if (ds->compact) {
     H[off]    = d[ds->l]*s[ds->l];
@@ -685,8 +705,8 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
       j=i+1;
       while (j<ds->n && (PetscAbsScalar(wr[i]-PetscConj(wr[j]))>PetscAbsScalar(wr[i])*PETSC_SQRT_MACHINE_EPSILON)) j++;
       if (j==ds->n) {
-        if (PetscAbsReal(PetscImaginaryPart(wr[i]))<PetscAbsScalar(wr[i])*PETSC_SQRT_MACHINE_EPSILON) wr[i]=PetscRealPart(wr[i]);
-        else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"Found complex without conjugate pair");
+        PetscCheck(PetscAbsReal(PetscImaginaryPart(wr[i]))<PetscAbsScalar(wr[i])*PETSC_SQRT_MACHINE_EPSILON,PETSC_COMM_SELF,PETSC_ERR_LIB,"Found complex without conjugate pair");
+        wr[i]=PetscRealPart(wr[i]);
       } else { /* complex eigenvalue */
         wr[j] = wr[i+1];
         if (PetscImaginaryPart(wr[i])<0) wr[i] = PetscConj(wr[i]);
@@ -697,10 +717,10 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
   SlepcCheckLapackInfo("hseqr",info);
   /* Compute Eigenvectors with Inverse Iteration */
-  ierr = DSGHIEPInverseIteration(ds,wr,wi);CHKERRQ(ierr);
+  PetscCall(DSGHIEPInverseIteration(ds,wr,wi));
 
   /* Recover eigenvalues from diagonal */
-  ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
+  PetscCall(DSGHIEPComplexEigs(ds,0,ds->l,wr,wi));
 #if defined(PETSC_USE_COMPLEX)
   if (wi) {
     for (i=ds->l;i<ds->n;i++) wi[i] = 0.0;
@@ -711,7 +731,6 @@ PetscErrorCode DSSolve_GHIEP_QR_II(DS ds,PetscScalar *wr,PetscScalar *wi)
 
 PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,off,nwu=0,n,lw,lwr,nwru=0;
   PetscBLASInt   n_,ld,info,lwork,ilo,ihi;
   PetscScalar    *H,*A,*B,*Q,*X;
@@ -722,30 +741,36 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(wi,3);
+  PetscValidScalarPointer(wi,3);
 #endif
   n = ds->n-ds->l;
-  ierr = PetscBLASIntCast(n,&n_);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(n,&n_));
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
   off = ds->l + ds->l*ld;
   A = ds->mat[DS_MAT_A];
   B = ds->mat[DS_MAT_B];
   Q = ds->mat[DS_MAT_Q];
   d = ds->rmat[DS_MAT_T];
   s = ds->rmat[DS_MAT_D];
+#if defined(PETSC_USE_DEBUG)
+  /* Check signature */
+  for (i=0;i<ds->n;i++) {
+    PetscReal de = (ds->compact)?s[i]:PetscRealPart(B[i*ld+i]);
+    PetscCheck(de==1.0 || de==-1.0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"Diagonal elements of the signature matrix must be 1 or -1");
+  }
+#endif
   lw = 14*ld+ld*ld;
   lwr = 7*ld;
-  ierr = DSAllocateWork_Private(ds,lw,lwr,0);CHKERRQ(ierr);
+  PetscCall(DSAllocateWork_Private(ds,lw,lwr,0));
   scale = ds->rwork+nwru;
   nwru += ld;
   rcde = ds->rwork+nwru;
   nwru += ld;
   rcdv = ds->rwork+nwru;
-  nwru += ld;
   /* Quick return if possible */
   if (n_ == 1) {
     for (i=0;i<=ds->l;i++) Q[i+i*ld] = 1.0;
-    ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
+    PetscCall(DSGHIEPComplexEigs(ds,0,ds->l,wr,wi));
     if (!ds->compact) {
       d[ds->l] = PetscRealPart(A[off]);
       s[ds->l] = PetscRealPart(B[off]);
@@ -758,7 +783,7 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   /* Form pseudo-symmetric matrix */
   H =  ds->work+nwu;
   nwu += n*n;
-  ierr = PetscArrayzero(H,n*n);CHKERRQ(ierr);
+  PetscCall(PetscArrayzero(H,n*n));
   if (ds->compact) {
     for (i=0;i<n-1;i++) {
       H[i+i*n]     = s[ds->l+i]*d[ds->l+i];
@@ -777,8 +802,8 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   }
 
   /* Compute eigenpairs */
-  ierr = PetscBLASIntCast(lw-nwu,&lwork);CHKERRQ(ierr);
-  ierr = DSAllocateMat_Private(ds,DS_MAT_X);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(lw-nwu,&lwork));
+  PetscCall(DSAllocateMat_Private(ds,DS_MAT_X));
   X = ds->mat[DS_MAT_X];
 #if !defined(PETSC_USE_COMPLEX)
   PetscStackCallBLAS("LAPACKgeevx",LAPACKgeevx_("B","N","V","N",&n_,H,&n_,wr+ds->l,wi+ds->l,NULL,&ld,X+off,&ld,&ilo,&ihi,scale,&nrm,rcde,rcdv,ds->work+nwu,&lwork,NULL,&info));
@@ -791,16 +816,15 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
     j=i+1;
     while (j<ds->n && (PetscAbsScalar(wr[i]-PetscConj(wr[j]))>PetscAbsScalar(wr[i])*PETSC_SQRT_MACHINE_EPSILON)) j++;
     if (j==ds->n) {
-      if (PetscAbsReal(PetscImaginaryPart(wr[i]))<PetscAbsScalar(wr[i])*PETSC_SQRT_MACHINE_EPSILON) {
-        wr[i]=PetscRealPart(wr[i]); /* real eigenvalue */
-        for (k=ds->l;k<ds->n;k++) {
-          X[k+i*ds->ld] = PetscRealPart(X[k+i*ds->ld]);
-        }
-      } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"Found complex without conjugate pair");
+      PetscCheck(PetscAbsReal(PetscImaginaryPart(wr[i]))<PetscAbsScalar(wr[i])*PETSC_SQRT_MACHINE_EPSILON,PETSC_COMM_SELF,PETSC_ERR_LIB,"Found complex without conjugate pair");
+      wr[i]=PetscRealPart(wr[i]); /* real eigenvalue */
+      for (k=ds->l;k<ds->n;k++) {
+        X[k+i*ds->ld] = PetscRealPart(X[k+i*ds->ld]);
+      }
     } else { /* complex eigenvalue */
       if (j!=i+1) {
         wr[j] = wr[i+1];
-        ierr = PetscArraycpy(X+j*ds->ld,X+(i+1)*ds->ld,ds->ld);CHKERRQ(ierr);
+        PetscCall(PetscArraycpy(X+j*ds->ld,X+(i+1)*ds->ld,ds->ld));
       }
       if (PetscImaginaryPart(wr[i])<0) {
         wr[i] = PetscConj(wr[i]);
@@ -822,10 +846,10 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   SlepcCheckLapackInfo("geevx",info);
 
   /* Compute real s-orthonormal basis */
-  ierr = DSGHIEPOrthogEigenv(ds,DS_MAT_X,wr,wi,PETSC_FALSE);CHKERRQ(ierr);
+  PetscCall(DSGHIEPOrthogEigenv(ds,DS_MAT_X,wr,wi,PETSC_FALSE));
 
   /* Recover eigenvalues from diagonal */
-  ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
+  PetscCall(DSGHIEPComplexEigs(ds,0,ds->l,wr,wi));
 #if defined(PETSC_USE_COMPLEX)
   if (wi) {
     for (i=ds->l;i<ds->n;i++) wi[i] = 0.0;
@@ -834,9 +858,60 @@ PetscErrorCode DSSolve_GHIEP_QR(DS ds,PetscScalar *wr,PetscScalar *wi)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode DSGetTruncateSize_GHIEP(DS ds,PetscInt l,PetscInt n,PetscInt *k)
+{
+  PetscReal *T = ds->rmat[DS_MAT_T];
+
+  PetscFunctionBegin;
+  if (T[l+(*k)-1+ds->ld] !=0.0) {
+    if (l+(*k)<n-1) (*k)++;
+    else (*k)--;
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode DSTruncate_GHIEP(DS ds,PetscInt n,PetscBool trim)
+{
+  PetscInt    i,ld=ds->ld,l=ds->l;
+  PetscScalar *A = ds->mat[DS_MAT_A];
+  PetscReal   *T = ds->rmat[DS_MAT_T],*b,*r,*omega;
+
+  PetscFunctionBegin;
+#if defined(PETSC_USE_DEBUG)
+  /* make sure diagonal 2x2 block is not broken */
+  PetscCheck(ds->state<DS_STATE_CONDENSED || n==0 || n==ds->n || T[n-1+ld]==0.0,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"The given size would break a 2x2 block, call DSGetTruncateSize() first");
+#endif
+  if (trim) {
+    if (!ds->compact && ds->extrarow) {   /* clean extra row */
+      for (i=l;i<ds->n;i++) A[ds->n+i*ld] = 0.0;
+    }
+    ds->l = 0;
+    ds->k = 0;
+    ds->n = n;
+    ds->t = ds->n;   /* truncated length equal to the new dimension */
+  } else {
+    if (!ds->compact && ds->extrarow && ds->k==ds->n) {
+      /* copy entries of extra row to the new position, then clean last row */
+      for (i=l;i<n;i++) A[n+i*ld] = A[ds->n+i*ld];
+      for (i=l;i<ds->n;i++) A[ds->n+i*ld] = 0.0;
+    }
+    if (ds->compact) {
+      b = T+ld;
+      r = T+2*ld;
+      omega = ds->rmat[DS_MAT_D];
+      b[n-1] = r[n-1];
+      b[n] = b[ds->n];
+      omega[n] = omega[ds->n];
+    }
+    ds->k = (ds->extrarow)? n: 0;
+    ds->t = ds->n;   /* truncated length equal to previous dimension */
+    ds->n = n;
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode DSSynchronize_GHIEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 {
-  PetscErrorCode ierr;
   PetscInt       ld=ds->ld,l=ds->l,k=0,kr=0;
   PetscMPIInt    n,rank,off=0,size,ldn,ld3,ld_;
 
@@ -846,49 +921,41 @@ PetscErrorCode DSSynchronize_GHIEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
   if (ds->state>DS_STATE_RAW) k += (ds->n-l)*ld;
   if (eigr) k += (ds->n-l);
   if (eigi) k += (ds->n-l);
-  ierr = DSAllocateWork_Private(ds,k+kr,0,0);CHKERRQ(ierr);
-  ierr = PetscMPIIntCast(k*sizeof(PetscScalar)+kr*sizeof(PetscReal),&size);CHKERRQ(ierr);
-  ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
-  ierr = PetscMPIIntCast(ld*(ds->n-l),&ldn);CHKERRQ(ierr);
-  ierr = PetscMPIIntCast(ld*3,&ld3);CHKERRQ(ierr);
-  ierr = PetscMPIIntCast(ld,&ld_);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ds),&rank);CHKERRQ(ierr);
+  PetscCall(DSAllocateWork_Private(ds,k+kr,0,0));
+  PetscCall(PetscMPIIntCast(k*sizeof(PetscScalar)+kr*sizeof(PetscReal),&size));
+  PetscCall(PetscMPIIntCast(ds->n-l,&n));
+  PetscCall(PetscMPIIntCast(ld*(ds->n-l),&ldn));
+  PetscCall(PetscMPIIntCast(ld*3,&ld3));
+  PetscCall(PetscMPIIntCast(ld,&ld_));
+  PetscCallMPI(MPI_Comm_rank(PetscObjectComm((PetscObject)ds),&rank));
   if (!rank) {
     if (ds->compact) {
-      ierr = MPI_Pack(ds->rmat[DS_MAT_T],ld3,MPIU_REAL,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-      ierr = MPI_Pack(ds->rmat[DS_MAT_D],ld_,MPIU_REAL,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      PetscCallMPI(MPI_Pack(ds->rmat[DS_MAT_T],ld3,MPIU_REAL,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
+      PetscCallMPI(MPI_Pack(ds->rmat[DS_MAT_D],ld_,MPIU_REAL,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
     } else {
-      ierr = MPI_Pack(ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-      ierr = MPI_Pack(ds->mat[DS_MAT_B]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
-    if (ds->state>DS_STATE_RAW) {
-      ierr = MPI_Pack(ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
-    if (eigr) {
-      ierr = MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
-    if (eigi) {
-      ierr = MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      PetscCallMPI(MPI_Pack(ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
+      PetscCallMPI(MPI_Pack(ds->mat[DS_MAT_B]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
     }
+    if (ds->state>DS_STATE_RAW) PetscCallMPI(MPI_Pack(ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
+    if (eigr) PetscCallMPI(MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
+#if !defined(PETSC_USE_COMPLEX)
+    if (eigi) PetscCallMPI(MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds)));
+#endif
   }
-  ierr = MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+  PetscCallMPI(MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds)));
   if (rank) {
     if (ds->compact) {
-      ierr = MPI_Unpack(ds->work,size,&off,ds->rmat[DS_MAT_T],ld3,MPIU_REAL,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-      ierr = MPI_Unpack(ds->work,size,&off,ds->rmat[DS_MAT_D],ld_,MPIU_REAL,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      PetscCallMPI(MPI_Unpack(ds->work,size,&off,ds->rmat[DS_MAT_T],ld3,MPIU_REAL,PetscObjectComm((PetscObject)ds)));
+      PetscCallMPI(MPI_Unpack(ds->work,size,&off,ds->rmat[DS_MAT_D],ld_,MPIU_REAL,PetscObjectComm((PetscObject)ds)));
     } else {
-      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_B]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
-    if (ds->state>DS_STATE_RAW) {
-      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
-    if (eigr) {
-      ierr = MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
-    }
-    if (eigi) {
-      ierr = MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      PetscCallMPI(MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds)));
+      PetscCallMPI(MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_B]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds)));
     }
+    if (ds->state>DS_STATE_RAW) PetscCallMPI(MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds)));
+    if (eigr) PetscCallMPI(MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds)));
+#if !defined(PETSC_USE_COMPLEX)
+    if (eigi) PetscCallMPI(MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds)));
+#endif
   }
   PetscFunctionReturn(0);
 }
@@ -901,18 +968,57 @@ PetscErrorCode DSHermitian_GHIEP(DS ds,DSMatType m,PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
+/*MC
+   DSGHIEP - Dense Generalized Hermitian Indefinite Eigenvalue Problem.
+
+   Level: beginner
+
+   Notes:
+   The problem is expressed as A*X = B*X*Lambda, where both A and B are
+   real symmetric (or complex Hermitian) and possibly indefinite. Lambda
+   is a diagonal matrix whose diagonal elements are the arguments of DSSolve().
+   After solve, A is overwritten with Lambda. Note that in the case of real
+   scalars, A is overwritten with a real representation of Lambda, i.e.,
+   complex conjugate eigenvalue pairs are stored as a 2x2 block in the
+   quasi-diagonal matrix.
+
+   In the intermediate state A is reduced to tridiagonal form and B is
+   transformed into a signature matrix. In compact storage format, these
+   matrices are stored in T and D, respectively.
+
+   Used DS matrices:
++  DS_MAT_A - first problem matrix
+.  DS_MAT_B - second problem matrix
+.  DS_MAT_T - symmetric tridiagonal matrix of the reduced pencil
+.  DS_MAT_D - diagonal matrix (signature) of the reduced pencil
+-  DS_MAT_Q - pseudo-orthogonal transformation that reduces (A,B) to
+   tridiagonal-diagonal form (intermediate step) or a real basis of eigenvectors
+
+   Implemented methods:
++  0 - QR iteration plus inverse iteration for the eigenvectors
+.  1 - HZ iteration
+-  2 - QR iteration plus pseudo-orthogonalization for the eigenvectors
+
+   References:
+.  1. - C. Campos and J. E. Roman, "Restarted Q-Arnoldi-type methods exploiting
+   symmetry in quadratic eigenvalue problems", BIT Numer. Math. 56(4):1213-1236, 2016.
+
+.seealso: DSCreate(), DSSetType(), DSType
+M*/
 SLEPC_EXTERN PetscErrorCode DSCreate_GHIEP(DS ds)
 {
   PetscFunctionBegin;
-  ds->ops->allocate      = DSAllocate_GHIEP;
-  ds->ops->view          = DSView_GHIEP;
-  ds->ops->vectors       = DSVectors_GHIEP;
-  ds->ops->solve[0]      = DSSolve_GHIEP_QR_II;
-  ds->ops->solve[1]      = DSSolve_GHIEP_HZ;
-  ds->ops->solve[2]      = DSSolve_GHIEP_QR;
-  ds->ops->sort          = DSSort_GHIEP;
-  ds->ops->synchronize   = DSSynchronize_GHIEP;
-  ds->ops->hermitian     = DSHermitian_GHIEP;
+  ds->ops->allocate        = DSAllocate_GHIEP;
+  ds->ops->view            = DSView_GHIEP;
+  ds->ops->vectors         = DSVectors_GHIEP;
+  ds->ops->solve[0]        = DSSolve_GHIEP_QR_II;
+  ds->ops->solve[1]        = DSSolve_GHIEP_HZ;
+  ds->ops->solve[2]        = DSSolve_GHIEP_QR;
+  ds->ops->sort            = DSSort_GHIEP;
+  ds->ops->synchronize     = DSSynchronize_GHIEP;
+  ds->ops->gettruncatesize = DSGetTruncateSize_GHIEP;
+  ds->ops->truncate        = DSTruncate_GHIEP;
+  ds->ops->update          = DSUpdateExtraRow_GHIEP;
+  ds->ops->hermitian       = DSHermitian_GHIEP;
   PetscFunctionReturn(0);
 }
-
