@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -40,31 +40,32 @@ static PetscErrorCode UnifiedRotation(PetscReal x,PetscReal y,PetscReal sygn,Pet
   } else {
     nrm = PetscMax(PetscAbs(x),PetscAbs(y));
     c = x/nrm; s = y/nrm;
+    PetscCheck(sygn==1.0 || sygn==-1,PETSC_COMM_SELF,PETSC_ERR_PLIB,"Value of sygn sent to transetup must be 1 or -1");
     if (sygn == 1.0) {  /* set up a rotator */
       nrm = PetscSqrtReal(c*c+s*s);
       c = c/nrm; s = s/nrm;
       /* rot = [c s; -s c]; */
       rot[0] = c; rot[1] = -s; rot[2] = s; rot[3] = c;
       *rcond = 1.0;
-    } else if (sygn == -1) {  /* set up a hyperbolic transformation */
+    } else {  /* sygn == -1, set up a hyperbolic transformation */
       nrm = c*c-s*s;
       if (nrm > 0) nrm = PetscSqrtReal(nrm);
-      else if (nrm < 0) {
+      else {
+        PetscCheck(nrm<0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"Breakdown in construction of hyperbolic transformation");
         nrm = PetscSqrtReal(-nrm);
         *swap = PETSC_TRUE;
-      } else SETERRQ(PETSC_COMM_SELF,1,"Breakdown in construction of hyperbolic transformation");
+      }
       c = c/nrm; s = s/nrm;
       /* rot = [c -s; -s c]; */
       rot[0] = c; rot[1] = -s; rot[2] = -s; rot[3] = c;
       *rcond = PetscAbs(PetscAbs(s)-PetscAbs(c))/(PetscAbs(s)+PetscAbs(c));
-    } else SETERRQ(PETSC_COMM_SELF,1,"Value of sygn sent to transetup must be 1 or -1");
+    }
   }
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode HZStep(PetscBLASInt ntop,PetscBLASInt nn,PetscReal tr,PetscReal dt,PetscReal *aa,PetscReal *bb,PetscReal *dd,PetscScalar *uu,PetscInt n,PetscInt ld,PetscBool *flag)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   one=1;
   PetscInt       k,jj,ii;
   PetscBLASInt   n_;
@@ -76,7 +77,7 @@ static PetscErrorCode HZStep(PetscBLASInt ntop,PetscBLASInt nn,PetscReal tr,Pets
   PetscFunctionBegin;
   *flag = PETSC_FALSE;
   worstcond = 1.0;
-  ierr = PetscBLASIntCast(n,&n_);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(n,&n_));
 
   /* Build initial bulge that sets step in motion */
   bulge10 = dd[ntop+1]*(aa[ntop]*(aa[ntop] - dd[ntop]*tr) + dt*dd[ntop]*dd[ntop]) + dd[ntop]*bb[ntop]*bb[ntop];
@@ -125,7 +126,7 @@ static PetscErrorCode HZStep(PetscBLASInt ntop,PetscBLASInt nn,PetscReal tr,Pets
         }
 
         /* Set up transforming matrix rot. */
-        ierr = UnifiedRotation(bulge20,bulge30,1,rot,&rcond,&swap);CHKERRQ(ierr);
+        PetscCall(UnifiedRotation(bulge20,bulge30,1,rot,&rcond,&swap));
 
         /* Apply transforming matrix rot to T. */
         bulge20 = rot[0]*bulge20 + rot[2]*bulge30;
@@ -152,7 +153,7 @@ static PetscErrorCode HZStep(PetscBLASInt ntop,PetscBLASInt nn,PetscReal tr,Pets
 
         /* Begin by setting up transforming matrix rot */
         sygn = dd[jj]*dd[jj+1];
-        ierr = UnifiedRotation(bulge10,bulge20,sygn,rot,&rcond,&swap);CHKERRQ(ierr);
+        PetscCall(UnifiedRotation(bulge10,bulge20,sygn,rot,&rcond,&swap));
         if (rcond<PETSC_MACHINE_EPSILON) {
           *flag = PETSC_TRUE;
           PetscFunctionReturn(0);
@@ -221,7 +222,6 @@ static PetscErrorCode HZStep(PetscBLASInt ntop,PetscBLASInt nn,PetscReal tr,Pets
 
 static PetscErrorCode HZIteration(PetscBLASInt nn,PetscBLASInt cgd,PetscReal *aa,PetscReal *bb,PetscReal *dd,PetscScalar *uu,PetscBLASInt ld)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   j2,one=1;
   PetscInt       its,nits,nstop,jj,ntop,nbot,ntry;
   PetscReal      htr,det,dis,dif,tn,kt,c,s,tr,dt;
@@ -252,12 +252,12 @@ static PetscErrorCode HZIteration(PetscBLASInt nn,PetscBLASInt cgd,PetscReal *aa
           dif = aa[ntop]-aa[nbot];
           if (2.0*PetscAbs(bb[ntop])<=dif) {
             tn = 2*bb[ntop]/dif;
-            tn = tn/(1.0 + PetscSqrtScalar(1.0+tn*tn));
+            tn = tn/(1.0 + PetscSqrtReal(1.0+tn*tn));
           } else {
             kt = dif/(2.0*bb[ntop]);
-            tn = PetscSign(kt)/(PetscAbs(kt)+PetscSqrtScalar(1.0+kt*kt));
+            tn = PetscSign(kt)/(PetscAbsReal(kt)+PetscSqrtReal(1.0+kt*kt));
           }
-          c = 1.0/PetscSqrtScalar(1.0 + tn*tn);
+          c = 1.0/PetscSqrtReal(1.0 + tn*tn);
           s = c*tn;
           aa[ntop] = aa[ntop] + tn*bb[ntop];
           aa[nbot] = aa[nbot] - tn*bb[ntop];
@@ -273,12 +273,10 @@ static PetscErrorCode HZIteration(PetscBLASInt nn,PetscBLASInt cgd,PetscReal *aa
       tr = aa[nbot-1]*dd[nbot-1] + aa[nbot]*dd[nbot];
       dt = dd[nbot-1]*dd[nbot]*(aa[nbot-1]*aa[nbot]-bb[nbot-1]*bb[nbot-1]);
       for (ntry=1;ntry<=6;ntry++) {
-        ierr = HZStep(ntop,nbot+1,tr,dt,aa,bb,dd,uu,nn,ld,&flag);CHKERRQ(ierr);
+        PetscCall(HZStep(ntop,nbot+1,tr,dt,aa,bb,dd,uu,nn,ld,&flag));
         if (!flag) break;
-        else if (ntry == 6) SETERRQ(PETSC_COMM_SELF,1,"Unable to complete hz step on six tries");
-        else {
-          tr = 0.9*tr; dt = 0.81*dt;
-        }
+        PetscCheck(ntry<6,PETSC_COMM_SELF,PETSC_ERR_CONV_FAILED,"Unable to complete hz step after six tries");
+        tr = 0.9*tr; dt = 0.81*dt;
       }
     }
   }
@@ -287,9 +285,8 @@ static PetscErrorCode HZIteration(PetscBLASInt nn,PetscBLASInt cgd,PetscReal *aa
 
 PetscErrorCode DSSolve_GHIEP_HZ(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
-  PetscErrorCode ierr;
   PetscInt       i,off;
-  PetscBLASInt   n1,ld;
+  PetscBLASInt   n1,ld = 0;
   PetscScalar    *A,*B,*Q;
   PetscReal      *d,*e,*s;
 
@@ -297,7 +294,7 @@ PetscErrorCode DSSolve_GHIEP_HZ(DS ds,PetscScalar *wr,PetscScalar *wi)
 #if !defined(PETSC_USE_COMPLEX)
   PetscValidScalarPointer(wi,3);
 #endif
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(ds->ld,&ld));
   n1  = ds->n - ds->l;
   off = ds->l + ds->l*ld;
   A   = ds->mat[DS_MAT_A];
@@ -306,10 +303,17 @@ PetscErrorCode DSSolve_GHIEP_HZ(DS ds,PetscScalar *wr,PetscScalar *wi)
   d   = ds->rmat[DS_MAT_T];
   e   = ds->rmat[DS_MAT_T] + ld;
   s   = ds->rmat[DS_MAT_D];
+#if defined(PETSC_USE_DEBUG)
+  /* Check signature */
+  for (i=0;i<ds->n;i++) {
+    PetscReal de = (ds->compact)?s[i]:PetscRealPart(B[i*ld+i]);
+    PetscCheck(de==1.0 || de==-1.0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"Diagonal elements of the signature matrix must be 1 or -1");
+  }
+#endif
   /* Quick return */
   if (n1 == 1) {
     for (i=0;i<=ds->l;i++) Q[i+i*ld] = 1.0;
-    ierr = DSGHIEPComplexEigs(ds,0,ds->l,wr,wi);CHKERRQ(ierr);
+    PetscCall(DSGHIEPComplexEigs(ds,0,ds->l,wr,wi));
     if (ds->compact) {
       wr[ds->l] = d[ds->l]/s[ds->l]; wi[ds->l] = 0.0;
     } else {
@@ -319,16 +323,14 @@ PetscErrorCode DSSolve_GHIEP_HZ(DS ds,PetscScalar *wr,PetscScalar *wi)
     PetscFunctionReturn(0);
   }
   /* Reduce to pseudotriadiagonal form */
-  ierr = DSIntermediate_GHIEP(ds);CHKERRQ(ierr);
-  ierr = HZIteration(ds->n,ds->l,d,e,s,Q,ld);CHKERRQ(ierr);
-  if (!ds->compact) {
-    ierr = DSSwitchFormat_GHIEP(ds,PETSC_FALSE);CHKERRQ(ierr);
-  }
-  /* Undo from diagonal the blocks whith real eigenvalues*/
-  ierr = DSGHIEPRealBlocks(ds);CHKERRQ(ierr);
+  PetscCall(DSIntermediate_GHIEP(ds));
+  PetscCall(HZIteration(ds->n,ds->l,d,e,s,Q,ld));
+  if (!ds->compact) PetscCall(DSSwitchFormat_GHIEP(ds,PETSC_FALSE));
+  /* Undo from diagonal the blocks with real eigenvalues*/
+  PetscCall(DSGHIEPRealBlocks(ds));
 
   /* Recover eigenvalues from diagonal */
-  ierr = DSGHIEPComplexEigs(ds,0,ds->n,wr,wi);CHKERRQ(ierr);
+  PetscCall(DSGHIEPComplexEigs(ds,0,ds->n,wr,wi));
 #if defined(PETSC_USE_COMPLEX)
   if (wi) {
     for (i=ds->l;i<ds->n;i++) wi[i] = 0.0;
@@ -337,4 +339,3 @@ PetscErrorCode DSSolve_GHIEP_HZ(DS ds,PetscScalar *wr,PetscScalar *wi)
   ds->t = ds->n;
   PetscFunctionReturn(0);
 }
-
