@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -51,7 +51,7 @@ static PetscErrorCode cutlr_(PetscBLASInt start,PetscBLASInt n,PetscBLASInt blkc
 
 /*  N      (input) INTEGER */
 /*         The sum of all the block sizes of the sequence to be cut = */
-/*         = sum_{i=1}^{BLKCT} BSIZES( I ). */
+/*         = sum_{i=1}^{BLKCT} BSIZES(I). */
 /*         N >= 3. */
 
 /*  BLKCT  (input) INTEGER */
@@ -60,7 +60,7 @@ static PetscErrorCode cutlr_(PetscBLASInt start,PetscBLASInt n,PetscBLASInt blkc
 
 /*  BSIZES (input) INTEGER array, dimension (BLKCT) */
 /*         The dimensions of the (quadratic) blocks of the sequence to be */
-/*         cut. sum_{i=1}^{BLKCT} BSIZES( I ) = N. */
+/*         cut. sum_{i=1}^{BLKCT} BSIZES(I) = N. */
 
 /*  RANKS  (input) INTEGER array, dimension (BLKCT-1) */
 /*         The ranks determining the approximations of the off-diagonal */
@@ -108,13 +108,9 @@ static PetscErrorCode cutlr_(PetscBLASInt start,PetscBLASInt n,PetscBLASInt blkc
   *lsum = 1;
   *cut = start;
 
-  if (start < 1) {
-    *info = -1;
-  } else if (n < 3) {
-    *info = -2;
-  } else if (blkct < 3) {
-    *info = -3;
-  }
+  if (start < 1) *info = -1;
+  else if (n < 3) *info = -2;
+  else if (blkct < 3) *info = -3;
   if (*info == 0) {
     ksum = 0;
     kchk = 0;
@@ -125,7 +121,7 @@ static PetscErrorCode cutlr_(PetscBLASInt start,PetscBLASInt n,PetscBLASInt blkc
     }
     if (ksum != n || kchk == 1) *info = -4;
   }
-  if (*info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %d in CUTLR",-(*info));
+  PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %" PetscBLASInt_FMT " in CUTLR",-(*info));
 
   /* determine smallest rank in the range considered */
 
@@ -159,13 +155,9 @@ static PetscErrorCode cutlr_(PetscBLASInt start,PetscBLASInt n,PetscBLASInt blkc
     }
   }
 
-  if (*cut < start || *cut >= start + blkct - 1) {
-    *info = 6;
-  } else if (*lsum < 1 || *lsum >= n) {
-    *info = 7;
-  } else if (*lblks < 1 || *lblks >= blkct) {
-    *info = 8;
-  }
+  if (*cut < start || *cut >= start + blkct - 1) *info = 6;
+  else if (*lsum < 1 || *lsum >= n) *info = 7;
+  else if (*lblks < 1 || *lblks >= blkct) *info = 8;
   PetscFunctionReturn(0);
 }
 
@@ -244,7 +236,7 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
 
 /*  RANK   (input) INTEGER array, dimension (NBLKS-1). */
 /*         The ranks of all the subdiagonal blocks contained in the array E. */
-/*         RANK( K ) <= MIN( KSIZES( K ), KSIZES( K+1 ) ) */
+/*         RANK(K) <= MIN(KSIZES(K), KSIZES(K+1)) */
 
 /*  L1E    (input) INTEGER */
 /*         The leading dimension of the array E.  L1E >= max(3,2*KMAX+1), */
@@ -256,9 +248,9 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
 
 /*  TOL    (input) DOUBLE PRECISION, TOL <= 1.0D-1 */
 /*         User specified deflation tolerance for the routine DMERG2. */
-/*         If ( 1.0D-1 >= TOL >= 20*EPS ) then TOL is used as */
+/*         If (1.0D-1 >= TOL >= 20*EPS) then TOL is used as */
 /*         the deflation tolerance in DSRTDF. */
-/*         If ( TOL < 20*EPS ) then the standard deflation tolerance from */
+/*         If (TOL < 20*EPS) then the standard deflation tolerance from */
 /*         LAPACK is used as the deflation tolerance in DSRTDF. */
 
 /*  EV     (output) DOUBLE PRECISION array, dimension (N) */
@@ -278,9 +270,9 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
 /*  LWORK   (input) INTEGER */
 /*          The dimension of the array WORK. */
 /*          In order to guarantee correct results in all cases, */
-/*          LWORK must be at least ( 2*N**2 + 3*N ). In many cases, */
+/*          LWORK must be at least (2*N**2 + 3*N). In many cases, */
 /*          less workspace is required. The absolute minimum required is */
-/*          ( N**2 + 3*N ). */
+/*          (N**2 + 3*N). */
 /*          If the workspace provided is not sufficient, the routine will */
 /*          return a corresponding error code and report how much workspace */
 /*          was missing (see INFO). */
@@ -289,11 +281,11 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
 
 /*  LIWORK  (input) INTEGER */
 /*          The dimension of the array IWORK. */
-/*          LIWORK must be at least ( 5*N + 3 + 4*NBLKS - 4 ): */
+/*          LIWORK must be at least (5*N + 3 + 4*NBLKS - 4): */
 /*                 5*KMAX+3 for DSYEVD, 5*N for ????, */
 /*                 4*NBLKS-4 for the preprocessing (merging order) */
 /*          Summarizing, the minimum integer workspace needed is */
-/*          MAX( 5*N, 5*KMAX + 3 ) + 4*NBLKS - 4 */
+/*          MAX(5*N, 5*KMAX + 3) + 4*NBLKS - 4 */
 
 /*  INFO   (output) INTEGER */
 /*          = 0:  successful exit. */
@@ -318,28 +310,19 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
 
 /*  ===================================================================== */
 
-#if defined(SLEPC_MISSING_LAPACK_LACPY) || defined(PETSC_MISSING_LAPACK_SYEV)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LACPY/SYEV - Lapack routine is unavailable");
-#else
   PetscBLASInt   i, j, k, np, rp1, ksk, one=1;
   PetscBLASInt   cut, mat1, kchk, kbrk, blks, kmax, icut, size, ksum, lsum;
   PetscBLASInt   lblks, rblks, isize, lwmin, ilsum;
-  PetscBLASInt   start, vstrt, istck1, istck2, istck3, merged;
+  PetscBLASInt   start, istck1, istck2, istck3, merged;
   PetscBLASInt   liwmin, matsiz, startp, istrtp;
   PetscReal      rho, done=1.0, dmone=-1.0;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   *info = 0;
 
-  if (*(unsigned char *)jobz != 'N' && *(unsigned char *)jobz != 'D') {
-    *info = -1;
-  } else if (n < 2) {
-    *info = -2;
-  } else if (nblks < 2 || nblks > n) {
-    *info = -3;
-  }
+  if (*(unsigned char *)jobz != 'N' && *(unsigned char *)jobz != 'D') *info = -1;
+  else if (n < 2) *info = -2;
+  else if (nblks < 2 || nblks > n) *info = -3;
   if (*info == 0) {
     ksum = 0;
     kmax = 0;
@@ -352,25 +335,15 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
     }
     lwmin = n*n + n * 3;
     liwmin = PetscMax(n * 5,kmax * 5 + 3) + 4*nblks - 4;
-    if (ksum != n || kchk == 1) {
-      *info = -4;
-    } else if (l1d < PetscMax(3,kmax)) {
-      *info = -6;
-    } else if (l2d < PetscMax(3,kmax)) {
-      *info = -7;
-    } else if (l1e < PetscMax(3,2*kmax + 1)) {
-      *info = -10;
-    } else if (l2e < PetscMax(3,2*kmax + 1)) {
-      *info = -11;
-    } else if (tol > .1) {
-      *info = -12;
-    } else if (ldz < PetscMax(1,n)) {
-      *info = -15;
-    } else if (lwork < lwmin) {
-      *info = -17;
-    } else if (liwork < liwmin) {
-      *info = -19;
-    }
+    if (ksum != n || kchk == 1) *info = -4;
+    else if (l1d < PetscMax(3,kmax)) *info = -6;
+    else if (l2d < PetscMax(3,kmax)) *info = -7;
+    else if (l1e < PetscMax(3,2*kmax + 1)) *info = -10;
+    else if (l2e < PetscMax(3,2*kmax + 1)) *info = -11;
+    else if (tol > .1) *info = -12;
+    else if (ldz < PetscMax(1,n)) *info = -15;
+    else if (lwork < lwmin) *info = -17;
+    else if (liwork < liwmin) *info = -19;
   }
   if (*info == 0) {
     for (k = 0; k < nblks-1; ++k) {
@@ -378,7 +351,7 @@ PetscErrorCode BDC_dibtdc_(const char *jobz,PetscBLASInt n,PetscBLASInt nblks,
     }
   }
 
-  if (*info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %d in DIBTDC",-(*info));
+  PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %" PetscBLASInt_FMT " in DIBTDC",-(*info));
 
 /* **************************************************************************** */
 
@@ -424,9 +397,8 @@ L200:
     /* chosen such that it yields the best balanced merging operation */
     /* among all the rank modifications with minimum rank. */
 
-    ierr = cutlr_(start, size, blks, &ksizes[start-1], &rank[start-1], &cut,
-                  &lsum, &lblks, info);CHKERRQ(ierr);
-    if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dibtdc: Error in cutlr, info = %d",*info);
+    PetscCall(cutlr_(start, size, blks, &ksizes[start-1], &rank[start-1], &cut, &lsum, &lblks, info));
+    PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_PLIB,"dibtdc: Error in cutlr, info = %" PetscBLASInt_FMT,*info);
 
   } else {
     cut = 1;
@@ -459,7 +431,7 @@ L200:
     size -= lsum;
     blks -= lblks;
 
-    if (blks <= 0) SETERRQ1(PETSC_COMM_SELF,1,"dibtdc: Error in preprocessing, blks = %d",blks);
+    PetscCheck(blks>0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"dibtdc: Error in preprocessing, blks = %" PetscBLASInt_FMT,blks);
 
     if (blks == 2) {
 
@@ -487,7 +459,7 @@ L200:
       } else {
 
         /* nothing left on the stack */
-        if (merged != nblks-1) SETERRQ(PETSC_COMM_SELF,1,"ERROR in preprocessing - not enough merges performed");
+        PetscCheck(merged==nblks-1,PETSC_COMM_SELF,PETSC_ERR_PLIB,"ERROR in preprocessing - not enough merges performed");
 
         /* exit preprocessing */
 
@@ -521,7 +493,7 @@ L200:
       iwork[isize + (nblks - 1) - merged-1] = size - lsum;
       iwork[ilsum + (nblks - 1) - merged-1] = ksizes[start + lblks-1];
     }
-    if (rblks <= 0) SETERRQ1(PETSC_COMM_SELF,1,"dibtdc: ERROR in preprocessing - rblks = %d",rblks);
+    PetscCheck(rblks>0,PETSC_COMM_SELF,PETSC_ERR_PLIB,"dibtdc: ERROR in preprocessing - rblks = %" PetscBLASInt_FMT,rblks);
 
     /* continue on the left side */
 
@@ -530,8 +502,8 @@ L200:
     goto L200;
   }
 
-  /*  SIZE = IWORK( ISIZE+NBLKS-2 ) */
-  /*  MAT1 = IWORK( ILSUM+NBLKS-2 ) */
+  /*  SIZE = IWORK(ISIZE+NBLKS-2) */
+  /*  MAT1 = IWORK(ILSUM+NBLKS-2) */
 
   /* Note: after the dimensions SIZE and MAT1 of the last merging */
   /* operation have been determined, an upper bound for the workspace */
@@ -539,15 +511,15 @@ L200:
   /* the last merging operation could be determined as follows */
   /* (based on (3.15) and (3.19) from UT-CS-00-447): */
 
-  /*  IF( MAT1.LE.N/2 ) THEN */
-  /*     WSPREQ = 3*N + 3/2*( SIZE-MAT1 )**2 + N*N/2 + MAT1*MAT1 */
+  /*  IF(MAT1.LE.N/2) THEN */
+  /*     WSPREQ = 3*N + 3/2*(SIZE-MAT1)**2 + N*N/2 + MAT1*MAT1 */
   /*  ELSE */
-  /*     WSPREQ = 3*N + 3/2*MAT1*MAT1 + N*N/2 + ( SIZE-MAT1 )**2 */
+  /*     WSPREQ = 3*N + 3/2*MAT1*MAT1 + N*N/2 + (SIZE-MAT1)**2 */
   /*  END IF */
 
-  /*  IF( LWORK-WSPREQ.LT.0 )THEN */
+  /*  IF(LWORK-WSPREQ.LT.0)THEN */
   /*          not enough work space provided */
-  /*     INFO = -200 - ( WSPREQ-LWORK ) */
+  /*     INFO = -200 - (WSPREQ-LWORK) */
   /*     RETURN */
   /*  END IF */
   /*  However, this is not really useful, since the actual check whether */
@@ -572,11 +544,11 @@ L200:
   /* (the correction will be made in Z and then the call of DSYEVD will */
   /*  overwrite it with the eigenvectors) */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, d, &l1d, z, &ldz));
+  for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) z[i+j*ldz] = d[i+j*l1d];
 
   /* copy D1 into WORK (in order to be able to restore it afterwards) */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, d, &l1d, work, &ksk));
+  for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) work[i+j*ksk] = d[i+j*l1d];
 
   /* copy V1 into the first RANK(1) columns of D1 and then */
   /* multiply with \Sigma1 */
@@ -586,7 +558,7 @@ L200:
     PetscStackCallBLAS("BLASscal",BLASscal_(&ksk, &e[i + rp1*l1e], &d[i*l1d], &one));
   }
 
-  /* multiply the first RANK( 1 ) columns of D1 with V1^T and */
+  /* multiply the first RANK(1) columns of D1 with V1^T and */
   /* subtract the result from the proper part of Z (previously */
   /* initialized with D1) */
 
@@ -595,14 +567,14 @@ L200:
 
   /* restore the original D1 from WORK */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, work, &ksk, d, &l1d));
+  for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) d[i+j*l1d] = work[i+j*ksk];
 
   /* eigenanalysis of block 1 (using DSYEVD) */
 
   PetscStackCallBLAS("LAPACKsyev",LAPACKsyev_("V", "L", &ksk, z, &ldz, ev, work, &lwork, info));
-  if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dibtdc: Error in DSYEVD for block 1, info = %d",*info);
+  SlepcCheckLapackInfo("syev",*info);
 
-  /* EV( 1: ) contains the eigenvalues in ascending order */
+  /* EV(1:) contains the eigenvalues in ascending order */
   /* (they are returned this way by DSYEVD) */
 
   for (i = 0; i < ksk; ++i) iwork[i] = i+1;
@@ -626,11 +598,11 @@ L200:
       /* (the correction will be made in Z and then the call of DSYEVD will */
       /*  overwrite it with the eigenvectors) */
 
-      PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, &d[k*l1d*l2d], &l1d, &z[np+np*ldz], &ldz));
+      for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) z[np+i+(np+j)*ldz] = d[i+(j+k*l2d)*l1d];
 
       /* copy Dk into WORK (in order to be able to restore it afterwards) */
 
-      PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, &d[k*l1d*l2d], &l1d, work, &ksk));
+      for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) work[i+j*ksk] = d[i+(j+k*l2d)*l1d];
 
       /* copy U(K-1) into the first RANK(K-1) columns of Dk and then */
       /* multiply with \Sigma(K-1) */
@@ -658,7 +630,7 @@ L200:
 
       /* multiply the first RANK(K) columns of Dk with Vk^T and */
       /* subtract the result from the proper part of Z (previously */
-      /* updated with [- U(k-1) \Sigma(k-1) U(k-1)^T] ) */
+      /* updated with [- U(k-1) \Sigma(k-1) U(k-1)^T]) */
 
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N", "T", &ksk, &ksk, &rank[k],
                     &dmone, &d[k*l1d*l2d], &l1d,
@@ -666,15 +638,15 @@ L200:
 
       /* restore the original Dk from WORK */
 
-      PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, work, &ksk, &d[k*l1d*l2d], &l1d));
+      for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) d[i+(j+k*l2d)*l1d] = work[i+j*ksk];
 
       /* eigenanalysis of block K (using dsyevd) */
 
       PetscStackCallBLAS("LAPACKsyev",LAPACKsyev_("V", "L", &ksk, &z[np+np*ldz],
                      &ldz, &ev[np], work, &lwork, info));
-      if (*info) SETERRQ2(PETSC_COMM_SELF,1,"dibtdc: Error in DSYEVD for block %d, info = %d",k,*info);
+      SlepcCheckLapackInfo("syev",*info);
 
-      /* EV( NPP1: ) contains the eigenvalues in ascending order */
+      /* EV(NPP1:) contains the eigenvalues in ascending order */
       /* (they are returned this way by DSYEVD) */
 
       for (i = 0; i < ksk; ++i) iwork[np + i] = i+1;
@@ -695,11 +667,11 @@ L200:
   /* (the correction will be made in Z and then the call of DSYEVD will */
   /* overwrite it with the eigenvectors) */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, &d[(nblks-1)*l1d*l2d], &l1d, &z[np+np*ldz], &ldz));
+  for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) z[np+i+(np+j)*ldz] = d[i+(j+(nblks-1)*l2d)*l1d];
 
   /* copy D(nblks) into WORK (in order to be able to restore it afterwards) */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, &d[(nblks-1)*l1d*l2d], &l1d, work, &ksk));
+  for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) work[i+j*ksk] = d[i+(j+(nblks-1)*l2d)*l1d];
 
   /* copy U(nblks-1) into the first RANK(nblks-1) columns of D(nblks) and then */
   /* multiply with \Sigma(nblks-1) */
@@ -714,7 +686,7 @@ L200:
 
   /* multiply the first RANK(nblks-1) columns of D(nblks) with U(nblks-1)^T */
   /* and subtract the result from the proper part of Z (previously */
-  /* initialized with D(nblks) ) */
+  /* initialized with D(nblks)) */
 
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N", "T", &ksk, &ksk, &rank[nblks - 2],
           &dmone, &d[(nblks-1)*l1d*l2d], &l1d,
@@ -722,24 +694,22 @@ L200:
 
   /* restore the original D(nblks) from WORK */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("L", &ksk, &ksk, work, &ksk, &d[(nblks-1)*l1d*l2d], &l1d));
+  for (j=0;j<ksk;j++) for (i=j;i<ksk;i++) d[i+(j+(nblks-1)*l2d)*l1d] = work[i+j*ksk];
 
   /* eigenanalysis of block NBLKS (using dsyevd) */
 
   PetscStackCallBLAS("LAPACKsyev",LAPACKsyev_("V", "L", &ksk, &z[np+np*ldz], &ldz, &ev[np], work, &lwork, info));
-  if (*info) SETERRQ2(PETSC_COMM_SELF,1,"dibtdc: Error in DSYEVD for block %d, info = %d",nblks,*info);
+  SlepcCheckLapackInfo("syev",*info);
 
-  /* EV( NPP1: ) contains the eigenvalues in ascending order */
+  /* EV(NPP1:) contains the eigenvalues in ascending order */
   /* (they are returned this way by DSYEVD) */
 
   for (i = 0; i < ksk; ++i) iwork[np + i] = i+1;
 
   /* note that from here on the entire workspace is available again */
 
-
   /* Perform all the merging operations. */
 
-  vstrt = 0;
   for (i = 0; i < nblks-1; ++i) {
 
     /* MATSIZ = total size of the current rank RANK modification problem */
@@ -748,33 +718,31 @@ L200:
     np = iwork[istrtp + i - 1];
     kbrk = iwork[icut + i - 1];
     mat1 = iwork[ilsum + i - 1];
-    vstrt += np;
 
     for (j = 0; j < rank[kbrk-1]; ++j) {
 
       /* NOTE: The parameter RHO in DMERG2 is modified in DSRTDF */
       /*       (multiplied by 2) ! In order not to change the */
-      /*       singular value stored in E( :, RANK( KBRK )+1, KBRK ), */
+      /*       singular value stored in E(:, RANK(KBRK)+1, KBRK), */
       /*       we do not pass on this variable as an argument to DMERG2, */
       /*       but we assign a separate variable RHO here which is passed */
       /*       on to DMERG2. */
       /*       Alternative solution in F90: */
-      /*       pass E( :,RANK( KBRK )+1,KBRK ) to an INTENT( IN ) parameter */
+      /*       pass E(:,RANK(KBRK)+1,KBRK) to an INTENT(IN) parameter */
       /*       in DMERG2. */
 
       rho = e[j + (rank[kbrk-1] + (kbrk-1)*l2e)*l1e];
 
-      /* eigenvectors are accumulated ( JOBZ.EQ.'D' ) */
+      /* eigenvectors are accumulated (JOBZ.EQ.'D') */
 
-      ierr = BDC_dmerg2_(jobz, j+1, matsiz, &ev[np-1], &z[np-1+(np-1)*ldz],
+      PetscCall(BDC_dmerg2_(jobz, j+1, matsiz, &ev[np-1], &z[np-1+(np-1)*ldz],
                     ldz, &iwork[np-1], &rho, &e[(j + (kbrk-1)*l2e)*l1e],
                     ksizes[kbrk], &e[(rank[kbrk-1]+j+1 + (kbrk-1)*l2e)*l1e],
-                    ksizes[kbrk-1], mat1, work, lwork, &iwork[n], tol, info, 1);
-                    CHKERRQ(ierr);
-      if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dibtdc: Error in dmerg2, info = %d",*info);
+                    ksizes[kbrk-1], mat1, work, lwork, &iwork[n], tol, info, 1));
+      PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_PLIB,"dibtdc: Error in dmerg2, info = %" PetscBLASInt_FMT,*info);
     }
 
-    /* at this point all RANK( KBRK ) rank-one modifications corresponding */
+    /* at this point all RANK(KBRK) rank-one modifications corresponding */
     /* to the current off-diagonal block are finished. */
     /* Move on to the next off-diagonal block. */
 
@@ -785,7 +753,7 @@ L200:
   /* to the permutation stored in IWORK. */
 
   /* copy eigenvalues and eigenvectors in ordered form into WORK */
-  /* (eigenvalues into WORK( 1:N ), eigenvectors into WORK( N+1:N+1+N^2 ) ) */
+  /* (eigenvalues into WORK(1:N), eigenvectors into WORK(N+1:N+1+N^2)) */
 
   for (i = 0; i < n; ++i) {
     j = iwork[i];
@@ -793,14 +761,12 @@ L200:
     PetscStackCallBLAS("BLAScopy",BLAScopy_(&n, &z[(j-1)*ldz], &one, &work[n*(i+1)], &one));
   }
 
-  /* copy ordered eigenvalues back from WORK( 1:N ) into EV */
+  /* copy ordered eigenvalues back from WORK(1:N) into EV */
 
   PetscStackCallBLAS("BLAScopy",BLAScopy_(&n, work, &one, ev, &one));
 
-  /* copy ordered eigenvectors back from WORK( N+1:N+1+N^2 ) into Z */
+  /* copy ordered eigenvectors back from WORK(N+1:N+1+N^2) into Z */
 
-  PetscStackCallBLAS("LAPACKlacpy",LAPACKlacpy_("A", &n, &n, &work[n], &n, z, &ldz));
+  for (j=0;j<n;j++) for (i=0;i<n;i++) z[i+j*ldz] = work[i+(j+1)*n];
   PetscFunctionReturn(0);
-#endif
 }
-
