@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -43,7 +43,7 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 /*  The dimension of the first subproblem is CUTPNT, the dimension of */
 /*  the second subproblem is N-CUTPNT. */
 
-/*  T = Q(in) ( EV(in) + RHO * Z*Z' ) Q'(in) = Q(out) * EV(out) * Q'(out) */
+/*  T = Q(in) (EV(in) + RHO * Z*Z') Q'(in) = Q(out) * EV(out) * Q'(out) */
 
 /*     where Z = Q'[V U']', where V is a row vector and U is a column */
 /*     vector with dimensions corresponding to the two underlying */
@@ -64,7 +64,7 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 /*        This routine also calculates the eigenvectors of the current */
 /*        problem. */
 
-/*        If( JOBZ.EQ.'D' ) then the final stage consists */
+/*        If(JOBZ.EQ.'D') then the final stage consists */
 /*        of computing the updated eigenvectors directly using the updated */
 /*        eigenvalues. The eigenvectors for the current problem are multiplied */
 /*        with the eigenvectors from the overall problem. */
@@ -107,7 +107,7 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 /*         subproblems in EV into ascending order. */
 /*         On exit, the permutation which will reintegrate the */
 /*         subproblems back into sorted order, */
-/*         i.e. EV( INDXQ( I = 1, N ) ) will be in ascending order. */
+/*         i.e. EV(INDXQ(I = 1, N)) will be in ascending order. */
 
 /*  RHO    (input/output) DOUBLE PRECISION */
 /*         The scalar in the rank-1 perturbation. Modified (multiplied */
@@ -150,9 +150,9 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 /*  LWORK  (input) INTEGER */
 /*         The dimension of the array WORK. */
 /*         In order to guarantee correct results in all cases, */
-/*         LWORK must be at least ( 2*N**2 + 3*N  ). In many cases, */
+/*         LWORK must be at least (2*N**2 + 3*N). In many cases, */
 /*         less workspace is required. The absolute minimum required is */
-/*         ( N**2 + 3*N ). */
+/*         (N**2 + 3*N). */
 /*         If the workspace provided is not sufficient, the routine will */
 /*         return a corresponding error code and report how much workspace */
 /*         was missing (see INFO). */
@@ -160,7 +160,7 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 /*               workspace is provided, and, if not, for computing how */
 /*               much workspace is needed. */
 
-/*  IWORK  (workspace) INTEGER array, dimension ( 4*N ) */
+/*  IWORK  (workspace) INTEGER array, dimension (4*N) */
 
 /*  TOL    (input) DOUBLE PRECISION */
 /*         User specified deflation tolerance for the routine DSRTDF. */
@@ -191,31 +191,21 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 
 /*  ===================================================================== */
 
-#if defined(SLEPC_MISSING_LAPACK_LAMRG)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LAMRG - Lapack routine is unavailable");
-#else
   PetscBLASInt   i, k, n1, n2, de, is, dz, iw, iz, iq2, nmc, cpp1;
   PetscBLASInt   indx, indxc, indxp, lwmin, idlmda;
   PetscBLASInt   spneed, coltyp, tmpcut, i__1, i__2, one=1, mone=-1;
   char           defl[1];
   PetscReal      done = 1.0, dzero = 0.0;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   *info = 0;
   lwmin = n*n + n * 3;
 
-  if (n < 0) {
-    *info = -3;
-  } else if (ldq < PetscMax(1,n)) {
-    *info = -6;
-  } else if (cutpnt < PetscMin(1,n) || cutpnt > PetscMax(1,n)) {
-    *info = -13;
-  } else if (lwork < lwmin) {
-    *info = -15;
-  }
-  if (*info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %d in DMERG2",-(*info));
+  if (n < 0) *info = -3;
+  else if (ldq < PetscMax(1,n)) *info = -6;
+  else if (cutpnt < PetscMin(1,n) || cutpnt > PetscMax(1,n)) *info = -13;
+  else if (lwork < lwmin) *info = -15;
+  PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %" PetscBLASInt_FMT " in DMERG2",-(*info));
 
 /* **************************************************************************** */
 
@@ -244,7 +234,7 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
   indxp = coltyp + n;
 
   /* If eigenvectors are to be accumulated in the divide-and-conquer */
-  /* process ( JOBZ.EQ.'D' ) form the z-vector which consists of */
+  /* process (JOBZ.EQ.'D') form the z-vector which consists of */
   /* Q_1^T * V and Q_2^T * U. */
 
   cpp1 = cutpnt + 1;
@@ -304,11 +294,10 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 
   /* call DSRTDF for deflation */
 
-  ierr = BDC_dsrtdf_(&k, n, n1, ev, q, ldq, indxq, rho, &work[iz],
+  PetscCall(BDC_dsrtdf_(&k, n, n1, ev, q, ldq, indxq, rho, &work[iz],
           &work[idlmda], &work[iw], &work[iq2], &iwork[indx],
-          &iwork[indxc], &iwork[indxp], &iwork[coltyp], tol, &dz, &de, info);
-          CHKERRQ(ierr);
-  if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dmerg2: error in dsrtdf, info = %d",*info);
+          &iwork[indxc], &iwork[indxp], &iwork[coltyp], tol, &dz, &de, info));
+  PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_LIB,"dmerg2: error in dsrtdf, info = %" PetscBLASInt_FMT,*info);
 
   if (k < n) {
 
@@ -324,7 +313,7 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
 
   /* Solve the Secular Equation. */
 
-  if (k != 0 || k == 0) {
+  if (k != 0) {
 
     /* ....not everything was deflated. */
 
@@ -338,23 +327,21 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
     i__2 = (iwork[coltyp+1] + iwork[coltyp + 2]) * k;
     spneed = is + PetscMax(i__1,i__2) - 1;
 
-    if (spneed > lwork) SETERRQ1(PETSC_COMM_SELF,1,"dmerg2: Workspace needed exceeds the workspace provided by %d numbers",spneed-lwork);
+    PetscCheck(spneed<=lwork,PETSC_COMM_SELF,PETSC_ERR_MEM,"dmerg2: Workspace needed exceeds the workspace provided by %" PetscBLASInt_FMT " numbers",spneed-lwork);
 
     /* calling DLAED3M for solving the secular equation. */
 
-    ierr = BDC_dlaed3m_(jobz, defl, k, n, tmpcut, ev, q, ldq,
+    PetscCall(BDC_dlaed3m_(jobz, defl, k, n, tmpcut, ev, q, ldq,
                 *rho, &work[idlmda], &work[iq2], &iwork[indxc], &iwork[coltyp],
-                &work[iw], &work[is], info, 1, 1);CHKERRQ(ierr);
-    if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dmerg2: error in dlaed3m, info = %d",*info);
+                &work[iw], &work[is], info, 1, 1));
+    PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_LIB,"dmerg2: error in dlaed3m, info = %" PetscBLASInt_FMT,*info);
 
     /* Prepare the INDXQ sorting permutation. */
 
     n1 = k;
     n2 = n - k;
     PetscStackCallBLAS("LAPACKlamrg",LAPACKlamrg_(&n1, &n2, ev, &one, &mone, indxq));
-    if (k == 0) {
-      for (i = 0; i < n; ++i) indxq[i] = i+1;
-    }
+    if (k == 0) for (i = 0; i < n; ++i) indxq[i] = i+1;
 
   } else {
 
@@ -363,6 +350,4 @@ PetscErrorCode BDC_dmerg2_(const char *jobz,PetscBLASInt rkct,PetscBLASInt n,
     for (i = 0; i < n; ++i) indxq[i] = i+1;
   }
   PetscFunctionReturn(0);
-#endif
 }
-
