@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -91,7 +91,7 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*  L1D     (input) INTEGER */
 /*          The leading dimension of the array D.  L1D >= max(3,KMAX), */
 /*          where KMAX is the dimension of the largest diagonal block, */
-/*          i.e.,  KMAX = max_I ( KSIZES(I) ). */
+/*          i.e.,  KMAX = max_I (KSIZES(I)). */
 
 /*  L2D     (input) INTEGER */
 /*          The second dimension of the array D.  L2D >= max(3,KMAX), */
@@ -114,7 +114,7 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*          the corresponding off-diagonal block in E(*,*,K) and therefore */
 /*          there are no restrictions on the rank of the approximation */
 /*          (only the "natural" restriction */
-/*          RANK( K ) .LE. MIN( KSIZES( K ),KSIZES( K+1 ) )). */
+/*          RANK(K) .LE. MIN(KSIZES(K),KSIZES(K+1))). */
 
 /*  L2E     (input) INTEGER */
 /*          The second dimension of the array E.  L2E >= max(3,2*KMAX+1), */
@@ -123,13 +123,13 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*          the corresponding off-diagonal block in E(*,*,K) and therefore */
 /*          there are no restrictions on the rank of the approximation */
 /*          (only the "natural" restriction */
-/*          RANK( K ) .LE. MIN( KSIZES( K ),KSIZES( K+1 ) )). */
+/*          RANK(K) .LE. MIN(KSIZES(K),KSIZES(K+1))). */
 
 /*  TOL     (input) DOUBLE PRECISION, TOL.LE.TOLMAX */
 /*          User specified tolerance for the residuals of the computed */
-/*          eigenpairs. If ( JOBACC.EQ.'A' ) then it is used to determine */
+/*          eigenpairs. If (JOBACC.EQ.'A') then it is used to determine */
 /*          TAU1 and TAU2; ignored otherwise. */
-/*          If ( TOL.LT.40*EPS .AND. JOBACC.EQ.'A' ) then TAU1 is set to machine */
+/*          If (TOL.LT.40*EPS .AND. JOBACC.EQ.'A') then TAU1 is set to machine */
 /*          epsilon and TAU2 is set to the standard deflation tolerance from */
 /*          LAPACK. */
 
@@ -139,20 +139,20 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*          The rank for each off-diagonal block is determined such that */
 /*          the resulting absolute eigenvalue error is less than or equal */
 /*          to TAU1. */
-/*          If ( JOBACC.EQ.'A' ) then TAU1 is determined automatically from */
+/*          If (JOBACC.EQ.'A') then TAU1 is determined automatically from */
 /*             TOL and the input value is ignored. */
-/*          If ( JOBACC.EQ.'M' .AND. TAU1.LT.20*EPS ) then TAU1 is set to */
+/*          If (JOBACC.EQ.'M' .AND. TAU1.LT.20*EPS) then TAU1 is set to */
 /*             machine epsilon. */
 
 /*  TAU2    (input) DOUBLE PRECISION, TAU2.LE.TOLMAX/2 */
 /*          User specified deflation tolerance for the routine DIBTDC. */
-/*          If ( 1.0D-1.GT.TAU2.GT.20*EPS ) then TAU2 is used as */
+/*          If (1.0D-1.GT.TAU2.GT.20*EPS) then TAU2 is used as */
 /*          the deflation tolerance in DSRTDF (EPS is the machine epsilon). */
-/*          If ( TAU2.LE.20*EPS ) then the standard deflation tolerance from */
+/*          If (TAU2.LE.20*EPS) then the standard deflation tolerance from */
 /*          LAPACK is used as the deflation tolerance in DSRTDF. */
-/*          If ( JOBACC.EQ.'A' ) then TAU2 is determined automatically from */
+/*          If (JOBACC.EQ.'A') then TAU2 is determined automatically from */
 /*             TOL and the input value is ignored. */
-/*          If ( JOBACC.EQ.'M' .AND. TAU2.LT.20*EPS ) then TAU2 is set to */
+/*          If (JOBACC.EQ.'M' .AND. TAU2.LT.20*EPS) then TAU2 is set to */
 /*             the standard deflation tolerance from LAPACK. */
 
 /*  EV      (output) DOUBLE PRECISION array, dimension (N) */
@@ -160,11 +160,11 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*          symmetric block tridiagonal matrix in ascending order. */
 
 /*  Z       (output) DOUBLE PRECISION array, dimension (LDZ,N) */
-/*          If ( JOBZ.EQ.'D' .AND. INFO = 0 ) */
+/*          If (JOBZ.EQ.'D' .AND. INFO = 0) */
 /*          then Z contains the orthonormal eigenvectors of the symmetric */
 /*          block tridiagonal matrix computed by the routine DIBTDC */
 /*          (accumulated in the divide-and-conquer process). */
-/*          If ( -199 < INFO < -99 ) then Z contains the orthonormal */
+/*          If (-199 < INFO < -99) then Z contains the orthonormal */
 /*          eigenvectors of the symmetric block tridiagonal matrix, */
 /*          computed without divide-and-conquer (quick returns). */
 /*          Otherwise not referenced. */
@@ -178,27 +178,26 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*          The dimension of the array WORK. */
 /*          If NBLKS.EQ.1, then LWORK has to be at least 2N^2+6N+1 */
 /*          (for the call of DSYEVD). */
-/*          If NBLKS.GE.2 and ( JOBZ.EQ.'D' ) then the absolute minimum */
-/*             required for DIBTDC is ( N**2 + 3*N ). This will not always */
+/*          If NBLKS.GE.2 and (JOBZ.EQ.'D') then the absolute minimum */
+/*             required for DIBTDC is (N**2 + 3*N). This will not always */
 /*             suffice, though, the routine will return a corresponding */
 /*             error code and report how much work space was missing (see */
 /*             INFO). */
 /*          In order to guarantee correct results in all cases where */
-/*          NBLKS.GE.2, LWORK must be at least ( 2*N**2 + 3*N ). */
+/*          NBLKS.GE.2, LWORK must be at least (2*N**2 + 3*N). */
 
 /*  IWORK   (workspace/output) INTEGER array, dimension (LIWORK) */
 
 /*  LIWORK  (input) INTEGER */
 /*          The dimension of the array IWORK. */
-/*          LIWORK must be at least ( 5*N + 5*NBLKS - 1 ) (for DIBTDC) */
+/*          LIWORK must be at least (5*N + 5*NBLKS - 1) (for DIBTDC) */
 /*          Note that this should also suffice for the call of DSYEVD on a */
-/*          diagonal block which requires ( 5*KMAX + 3 ). */
-
+/*          diagonal block which requires (5*KMAX + 3). */
 
 /*  MINGAP  (output) DOUBLE PRECISION */
 /*          The minimum "gap" between the approximate eigenvalues */
-/*          computed, i.e., MIN( ABS( EV( I+1 )-EV( I ) ) for I=1,2,..., N-1 */
-/*          IF ( MINGAP.LE.TOL/10 ) THEN a warning flag is returned in INFO, */
+/*          computed, i.e., MIN( ABS(EV(I+1)-EV(I)) for I=1,2,..., N-1 */
+/*          IF (MINGAP.LE.TOL/10) THEN a warning flag is returned in INFO, */
 /*          because the computed eigenvectors may be unreliable individually */
 /*          (only the subspaces spanned are approximated reliably). */
 
@@ -257,17 +256,12 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 /*                          NOTE: in the routine DIBTDC, the value */
 /*                                1.D-1 is hardcoded for TOLMAX ! */
 
-#if defined(SLEPC_MISSING_LAPACK_SYEVD) || defined(PETSC_MISSING_LAPACK_GESVD) || defined(SLEPC_MISSING_LAPACK_LASET) || defined(SLEPC_MISSING_LAPACK_LASCL)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"SYEVD/GESVD/LASET/LASCL - Lapack routine is unavailable");
-#else
   PetscBLASInt   i, j, k, i1, iwspc, lwmin, start;
-  PetscBLASInt   ii, ip, jp, nk, rk, np, iu, rp1, ldu;
-  PetscBLASInt   ksk, ivt, iend, kchk, kmax, one=1, zero=0;
-  PetscBLASInt   ldvt, ksum, kskp1, spneed, nrblks, liwmin, isvals;
-  PetscReal      p, d2, eps, dmax, emax, done = 1.0, dzero = 0.0;
+  PetscBLASInt   ii, ip, nk, rk, np, iu, rp1, ldu;
+  PetscBLASInt   ksk, ivt, iend, kchk=0, kmax=0, one=1, zero=0;
+  PetscBLASInt   ldvt, ksum=0, kskp1, spneed, nrblks, liwmin, isvals;
+  PetscReal      p, d2, eps, dmax, emax, done = 1.0;
   PetscReal      dnrm, tiny, anorm, exdnrm=0, dropsv, absdiff;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   /* Determine machine epsilon. */
@@ -275,19 +269,11 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
 
   *info = 0;
 
-  if (*(unsigned char *)jobz != 'N' && *(unsigned char *)jobz != 'D') {
-    *info = -1;
-  } else if (*(unsigned char *)jobacc != 'A' && *(unsigned char *)jobacc != 'M') {
-    *info = -2;
-  } else if (n < 1) {
-    *info = -3;
-  } else if (nblks < 1 || nblks > n) {
-    *info = -4;
-  }
+  if (*(unsigned char *)jobz != 'N' && *(unsigned char *)jobz != 'D') *info = -1;
+  else if (*(unsigned char *)jobacc != 'A' && *(unsigned char *)jobacc != 'M') *info = -2;
+  else if (n < 1) *info = -3;
+  else if (nblks < 1 || nblks > n) *info = -4;
   if (*info == 0) {
-    ksum = 0;
-    kmax = 0;
-    kchk = 0;
     for (k = 0; k < nblks; ++k) {
       ksk = ksizes[k];
       ksum += ksk;
@@ -297,38 +283,25 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
     if (nblks == 1) lwmin = 2*n*n + n*6 + 1;
     else lwmin = n*n + n*3;
     liwmin = n * 5 + nblks * 5 - 4;
-    if (ksum != n || kchk == 1) {
-      *info = -5;
-    } else if (l1d < PetscMax(3,kmax)) {
-      *info = -7;
-    } else if (l2d < PetscMax(3,kmax)) {
-      *info = -8;
-    } else if (l1e < PetscMax(3,2*kmax+1)) {
-      *info = -10;
-    } else if (l2e < PetscMax(3,2*kmax+1)) {
-      *info = -11;
-    } else if (*(unsigned char *)jobacc == 'A' && tol > TOLMAX) {
-      *info = -12;
-    } else if (*(unsigned char *)jobacc == 'M' && tau1 > TOLMAX/2) {
-      *info = -13;
-    } else if (*(unsigned char *)jobacc == 'M' && tau2 > TOLMAX/2) {
-      *info = -14;
-    } else if (ldz < PetscMax(1,n)) {
-      *info = -17;
-    } else if (lwork < lwmin) {
-      *info = -19;
-    } else if (liwork < liwmin) {
-      *info = -21;
-    }
+    if (ksum != n || kchk == 1) *info = -5;
+    else if (l1d < PetscMax(3,kmax)) *info = -7;
+    else if (l2d < PetscMax(3,kmax)) *info = -8;
+    else if (l1e < PetscMax(3,2*kmax+1)) *info = -10;
+    else if (l2e < PetscMax(3,2*kmax+1)) *info = -11;
+    else if (*(unsigned char *)jobacc == 'A' && tol > TOLMAX) *info = -12;
+    else if (*(unsigned char *)jobacc == 'M' && tau1 > TOLMAX/2) *info = -13;
+    else if (*(unsigned char *)jobacc == 'M' && tau2 > TOLMAX/2) *info = -14;
+    else if (ldz < PetscMax(1,n)) *info = -17;
+    else if (lwork < lwmin) *info = -19;
+    else if (liwork < liwmin) *info = -21;
   }
 
-  if (*info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %d in DSBTDC",-(*info));
+  PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Wrong argument %" PetscBLASInt_FMT " in DSBTDC",-(*info));
 
   /* Quick return if possible */
 
   if (n == 1) {
-    ev[0] = d[0];
-    z[0] = 1.;
+    ev[0] = d[0]; z[0] = 1.;
     *info = -101;
     PetscFunctionReturn(0);
   }
@@ -337,14 +310,13 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
   /* dense solver (in this case KSIZES(1) = N). */
 
   if (nblks == 1) {
-    ierr = PetscPrintf(PETSC_COMM_WORLD," dsbtdc: This branch still needs to be checked!\n");CHKERRQ(ierr);
     for (i = 0; i < n; ++i) {
       for (j = 0; j <= i; ++j) {
         z[i + j*ldz] = d[i + j*l1d];
       }
     }
     PetscStackCallBLAS("LAPACKsyevd",LAPACKsyevd_("V", "L", &n, z, &ldz, ev, work, &lwork, iwork, &liwork, info));
-    if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: Error in DSYEVD, info = %d",*info);
+    SlepcCheckLapackInfo("syevd",*info);
     *info = -102;
     PetscFunctionReturn(0);
   }
@@ -360,12 +332,13 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
   /* Initialize Z as the identity matrix */
 
   if (*(unsigned char *)jobz == 'D') {
-    PetscStackCallBLAS("LAPACKlaset",LAPACKlaset_("Full", &n, &n, &dzero, &done, z, &ldz));
+    for (j=0;j<n;j++) for (i=0;i<n;i++) z[i+j*ldz] = 0.0;
+    for (i=0;i<n;i++) z[i+i*ldz] = 1.0;
   }
 
   /* Determine the off-diagonal ranks, form and store the lower rank */
   /* approximations based on the tolerance parameters, the */
-  /* RANK( K ) largest singular values and the associated singular */
+  /* RANK(K) largest singular values and the associated singular */
   /* vectors of each subdiagonal block. Also find the maximum norm of */
   /* the subdiagonal blocks (in EMAX). */
 
@@ -402,10 +375,10 @@ PetscErrorCode BDC_dsbtdc_(const char *jobz,const char *jobacc,PetscBLASInt n,
     PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("O", "S", &kskp1, &ksk,
             &e[k*l1e*l2e], &l1e, &work[isvals],
             &work[iu], &ldu, &work[ivt], &ldvt, &work[iwspc], &i1, info));
-    if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: Error in DGESVD, info = %d",*info);
+    SlepcCheckLapackInfo("gesvd",*info);
 
     /* Note that after the return from DGESVD U is stored in */
-    /* E(*,*,K), and V^{\top} is stored in WORK( IVT, IVT+1, .... ) */
+    /* E(*,*,K), and V^{\top} is stored in WORK(IVT, IVT+1, ....) */
 
     /* determine the ranks RANK() for the approximations */
 
@@ -418,8 +391,7 @@ L8:
       /* the error caused by dropping singular value RK is */
       /* small enough, try to reduce the rank by one more */
 
-      --rk;
-      if (rk > 0) goto L8;
+      if (--rk > 0) goto L8;
       else iwork[k] = 0;
     } else {
 
@@ -432,7 +404,7 @@ L8:
 
 /* ************************************************************************** */
 
-    /* Store the first RANK( K ) terms of the SVD of the current */
+    /* Store the first RANK(K) terms of the SVD of the current */
     /* off-diagonal block. */
     /* NOTE that here it is required that L1E, L2E >= 2*KMAX+1 in order */
     /* to have enough space for storing singular vectors and values up */
@@ -446,7 +418,7 @@ L8:
     rp1 = iwork[k];
     for (j = 0; j < iwork[k]; ++j) {
 
-      /* store sigma_J in E( J,RANK( K )+1,K ) */
+      /* store sigma_J in E(J,RANK(K)+1,K) */
 
       e[j + (rp1 + k*l2e)* l1e] = work[isvals + j];
 
@@ -456,7 +428,7 @@ L8:
         emax = e[j + (rp1 + k*l2e)*l1e];
       }
 
-      /* store v_J in E( :,RANK( K )+1+J,K ) */
+      /* store v_J in E(:,RANK(K)+1+J,K) */
       /* (note that WORK contains V^{\top} and therefore */
       /* we need to read rowwise !) */
 
@@ -496,7 +468,7 @@ L8:
 /* **************************************************************** */
 
   /* ....Identify irreducible parts of the block tridiagonal matrix */
-  /* [while ( START <= NBLKS )].... */
+  /* [while (START <= NBLKS)].... */
 
   start = 0;
   np = 0;
@@ -515,7 +487,7 @@ L20:
     if (iend < nblks) {
       rk = iwork[iend];
 
-      /* NOTE: if RANK( IEND ).EQ.0 then decoupling happens due to */
+      /* NOTE: if RANK(IEND).EQ.0 then decoupling happens due to */
       /*       reduced accuracy requirements ! (because in this case */
       /*       we would not merge the corresponding two diagonal blocks) */
 
@@ -570,21 +542,18 @@ L20:
 
       for (i = 0; i < nk; ++i) {
         ip = np + i + 1;
-        for (j = 0; j <= i; ++j) {
-          jp = np + j + 1;
-          z[ip + jp*ldz] = d[i + (j + start*l2d)*l1d];
-        }
+        for (j = 0; j <= i; ++j) z[ip + (np+j+1)*ldz] = d[i + (j + start*l2d)*l1d];
       }
 
       /* check whether there is enough workspace */
 
       spneed = 2*nk*nk + nk * 6 + 1;
-      if (spneed > lwork) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: not enough workspace for DSYEVD, info = %d",lwork - 200 - spneed);
+      PetscCheck(spneed<=lwork,PETSC_COMM_SELF,PETSC_ERR_MEM,"dsbtdc: not enough workspace for DSYEVD, info = %" PetscBLASInt_FMT,lwork - 200 - spneed);
 
       PetscStackCallBLAS("LAPACKsyevd",LAPACKsyevd_("V", "L", &nk,
                     &z[np + np*ldz], &ldz, &ev[np],
                     work, &lwork, &iwork[nblks-1], &liwork, info));
-      if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: Error in DSYEVD, info = %d",*info);
+      SlepcCheckLapackInfo("syevd",*info);
       start = iend + 1;
       np += nk;
 
@@ -622,7 +591,7 @@ L20:
       /* scale the diagonal block */
       PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("L", &zero, &zero,
                     &anorm, &done, &ksk, &ksk, &d[k*l2d*l1d], &l1d, info));
-      if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: Error in DLASCL, info = %d",*info);
+      SlepcCheckLapackInfo("lascl",*info);
 
       /* scale the (approximated) off-diagonal block by dividing its */
       /* singular values */
@@ -639,11 +608,10 @@ L20:
     /* call the block-tridiagonal divide-and-conquer on the */
     /* irreducible subproblem which has been identified */
 
-    ierr = BDC_dibtdc_(jobz, nk, nrblks, &ksizes[start], &d[start*l1d*l2d], l1d, l2d,
+    PetscCall(BDC_dibtdc_(jobz, nk, nrblks, &ksizes[start], &d[start*l1d*l2d], l1d, l2d,
                 &e[start*l2e*l1e], &iwork[start], l1e, l2e, tau2, &ev[np],
-                &z[np + np*ldz], ldz, work, lwork, &iwork[nblks-1], liwork, info, 1);
-                CHKERRQ(ierr);
-    if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: Error in DIBTDC, info = %d",*info);
+                &z[np + np*ldz], ldz, work, lwork, &iwork[nblks-1], liwork, info, 1));
+    PetscCheck(!*info,PETSC_COMM_SELF,PETSC_ERR_LIB,"dsbtdc: Error in DIBTDC, info = %" PetscBLASInt_FMT,*info);
 
 /* ************************************************************************** */
 
@@ -651,7 +619,7 @@ L20:
 
     PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G", &zero, &zero, &done,
             &anorm, &nk, &one, &ev[np], &nk, info));
-    if (*info) SETERRQ1(PETSC_COMM_SELF,1,"dsbtdc: Error in DLASCL, info = %d",*info);
+    SlepcCheckLapackInfo("lascl",*info);
 
     start = iend + 1;
     np += nk;
@@ -666,7 +634,7 @@ L20:
   /* (and the associated eigenvectors) across the irreducible parts */
   /* into ascending order.... */
 
-  /*  IF( NRBLKS.LT.NBLKS )THEN */
+  /*  IF(NRBLKS.LT.NBLKS)THEN */
 
   /*    Use Selection Sort to minimize swaps of eigenvectors */
 
@@ -691,13 +659,12 @@ L20:
   /*    approximations).............................................. */
 
   *mingap = ev[1] - ev[0];
-  if (*mingap < 0.) SETERRQ2(PETSC_COMM_SELF,1,"dsbtdc: Eigenvalue approximations are not ordered properly. Approximation %d is larger than approximation %d.",1,2);
+  PetscCheck(*mingap>=0.,PETSC_COMM_SELF,PETSC_ERR_LIB,"dsbtdc: Eigenvalue approximations are not ordered properly. Approximation 1 is larger than approximation 2.");
   *mingapi = 1;
   for (i = 2; i < n; ++i) {
     absdiff = ev[i] - ev[i-1];
-    if (absdiff < 0.) {
-      SETERRQ2(PETSC_COMM_SELF,1,"dsbtdc: Eigenvalue approximations are not ordered properly. Approximation %d is larger than approximation %d.",i,i+1);
-    } else if (absdiff < *mingap) {
+    PetscCheck(absdiff>=0.,PETSC_COMM_SELF,PETSC_ERR_LIB,"dsbtdc: Eigenvalue approximations are not ordered properly. Approximation %" PetscBLASInt_FMT " is larger than approximation %" PetscBLASInt_FMT ".",i,i+1);
+    else if (absdiff < *mingap) {
       *mingap = absdiff;
       *mingapi = i;
     }
@@ -708,6 +675,4 @@ L20:
 
   if (*mingap <= tol / 10) *info = -103;
   PetscFunctionReturn(0);
-#endif
 }
-
