@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -62,8 +62,6 @@ PetscErrorCode DSGetLeadingDimension(DS ds,PetscInt *ld)
 @*/
 PetscErrorCode DSSetState(DS ds,DSStateType state)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidLogicalCollectiveEnum(ds,state,2);
@@ -72,7 +70,7 @@ PetscErrorCode DSSetState(DS ds,DSStateType state)
     case DS_STATE_INTERMEDIATE:
     case DS_STATE_CONDENSED:
     case DS_STATE_TRUNCATED:
-      if (ds->state<state) { ierr = PetscInfo(ds,"DS state has been increased\n");CHKERRQ(ierr); }
+      if (ds->state!=state) PetscCall(PetscInfo(ds,"State has changed from %s to %s\n",DSStateTypes[ds->state],DSStateTypes[state]));
       ds->state = state;
       break;
     default:
@@ -113,56 +111,53 @@ PetscErrorCode DSGetState(DS ds,DSStateType *state)
    Input Parameters:
 +  ds - the direct solver context
 .  n  - the new size
-.  m  - the new column size (only for DSSVD)
 .  l  - number of locked (inactive) leading columns
 -  k  - intermediate dimension (e.g., position of arrow)
 
    Notes:
    The internal arrays are not reallocated.
 
-   The value m is not used except in the case of DSSVD, pass 0 otherwise.
+   Some DS types have additional dimensions, e.g. the number of columns
+   in DSSVD. For these, you should call a specific interface function.
 
    Level: intermediate
 
-.seealso: DSGetDimensions(), DSAllocate()
+.seealso: DSGetDimensions(), DSAllocate(), DSSVDSetDimensions()
 @*/
-PetscErrorCode DSSetDimensions(DS ds,PetscInt n,PetscInt m,PetscInt l,PetscInt k)
+PetscErrorCode DSSetDimensions(DS ds,PetscInt n,PetscInt l,PetscInt k)
 {
+  PetscInt       on,ol,ok;
+  PetscBool      issvd;
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   PetscValidLogicalCollectiveInt(ds,n,2);
-  PetscValidLogicalCollectiveInt(ds,m,3);
-  PetscValidLogicalCollectiveInt(ds,l,4);
-  PetscValidLogicalCollectiveInt(ds,k,5);
+  PetscValidLogicalCollectiveInt(ds,l,3);
+  PetscValidLogicalCollectiveInt(ds,k,4);
+  on = ds->n; ol = ds->l; ok = ds->k;
   if (n==PETSC_DECIDE || n==PETSC_DEFAULT) {
     ds->n = ds->ld;
   } else {
-    if (n<0 || n>ds->ld) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of n. Must be between 0 and ld");
-    if (ds->extrarow && n+1>ds->ld) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"A value of n equal to ld leaves no room for extra row");
+    PetscCheck(n>=0 && n<=ds->ld,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of n. Must be between 0 and ld");
+    PetscCall(PetscObjectTypeCompareAny((PetscObject)ds,&issvd,DSSVD,DSGSVD,""));  /* SVD and GSVD have extra column instead of extra row */
+    PetscCheck(!ds->extrarow || n<ds->ld || issvd,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"A value of n equal to ld leaves no room for extra row");
     ds->n = n;
   }
   ds->t = ds->n;   /* truncated length equal to the new dimension */
-  if (m) {
-    if (m==PETSC_DECIDE || m==PETSC_DEFAULT) {
-      ds->m = ds->ld;
-    } else {
-      if (m<0 || m>ds->ld) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of m. Must be between 0 and ld");
-      ds->m = m;
-    }
-  }
   if (l==PETSC_DECIDE || l==PETSC_DEFAULT) {
     ds->l = 0;
   } else {
-    if (l<0 || l>ds->n) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of l. Must be between 0 and n");
+    PetscCheck(l>=0 && l<=ds->n,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of l. Must be between 0 and n");
     ds->l = l;
   }
   if (k==PETSC_DECIDE || k==PETSC_DEFAULT) {
     ds->k = ds->n/2;
   } else {
-    if (k<0 || k>ds->n) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of k. Must be between 0 and n");
+    PetscCheck(k>=0 || k<=ds->n,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of k. Must be between 0 and n");
     ds->k = k;
   }
+  if (on!=ds->n || ol!=ds->l || ok!=ds->k) PetscCall(PetscInfo(ds,"New dimensions are: n=%" PetscInt_FMT ", l=%" PetscInt_FMT ", k=%" PetscInt_FMT "\n",ds->n,ds->l,ds->k));
   PetscFunctionReturn(0);
 }
 
@@ -174,9 +169,8 @@ PetscErrorCode DSSetDimensions(DS ds,PetscInt n,PetscInt m,PetscInt l,PetscInt k
    Input Parameter:
 .  ds - the direct solver context
 
-   Output Parameter:
+   Output Parameters:
 +  n  - the current size
-.  m  - the current column size (only for DSSVD)
 .  l  - number of locked (inactive) leading columns
 .  k  - intermediate dimension (e.g., position of arrow)
 -  t  - truncated length
@@ -185,17 +179,19 @@ PetscErrorCode DSSetDimensions(DS ds,PetscInt n,PetscInt m,PetscInt l,PetscInt k
    The t parameter makes sense only if DSTruncate() has been called.
    Otherwise its value equals n.
 
+   Some DS types have additional dimensions, e.g. the number of columns
+   in DSSVD. For these, you should call a specific interface function.
+
    Level: intermediate
 
-.seealso: DSSetDimensions(), DSTruncate()
+.seealso: DSSetDimensions(), DSTruncate(), DSSVDGetDimensions()
 @*/
-PetscErrorCode DSGetDimensions(DS ds,PetscInt *n,PetscInt *m,PetscInt *l,PetscInt *k,PetscInt *t)
+PetscErrorCode DSGetDimensions(DS ds,PetscInt *n,PetscInt *l,PetscInt *k,PetscInt *t)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   if (n) *n = ds->n;
-  if (m) *m = ds->m;
   if (l) *l = ds->l;
   if (k) *k = ds->k;
   if (t) *t = ds->t;
@@ -208,36 +204,50 @@ PetscErrorCode DSGetDimensions(DS ds,PetscInt *n,PetscInt *m,PetscInt *l,PetscIn
    Logically Collective on ds
 
    Input Parameters:
-+  ds - the direct solver context
--  n  - the new size
++  ds   - the direct solver context
+.  n    - the new size
+-  trim - a flag to indicate if the factorization must be trimmed
 
    Note:
-   The new size is set to n. In cases where the extra row is meaningful,
-   the first n elements are kept as the extra row for the new system.
+   The new size is set to n. Note that in some cases the new size could
+   be n+1 or n-1 to avoid breaking a 2x2 diagonal block (e.g. in real
+   Schur form). In cases where the extra row is meaningful, the first
+   n elements are kept as the extra row for the new system.
+
+   If the flag trim is turned on, it resets the locked and intermediate
+   dimensions to zero, see DSSetDimensions(), and sets the state to RAW.
+   It also cleans the extra row if being used.
+
+   The typical usage of trim=true is to truncate the Schur decomposition
+   at the end of a Krylov iteration. In this case, the state must be
+   changed to RAW so that DSVectors() computes eigenvectors from scratch.
 
    Level: advanced
 
 .seealso: DSSetDimensions(), DSSetExtraRow(), DSStateType
 @*/
-PetscErrorCode DSTruncate(DS ds,PetscInt n)
+PetscErrorCode DSTruncate(DS ds,PetscInt n,PetscBool trim)
 {
-  PetscErrorCode ierr;
+  DSStateType    old;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  DSCheckSolved(ds,1);
   PetscValidLogicalCollectiveInt(ds,n,2);
-  if (!ds->ops->truncate) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  if (n<ds->l || n>ds->n) SETERRQ3(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of n (%D). Must be between l (%D) and n (%D)",n,ds->l,ds->n);
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->truncate)(ds,n);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ds->state = DS_STATE_TRUNCATED;
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscValidLogicalCollectiveBool(ds,trim,3);
+  PetscCheck(ds->ops->truncate,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCheck(n>=ds->l && n<=ds->n,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of n (%" PetscInt_FMT "). Must be between l (%" PetscInt_FMT ") and n (%" PetscInt_FMT ")",n,ds->l,ds->n);
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->truncate)(ds,n,trim));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
+  PetscCall(PetscInfo(ds,"Decomposition %s to size n=%" PetscInt_FMT "\n",trim?"trimmed":"truncated",ds->n));
+  old = ds->state;
+  ds->state = trim? DS_STATE_RAW: DS_STATE_TRUNCATED;
+  if (old!=ds->state) PetscCall(PetscInfo(ds,"State has changed from %s to %s\n",DSStateTypes[old],DSStateTypes[ds->state]));
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -263,16 +273,14 @@ PetscErrorCode DSTruncate(DS ds,PetscInt n)
 @*/
 PetscErrorCode DSMatGetSize(DS ds,DSMatType t,PetscInt *m,PetscInt *n)
 {
-  PetscErrorCode ierr;
   PetscInt       rows,cols;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckValidMat(ds,t,2);
-  if (ds->ops->matgetsize) {
-    ierr = (*ds->ops->matgetsize)(ds,t,&rows,&cols);CHKERRQ(ierr);
-  } else {
+  if (ds->ops->matgetsize) PetscCall((*ds->ops->matgetsize)(ds,t,&rows,&cols));
+  else {
     if (ds->state==DS_STATE_TRUNCATED && t>=DS_MAT_Q) rows = ds->t;
     else rows = (t==DS_MAT_A && ds->extrarow)? ds->n+1: ds->n;
     cols = ds->n;
@@ -304,16 +312,69 @@ PetscErrorCode DSMatGetSize(DS ds,DSMatType t,PetscInt *m,PetscInt *n)
 @*/
 PetscErrorCode DSMatIsHermitian(DS ds,DSMatType t,PetscBool *flg)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckValidMat(ds,t,2);
   PetscValidBoolPointer(flg,3);
-  if (ds->ops->hermitian) {
-    ierr = (*ds->ops->hermitian)(ds,t,flg);CHKERRQ(ierr);
-  } else *flg = PETSC_FALSE;
+  if (ds->ops->hermitian) PetscCall((*ds->ops->hermitian)(ds,t,flg));
+  else *flg = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode DSGetTruncateSize_Default(DS ds,PetscInt l,PetscInt n,PetscInt *k)
+{
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar *A = ds->mat[DS_MAT_A];
+#endif
+
+  PetscFunctionBegin;
+#if !defined(PETSC_USE_COMPLEX)
+  if (A[l+(*k)+(l+(*k)-1)*ds->ld] != 0.0) {
+    if (l+(*k)<n-1) (*k)++;
+    else (*k)--;
+  }
+#endif
+  PetscFunctionReturn(0);
+}
+
+/*@
+   DSGetTruncateSize - Gets the correct size to be used in DSTruncate()
+   to avoid breaking a 2x2 block.
+
+   Not Collective
+
+   Input Parameters:
++  ds - the direct solver context
+.  l  - the size of the locked part (set to 0 to use ds->l)
+.  n  - the total matrix size (set to 0 to use ds->n)
+-  k  - the wanted truncation size
+
+   Output Parameter:
+.  k  - the possibly modified value of the truncation size
+
+   Notes:
+   This should be called before DSTruncate() to make sure that the truncation
+   does not break a 2x2 block corresponding to a complex conjugate eigenvalue.
+
+   The total size is n (either user-provided or ds->n if 0 is passed). The
+   size where the truncation is intended is equal to l+k (where l can be
+   equal to the locked size ds->l if set to 0). Then if there is a 2x2 block
+   at the l+k limit, the value of k is increased (or decreased) by 1.
+
+   Level: advanced
+
+.seealso: DSTruncate(), DSSetDimensions()
+@*/
+PetscErrorCode DSGetTruncateSize(DS ds,PetscInt l,PetscInt n,PetscInt *k)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ds,DS_CLASSID,1);
+  DSCheckAlloc(ds,1);
+  PetscValidLogicalCollectiveInt(ds,l,2);
+  PetscValidLogicalCollectiveInt(ds,n,3);
+  PetscValidIntPointer(k,4);
+  if (ds->ops->gettruncatesize) PetscCall((*ds->ops->gettruncatesize)(ds,l?l:ds->l,n?n:ds->n,k));
   PetscFunctionReturn(0);
 }
 
@@ -346,7 +407,6 @@ PetscErrorCode DSMatIsHermitian(DS ds,DSMatType t,PetscBool *flg)
 @*/
 PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
 {
-  PetscErrorCode ierr;
   PetscInt       j,rows,cols,arows,acols;
   PetscBool      create=PETSC_FALSE,flg;
   PetscScalar    *pA,*M;
@@ -356,34 +416,30 @@ PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
   DSCheckAlloc(ds,1);
   DSCheckValidMat(ds,m,2);
   PetscValidPointer(A,3);
-  if (m==DS_MAT_T || m==DS_MAT_D) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Not implemented for T or D matrices");
+  PetscCheck(m!=DS_MAT_T && m!=DS_MAT_D,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Not implemented for T or D matrices");
 
-  ierr = DSMatGetSize(ds,m,&rows,&cols);CHKERRQ(ierr);
+  PetscCall(DSMatGetSize(ds,m,&rows,&cols));
   if (!ds->omat[m]) create=PETSC_TRUE;
   else {
-    ierr = MatGetSize(ds->omat[m],&arows,&acols);CHKERRQ(ierr);
+    PetscCall(MatGetSize(ds->omat[m],&arows,&acols));
     if (arows!=rows || acols!=cols) {
-      ierr = MatDestroy(&ds->omat[m]);CHKERRQ(ierr);
+      PetscCall(MatDestroy(&ds->omat[m]));
       create=PETSC_TRUE;
     }
   }
-  if (create) {
-    ierr = MatCreateSeqDense(PETSC_COMM_SELF,rows,cols,NULL,&ds->omat[m]);CHKERRQ(ierr);
-  }
+  if (create) PetscCall(MatCreateSeqDense(PETSC_COMM_SELF,rows,cols,NULL,&ds->omat[m]));
 
   /* set Hermitian flag */
-  ierr = DSMatIsHermitian(ds,m,&flg);CHKERRQ(ierr);
-  ierr = MatSetOption(ds->omat[m],MAT_HERMITIAN,flg);CHKERRQ(ierr);
+  PetscCall(DSMatIsHermitian(ds,m,&flg));
+  PetscCall(MatSetOption(ds->omat[m],MAT_HERMITIAN,flg));
 
   /* copy entries */
-  ierr = PetscObjectReference((PetscObject)ds->omat[m]);CHKERRQ(ierr);
+  PetscCall(PetscObjectReference((PetscObject)ds->omat[m]));
   *A = ds->omat[m];
   M  = ds->mat[m];
-  ierr = MatDenseGetArray(*A,&pA);CHKERRQ(ierr);
-  for (j=0;j<cols;j++) {
-    ierr = PetscArraycpy(pA+j*rows,M+j*ds->ld,rows);CHKERRQ(ierr);
-  }
-  ierr = MatDenseRestoreArray(*A,&pA);CHKERRQ(ierr);
+  PetscCall(MatDenseGetArray(*A,&pA));
+  for (j=0;j<cols;j++) PetscCall(PetscArraycpy(pA+j*rows,M+j*ds->ld,rows));
+  PetscCall(MatDenseRestoreArray(*A,&pA));
   PetscFunctionReturn(0);
 }
 
@@ -411,7 +467,6 @@ PetscErrorCode DSGetMat(DS ds,DSMatType m,Mat *A)
 @*/
 PetscErrorCode DSRestoreMat(DS ds,DSMatType m,Mat *A)
 {
-  PetscErrorCode ierr;
   PetscInt       j,rows,cols;
   PetscScalar    *pA,*M;
 
@@ -420,17 +475,15 @@ PetscErrorCode DSRestoreMat(DS ds,DSMatType m,Mat *A)
   DSCheckAlloc(ds,1);
   DSCheckValidMat(ds,m,2);
   PetscValidPointer(A,3);
-  if (!ds->omat[m]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"DSRestoreMat must match a previous call to DSGetMat");
-  if (ds->omat[m]!=*A) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Mat argument is not the same as the one obtained with DSGetMat");
+  PetscCheck(ds->omat[m],PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"DSRestoreMat must match a previous call to DSGetMat");
+  PetscCheck(ds->omat[m]==*A,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Mat argument is not the same as the one obtained with DSGetMat");
 
-  ierr = MatGetSize(*A,&rows,&cols);CHKERRQ(ierr);
+  PetscCall(MatGetSize(*A,&rows,&cols));
   M  = ds->mat[m];
-  ierr = MatDenseGetArray(*A,&pA);CHKERRQ(ierr);
-  for (j=0;j<cols;j++) {
-    ierr = PetscArraycpy(M+j*ds->ld,pA+j*rows,rows);CHKERRQ(ierr);
-  }
-  ierr = MatDenseRestoreArray(*A,&pA);CHKERRQ(ierr);
-  ierr = MatDestroy(A);CHKERRQ(ierr);
+  PetscCall(MatDenseGetArray(*A,&pA));
+  for (j=0;j<cols;j++) PetscCall(PetscArraycpy(M+j*ds->ld,pA+j*rows,rows));
+  PetscCall(MatDenseRestoreArray(*A,&pA));
+  PetscCall(MatDestroy(A));
   PetscFunctionReturn(0);
 }
 
@@ -458,7 +511,7 @@ PetscErrorCode DSGetArray(DS ds,DSMatType m,PetscScalar *a[])
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   DSCheckValidMat(ds,m,2);
-  PetscValidPointer(a,2);
+  PetscValidPointer(a,3);
   *a = ds->mat[m];
   CHKMEMQ;
   PetscFunctionReturn(0);
@@ -480,16 +533,14 @@ PetscErrorCode DSGetArray(DS ds,DSMatType m,PetscScalar *a[])
 @*/
 PetscErrorCode DSRestoreArray(DS ds,DSMatType m,PetscScalar *a[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   DSCheckValidMat(ds,m,2);
-  PetscValidPointer(a,2);
+  PetscValidPointer(a,3);
   CHKMEMQ;
   *a = 0;
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -517,7 +568,7 @@ PetscErrorCode DSGetArrayReal(DS ds,DSMatType m,PetscReal *a[])
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   DSCheckValidMatReal(ds,m,2);
-  PetscValidPointer(a,2);
+  PetscValidPointer(a,3);
   *a = ds->rmat[m];
   CHKMEMQ;
   PetscFunctionReturn(0);
@@ -539,16 +590,14 @@ PetscErrorCode DSGetArrayReal(DS ds,DSMatType m,PetscReal *a[])
 @*/
 PetscErrorCode DSRestoreArrayReal(DS ds,DSMatType m,PetscReal *a[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   DSCheckAlloc(ds,1);
   DSCheckValidMatReal(ds,m,2);
-  PetscValidPointer(a,2);
+  PetscValidPointer(a,3);
   CHKMEMQ;
   *a = 0;
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -572,22 +621,22 @@ PetscErrorCode DSRestoreArrayReal(DS ds,DSMatType m,PetscReal *a[])
 @*/
 PetscErrorCode DSSolve(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   PetscValidScalarPointer(eigr,2);
   if (ds->state>=DS_STATE_CONDENSED) PetscFunctionReturn(0);
-  if (!ds->ops->solve[ds->method]) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The specified method number does not exist for this DS");
-  ierr = PetscLogEventBegin(DS_Solve,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->solve[ds->method])(ds,eigr,eigi);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Solve,ds,0,0,0);CHKERRQ(ierr);
+  PetscCheck(ds->ops->solve[ds->method],PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"The specified method number does not exist for this DS");
+  PetscCall(PetscInfo(ds,"Starting solve with problem sizes: n=%" PetscInt_FMT ", l=%" PetscInt_FMT ", k=%" PetscInt_FMT "\n",ds->n,ds->l,ds->k));
+  PetscCall(PetscLogEventBegin(DS_Solve,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->solve[ds->method])(ds,eigr,eigi));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Solve,ds,0,0,0));
+  PetscCall(PetscInfo(ds,"State has changed from %s to CONDENSED\n",DSStateTypes[ds->state]));
   ds->state = DS_STATE_CONDENSED;
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -626,7 +675,6 @@ PetscErrorCode DSSolve(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 @*/
 PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
-  PetscErrorCode ierr;
   PetscInt       i;
 
   PetscFunctionBegin;
@@ -635,18 +683,19 @@ PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,
   DSCheckSolved(ds,1);
   PetscValidScalarPointer(eigr,2);
   if (rr) PetscValidScalarPointer(rr,4);
-  if (ds->state==DS_STATE_TRUNCATED) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot sort a truncated DS");
-  if (!ds->ops->sort) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  if (!ds->sc) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Must provide a sorting criterion first");
-  if (k && !rr) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Argument k can only be used together with rr");
+  PetscCheck(ds->state<DS_STATE_TRUNCATED,PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot sort a truncated DS");
+  PetscCheck(ds->ops->sort,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCheck(ds->sc,PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Must provide a sorting criterion first");
+  PetscCheck(!k || rr,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Argument k can only be used together with rr");
 
   for (i=0;i<ds->n;i++) ds->perm[i] = i;   /* initialize to trivial permutation */
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->sort)(ds,eigr,eigi,rr,ri,k);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->sort)(ds,eigr,eigi,rr,ri,k));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
+  PetscCall(PetscInfo(ds,"Finished sorting\n"));
   PetscFunctionReturn(0);
 }
 
@@ -675,23 +724,22 @@ PetscErrorCode DSSort(DS ds,PetscScalar *eigr,PetscScalar *eigi,PetscScalar *rr,
 @*/
 PetscErrorCode DSSortWithPermutation(DS ds,PetscInt *perm,PetscScalar *eigr,PetscScalar *eigi)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckSolved(ds,1);
   PetscValidIntPointer(perm,2);
   PetscValidScalarPointer(eigr,3);
-  if (ds->state==DS_STATE_TRUNCATED) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot sort a truncated DS");
-  if (!ds->ops->sortperm) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCheck(ds->state<DS_STATE_TRUNCATED,PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Cannot sort a truncated DS");
+  PetscCheck(ds->ops->sortperm,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
 
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->sortperm)(ds,perm,eigr,eigi);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->sortperm)(ds,perm,eigr,eigi));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
+  PetscCall(PetscInfo(ds,"Finished sorting\n"));
   PetscFunctionReturn(0);
 }
 
@@ -702,7 +750,7 @@ PetscErrorCode DSSortWithPermutation(DS ds,PetscInt *perm,PetscScalar *eigr,Pets
    Collective on ds
 
    Input Parameter:
-+  ds   - the direct solver context
+.  ds   - the direct solver context
 
    Input/Output Parameters:
 +  eigr - (optional) array with the computed eigenvalues (real part)
@@ -727,21 +775,19 @@ PetscErrorCode DSSortWithPermutation(DS ds,PetscInt *perm,PetscScalar *eigr,Pets
 @*/
 PetscErrorCode DSSynchronize(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 {
-  PetscErrorCode ierr;
   PetscMPIInt    size;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRQ(ierr);
+  PetscCallMPI(MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size));
   if (size>1 && ds->pmode==DS_PARALLEL_SYNCHRONIZED) {
-    ierr = PetscLogEventBegin(DS_Synchronize,ds,0,0,0);CHKERRQ(ierr);
-    if (ds->ops->synchronize) {
-      ierr = (*ds->ops->synchronize)(ds,eigr,eigi);CHKERRQ(ierr);
-    }
-    ierr = PetscLogEventEnd(DS_Synchronize,ds,0,0,0);CHKERRQ(ierr);
-    ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+    PetscCall(PetscLogEventBegin(DS_Synchronize,ds,0,0,0));
+    if (ds->ops->synchronize) PetscCall((*ds->ops->synchronize)(ds,eigr,eigi));
+    PetscCall(PetscLogEventEnd(DS_Synchronize,ds,0,0,0));
+    PetscCall(PetscObjectStateIncrease((PetscObject)ds));
+    PetscCall(PetscInfo(ds,"Synchronization completed (%s)\n",DSParallelTypes[ds->pmode]));
   }
   PetscFunctionReturn(0);
 }
@@ -757,13 +803,13 @@ PetscErrorCode DSSynchronize(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 -  mat - the matrix, used to indicate which vectors are required
 
    Input/Output Parameter:
--  j   - (optional) index of vector to be computed
+.  j   - (optional) index of vector to be computed
 
    Output Parameter:
 .  rnorm - (optional) computed residual norm
 
    Notes:
-   Allowed values for mat are DS_MAT_X, DS_MAT_Y, DS_MAT_U and DS_MAT_VT, to
+   Allowed values for mat are DS_MAT_X, DS_MAT_Y, DS_MAT_U and DS_MAT_V, to
    compute right or left eigenvectors, or left or right singular vectors,
    respectively.
 
@@ -785,23 +831,22 @@ PetscErrorCode DSSynchronize(DS ds,PetscScalar eigr[],PetscScalar eigi[])
 @*/
 PetscErrorCode DSVectors(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   PetscValidLogicalCollectiveEnum(ds,mat,2);
-  if (mat>=DS_NUM_MAT) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
-  if (!ds->ops->vectors) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  if (rnorm && !j) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Must give a value of j");
-  if (!ds->mat[mat]) { ierr = DSAllocateMat_Private(ds,mat);CHKERRQ(ierr); }
-  ierr = PetscLogEventBegin(DS_Vectors,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->vectors)(ds,mat,j,rnorm);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Vectors,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCheck(mat<DS_NUM_MAT,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Invalid matrix");
+  PetscCheck(ds->ops->vectors,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCheck(!rnorm || j,PetscObjectComm((PetscObject)ds),PETSC_ERR_ORDER,"Must give a value of j");
+  if (!ds->mat[mat]) PetscCall(DSAllocateMat_Private(ds,mat));
+  if (!j) PetscCall(PetscInfo(ds,"Computing all vectors on %s\n",DSMatName[mat]));
+  PetscCall(PetscLogEventBegin(DS_Vectors,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->vectors)(ds,mat,j,rnorm));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Vectors,ds,0,0,0));
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -820,19 +865,18 @@ PetscErrorCode DSVectors(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 @*/
 PetscErrorCode DSUpdateExtraRow(DS ds)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  if (!ds->ops->update) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  if (!ds->extrarow) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Should have called DSSetExtraRow");
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->update)(ds);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
+  PetscCheck(ds->ops->update,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCheck(ds->extrarow,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONGSTATE,"Should have called DSSetExtraRow");
+  PetscCall(PetscInfo(ds,"Updating extra row\n"));
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->update)(ds));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
   PetscFunctionReturn(0);
 }
 
@@ -852,19 +896,18 @@ PetscErrorCode DSUpdateExtraRow(DS ds)
 @*/
 PetscErrorCode DSCond(DS ds,PetscReal *cond)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
   PetscValidRealPointer(cond,2);
-  if (!ds->ops->cond) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->cond)(ds,cond);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
+  PetscCheck(ds->ops->cond,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->cond)(ds,cond));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
+  PetscCall(PetscInfo(ds,"Computed condition number = %g\n",(double)*cond));
   PetscFunctionReturn(0);
 }
 
@@ -897,23 +940,25 @@ PetscErrorCode DSCond(DS ds,PetscReal *cond)
    translation done previously. In that case, parameter tau is ignored.
 
    Level: developer
+
+.seealso: DSTranslateRKS()
 @*/
 PetscErrorCode DSTranslateHarmonic(DS ds,PetscScalar tau,PetscReal beta,PetscBool recover,PetscScalar *g,PetscReal *gamma)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  if (!ds->ops->transharm) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->transharm)(ds,tau,beta,recover,g,gamma);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
+  PetscCheck(ds->ops->transharm,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  if (recover) PetscCall(PetscInfo(ds,"Undoing the translation\n"));
+  else PetscCall(PetscInfo(ds,"Computing the translation\n"));
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->transharm)(ds,tau,beta,recover,g,gamma));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
   ds->state = DS_STATE_RAW;
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -940,24 +985,25 @@ PetscErrorCode DSTranslateHarmonic(DS ds,PetscScalar tau,PetscReal beta,PetscBoo
    Krylov basis.
 
    Level: developer
+
+.seealso: DSTranslateHarmonic()
 @*/
 PetscErrorCode DSTranslateRKS(DS ds,PetscScalar alpha)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ds,DS_CLASSID,1);
   PetscValidType(ds,1);
   DSCheckAlloc(ds,1);
-  if (!ds->ops->transrks) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
-  ierr = PetscLogEventBegin(DS_Other,ds,0,0,0);CHKERRQ(ierr);
-  ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
-  ierr = (*ds->ops->transrks)(ds,alpha);CHKERRQ(ierr);
-  ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(DS_Other,ds,0,0,0);CHKERRQ(ierr);
+  PetscCheck(ds->ops->transrks,PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"DS type %s",((PetscObject)ds)->type_name);
+  PetscCall(PetscInfo(ds,"Translating with alpha=%g\n",(double)PetscRealPart(alpha)));
+  PetscCall(PetscLogEventBegin(DS_Other,ds,0,0,0));
+  PetscCall(PetscFPTrapPush(PETSC_FP_TRAP_OFF));
+  PetscCall((*ds->ops->transrks)(ds,alpha));
+  PetscCall(PetscFPTrapPop());
+  PetscCall(PetscLogEventEnd(DS_Other,ds,0,0,0));
   ds->state   = DS_STATE_RAW;
   ds->compact = PETSC_FALSE;
-  ierr = PetscObjectStateIncrease((PetscObject)ds);CHKERRQ(ierr);
+  PetscCall(PetscObjectStateIncrease((PetscObject)ds));
   PetscFunctionReturn(0);
 }
 
@@ -989,7 +1035,6 @@ PetscErrorCode DSTranslateRKS(DS ds,PetscScalar alpha)
 @*/
 PetscErrorCode DSCopyMat(DS ds,DSMatType m,PetscInt mr,PetscInt mc,Mat A,PetscInt Ar,PetscInt Ac,PetscInt rows,PetscInt cols,PetscBool out)
 {
-  PetscErrorCode ierr;
   PetscInt       j,mrows,mcols,arows,acols;
   PetscScalar    *pA,*M;
 
@@ -1008,26 +1053,22 @@ PetscErrorCode DSCopyMat(DS ds,DSMatType m,PetscInt mr,PetscInt mc,Mat A,PetscIn
   PetscValidLogicalCollectiveBool(ds,out,10);
   if (!rows || !cols) PetscFunctionReturn(0);
 
-  ierr = DSMatGetSize(ds,m,&mrows,&mcols);CHKERRQ(ierr);
-  ierr = MatGetSize(A,&arows,&acols);CHKERRQ(ierr);
-  if (m==DS_MAT_T || m==DS_MAT_D) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Not implemented for T or D matrices");
-  if (mr<0 || mr>=mrows) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial row in m");
-  if (mc<0 || mc>=mcols) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial column in m");
-  if (Ar<0 || Ar>=arows) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial row in A");
-  if (Ac<0 || Ac>=acols) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial column in A");
-  if (mr+rows>mrows || Ar+rows>arows) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid number of rows");
-  if (mc+cols>mcols || Ac+cols>acols) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid number of columns");
+  PetscCall(DSMatGetSize(ds,m,&mrows,&mcols));
+  PetscCall(MatGetSize(A,&arows,&acols));
+  PetscCheck(m!=DS_MAT_T && m!=DS_MAT_D,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Not implemented for T or D matrices");
+  PetscCheck(mr>=0 && mr<mrows,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial row in m");
+  PetscCheck(mc>=0 && mc<mcols,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial column in m");
+  PetscCheck(Ar>=0 && Ar<arows,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial row in A");
+  PetscCheck(Ac>=0 && Ac<acols,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid initial column in A");
+  PetscCheck(mr+rows<=mrows && Ar+rows<=arows,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid number of rows");
+  PetscCheck(mc+cols<=mcols && Ac+cols<=acols,PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid number of columns");
 
   M  = ds->mat[m];
-  ierr = MatDenseGetArray(A,&pA);CHKERRQ(ierr);
+  PetscCall(MatDenseGetArray(A,&pA));
   for (j=0;j<cols;j++) {
-    if (out) {
-      ierr = PetscArraycpy(pA+(Ac+j)*arows+Ar,M+(mc+j)*ds->ld+mr,rows);CHKERRQ(ierr);
-    } else {
-      ierr = PetscArraycpy(M+(mc+j)*ds->ld+mr,pA+(Ac+j)*arows+Ar,rows);CHKERRQ(ierr);
-    }
+    if (out) PetscCall(PetscArraycpy(pA+(Ac+j)*arows+Ar,M+(mc+j)*ds->ld+mr,rows));
+    else PetscCall(PetscArraycpy(M+(mc+j)*ds->ld+mr,pA+(Ac+j)*arows+Ar,rows));
   }
-  ierr = MatDenseRestoreArray(A,&pA);CHKERRQ(ierr);
+  PetscCall(MatDenseRestoreArray(A,&pA));
   PetscFunctionReturn(0);
 }
-
