@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -35,7 +35,7 @@ static PetscErrorCode SlepcMatDenseSqrt(PetscBLASInt n,PetscScalar *T,PetscBLASI
 #else
     sj = (j==n-1 || T[j+1+j*ld] == 0.0)? 1: 2;
     if (sj==1) {
-      if (T[j+j*ld]<0.0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_USER_INPUT,"Matrix has a real negative eigenvalue, no real primary square root exists");
+      PetscCheck(T[j+j*ld]>=0.0,PETSC_COMM_SELF,PETSC_ERR_USER_INPUT,"Matrix has a real negative eigenvalue, no real primary square root exists");
       T[j+j*ld] = PetscSqrtReal(T[j+j*ld]);
     } else {
       /* square root of 2x2 block */
@@ -65,7 +65,7 @@ static PetscErrorCode SlepcMatDenseSqrt(PetscBLASInt n,PetscScalar *T,PetscBLASI
       if (r) PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&si,&sj,&r,&mone,T+i+(i+si)*ld,&ld,T+i+si+j*ld,&ld,&one,T+i+j*ld,&ld));
       PetscStackCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","N",&ione,&si,&sj,T+i+i*ld,&ld,T+j+j*ld,&ld,T+i+j*ld,&ld,&scal,&info));
       SlepcCheckLapackInfo("trsyl",info);
-      if (scal!=1.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
+      PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",(double)scal);
     }
     if (sj==2) j++;
   }
@@ -81,7 +81,6 @@ static PetscErrorCode SlepcMatDenseSqrt(PetscBLASInt n,PetscScalar *T,PetscBLASI
  */
 PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,PetscBool firstonly)
 {
-  PetscErrorCode ierr;
   PetscBLASInt   i,j,k,r,ione=1,sdim,lwork,*s,*p,info,bs=BLOCKSIZE;
   PetscScalar    *wr,*W,*Q,*work,one=1.0,zero=0.0,mone=-1.0;
   PetscInt       m,nblk;
@@ -100,10 +99,10 @@ PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,
 
   /* compute Schur decomposition A*Q = Q*T */
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscMalloc7(m,&wr,m,&wi,m*k,&W,m*m,&Q,lwork,&work,nblk,&s,nblk,&p);CHKERRQ(ierr);
+  PetscCall(PetscMalloc7(m,&wr,m,&wi,m*k,&W,m*m,&Q,lwork,&work,nblk,&s,nblk,&p));
   PetscStackCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,T,&ld,&sdim,wr,wi,Q,&ld,work,&lwork,NULL,&info));
 #else
-  ierr = PetscMalloc7(m,&wr,m,&rwork,m*k,&W,m*m,&Q,lwork,&work,nblk,&s,nblk,&p);CHKERRQ(ierr);
+  PetscCall(PetscMalloc7(m,&wr,m,&rwork,m*k,&W,m*m,&Q,lwork,&work,nblk,&s,nblk,&p));
   PetscStackCallBLAS("LAPACKgees",LAPACKgees_("V","N",NULL,&n,T,&ld,&sdim,wr,Q,&ld,work,&lwork,rwork,NULL,&info));
 #endif
   SlepcCheckLapackInfo("gees",info);
@@ -124,14 +123,14 @@ PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,
 
   for (j=0;j<nblk;j++) {
     /* evaluate f(T_jj) */
-    ierr = SlepcMatDenseSqrt(s[j],T+p[j]+p[j]*ld,ld);CHKERRQ(ierr);
+    PetscCall(SlepcMatDenseSqrt(s[j],T+p[j]+p[j]*ld,ld));
     for (i=j-1;i>=0;i--) {
       /* solve Sylvester equation for block (i,j) */
       r = p[j]-p[i]-s[i];
       if (r) PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",s+i,s+j,&r,&mone,T+p[i]+(p[i]+s[i])*ld,&ld,T+p[i]+s[i]+p[j]*ld,&ld,&one,T+p[i]+p[j]*ld,&ld));
       PetscStackCallBLAS("LAPACKtrsyl",LAPACKtrsyl_("N","N",&ione,s+i,s+j,T+p[i]+p[i]*ld,&ld,T+p[j]+p[j]*ld,&ld,T+p[i]+p[j]*ld,&ld,&scal,&info));
       SlepcCheckLapackInfo("trsyl",info);
-      if (scal!=1.0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",scal);
+      PetscCheck(scal==1.0,PETSC_COMM_SELF,PETSC_ERR_SUP,"Current implementation cannot handle scale factor %g",(double)scal);
     }
   }
 
@@ -140,12 +139,12 @@ PetscErrorCode FNSqrtmSchur(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLASInt ld,
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&k,&n,&one,Q,&ld,W,&ld,&zero,T,&ld));
 
   /* flop count: Schur decomposition, triangular square root, and backtransform */
-  ierr = PetscLogFlops(25.0*n*n*n+n*n*n/3.0+4.0*n*n*k);CHKERRQ(ierr);
+  PetscCall(PetscLogFlops(25.0*n*n*n+n*n*n/3.0+4.0*n*n*k));
 
 #if !defined(PETSC_USE_COMPLEX)
-  ierr = PetscFree7(wr,wi,W,Q,work,s,p);CHKERRQ(ierr);
+  PetscCall(PetscFree7(wr,wi,W,Q,work,s,p));
 #else
-  ierr = PetscFree7(wr,rwork,W,Q,work,s,p);CHKERRQ(ierr);
+  PetscCall(PetscFree7(wr,rwork,W,Q,work,s,p));
 #endif
   PetscFunctionReturn(0);
 }
@@ -165,29 +164,28 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
   PetscBLASInt       N,i,it,*piv=NULL,info,query=-1,lwork;
   const PetscBLASInt one=1;
   PetscBool          converged=PETSC_FALSE,scale=PETSC_FALSE;
-  PetscErrorCode     ierr;
   unsigned int       ftz;
 
   PetscFunctionBegin;
   N = n*n;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
-  ierr = SlepcSetFlushToZero(&ftz);CHKERRQ(ierr);
+  PetscCall(SlepcSetFlushToZero(&ftz));
 
   /* query work size */
   PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,M,&ld,piv,&work1,&query,&info));
-  ierr = PetscBLASIntCast((PetscInt)PetscRealPart(work1),&lwork);CHKERRQ(ierr);
-  ierr = PetscMalloc5(lwork,&work,n,&piv,n*n,&Told,n*n,&M,n*n,&invM);CHKERRQ(ierr);
-  ierr = PetscArraycpy(M,T,n*n);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast((PetscInt)PetscRealPart(work1),&lwork));
+  PetscCall(PetscMalloc5(lwork,&work,n,&piv,n*n,&Told,n*n,&M,n*n,&invM));
+  PetscCall(PetscArraycpy(M,T,n*n));
 
   if (inv) {  /* start recurrence with I instead of A */
-    ierr = PetscArrayzero(T,n*n);CHKERRQ(ierr);
+    PetscCall(PetscArrayzero(T,n*n));
     for (i=0;i<n;i++) T[i+i*ld] += 1.0;
   }
 
   for (it=0;it<DBMAXIT && !converged;it++) {
 
     if (scale) {  /* g = (abs(det(M)))^(-1/(2*n)) */
-      ierr = PetscArraycpy(invM,M,n*n);CHKERRQ(ierr);
+      PetscCall(PetscArraycpy(invM,M,n*n));
       PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,invM,&ld,piv,&info));
       SlepcCheckLapackInfo("getrf",info);
       prod = invM[0];
@@ -198,17 +196,17 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
       PetscStackCallBLAS("BLASscal",BLASscal_(&N,&alpha,T,&one));
       alpha = g*g;
       PetscStackCallBLAS("BLASscal",BLASscal_(&N,&alpha,M,&one));
-      ierr = PetscLogFlops(2.0*n*n*n/3.0+2.0*n*n);CHKERRQ(ierr);
+      PetscCall(PetscLogFlops(2.0*n*n*n/3.0+2.0*n*n));
     }
 
-    ierr = PetscArraycpy(Told,T,n*n);CHKERRQ(ierr);
-    ierr = PetscArraycpy(invM,M,n*n);CHKERRQ(ierr);
+    PetscCall(PetscArraycpy(Told,T,n*n));
+    PetscCall(PetscArraycpy(invM,M,n*n));
 
     PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,invM,&ld,piv,&info));
     SlepcCheckLapackInfo("getrf",info);
     PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,invM,&ld,piv,work,&lwork,&info));
     SlepcCheckLapackInfo("getri",info);
-    ierr = PetscLogFlops(2.0*n*n*n/3.0+4.0*n*n*n/3.0);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(2.0*n*n*n/3.0+4.0*n*n*n/3.0));
 
     for (i=0;i<n;i++) invM[i+i*ld] += 1.0;
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&spfive,Told,&ld,invM,&ld,&szero,T,&ld));
@@ -217,7 +215,7 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
     PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&N,&sone,invM,&one,M,&one));
     PetscStackCallBLAS("BLASscal",BLASscal_(&N,&sp25,M,&one));
     for (i=0;i<n;i++) M[i+i*ld] -= 0.5;
-    ierr = PetscLogFlops(2.0*n*n*n+2.0*n*n);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(2.0*n*n*n+2.0*n*n));
 
     Mres = LAPACKlange_("F",&n,&n,M,&n,rwork);
     for (i=0;i<n;i++) M[i+i*ld] += 1.0;
@@ -227,18 +225,18 @@ PetscErrorCode FNSqrtmDenmanBeavers(FN fn,PetscBLASInt n,PetscScalar *T,PetscBLA
       PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&N,&smone,T,&one,Told,&one));
       fnormdiff = LAPACKlange_("F",&n,&n,Told,&n,rwork);
       fnormT = LAPACKlange_("F",&n,&n,T,&n,rwork);
-      ierr = PetscLogFlops(7.0*n*n);CHKERRQ(ierr);
+      PetscCall(PetscLogFlops(7.0*n*n));
       reldiff = fnormdiff/fnormT;
-      ierr = PetscInfo4(fn,"it: %D reldiff: %g scale: %g tol*scale: %g\n",it,(double)reldiff,(double)g,(double)tol*g);CHKERRQ(ierr);
+      PetscCall(PetscInfo(fn,"it: %" PetscBLASInt_FMT " reldiff: %g scale: %g tol*scale: %g\n",it,(double)reldiff,(double)g,(double)(tol*g)));
       if (reldiff<1e-2) scale = PETSC_FALSE;  /* Switch off scaling */
     }
 
     if (Mres<=tol) converged = PETSC_TRUE;
   }
 
-  if (Mres>tol) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations",DBMAXIT);
-  ierr = PetscFree5(work,piv,Told,M,invM);CHKERRQ(ierr);
-  ierr = SlepcResetFlushToZero(&ftz);CHKERRQ(ierr);
+  PetscCheck(Mres<=tol,PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations",DBMAXIT);
+  PetscCall(PetscFree5(work,piv,Told,M,invM));
+  PetscCall(SlepcResetFlushToZero(&ftz));
   PetscFunctionReturn(0);
 }
 
@@ -255,38 +253,37 @@ PetscErrorCode FNSqrtmNewtonSchulz(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLAS
   PetscReal      sqrtnrm,tol,Yres=0.0,nrm,rwork[1],done=1.0;
   PetscBLASInt   info,i,it,N,one=1,zero=0;
   PetscBool      converged=PETSC_FALSE;
-  PetscErrorCode ierr;
   unsigned int   ftz;
 
   PetscFunctionBegin;
   N = n*n;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
-  ierr = SlepcSetFlushToZero(&ftz);CHKERRQ(ierr);
+  PetscCall(SlepcSetFlushToZero(&ftz));
 
-  ierr = PetscMalloc4(N,&Yold,N,&Z,N,&Zold,N,&M);CHKERRQ(ierr);
+  PetscCall(PetscMalloc4(N,&Yold,N,&Z,N,&Zold,N,&M));
 
   /* scale A so that ||I-A|| < 1 */
-  ierr = PetscArraycpy(Z,A,N);CHKERRQ(ierr);
+  PetscCall(PetscArraycpy(Z,A,N));
   for (i=0;i<n;i++) Z[i+i*ld] -= 1.0;
   nrm = LAPACKlange_("fro",&n,&n,Z,&n,rwork);
   sqrtnrm = PetscSqrtReal(nrm);
   PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&nrm,&done,&N,&one,A,&N,&info));
   SlepcCheckLapackInfo("lascl",info);
   tol *= nrm;
-  ierr = PetscInfo2(fn,"||I-A||_F = %g, new tol: %g\n",(double)nrm,(double)tol);CHKERRQ(ierr);
-  ierr = PetscLogFlops(2.0*n*n);CHKERRQ(ierr);
+  PetscCall(PetscInfo(fn,"||I-A||_F = %g, new tol: %g\n",(double)nrm,(double)tol));
+  PetscCall(PetscLogFlops(2.0*n*n));
 
   /* Z = I */
-  ierr = PetscArrayzero(Z,N);CHKERRQ(ierr);
+  PetscCall(PetscArrayzero(Z,N));
   for (i=0;i<n;i++) Z[i+i*ld] = 1.0;
 
   for (it=0;it<NSMAXIT && !converged;it++) {
     /* Yold = Y, Zold = Z */
-    ierr = PetscArraycpy(Yold,Y,N);CHKERRQ(ierr);
-    ierr = PetscArraycpy(Zold,Z,N);CHKERRQ(ierr);
+    PetscCall(PetscArraycpy(Yold,Y,N));
+    PetscCall(PetscArraycpy(Zold,Z,N));
 
     /* M = (3*I-Zold*Yold) */
-    ierr = PetscArrayzero(M,N);CHKERRQ(ierr);
+    PetscCall(PetscArrayzero(M,N));
     for (i=0;i<n;i++) M[i+i*ld] = sthree;
     PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&smone,Zold,&ld,Yold,&ld,&sone,M,&ld));
 
@@ -297,24 +294,24 @@ PetscErrorCode FNSqrtmNewtonSchulz(FN fn,PetscBLASInt n,PetscScalar *A,PetscBLAS
     /* reldiff = norm(Y-Yold,'fro')/norm(Y,'fro') */
     PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&N,&smone,Y,&one,Yold,&one));
     Yres = LAPACKlange_("fro",&n,&n,Yold,&n,rwork);
-    if (PetscIsNanReal(Yres)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"The computed norm is not-a-number");
+    PetscCheck(!PetscIsNanReal(Yres),PETSC_COMM_SELF,PETSC_ERR_FP,"The computed norm is not-a-number");
     if (Yres<=tol) converged = PETSC_TRUE;
-    ierr = PetscInfo2(fn,"it: %D res: %g\n",it,(double)Yres);CHKERRQ(ierr);
+    PetscCall(PetscInfo(fn,"it: %" PetscBLASInt_FMT " res: %g\n",it,(double)Yres));
 
-    ierr = PetscLogFlops(6.0*n*n*n+2.0*n*n);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(6.0*n*n*n+2.0*n*n));
   }
 
-  if (Yres>tol) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations",NSMAXIT);
+  PetscCheck(Yres<=tol,PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations",NSMAXIT);
 
   /* undo scaling */
   if (inv) {
-    ierr = PetscArraycpy(A,Z,N);CHKERRQ(ierr);
+    PetscCall(PetscArraycpy(A,Z,N));
     PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&sqrtnrm,&done,&N,&one,A,&N,&info));
   } else PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&done,&sqrtnrm,&N,&one,A,&N,&info));
   SlepcCheckLapackInfo("lascl",info);
 
-  ierr = PetscFree4(Yold,Z,Zold,M);CHKERRQ(ierr);
-  ierr = SlepcResetFlushToZero(&ftz);CHKERRQ(ierr);
+  PetscCall(PetscFree4(Yold,Z,Zold,M));
+  PetscCall(SlepcResetFlushToZero(&ftz));
   PetscFunctionReturn(0);
 }
 
@@ -338,86 +335,83 @@ PetscErrorCode FNSqrtmNewtonSchulz_CUDA(FN fn,PetscBLASInt n,PetscScalar *A,Pets
   const PetscBLASInt one=1,zero=0;
   PetscBool          converged=PETSC_FALSE;
   cublasHandle_t     cublasv2handle;
-  PetscErrorCode     ierr;
-  cublasStatus_t     cberr;
-  cudaError_t        cerr;
 
   PetscFunctionBegin;
-  ierr = PetscCUDAInitializeCheck();CHKERRQ(ierr); /* For CUDA event timers */
-  ierr = PetscCUBLASGetHandle(&cublasv2handle);CHKERRQ(ierr);
+  PetscCall(PetscDeviceInitialize(PETSC_DEVICE_CUDA)); /* For CUDA event timers */
+  PetscCall(PetscCUBLASGetHandle(&cublasv2handle));
   N = n*n;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
 
-  cerr = cudaMalloc((void **)&d_A,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_Yold,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_Z,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_Zold,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_M,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
+  PetscCallCUDA(cudaMalloc((void **)&d_A,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_Yold,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_Z,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_Zold,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_M,sizeof(PetscScalar)*N));
 
-  ierr = PetscLogGpuTimeBegin();CHKERRQ(ierr);
+  PetscCall(PetscLogGpuTimeBegin());
 
   /* Y = A; */
-  cerr = cudaMemcpy(d_A,A,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice);CHKERRCUDA(cerr);
+  PetscCallCUDA(cudaMemcpy(d_A,A,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice));
   /* Z = I; */
-  cerr = cudaMemset(d_Z,zero,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  ierr = set_diagonal(n,d_Z,ld,sone);CHKERRQ(cerr);
+  PetscCallCUDA(cudaMemset(d_Z,zero,sizeof(PetscScalar)*N));
+  PetscCall(set_diagonal(n,d_Z,ld,sone));
 
   /* scale A so that ||I-A|| < 1 */
-  cberr = cublasXaxpy(cublasv2handle,N,&smone,d_A,one,d_Z,one);CHKERRCUBLAS(cberr);
-  cberr = cublasXnrm2(cublasv2handle,N,d_Z,one,&nrm);CHKERRCUBLAS(cberr);
+  PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&smone,d_A,one,d_Z,one));
+  PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_Z,one,&nrm));
   sqrtnrm = PetscSqrtReal(nrm);
   alpha = 1.0/nrm;
-  cberr = cublasXscal(cublasv2handle,N,&alpha,d_A,one);CHKERRCUBLAS(cberr);
+  PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&alpha,d_A,one));
   tol *= nrm;
-  ierr = PetscInfo2(fn,"||I-A||_F = %g, new tol: %g\n",(double)nrm,(double)tol);CHKERRQ(ierr);
-  ierr = PetscLogGpuFlops(2.0*n*n);CHKERRQ(ierr);
+  PetscCall(PetscInfo(fn,"||I-A||_F = %g, new tol: %g\n",(double)nrm,(double)tol));
+  PetscCall(PetscLogGpuFlops(2.0*n*n));
 
   /* Z = I; */
-  cerr = cudaMemset(d_Z,zero,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  ierr = set_diagonal(n,d_Z,ld,sone);CHKERRQ(cerr);
+  PetscCallCUDA(cudaMemset(d_Z,zero,sizeof(PetscScalar)*N));
+  PetscCall(set_diagonal(n,d_Z,ld,sone));
 
   for (it=0;it<NSMAXIT && !converged;it++) {
     /* Yold = Y, Zold = Z */
-    cerr = cudaMemcpy(d_Yold,d_A,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice);CHKERRCUDA(cerr);
-    cerr = cudaMemcpy(d_Zold,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice);CHKERRCUDA(cerr);
+    PetscCallCUDA(cudaMemcpy(d_Yold,d_A,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
+    PetscCallCUDA(cudaMemcpy(d_Zold,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
 
     /* M = (3*I - Zold*Yold) */
-    cerr = cudaMemset(d_M,zero,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-    ierr = set_diagonal(n,d_M,ld,sthree);CHKERRQ(cerr);
-    cberr = cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&smone,d_Zold,ld,d_Yold,ld,&sone,d_M,ld);CHKERRCUBLAS(cberr);
+    PetscCallCUDA(cudaMemset(d_M,zero,sizeof(PetscScalar)*N));
+    PetscCall(set_diagonal(n,d_M,ld,sthree));
+    PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&smone,d_Zold,ld,d_Yold,ld,&sone,d_M,ld));
 
     /* Y = (1/2) * Yold * M, Z = (1/2) * M * Zold */
-    cberr = cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_Yold,ld,d_M,ld,&szero,d_A,ld);CHKERRCUBLAS(cberr);
-    cberr = cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_M,ld,d_Zold,ld,&szero,d_Z,ld);CHKERRCUBLAS(cberr);
+    PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_Yold,ld,d_M,ld,&szero,d_A,ld));
+    PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_M,ld,d_Zold,ld,&szero,d_Z,ld));
 
     /* reldiff = norm(Y-Yold,'fro')/norm(Y,'fro') */
-    cberr = cublasXaxpy(cublasv2handle,N,&smone,d_A,one,d_Yold,one);CHKERRCUBLAS(cberr);
-    cberr = cublasXnrm2(cublasv2handle,N,d_Yold,one,&Yres);CHKERRCUBLAS(cberr);
-    if (PetscIsNanReal(Yres)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FP,"The computed norm is not-a-number");
+    PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&smone,d_A,one,d_Yold,one));
+    PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_Yold,one,&Yres));
+    PetscCheck(!PetscIsNanReal(Yres),PETSC_COMM_SELF,PETSC_ERR_FP,"The computed norm is not-a-number");
     if (Yres<=tol) converged = PETSC_TRUE;
-    ierr = PetscInfo2(fn,"it: %D res: %g\n",it,(double)Yres);CHKERRQ(ierr);
+    PetscCall(PetscInfo(fn,"it: %" PetscInt_FMT " res: %g\n",it,(double)Yres));
 
-    ierr = PetscLogGpuFlops(6.0*n*n*n+2.0*n*n);CHKERRQ(ierr);
+    PetscCall(PetscLogGpuFlops(6.0*n*n*n+2.0*n*n));
   }
 
-  if (Yres>tol) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations", NSMAXIT);
+  PetscCheck(Yres<=tol,PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations", NSMAXIT);
 
   /* undo scaling */
   if (inv) {
     sqrtnrm = 1.0/sqrtnrm;
-    cberr = cublasXscal(cublasv2handle,N,&sqrtnrm,d_Z,one);CHKERRCUBLAS(cberr);
-    cerr = cudaMemcpy(A,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost);CHKERRCUDA(cerr);
+    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&sqrtnrm,d_Z,one));
+    PetscCallCUDA(cudaMemcpy(A,d_Z,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
   } else {
-    cberr = cublasXscal(cublasv2handle,N,&sqrtnrm,d_A,one);CHKERRCUBLAS(cberr);
-    cerr = cudaMemcpy(A,d_A,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost);CHKERRCUDA(cerr);
+    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&sqrtnrm,d_A,one));
+    PetscCallCUDA(cudaMemcpy(A,d_A,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
   }
 
-  ierr = PetscLogGpuTimeEnd();CHKERRQ(ierr);
-  cerr = cudaFree(d_A);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_Yold);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_Z);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_Zold);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_M);CHKERRCUDA(cerr);
+  PetscCall(PetscLogGpuTimeEnd());
+  PetscCallCUDA(cudaFree(d_A));
+  PetscCallCUDA(cudaFree(d_Yold));
+  PetscCallCUDA(cudaFree(d_Z));
+  PetscCallCUDA(cudaFree(d_Zold));
+  PetscCallCUDA(cudaFree(d_M));
   PetscFunctionReturn(0);
 }
 
@@ -434,17 +428,13 @@ PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,Pe
   PetscScalar    *d_T,*d_Told,*d_M,*d_invM,*d_work,zero=0.0,sone=1.0,smone=-1.0,spfive=0.5,sneg_pfive=-0.5,sp25=0.25,alpha;
   PetscReal      tol,Mres=0.0,detM,g,reldiff,fnormdiff,fnormT,prod;
   PetscInt       i,it,lwork,nb;
-  PetscBLASInt   N,one=1,info,*piv=NULL;
+  PetscBLASInt   N,one=1,*piv=NULL;
   PetscBool      converged=PETSC_FALSE,scale=PETSC_FALSE;
   cublasHandle_t cublasv2handle;
-  PetscErrorCode ierr;
-  cublasStatus_t cberr;
-  cudaError_t    cerr;
-  magma_int_t    mierr;
 
   PetscFunctionBegin;
-  ierr = PetscCUDAInitializeCheck();CHKERRQ(ierr); /* For CUDA event timers */
-  ierr = PetscCUBLASGetHandle(&cublasv2handle);CHKERRQ(ierr);
+  PetscCall(PetscDeviceInitialize(PETSC_DEVICE_CUDA)); /* For CUDA event timers */
+  PetscCall(PetscCUBLASGetHandle(&cublasv2handle));
   magma_init();
   N = n*n;
   tol = PetscSqrtReal((PetscReal)n)*PETSC_MACHINE_EPSILON/2;
@@ -452,91 +442,83 @@ PetscErrorCode FNSqrtmDenmanBeavers_CUDAm(FN fn,PetscBLASInt n,PetscScalar *T,Pe
   /* query work size */
   nb = magma_get_xgetri_nb(n);
   lwork = nb*n;
-  ierr = PetscMalloc1(n,&piv);CHKERRQ(ierr);
-  cerr = cudaMalloc((void **)&d_work,sizeof(PetscScalar)*lwork);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_T,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_Told,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_M,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-  cerr = cudaMalloc((void **)&d_invM,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
+  PetscCall(PetscMalloc1(n,&piv));
+  PetscCallCUDA(cudaMalloc((void **)&d_work,sizeof(PetscScalar)*lwork));
+  PetscCallCUDA(cudaMalloc((void **)&d_T,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_Told,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_M,sizeof(PetscScalar)*N));
+  PetscCallCUDA(cudaMalloc((void **)&d_invM,sizeof(PetscScalar)*N));
 
-  ierr = PetscLogGpuTimeBegin();CHKERRQ(ierr);
-  cerr = cudaMemcpy(d_M,T,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice);CHKERRCUDA(cerr);
+  PetscCall(PetscLogGpuTimeBegin());
+  PetscCallCUDA(cudaMemcpy(d_M,T,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice));
   if (inv) {  /* start recurrence with I instead of A */
-    cerr = cudaMemset(d_T,zero,sizeof(PetscScalar)*N);CHKERRCUDA(cerr);
-    ierr = set_diagonal(n,d_T,ld,1.0);CHKERRQ(cerr);
-  } else {
-    cerr = cudaMemcpy(d_T,T,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice);CHKERRCUDA(cerr);
-  }
+    PetscCallCUDA(cudaMemset(d_T,zero,sizeof(PetscScalar)*N));
+    PetscCall(set_diagonal(n,d_T,ld,1.0));
+  } else PetscCallCUDA(cudaMemcpy(d_T,T,sizeof(PetscScalar)*N,cudaMemcpyHostToDevice));
 
   for (it=0;it<DBMAXIT && !converged;it++) {
 
     if (scale) { /* g = (abs(det(M)))^(-1/(2*n)); */
-      cerr = cudaMemcpy(d_invM,d_M,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice);CHKERRCUDA(cerr);
-      mierr = magma_xgetrf_gpu(n,n,d_invM,ld,piv,&info);CHKERRMAGMA(mierr);
-      if (info < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"LAPACKgetrf: Illegal value on argument %d",PetscAbsInt(info));
-      if (info > 0) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"LAPACKgetrf: Matrix is singular. U(%d,%d) is zero",info,info);
+      PetscCallCUDA(cudaMemcpy(d_invM,d_M,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
+      PetscCallMAGMA(magma_xgetrf_gpu,n,n,d_invM,ld,piv);
 
       /* XXX pending */
-//      ierr = mult_diagonal(d_invM,n,ld,&detM);CHKERRQ(cerr);
-      cerr = cudaMemcpy(T,d_invM,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost);CHKERRCUDA(cerr);
+//      PetscCall(mult_diagonal(d_invM,n,ld,&detM));
+      PetscCallCUDA(cudaMemcpy(T,d_invM,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
       prod = T[0];
       for (i=1;i<n;i++) { prod *= T[i+i*ld]; }
       detM = PetscAbsReal(prod);
       g = PetscPowReal(detM,-1.0/(2.0*n));
       alpha = g;
-      cberr = cublasXscal(cublasv2handle,N,&alpha,d_T,one);CHKERRCUBLAS(cberr);
+      PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&alpha,d_T,one));
       alpha = g*g;
-      cberr = cublasXscal(cublasv2handle,N,&alpha,d_M,one);CHKERRCUBLAS(cberr);
-      ierr = PetscLogGpuFlops(2.0*n*n*n/3.0+2.0*n*n);CHKERRQ(ierr);
+      PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&alpha,d_M,one));
+      PetscCall(PetscLogGpuFlops(2.0*n*n*n/3.0+2.0*n*n));
     }
 
-    cerr = cudaMemcpy(d_Told,d_T,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice);CHKERRCUDA(cerr);
-    cerr = cudaMemcpy(d_invM,d_M,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice);CHKERRCUDA(cerr);
+    PetscCallCUDA(cudaMemcpy(d_Told,d_T,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
+    PetscCallCUDA(cudaMemcpy(d_invM,d_M,sizeof(PetscScalar)*N,cudaMemcpyDeviceToDevice));
 
-    mierr = magma_xgetrf_gpu(n,n,d_invM,ld,piv,&info);CHKERRMAGMA(mierr);
-    if (info < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"LAPACKgetrf: Illegal value on argument %d",PetscAbsInt(info));
-    if (info > 0) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"LAPACKgetrf: Matrix is singular. U(%d,%d) is zero",info,info);
-    mierr = magma_xgetri_gpu(n,d_invM,ld,piv,d_work,lwork,&info);CHKERRMAGMA(mierr);
-    if (info < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"LAPACKgetri: Illegal value on argument %d",PetscAbsInt(info));
-    if (info > 0) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"LAPACKgetri: Matrix is singular. U(%d,%d) is zero",info,info);
-    ierr = PetscLogGpuFlops(2.0*n*n*n/3.0+4.0*n*n*n/3.0);CHKERRQ(ierr);
+    PetscCallMAGMA(magma_xgetrf_gpu,n,n,d_invM,ld,piv);
+    PetscCallMAGMA(magma_xgetri_gpu,n,d_invM,ld,piv,d_work,lwork);
+    PetscCall(PetscLogGpuFlops(2.0*n*n*n/3.0+4.0*n*n*n/3.0));
 
-    ierr = shift_diagonal(n,d_invM,ld,sone);CHKERRQ(cerr);
-    cberr = cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_Told,ld,d_invM,ld,&zero,d_T,ld);CHKERRCUBLAS(cberr);
-    ierr = shift_diagonal(n,d_invM,ld,smone);CHKERRQ(cerr);
+    PetscCall(shift_diagonal(n,d_invM,ld,sone));
+    PetscCallCUBLAS(cublasXgemm(cublasv2handle,CUBLAS_OP_N,CUBLAS_OP_N,n,n,n,&spfive,d_Told,ld,d_invM,ld,&zero,d_T,ld));
+    PetscCall(shift_diagonal(n,d_invM,ld,smone));
 
-    cberr = cublasXaxpy(cublasv2handle,N,&sone,d_invM,one,d_M,one);CHKERRCUBLAS(cberr);
-    cberr = cublasXscal(cublasv2handle,N,&sp25,d_M,one);CHKERRCUBLAS(cberr);
-    ierr = shift_diagonal(n,d_M,ld,sneg_pfive);CHKERRQ(cerr);
-    ierr = PetscLogGpuFlops(2.0*n*n*n+2.0*n*n);CHKERRQ(ierr);
+    PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&sone,d_invM,one,d_M,one));
+    PetscCallCUBLAS(cublasXscal(cublasv2handle,N,&sp25,d_M,one));
+    PetscCall(shift_diagonal(n,d_M,ld,sneg_pfive));
+    PetscCall(PetscLogGpuFlops(2.0*n*n*n+2.0*n*n));
 
-    cberr = cublasXnrm2(cublasv2handle,N,d_M,one,&Mres);CHKERRCUBLAS(cberr);
-    ierr = shift_diagonal(n,d_M,ld,sone);CHKERRQ(cerr);
+    PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_M,one,&Mres));
+    PetscCall(shift_diagonal(n,d_M,ld,sone));
 
     if (scale) {
       // reldiff = norm(T - Told,'fro')/norm(T,'fro');
-      cberr = cublasXaxpy(cublasv2handle,N,&smone,d_T,one,d_Told,one);CHKERRCUBLAS(cberr);
-      cberr = cublasXnrm2(cublasv2handle,N,d_Told,one,&fnormdiff);CHKERRCUBLAS(cberr);
-      cberr = cublasXnrm2(cublasv2handle,N,d_T,one,&fnormT);CHKERRCUBLAS(cberr);
-      ierr = PetscLogGpuFlops(7.0*n*n);CHKERRQ(ierr);
+      PetscCallCUBLAS(cublasXaxpy(cublasv2handle,N,&smone,d_T,one,d_Told,one));
+      PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_Told,one,&fnormdiff));
+      PetscCallCUBLAS(cublasXnrm2(cublasv2handle,N,d_T,one,&fnormT));
+      PetscCall(PetscLogGpuFlops(7.0*n*n));
       reldiff = fnormdiff/fnormT;
-      ierr = PetscInfo4(fn,"it: %D reldiff: %g scale: %g tol*scale: %g\n",it,(double)reldiff,(double)g,(double)tol*g);CHKERRQ(ierr);
+      PetscCall(PetscInfo(fn,"it: %" PetscInt_FMT " reldiff: %g scale: %g tol*scale: %g\n",it,(double)reldiff,(double)g,(double)tol*g));
       if (reldiff<1e-2) scale = PETSC_FALSE; /* Switch to no scaling. */
     }
 
-    ierr = PetscInfo2(fn,"it: %D Mres: %g\n",it,(double)Mres);
+    PetscCall(PetscInfo(fn,"it: %" PetscInt_FMT " Mres: %g\n",it,(double)Mres));
     if (Mres<=tol) converged = PETSC_TRUE;
   }
 
-  if (Mres>tol) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations", DBMAXIT);
-  cerr = cudaMemcpy(T,d_T,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost);CHKERRCUDA(cerr);
-  ierr = PetscLogGpuTimeEnd();CHKERRQ(ierr);
-  ierr = PetscFree(piv);CHKERRQ(ierr);
-  cerr = cudaFree(d_work);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_T);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_Told);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_M);CHKERRCUDA(cerr);
-  cerr = cudaFree(d_invM);CHKERRCUDA(cerr);
+  PetscCheck(Mres<=tol,PETSC_COMM_SELF,PETSC_ERR_LIB,"SQRTM not converged after %d iterations", DBMAXIT);
+  PetscCallCUDA(cudaMemcpy(T,d_T,sizeof(PetscScalar)*N,cudaMemcpyDeviceToHost));
+  PetscCall(PetscLogGpuTimeEnd());
+  PetscCall(PetscFree(piv));
+  PetscCallCUDA(cudaFree(d_work));
+  PetscCallCUDA(cudaFree(d_T));
+  PetscCallCUDA(cudaFree(d_Told));
+  PetscCallCUDA(cudaFree(d_M));
+  PetscCallCUDA(cudaFree(d_invM));
   magma_finalize();
   PetscFunctionReturn(0);
 }
@@ -555,7 +537,6 @@ static PetscErrorCode SlepcNormEst1(PetscBLASInt n,PetscScalar *A,PetscInt m,Pet
   PetscScalar    *X,*Y,*Z,*S,*S_old,*aux,val,sone=1.0,szero=0.0;
   PetscReal      est=0.0,est_old,vals[2]={0.0,0.0},*zvals,maxzval[2],raux;
   PetscBLASInt   i,j,t=2,it=0,ind[2],est_j=0,m1;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   X = work;
@@ -567,7 +548,7 @@ static PetscErrorCode SlepcNormEst1(PetscBLASInt n,PetscScalar *A,PetscInt m,Pet
 
   for (i=0;i<n;i++) {  /* X has columns of unit 1-norm */
     X[i] = 1.0/n;
-    ierr = PetscRandomGetValue(rand,&val);CHKERRQ(ierr);
+    PetscCall(PetscRandomGetValue(rand,&val));
     if (PetscRealPart(val) < 0.5) X[i+n] = -1.0/n;
     else X[i+n] = 1.0/n;
   }
@@ -622,7 +603,7 @@ static PetscErrorCode SlepcNormEst1(PetscBLASInt n,PetscScalar *A,PetscInt m,Pet
   }
   *nrm = est;
   /* Flop count is roughly (it * 2*m * t*gemv) = 4*its*m*t*n*n */
-  ierr = PetscLogFlops(4.0*it*m*t*n*n);CHKERRQ(ierr);
+  PetscCall(PetscLogFlops(4.0*it*m*t*n*n));
   PetscFunctionReturn(0);
 }
 
@@ -637,13 +618,12 @@ PetscErrorCode SlepcNormAm(PetscBLASInt n,PetscScalar *A,PetscInt m,PetscScalar
   PetscReal      rwork[1],tmp;
   PetscBLASInt   i,j,one=1;
   PetscBool      isrealpos=PETSC_TRUE;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (n<SMALLN) {   /* compute matrix power explicitly */
     if (m==1) {
       *nrm = LAPACKlange_("O",&n,&n,A,&n,rwork);
-      ierr = PetscLogFlops(1.0*n*n);CHKERRQ(ierr);
+      PetscCall(PetscLogFlops(1.0*n*n));
     } else {  /* m>=2 */
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&sone,A,&n,A,&n,&szero,v,&n));
       for (j=0;j<m-2;j++) {
@@ -651,7 +631,7 @@ PetscErrorCode SlepcNormAm(PetscBLASInt n,PetscScalar *A,PetscInt m,PetscScalar
         SWAP(v,w,aux);
       }
       *nrm = LAPACKlange_("O",&n,&n,v,&n,rwork);
-      ierr = PetscLogFlops(2.0*n*n*n*(m-1)+1.0*n*n);CHKERRQ(ierr);
+      PetscCall(PetscLogFlops(2.0*n*n*n*(m-1)+1.0*n*n));
     }
   } else {
     for (i=0;i<n;i++)
@@ -667,13 +647,10 @@ PetscErrorCode SlepcNormAm(PetscBLASInt n,PetscScalar *A,PetscInt m,PetscScalar
         PetscStackCallBLAS("BLASgemv",BLASgemv_("C",&n,&n,&sone,A,&n,v,&one,&szero,w,&one));
         SWAP(v,w,aux);
       }
-      ierr = PetscLogFlops(2.0*n*n*m);CHKERRQ(ierr);
+      PetscCall(PetscLogFlops(2.0*n*n*m));
       *nrm = 0.0;
       for (i=0;i<n;i++) if ((tmp = PetscAbsScalar(v[i])) > *nrm) *nrm = tmp;   /* norm(v,inf) */
-    } else {
-      ierr = SlepcNormEst1(n,A,m,work,rand,nrm);CHKERRQ(ierr);
-    }
+    } else PetscCall(SlepcNormEst1(n,A,m,work,rand,nrm));
   }
   PetscFunctionReturn(0);
 }
-
