@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -39,96 +39,92 @@ PetscErrorCode FNEvaluateFunction_Rational(FN fn,PetscScalar x,PetscScalar *y)
     q = ctx->qcoeff[0];
     for (i=1;i<ctx->nq;i++)
       q = ctx->qcoeff[i]+x*q;
-    if (q==0.0) SETERRQ(PETSC_COMM_SELF,1,"Function not defined in the requested value");
+    PetscCheck(q!=0.0,PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Function not defined in the requested value");
     *y = p/q;
   }
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode FNEvaluateFunctionMat_Rational_Private(FN fn,PetscScalar *Aa,PetscScalar *Ba,PetscInt m,PetscBool firstonly)
+static PetscErrorCode FNEvaluateFunctionMat_Rational_Private(FN fn,const PetscScalar *Aa,PetscScalar *Ba,PetscInt m,PetscBool firstonly)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscBLASInt   n,k,ld,*ipiv,info;
   PetscInt       i,j;
   PetscScalar    *W,*P,*Q,one=1.0,zero=0.0;
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(m,&n);CHKERRQ(ierr);
+  PetscCall(PetscBLASIntCast(m,&n));
   ld = n;
   k  = firstonly? 1: n;
-  if (Aa==Ba) {
-    ierr = PetscMalloc4(m*m,&P,m*m,&Q,m*m,&W,ld,&ipiv);CHKERRQ(ierr);
-  } else {
+  if (Aa==Ba) PetscCall(PetscMalloc4(m*m,&P,m*m,&Q,m*m,&W,ld,&ipiv));
+  else {
     P = Ba;
-    ierr = PetscMalloc3(m*m,&Q,m*m,&W,ld,&ipiv);CHKERRQ(ierr);
+    PetscCall(PetscMalloc3(m*m,&Q,m*m,&W,ld,&ipiv));
   }
-  ierr = PetscArrayzero(P,m*m);CHKERRQ(ierr);
+  PetscCall(PetscArrayzero(P,m*m));
   if (!ctx->np) {
     for (i=0;i<m;i++) P[i+i*ld] = 1.0;
   } else {
     for (i=0;i<m;i++) P[i+i*ld] = ctx->pcoeff[0];
     for (j=1;j<ctx->np;j++) {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&one,P,&ld,Aa,&ld,&zero,W,&ld));
-      ierr = PetscArraycpy(P,W,m*m);CHKERRQ(ierr);
+      PetscCall(PetscArraycpy(P,W,m*m));
       for (i=0;i<m;i++) P[i+i*ld] += ctx->pcoeff[j];
     }
-    ierr = PetscLogFlops(2.0*n*n*n*(ctx->np-1));CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(2.0*n*n*n*(ctx->np-1)));
   }
   if (ctx->nq) {
-    ierr = PetscArrayzero(Q,m*m);CHKERRQ(ierr);
+    PetscCall(PetscArrayzero(Q,m*m));
     for (i=0;i<m;i++) Q[i+i*ld] = ctx->qcoeff[0];
     for (j=1;j<ctx->nq;j++) {
       PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&n,&n,&n,&one,Q,&ld,Aa,&ld,&zero,W,&ld));
-      ierr = PetscArraycpy(Q,W,m*m);CHKERRQ(ierr);
+      PetscCall(PetscArraycpy(Q,W,m*m));
       for (i=0;i<m;i++) Q[i+i*ld] += ctx->qcoeff[j];
     }
     PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&n,&k,Q,&ld,ipiv,P,&ld,&info));
     SlepcCheckLapackInfo("gesv",info);
-    ierr = PetscLogFlops(2.0*n*n*n*(ctx->nq-1)+2.0*n*n*n/3.0+2.0*n*n*k);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(2.0*n*n*n*(ctx->nq-1)+2.0*n*n*n/3.0+2.0*n*n*k));
   }
   if (Aa==Ba) {
-    ierr = PetscArraycpy(Aa,P,m*k);CHKERRQ(ierr);
-    ierr = PetscFree4(P,Q,W,ipiv);CHKERRQ(ierr);
-  } else {
-    ierr = PetscFree3(Q,W,ipiv);CHKERRQ(ierr);
-  }
+    PetscCall(PetscArraycpy(Ba,P,m*k));
+    PetscCall(PetscFree4(P,Q,W,ipiv));
+  } else PetscCall(PetscFree3(Q,W,ipiv));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode FNEvaluateFunctionMat_Rational(FN fn,Mat A,Mat B)
 {
-  PetscErrorCode ierr;
-  PetscInt       m;
-  PetscScalar    *Aa,*Ba;
+  PetscInt          m;
+  const PetscScalar *Aa;
+  PetscScalar       *Ba;
 
   PetscFunctionBegin;
-  ierr = MatDenseGetArray(A,&Aa);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
-  ierr = MatGetSize(A,&m,NULL);CHKERRQ(ierr);
-  ierr = FNEvaluateFunctionMat_Rational_Private(fn,Aa,Ba,m,PETSC_FALSE);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(A,&Aa);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
+  PetscCall(MatDenseGetArrayRead(A,&Aa));
+  PetscCall(MatDenseGetArray(B,&Ba));
+  PetscCall(MatGetSize(A,&m,NULL));
+  PetscCall(FNEvaluateFunctionMat_Rational_Private(fn,Aa,Ba,m,PETSC_FALSE));
+  PetscCall(MatDenseRestoreArrayRead(A,&Aa));
+  PetscCall(MatDenseRestoreArray(B,&Ba));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode FNEvaluateFunctionMatVec_Rational(FN fn,Mat A,Vec v)
 {
-  PetscErrorCode ierr;
-  PetscInt       m;
-  PetscScalar    *Aa,*Ba;
-  Mat            B;
+  PetscInt          m;
+  const PetscScalar *Aa;
+  PetscScalar       *Ba;
+  Mat               B;
 
   PetscFunctionBegin;
-  ierr = FN_AllocateWorkMat(fn,A,&B);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(A,&Aa);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(B,&Ba);CHKERRQ(ierr);
-  ierr = MatGetSize(A,&m,NULL);CHKERRQ(ierr);
-  ierr = FNEvaluateFunctionMat_Rational_Private(fn,Aa,Ba,m,PETSC_TRUE);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(A,&Aa);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(B,&Ba);CHKERRQ(ierr);
-  ierr = MatGetColumnVector(B,v,0);CHKERRQ(ierr);
-  ierr = FN_FreeWorkMat(fn,&B);CHKERRQ(ierr);
+  PetscCall(FN_AllocateWorkMat(fn,A,&B));
+  PetscCall(MatDenseGetArrayRead(A,&Aa));
+  PetscCall(MatDenseGetArray(B,&Ba));
+  PetscCall(MatGetSize(A,&m,NULL));
+  PetscCall(FNEvaluateFunctionMat_Rational_Private(fn,Aa,Ba,m,PETSC_TRUE));
+  PetscCall(MatDenseRestoreArrayRead(A,&Aa));
+  PetscCall(MatDenseRestoreArray(B,&Ba));
+  PetscCall(MatGetColumnVector(B,v,0));
+  PetscCall(FN_FreeWorkMat(fn,&B));
   PetscFunctionReturn(0);
 }
 
@@ -158,7 +154,7 @@ PetscErrorCode FNEvaluateDerivative_Rational(FN fn,PetscScalar x,PetscScalar *yp
       qp = q+x*qp;
       q = ctx->qcoeff[i]+x*q;
     }
-    if (q==0.0) SETERRQ(PETSC_COMM_SELF,1,"Derivative not defined in the requested value");
+    PetscCheck(q!=0.0,PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Derivative not defined in the requested value");
     *yp = (pp*q-p*qp)/(q*q);
   }
   PetscFunctionReturn(0);
@@ -166,66 +162,64 @@ PetscErrorCode FNEvaluateDerivative_Rational(FN fn,PetscScalar x,PetscScalar *yp
 
 PetscErrorCode FNView_Rational(FN fn,PetscViewer viewer)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscBool      isascii;
   PetscInt       i;
   char           str[50];
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii));
   if (isascii) {
     if (fn->alpha!=(PetscScalar)1.0 || fn->beta!=(PetscScalar)1.0) {
-      ierr = SlepcSNPrintfScalar(str,50,fn->alpha,PETSC_FALSE);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"  Scale factors: alpha=%s,",str);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
-      ierr = SlepcSNPrintfScalar(str,50,fn->beta,PETSC_FALSE);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer," beta=%s\n",str);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      PetscCall(SlepcSNPrintfScalar(str,sizeof(str),fn->alpha,PETSC_FALSE));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"  Scale factors: alpha=%s,",str));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
+      PetscCall(SlepcSNPrintfScalar(str,sizeof(str),fn->beta,PETSC_FALSE));
+      PetscCall(PetscViewerASCIIPrintf(viewer," beta=%s\n",str));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
     }
     if (!ctx->nq) {
-      if (!ctx->np) {
-        ierr = PetscViewerASCIIPrintf(viewer,"  Constant: 1.0\n");CHKERRQ(ierr);
-      } else if (ctx->np==1) {
-        ierr = SlepcSNPrintfScalar(str,50,ctx->pcoeff[0],PETSC_FALSE);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"  Constant: %s\n",str);CHKERRQ(ierr);
+      if (!ctx->np) PetscCall(PetscViewerASCIIPrintf(viewer,"  Constant: 1.0\n"));
+      else if (ctx->np==1) {
+        PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->pcoeff[0],PETSC_FALSE));
+        PetscCall(PetscViewerASCIIPrintf(viewer,"  Constant: %s\n",str));
       } else {
-        ierr = PetscViewerASCIIPrintf(viewer,"  Polynomial: ");CHKERRQ(ierr);
-        ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
+        PetscCall(PetscViewerASCIIPrintf(viewer,"  Polynomial: "));
+        PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
         for (i=0;i<ctx->np-1;i++) {
-          ierr = SlepcSNPrintfScalar(str,50,ctx->pcoeff[i],PETSC_TRUE);CHKERRQ(ierr);
-          ierr = PetscViewerASCIIPrintf(viewer,"%s*x^%1D",str,ctx->np-i-1);CHKERRQ(ierr);
+          PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->pcoeff[i],PETSC_TRUE));
+          PetscCall(PetscViewerASCIIPrintf(viewer,"%s*x^%1" PetscInt_FMT,str,ctx->np-i-1));
         }
-        ierr = SlepcSNPrintfScalar(str,50,ctx->pcoeff[ctx->np-1],PETSC_TRUE);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"%s\n",str);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+        PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->pcoeff[ctx->np-1],PETSC_TRUE));
+        PetscCall(PetscViewerASCIIPrintf(viewer,"%s\n",str));
+        PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
       }
     } else if (!ctx->np) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Inverse polinomial: 1 / (");CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"  Inverse polinomial: 1 / ("));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
       for (i=0;i<ctx->nq-1;i++) {
-        ierr = SlepcSNPrintfScalar(str,50,ctx->qcoeff[i],PETSC_TRUE);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"%s*x^%1D",str,ctx->nq-i-1);CHKERRQ(ierr);
+        PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->qcoeff[i],PETSC_TRUE));
+        PetscCall(PetscViewerASCIIPrintf(viewer,"%s*x^%1" PetscInt_FMT,str,ctx->nq-i-1));
       }
-      ierr = SlepcSNPrintfScalar(str,50,ctx->qcoeff[ctx->nq-1],PETSC_TRUE);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"%s)\n",str);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->qcoeff[ctx->nq-1],PETSC_TRUE));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"%s)\n",str));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Rational function: (");CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      PetscCall(PetscViewerASCIIPrintf(viewer,"  Rational function: ("));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_FALSE));
       for (i=0;i<ctx->np-1;i++) {
-        ierr = SlepcSNPrintfScalar(str,50,ctx->pcoeff[i],PETSC_TRUE);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"%s*x^%1D",str,ctx->np-i-1);CHKERRQ(ierr);
+        PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->pcoeff[i],PETSC_TRUE));
+        PetscCall(PetscViewerASCIIPrintf(viewer,"%s*x^%1" PetscInt_FMT,str,ctx->np-i-1));
       }
-      ierr = SlepcSNPrintfScalar(str,50,ctx->pcoeff[ctx->np-1],PETSC_TRUE);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"%s) / (",str);CHKERRQ(ierr);
+      PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->pcoeff[ctx->np-1],PETSC_TRUE));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"%s) / (",str));
       for (i=0;i<ctx->nq-1;i++) {
-        ierr = SlepcSNPrintfScalar(str,50,ctx->qcoeff[i],PETSC_TRUE);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(viewer,"%s*x^%1D",str,ctx->nq-i-1);CHKERRQ(ierr);
+        PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->qcoeff[i],PETSC_TRUE));
+        PetscCall(PetscViewerASCIIPrintf(viewer,"%s*x^%1" PetscInt_FMT,str,ctx->nq-i-1));
       }
-      ierr = SlepcSNPrintfScalar(str,50,ctx->qcoeff[ctx->nq-1],PETSC_TRUE);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer,"%s)\n",str);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      PetscCall(SlepcSNPrintfScalar(str,sizeof(str),ctx->qcoeff[ctx->nq-1],PETSC_TRUE));
+      PetscCall(PetscViewerASCIIPrintf(viewer,"%s)\n",str));
+      PetscCall(PetscViewerASCIIUseTabs(viewer,PETSC_TRUE));
     }
   }
   PetscFunctionReturn(0);
@@ -233,17 +227,16 @@ PetscErrorCode FNView_Rational(FN fn,PetscViewer viewer)
 
 static PetscErrorCode FNRationalSetNumerator_Rational(FN fn,PetscInt np,PetscScalar *pcoeff)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscInt       i;
 
   PetscFunctionBegin;
-  if (np<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument np cannot be negative");
+  PetscCheck(np>=0,PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument np cannot be negative");
   ctx->np = np;
-  ierr = PetscFree(ctx->pcoeff);CHKERRQ(ierr);
+  PetscCall(PetscFree(ctx->pcoeff));
   if (np) {
-    ierr = PetscMalloc1(np,&ctx->pcoeff);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)fn,np*sizeof(PetscScalar));CHKERRQ(ierr);
+    PetscCall(PetscMalloc1(np,&ctx->pcoeff));
+    PetscCall(PetscLogObjectMemory((PetscObject)fn,np*sizeof(PetscScalar)));
     for (i=0;i<np;i++) ctx->pcoeff[i] = pcoeff[i];
   }
   PetscFunctionReturn(0);
@@ -275,19 +268,16 @@ static PetscErrorCode FNRationalSetNumerator_Rational(FN fn,PetscInt np,PetscSca
 @*/
 PetscErrorCode FNRationalSetNumerator(FN fn,PetscInt np,PetscScalar *pcoeff)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,np,2);
   if (np) PetscValidScalarPointer(pcoeff,3);
-  ierr = PetscTryMethod(fn,"FNRationalSetNumerator_C",(FN,PetscInt,PetscScalar*),(fn,np,pcoeff));CHKERRQ(ierr);
+  PetscTryMethod(fn,"FNRationalSetNumerator_C",(FN,PetscInt,PetscScalar*),(fn,np,pcoeff));
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode FNRationalGetNumerator_Rational(FN fn,PetscInt *np,PetscScalar *pcoeff[])
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscInt       i;
 
@@ -296,7 +286,7 @@ static PetscErrorCode FNRationalGetNumerator_Rational(FN fn,PetscInt *np,PetscSc
   if (pcoeff) {
     if (!ctx->np) *pcoeff = NULL;
     else {
-      ierr = PetscMalloc1(ctx->np,pcoeff);CHKERRQ(ierr);
+      PetscCall(PetscMalloc1(ctx->np,pcoeff));
       for (i=0;i<ctx->np;i++) (*pcoeff)[i] = ctx->pcoeff[i];
     }
   }
@@ -327,27 +317,24 @@ static PetscErrorCode FNRationalGetNumerator_Rational(FN fn,PetscInt *np,PetscSc
 @*/
 PetscErrorCode FNRationalGetNumerator(FN fn,PetscInt *np,PetscScalar *pcoeff[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
-  ierr = PetscUseMethod(fn,"FNRationalGetNumerator_C",(FN,PetscInt*,PetscScalar**),(fn,np,pcoeff));CHKERRQ(ierr);
+  PetscUseMethod(fn,"FNRationalGetNumerator_C",(FN,PetscInt*,PetscScalar**),(fn,np,pcoeff));
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode FNRationalSetDenominator_Rational(FN fn,PetscInt nq,PetscScalar *qcoeff)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscInt       i;
 
   PetscFunctionBegin;
-  if (nq<0) SETERRQ(PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument nq cannot be negative");
+  PetscCheck(nq>=0,PetscObjectComm((PetscObject)fn),PETSC_ERR_ARG_OUTOFRANGE,"Argument nq cannot be negative");
   ctx->nq = nq;
-  ierr = PetscFree(ctx->qcoeff);CHKERRQ(ierr);
+  PetscCall(PetscFree(ctx->qcoeff));
   if (nq) {
-    ierr = PetscMalloc1(nq,&ctx->qcoeff);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)fn,nq*sizeof(PetscScalar));CHKERRQ(ierr);
+    PetscCall(PetscMalloc1(nq,&ctx->qcoeff));
+    PetscCall(PetscLogObjectMemory((PetscObject)fn,nq*sizeof(PetscScalar)));
     for (i=0;i<nq;i++) ctx->qcoeff[i] = qcoeff[i];
   }
   PetscFunctionReturn(0);
@@ -379,19 +366,16 @@ static PetscErrorCode FNRationalSetDenominator_Rational(FN fn,PetscInt nq,PetscS
 @*/
 PetscErrorCode FNRationalSetDenominator(FN fn,PetscInt nq,PetscScalar *qcoeff)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
   PetscValidLogicalCollectiveInt(fn,nq,2);
   if (nq) PetscValidScalarPointer(qcoeff,3);
-  ierr = PetscTryMethod(fn,"FNRationalSetDenominator_C",(FN,PetscInt,PetscScalar*),(fn,nq,qcoeff));CHKERRQ(ierr);
+  PetscTryMethod(fn,"FNRationalSetDenominator_C",(FN,PetscInt,PetscScalar*),(fn,nq,qcoeff));
   PetscFunctionReturn(0);
 }
 
 static PetscErrorCode FNRationalGetDenominator_Rational(FN fn,PetscInt *nq,PetscScalar *qcoeff[])
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
   PetscInt       i;
 
@@ -400,7 +384,7 @@ static PetscErrorCode FNRationalGetDenominator_Rational(FN fn,PetscInt *nq,Petsc
   if (qcoeff) {
     if (!ctx->nq) *qcoeff = NULL;
     else {
-      ierr = PetscMalloc1(ctx->nq,qcoeff);CHKERRQ(ierr);
+      PetscCall(PetscMalloc1(ctx->nq,qcoeff));
       for (i=0;i<ctx->nq;i++) (*qcoeff)[i] = ctx->qcoeff[i];
     }
   }
@@ -431,56 +415,52 @@ static PetscErrorCode FNRationalGetDenominator_Rational(FN fn,PetscInt *nq,Petsc
 @*/
 PetscErrorCode FNRationalGetDenominator(FN fn,PetscInt *nq,PetscScalar *qcoeff[])
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(fn,FN_CLASSID,1);
-  ierr = PetscUseMethod(fn,"FNRationalGetDenominator_C",(FN,PetscInt*,PetscScalar**),(fn,nq,qcoeff));CHKERRQ(ierr);
+  PetscUseMethod(fn,"FNRationalGetDenominator_C",(FN,PetscInt*,PetscScalar**),(fn,nq,qcoeff));
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode FNSetFromOptions_Rational(PetscOptionItems *PetscOptionsObject,FN fn)
 {
-  PetscErrorCode ierr;
 #define PARMAX 10
   PetscScalar    array[PARMAX];
   PetscInt       i,k;
   PetscBool      flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead(PetscOptionsObject,"FN Rational Options");CHKERRQ(ierr);
+  PetscCall(PetscOptionsHead(PetscOptionsObject,"FN Rational Options"));
 
     k = PARMAX;
     for (i=0;i<k;i++) array[i] = 0;
-    ierr = PetscOptionsScalarArray("-fn_rational_numerator","Numerator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetNumerator",array,&k,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = FNRationalSetNumerator(fn,k,array);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsScalarArray("-fn_rational_numerator","Numerator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetNumerator",array,&k,&flg));
+    if (flg) PetscCall(FNRationalSetNumerator(fn,k,array));
 
     k = PARMAX;
     for (i=0;i<k;i++) array[i] = 0;
-    ierr = PetscOptionsScalarArray("-fn_rational_denominator","Denominator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetDenominator",array,&k,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = FNRationalSetDenominator(fn,k,array);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsScalarArray("-fn_rational_denominator","Denominator coefficients (one or more scalar values separated with a comma without spaces)","FNRationalSetDenominator",array,&k,&flg));
+    if (flg) PetscCall(FNRationalSetDenominator(fn,k,array));
 
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  PetscCall(PetscOptionsTail());
   PetscFunctionReturn(0);
 }
 
 PetscErrorCode FNDuplicate_Rational(FN fn,MPI_Comm comm,FN *newfn)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data,*ctx2 = (FN_RATIONAL*)(*newfn)->data;
   PetscInt       i;
 
   PetscFunctionBegin;
   ctx2->np = ctx->np;
   if (ctx->np) {
-    ierr = PetscMalloc1(ctx->np,&ctx2->pcoeff);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)(*newfn),ctx->np*sizeof(PetscScalar));CHKERRQ(ierr);
+    PetscCall(PetscMalloc1(ctx->np,&ctx2->pcoeff));
+    PetscCall(PetscLogObjectMemory((PetscObject)(*newfn),ctx->np*sizeof(PetscScalar)));
     for (i=0;i<ctx->np;i++) ctx2->pcoeff[i] = ctx->pcoeff[i];
   }
   ctx2->nq = ctx->nq;
   if (ctx->nq) {
-    ierr = PetscMalloc1(ctx->nq,&ctx2->qcoeff);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)(*newfn),ctx->nq*sizeof(PetscScalar));CHKERRQ(ierr);
+    PetscCall(PetscMalloc1(ctx->nq,&ctx2->qcoeff));
+    PetscCall(PetscLogObjectMemory((PetscObject)(*newfn),ctx->nq*sizeof(PetscScalar)));
     for (i=0;i<ctx->nq;i++) ctx2->qcoeff[i] = ctx->qcoeff[i];
   }
   PetscFunctionReturn(0);
@@ -488,27 +468,25 @@ PetscErrorCode FNDuplicate_Rational(FN fn,MPI_Comm comm,FN *newfn)
 
 PetscErrorCode FNDestroy_Rational(FN fn)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx = (FN_RATIONAL*)fn->data;
 
   PetscFunctionBegin;
-  ierr = PetscFree(ctx->pcoeff);CHKERRQ(ierr);
-  ierr = PetscFree(ctx->qcoeff);CHKERRQ(ierr);
-  ierr = PetscFree(fn->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetNumerator_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetNumerator_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetDenominator_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetDenominator_C",NULL);CHKERRQ(ierr);
+  PetscCall(PetscFree(ctx->pcoeff));
+  PetscCall(PetscFree(ctx->qcoeff));
+  PetscCall(PetscFree(fn->data));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetNumerator_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetNumerator_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetDenominator_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetDenominator_C",NULL));
   PetscFunctionReturn(0);
 }
 
 SLEPC_EXTERN PetscErrorCode FNCreate_Rational(FN fn)
 {
-  PetscErrorCode ierr;
   FN_RATIONAL    *ctx;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(fn,&ctx);CHKERRQ(ierr);
+  PetscCall(PetscNewLog(fn,&ctx));
   fn->data = (void*)ctx;
 
   fn->ops->evaluatefunction          = FNEvaluateFunction_Rational;
@@ -519,10 +497,9 @@ SLEPC_EXTERN PetscErrorCode FNCreate_Rational(FN fn)
   fn->ops->view                      = FNView_Rational;
   fn->ops->duplicate                 = FNDuplicate_Rational;
   fn->ops->destroy                   = FNDestroy_Rational;
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetNumerator_C",FNRationalSetNumerator_Rational);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetNumerator_C",FNRationalGetNumerator_Rational);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetDenominator_C",FNRationalSetDenominator_Rational);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetDenominator_C",FNRationalGetDenominator_Rational);CHKERRQ(ierr);
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetNumerator_C",FNRationalSetNumerator_Rational));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetNumerator_C",FNRationalGetNumerator_Rational));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalSetDenominator_C",FNRationalSetDenominator_Rational));
+  PetscCall(PetscObjectComposeFunction((PetscObject)fn,"FNRationalGetDenominator_C",FNRationalGetDenominator_Rational));
   PetscFunctionReturn(0);
 }
-
