@@ -1,20 +1,22 @@
 !
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !  SLEPc - Scalable Library for Eigenvalue Problem Computations
-!  Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
+!  Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 !
 !  This file is part of SLEPc.
 !  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 !  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 !
-!  Include file for Fortran use of the RG object in SLEPc
+!  Used by slepcrgmod.F90 to create Fortran module file
 !
 #include "slepc/finclude/slepcrg.h"
 
       type tRG
-        PetscFortranAddr:: v
+        PetscFortranAddr:: v PETSC_FORTRAN_TYPE_INITIALIZE
       end type tRG
 
-!
-!  End of Fortran include file for the RG package in SLEPc
-!
+      RG, parameter :: SLEPC_NULL_RG = tRG(0)
+
+      PetscEnum, parameter :: RG_QUADRULE_TRAPEZOIDAL = 1
+      PetscEnum, parameter :: RG_QUADRULE_CHEBYSHEV   = 2
+
