@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -43,21 +43,20 @@ static PetscErrorCode FILTLAN_ExpandNewtonPolynomialInChebyshevBasis(PetscInt,Pe
    in general, if there are k x(i)'s with the same value x0, then
        the j-th order derivative of P(z) is zero at z=x0 for j=1,...,k-1
 */
-PETSC_STATIC_INLINE PetscErrorCode FILTLAN_NewtonPolynomial(PetscInt n,PetscReal *x,PetscReal *y,PetscReal *sa,PetscReal *sf)
+static inline PetscErrorCode FILTLAN_NewtonPolynomial(PetscInt n,PetscReal *x,PetscReal *y,PetscReal *sa,PetscReal *sf)
 {
-  PetscErrorCode ierr;
   PetscReal      d,*sx=x,*sy=y;
   PetscInt       j,k;
 
   PetscFunctionBegin;
-  ierr = PetscArraycpy(sf,sy,n);CHKERRQ(ierr);
+  PetscCall(PetscArraycpy(sf,sy,n));
 
   /* apply Newton's finite difference method */
   sa[0] = sf[0];
   for (j=1;j<n;j++) {
     for (k=n-1;k>=j;k--) {
       d = sx[k]-sx[k-j];
-      if (d == 0.0) sf[k] = 0.0;  /* assume that the derivative is 0.0 and apply the Hermite interpolation */
+      if (PetscUnlikely(d == 0.0)) sf[k] = 0.0;  /* assume that the derivative is 0.0 and apply the Hermite interpolation */
       else sf[k] = (sf[k]-sf[k-1]) / d;
     }
     sa[j] = sf[j];
@@ -102,7 +101,6 @@ PETSC_STATIC_INLINE PetscErrorCode FILTLAN_NewtonPolynomial(PetscInt n,PetscReal
 */
 static PetscErrorCode FILTLAN_HermiteBaseFilterInChebyshevBasis(PetscReal *baseFilter,PetscReal *intv,PetscInt npoints,const PetscInt *HighLowFlags,PetscInt baseDeg)
 {
-  PetscErrorCode ierr;
   PetscInt       m,ii,jj;
   PetscReal      flag,flag0,flag2,aa,bb,*px,*py,*sx,*sy,*pp,*qq,*sq,*sbf,*work,*currentPoint = intv;
   const PetscInt *hilo = HighLowFlags;
@@ -110,7 +108,7 @@ static PetscErrorCode FILTLAN_HermiteBaseFilterInChebyshevBasis(PetscReal *baseF
   PetscFunctionBegin;
   m = 2*baseDeg+2;
   jj = npoints-1;  /* jj is initialized as the number of intervals */
-  ierr = PetscMalloc5(m,&px,m,&py,m,&pp,m,&qq,m,&work);CHKERRQ(ierr);
+  PetscCall(PetscMalloc5(m,&px,m,&py,m,&pp,m,&qq,m,&work));
   sbf = baseFilter;
 
   while (jj--) {  /* the main loop to compute the Chebyshev coefficients */
@@ -144,13 +142,13 @@ static PetscErrorCode FILTLAN_HermiteBaseFilterInChebyshevBasis(PetscReal *baseF
       }
 
       /* build a Newton polynomial (indeed, the generalized Hermite interpolating polynomial) with x[] and y[] */
-      ierr = FILTLAN_NewtonPolynomial(m,px,py,pp,work);CHKERRQ(ierr);
+      PetscCall(FILTLAN_NewtonPolynomial(m,px,py,pp,work));
 
       /* pp contains coefficients of the Newton polynomial P(z) in the current interval [aa,bb], where
          P(z) = pp(1) + pp(2)*(z-px(1)) + pp(3)*(z-px(1))*(z-px(2)) + ... + pp(n)*(z-px(1))*...*(z-px(n-1)) */
 
       /* translate the Newton coefficients to the Chebyshev coefficients */
-      ierr = FILTLAN_ExpandNewtonPolynomialInChebyshevBasis(m,aa,bb,pp,px,qq,work);CHKERRQ(ierr);
+      PetscCall(FILTLAN_ExpandNewtonPolynomialInChebyshevBasis(m,aa,bb,pp,px,qq,work));
       /* qq contains coefficients of the polynomial in [aa,bb] in the `translated' Chebyshev basis */
 
       /* copy the Chebyshev coefficients to baseFilter
@@ -174,7 +172,7 @@ static PetscErrorCode FILTLAN_HermiteBaseFilterInChebyshevBasis(PetscReal *baseF
       currentPoint++;
     }
   }
-  ierr = PetscFree5(px,py,pp,qq,work);CHKERRQ(ierr);
+  PetscCall(PetscFree5(px,py,pp,qq,work));
   PetscFunctionReturn(0);
 }
 
@@ -208,7 +206,6 @@ static PetscErrorCode FILTLAN_HermiteBaseFilterInChebyshevBasis(PetscReal *baseF
 */
 static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame,PetscInt polyDeg,PetscInt baseDeg,FILTLAN_IOP opts,FILTLAN_PFI filterInfo)
 {
-  PetscErrorCode  ierr;
   PetscReal       intv[6],x,y,z1,z2,c,c1,c2,fc,fc2,halfPlateau,leftDelta,rightDelta,gridSize;
   PetscReal       yLimit,ySummit,yLeftLimit,yRightLimit,bottom,qIndex,*baseFilter,*polyFilter;
   PetscReal       yLimitGap=0.0,yLeftSummit=0.0,yLeftBottom=0.0,yRightSummit=0.0,yRightBottom=0.0;
@@ -219,13 +216,13 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
   const PetscInt  numLookMore = 2*(PetscInt)(0.5+(PetscLogReal(2.0)/PetscLogReal(opts->shiftStepExpanRate)));
 
   PetscFunctionBegin;
-  if (a>a1 || a1>b1 || b1>b) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Values in the frame vector should be non-decreasing");
-  if (a1 == b1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"The range of wanted eigenvalues cannot be of size zero");
+  PetscCheck(a<=a1 && a1<=b1 && b1<=b,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Values in the frame vector should be non-decreasing");
+  PetscCheck(a1!=b1,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"The range of wanted eigenvalues cannot be of size zero");
   filterInfo->filterType = 2;      /* mid-pass filter, for interior eigenvalues */
   if (b == b1) {
-    if (a == a1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"A polynomial filter should not cover all eigenvalues");
+    PetscCheck(a!=a1,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"A polynomial filter should not cover all eigenvalues");
     filterInfo->filterType = 1;    /* high-pass filter, for largest eigenvalues */
-  } else if (a == a1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"filterType==3 for smallest eigenvalues should be pre-converted to filterType==1 for largest eigenvalues");
+  } else PetscCheck(a!=a1,PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"filterType==3 for smallest eigenvalues should be pre-converted to filterType==1 for largest eigenvalues");
 
   /* the following recipe follows Yousef Saad (2005, 2006) with a few minor adaptations / enhancements */
   halfPlateau = 0.5*(b1-a1)*opts->initialPlateau;    /* half length of the "plateau" of the (dual) base filter */
@@ -251,7 +248,7 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
   filterInfo->filterOK = 0;  /* not yet found any OK filter */
 
   /* allocate matrices */
-  ierr = PetscMalloc2((polyDeg+2)*(npoints-1),&polyFilter,(2*baseDeg+2)*(npoints-1),&baseFilter);CHKERRQ(ierr);
+  PetscCall(PetscMalloc2((polyDeg+2)*(npoints-1),&polyFilter,(2*baseDeg+2)*(npoints-1),&baseFilter));
 
   /* initialize the intervals, mainly for the case opts->maxOuterIter == 0 */
   intervals[0] = intv[0];
@@ -285,14 +282,14 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
       c1 = z1 + halfPlateau;
       intv[2] = z1;           /* i.e. c1 - halfPlateau */
       intv[3] = c1 + halfPlateau;
-      ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,6,HighLowFlags,baseDeg);CHKERRQ(ierr);
-      ierr = FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,6,opts->intervalWeights,polyDeg);CHKERRQ(ierr);
+      PetscCall(FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,6,HighLowFlags,baseDeg));
+      PetscCall(FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,6,opts->intervalWeights,polyDeg));
       /* fc1 = FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(polyFilter,polyDeg+2,intv,npoints,b1) - FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(polyFilter,polyDeg+2,intv,npoints,a1); */
       c2 = z2 - halfPlateau;
       intv[2] = c2 - halfPlateau;
       intv[3] = z2;           /* i.e. c2 + halfPlateau */
-      ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,6,HighLowFlags,baseDeg);CHKERRQ(ierr);
-      ierr = FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,6,opts->intervalWeights,polyDeg);CHKERRQ(ierr);
+      PetscCall(FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,6,HighLowFlags,baseDeg));
+      PetscCall(FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,6,opts->intervalWeights,polyDeg));
       fc2 = FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(polyFilter,polyDeg+2,intv,npoints,b1) - FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(polyFilter,polyDeg+2,intv,npoints,a1);
       yLimitGap = PETSC_MAX_REAL;
       ii = opts->maxInnerIter;
@@ -301,8 +298,8 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
         c = (c1+c2) / 2.0;
         intv[2] = c - halfPlateau;
         intv[3] = c + halfPlateau;
-        ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,6,HighLowFlags,baseDeg);CHKERRQ(ierr);
-        ierr = FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,6,opts->intervalWeights,polyDeg);CHKERRQ(ierr);
+        PetscCall(FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,6,HighLowFlags,baseDeg));
+        PetscCall(FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,6,opts->intervalWeights,polyDeg));
         fc = FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(polyFilter,polyDeg+2,intv,npoints,b1) - FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(polyFilter,polyDeg+2,intv,npoints,a1);
         if (fc*fc2 < 0.0) {
           c1 = c;
@@ -316,8 +313,8 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
     } else {  /* filterType == 1 (or 3 with conversion) for extreme eigenvalues */
       intv[1] = z1;
       intv[2] = z1 + (b1-z1)*opts->transIntervalRatio;
-      ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,4,HighLowFlags,baseDeg);CHKERRQ(ierr);
-      ierr = FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,4,opts->intervalWeights,polyDeg);CHKERRQ(ierr);
+      PetscCall(FILTLAN_HermiteBaseFilterInChebyshevBasis(baseFilter,intv,4,HighLowFlags,baseDeg));
+      PetscCall(FILTLAN_FilteredConjugateResidualPolynomial(polyFilter,baseFilter,2*baseDeg+2,intv,4,opts->intervalWeights,polyDeg));
     }
     /* polyFilter contains the coefficients of the polynomial filter which approximates phi(x)
        expanded in the `translated' Chebyshev basis */
@@ -448,10 +445,10 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
       else halfPlateau /= PetscSqrtReal(opts->plateauShrinkRate);
     }
   }
-  if (!filterInfo->filterOK) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_CONV_FAILED,"STFILTER cannot get the filter specified; please adjust your filter parameters (e.g. increasing the polynomial degree)");
+  PetscCheck(filterInfo->filterOK,PETSC_COMM_SELF,PETSC_ERR_CONV_FAILED,"STFILTER cannot get the filter specified; please adjust your filter parameters (e.g. increasing the polynomial degree)");
 
   filterInfo->totalNumIter = numIter;
-  ierr = PetscFree2(polyFilter,baseFilter);CHKERRQ(ierr);
+  PetscCall(PetscFree2(polyFilter,baseFilter));
   PetscFunctionReturn(0);
 }
 
@@ -476,7 +473,7 @@ static PetscErrorCode FILTLAN_GetIntervals(PetscReal *intervals,PetscReal *frame
    a vector q containing the Chebyshev coefficients:
        P(z) = q(1)*S_0(z) + q(2)*S_1(z) + ... + q(n)*S_{n-1}(z)
 */
-PETSC_STATIC_INLINE PetscErrorCode FILTLAN_ExpandNewtonPolynomialInChebyshevBasis(PetscInt n,PetscReal aa,PetscReal bb,PetscReal *a,PetscReal *x,PetscReal *q,PetscReal *q2)
+static inline PetscErrorCode FILTLAN_ExpandNewtonPolynomialInChebyshevBasis(PetscInt n,PetscReal aa,PetscReal bb,PetscReal *a,PetscReal *x,PetscReal *q,PetscReal *q2)
 {
   PetscInt  m,mm;
   PetscReal *sa,*sx,*sq,*sq2,c,c2,h,h2;
@@ -540,7 +537,7 @@ PETSC_STATIC_INLINE PetscErrorCode FILTLAN_ExpandNewtonPolynomialInChebyshevBasi
    output:
    the evaluated value of P(z) at z=z0
 */
-PETSC_STATIC_INLINE PetscReal FILTLAN_PolynomialEvaluationInChebyshevBasis(PetscReal *pp,PetscInt m,PetscInt idx,PetscReal z0,PetscReal aa,PetscReal bb)
+static inline PetscReal FILTLAN_PolynomialEvaluationInChebyshevBasis(PetscReal *pp,PetscInt m,PetscInt idx,PetscReal z0,PetscReal aa,PetscReal bb)
 {
   PetscInt  ii,deg1;
   PetscReal y,zz,t0,t1,t2,*sc;
@@ -594,7 +591,7 @@ PETSC_STATIC_INLINE PetscReal FILTLAN_PolynomialEvaluationInChebyshevBasis(Petsc
    note that if z0 falls below the first interval, then the polynomial in the first interval will be used to evaluate P(z0)
              if z0 flies over  the last  interval, then the polynomial in the last  interval will be used to evaluate P(z0)
 */
-PETSC_STATIC_INLINE PetscReal FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(PetscReal *pp,PetscInt m,PetscReal *intv,PetscInt npoints,PetscReal z0)
+static inline PetscReal FILTLAN_PiecewisePolynomialEvaluationInChebyshevBasis(PetscReal *pp,PetscInt m,PetscReal *intv,PetscInt npoints,PetscReal z0)
 {
   PetscReal *sintv,aa,bb,resul;
   PetscInt  idx;
@@ -654,14 +651,14 @@ PETSC_STATIC_INLINE PetscReal FILTLAN_PiecewisePolynomialEvaluationInChebyshevBa
 
    note that for unit weights, pass an empty vector of intervalWeights (i.e. of length 0)
 */
-PETSC_STATIC_INLINE PetscReal FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(PetscReal *pp,PetscInt prows,PetscInt pcols,PetscInt ldp,PetscReal *qq,PetscInt qrows,PetscInt qcols,PetscInt ldq,const PetscReal *intervalWeights)
+static inline PetscReal FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(PetscReal *pp,PetscInt prows,PetscInt pcols,PetscInt ldp,PetscReal *qq,PetscInt qrows,PetscInt qcols,PetscInt ldq,const PetscReal *intervalWeights)
 {
   PetscInt  nintv,deg1,i,k;
   PetscReal *sp,*sq,ans=0.0,ans2;
 
   PetscFunctionBegin;
   deg1 = PetscMin(prows,qrows);  /* number of effective coefficients, one more than the effective polynomial degree */
-  if (!deg1) PetscFunctionReturn(0.0);
+  if (PetscUnlikely(!deg1)) PetscFunctionReturn(0.0);
   nintv = PetscMin(pcols,qcols);  /* number of intervals */
 
   /* scaled by intervalWeights(i) in the i-th interval (we assume intervalWeights[] are always provided).
@@ -695,7 +692,7 @@ PETSC_STATIC_INLINE PetscReal FILTLAN_PiecewisePolynomialInnerProductInChebyshev
 
    the returned matrix is qq which represents Q(z) = z*P(z)
 */
-PETSC_STATIC_INLINE PetscErrorCode FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(PetscReal *pp,PetscInt deg1,PetscInt ldp,PetscReal *intv,PetscInt nintv,PetscReal *qq,PetscInt ldq)
+static inline PetscErrorCode FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(PetscReal *pp,PetscInt deg1,PetscInt ldp,PetscReal *intv,PetscInt nintv,PetscReal *qq,PetscInt ldq)
 {
   PetscInt  i,j;
   PetscReal c,h,h2,tmp,*sp,*sq,*sp2,*sq2;
@@ -732,18 +729,17 @@ PETSC_STATIC_INLINE PetscErrorCode FILTLAN_PiecewisePolynomialInChebyshevBasisMu
 
     A,B are nxk
 */
-PETSC_STATIC_INLINE PetscErrorCode Mat_AXPY_BLAS(PetscInt n,PetscInt k,PetscReal alpha,const PetscReal *A,PetscInt lda,PetscReal beta,PetscReal *B,PetscInt ldb)
+static inline PetscErrorCode Mat_AXPY_BLAS(PetscInt n,PetscInt k,PetscReal alpha,const PetscReal *A,PetscInt lda,PetscReal beta,PetscReal *B,PetscInt ldb)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j;
 
   PetscFunctionBegin;
   if (beta==(PetscReal)1.0) {
     for (j=0;j<k;j++) for (i=0;i<n;i++) B[i+j*ldb] += alpha*A[i+j*lda];
-    ierr = PetscLogFlops(2.0*n*k);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(2.0*n*k));
   } else {
     for (j=0;j<k;j++) for (i=0;i<n;i++) B[i+j*ldb] = alpha*A[i+j*lda] + beta*B[i+j*ldb];
-    ierr = PetscLogFlops(3.0*n*k);CHKERRQ(ierr);
+    PetscCall(PetscLogFlops(3.0*n*k));
   }
   PetscFunctionReturn(0);
 }
@@ -788,15 +784,14 @@ PETSC_STATIC_INLINE PetscErrorCode Mat_AXPY_BLAS(PetscInt n,PetscInt k,PetscReal
 */
 static PetscErrorCode FILTLAN_FilteredConjugateResidualPolynomial(PetscReal *cpol,PetscReal *baseFilter,PetscInt nbase,PetscReal *intv,PetscInt m,PetscReal *intervalWeights,PetscInt niter)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,srpol,scpol,sarpol,sppol,sappol,ld,nintv;
   PetscReal      rho,rho0,rho1,den,bet,alp,alp0,*ppol,*rpol,*appol,*arpol;
 
   PetscFunctionBegin;
   nintv = m-1;
   ld = niter+2;  /* leading dimension */
-  ierr = PetscCalloc4(ld*nintv,&ppol,ld*nintv,&rpol,ld*nintv,&appol,ld*nintv,&arpol);CHKERRQ(ierr);
-  ierr = PetscArrayzero(cpol,ld*nintv);CHKERRQ(ierr);
+  PetscCall(PetscCalloc4(ld*nintv,&ppol,ld*nintv,&rpol,ld*nintv,&appol,ld*nintv,&arpol));
+  PetscCall(PetscArrayzero(cpol,ld*nintv));
   /* initialize polynomial ppol to be 1 (i.e. multiplicative identity) in all intervals */
   sppol = 2;
   srpol = 2;
@@ -811,7 +806,7 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualPolynomial(PetscReal *cpo
      cpol is the "corrected" residual polynomial (result of this function) */
   sappol = 3;
   sarpol = 3;
-  ierr = FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(ppol,sppol,ld,intv,nintv,appol,ld);CHKERRQ(ierr);
+  PetscCall(FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(ppol,sppol,ld,intv,nintv,appol,ld));
   for (i=0;i<3;i++) for (j=0;j<nintv;j++) arpol[i+j*ld] = appol[i+j*ld];
   rho = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(rpol,srpol,nintv,ld,arpol,sarpol,nintv,ld,intervalWeights);
   for (i=0;i<niter;i++) {
@@ -821,20 +816,20 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualPolynomial(PetscReal *cpo
     alp  = (rho-rho1)/den;
     srpol++;
     scpol++;
-    ierr = Mat_AXPY_BLAS(srpol,nintv,-alp0,appol,ld,1.0,rpol,ld);CHKERRQ(ierr);
-    ierr = Mat_AXPY_BLAS(scpol,nintv,-alp,appol,ld,1.0,cpol,ld);CHKERRQ(ierr);
+    PetscCall(Mat_AXPY_BLAS(srpol,nintv,-alp0,appol,ld,1.0,rpol,ld));
+    PetscCall(Mat_AXPY_BLAS(scpol,nintv,-alp,appol,ld,1.0,cpol,ld));
     if (i+1 == niter) break;
     sarpol++;
-    ierr = FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(rpol,srpol,ld,intv,nintv,arpol,ld);CHKERRQ(ierr);
+    PetscCall(FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(rpol,srpol,ld,intv,nintv,arpol,ld));
     rho0 = rho;
     rho = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(rpol,srpol,nintv,ld,arpol,sarpol,nintv,ld,intervalWeights);
     bet  = rho / rho0;
     sppol++;
     sappol++;
-    ierr = Mat_AXPY_BLAS(sppol,nintv,1.0,rpol,ld,bet,ppol,ld);CHKERRQ(ierr);
-    ierr = Mat_AXPY_BLAS(sappol,nintv,1.0,arpol,ld,bet,appol,ld);CHKERRQ(ierr);
+    PetscCall(Mat_AXPY_BLAS(sppol,nintv,1.0,rpol,ld,bet,ppol,ld));
+    PetscCall(Mat_AXPY_BLAS(sappol,nintv,1.0,arpol,ld,bet,appol,ld));
   }
-  ierr = PetscFree4(ppol,rpol,appol,arpol);CHKERRQ(ierr);
+  PetscCall(PetscFree4(ppol,rpol,appol,arpol));
   PetscFunctionReturn(0);
 }
 
@@ -884,7 +879,6 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualPolynomial(PetscReal *cpo
 */
 static PetscErrorCode FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProduct(Mat A,Vec b,Vec x,PetscReal *baseFilter,PetscInt nbase,PetscReal *intv,PetscInt nintv,PetscReal *intervalWeights,PetscInt niter,Vec *work)
 {
-  PetscErrorCode ierr;
   PetscInt       i,j,srpol,scpol,sarpol,sppol,sappol,ld;
   PetscReal      rho,rho0,rho00,rho1,den,bet,alp,alp0,*cpol,*ppol,*rpol,*appol,*arpol,tol=0.0;
   Vec            r=work[0],p=work[1],ap=work[2],w=work[3];
@@ -892,7 +886,7 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorPro
 
   PetscFunctionBegin;
   ld = niter+3;  /* leading dimension */
-  ierr = PetscCalloc5(ld*nintv,&ppol,ld*nintv,&rpol,ld*nintv,&cpol,ld*nintv,&appol,ld*nintv,&arpol);CHKERRQ(ierr);
+  PetscCall(PetscCalloc5(ld*nintv,&ppol,ld*nintv,&rpol,ld*nintv,&cpol,ld*nintv,&appol,ld*nintv,&arpol));
   /* initialize polynomial ppol to be 1 (i.e. multiplicative identity) in all intervals */
   sppol = 2;
   srpol = 2;
@@ -907,17 +901,17 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorPro
      cpol is the "corrected" residual polynomial */
   sappol = 3;
   sarpol = 3;
-  ierr = FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(ppol,sppol,ld,intv,nintv,appol,ld);CHKERRQ(ierr);
+  PetscCall(FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(ppol,sppol,ld,intv,nintv,appol,ld));
   for (i=0;i<3;i++) for (j=0;j<nintv;j++) arpol[i+j*ld] = appol[i+j*ld];
   rho00 = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(rpol,srpol,nintv,ld,arpol,sarpol,nintv,ld,intervalWeights);
   rho = rho00;
 
   /* corrected CR in vector space */
   /* we assume x0 is always 0 */
-  ierr = VecSet(x,0.0);CHKERRQ(ierr);
-  ierr = VecCopy(b,r);CHKERRQ(ierr);     /* initial residual r = b-A*x0 */
-  ierr = VecCopy(r,p);CHKERRQ(ierr);     /* p = r */
-  ierr = MatMult(A,p,ap);CHKERRQ(ierr);  /* ap = A*p */
+  PetscCall(VecSet(x,0.0));
+  PetscCall(VecCopy(b,r));     /* initial residual r = b-A*x0 */
+  PetscCall(VecCopy(r,p));     /* p = r */
+  PetscCall(MatMult(A,p,ap));  /* ap = A*p */
 
   for (i=0;i<niter;i++) {
     /* iteration in the polynomial space */
@@ -927,34 +921,34 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorPro
     alp  = (rho-rho1)/den;
     srpol++;
     scpol++;
-    ierr = Mat_AXPY_BLAS(srpol,nintv,-alp0,appol,ld,1.0,rpol,ld);CHKERRQ(ierr);
-    ierr = Mat_AXPY_BLAS(scpol,nintv,-alp,appol,ld,1.0,cpol,ld);CHKERRQ(ierr);
+    PetscCall(Mat_AXPY_BLAS(srpol,nintv,-alp0,appol,ld,1.0,rpol,ld));
+    PetscCall(Mat_AXPY_BLAS(scpol,nintv,-alp,appol,ld,1.0,cpol,ld));
     sarpol++;
-    ierr = FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(rpol,srpol,ld,intv,nintv,arpol,ld);CHKERRQ(ierr);
+    PetscCall(FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(rpol,srpol,ld,intv,nintv,arpol,ld));
     rho0 = rho;
     rho = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(rpol,srpol,nintv,ld,arpol,sarpol,nintv,ld,intervalWeights);
 
     /* update x in the vector space */
     alpha = alp;
-    ierr = VecAXPY(x,alpha,p);CHKERRQ(ierr);   /* x += alp*p */
+    PetscCall(VecAXPY(x,alpha,p));   /* x += alp*p */
     if (rho < tol*rho00) break;
 
     /* finish the iteration in the polynomial space */
     bet = rho / rho0;
     sppol++;
     sappol++;
-    ierr = Mat_AXPY_BLAS(sppol,nintv,1.0,rpol,ld,bet,ppol,ld);CHKERRQ(ierr);
-    ierr = Mat_AXPY_BLAS(sappol,nintv,1.0,arpol,ld,bet,appol,ld);CHKERRQ(ierr);
+    PetscCall(Mat_AXPY_BLAS(sppol,nintv,1.0,rpol,ld,bet,ppol,ld));
+    PetscCall(Mat_AXPY_BLAS(sappol,nintv,1.0,arpol,ld,bet,appol,ld));
 
     /* finish the iteration in the vector space */
     alpha = -alp0;
-    ierr = VecAXPY(r,alpha,ap);CHKERRQ(ierr);    /* r -= alp0*ap */
+    PetscCall(VecAXPY(r,alpha,ap));    /* r -= alp0*ap */
     alpha = bet;
-    ierr = VecAYPX(p,alpha,r);CHKERRQ(ierr);     /* p = r + bet*p */
-    ierr = MatMult(A,r,w);CHKERRQ(ierr);         /* ap = A*r + bet*ap */
-    ierr = VecAYPX(ap,alpha,w);CHKERRQ(ierr);
+    PetscCall(VecAYPX(p,alpha,r));     /* p = r + bet*p */
+    PetscCall(MatMult(A,r,w));         /* ap = A*r + bet*ap */
+    PetscCall(VecAYPX(ap,alpha,w));
   }
-  ierr = PetscFree5(ppol,rpol,cpol,appol,arpol);CHKERRQ(ierr);
+  PetscCall(PetscFree5(ppol,rpol,cpol,appol,arpol));
   PetscFunctionReturn(0);
 }
 
@@ -963,18 +957,126 @@ static PetscErrorCode FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorPro
 */
 PetscErrorCode MatMult_FILTLAN(Mat A,Vec x,Vec y)
 {
-  PetscErrorCode ierr;
   ST             st;
   ST_FILTER      *ctx;
   PetscInt       npoints;
 
   PetscFunctionBegin;
-  ierr = MatShellGetContext(A,&st);CHKERRQ(ierr);
+  PetscCall(MatShellGetContext(A,&st));
   ctx = (ST_FILTER*)st->data;
   npoints = (ctx->filterInfo->filterType == 2)? 6: 4;
-  ierr = FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProduct(ctx->T,x,y,ctx->baseFilter,2*ctx->baseDegree+2,ctx->intervals,npoints-1,ctx->opts->intervalWeights,ctx->polyDegree,st->work);CHKERRQ(ierr);
-  ierr = VecCopy(y,st->work[0]);CHKERRQ(ierr);
-  ierr = MatMult(ctx->T,st->work[0],y);CHKERRQ(ierr);
+  PetscCall(FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProduct(ctx->T,x,y,ctx->baseFilter,2*ctx->baseDegree+2,ctx->intervals,npoints-1,ctx->opts->intervalWeights,ctx->polyDegree,st->work));
+  PetscCall(VecCopy(y,st->work[0]));
+  PetscCall(MatMult(ctx->T,st->work[0],y));
+  PetscFunctionReturn(0);
+}
+
+/* Block version of FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProduct */
+static PetscErrorCode FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProductBlock(Mat A,Mat B,Mat C,PetscReal *baseFilter,PetscInt nbase,PetscReal *intv,PetscInt nintv,PetscReal *intervalWeights,PetscInt niter,Vec *work,Mat R,Mat P,Mat AP,Mat W)
+{
+  PetscInt       i,j,srpol,scpol,sarpol,sppol,sappol,ld;
+  PetscReal      rho,rho0,rho00,rho1,den,bet,alp,alp0,*cpol,*ppol,*rpol,*appol,*arpol,tol=0.0;
+  PetscScalar    alpha;
+
+  PetscFunctionBegin;
+  ld = niter+3;  /* leading dimension */
+  PetscCall(PetscCalloc5(ld*nintv,&ppol,ld*nintv,&rpol,ld*nintv,&cpol,ld*nintv,&appol,ld*nintv,&arpol));
+  /* initialize polynomial ppol to be 1 (i.e. multiplicative identity) in all intervals */
+  sppol = 2;
+  srpol = 2;
+  scpol = 2;
+  for (j=0;j<nintv;j++) {
+    ppol[j*ld] = 1.0;
+    rpol[j*ld] = 1.0;
+    cpol[j*ld] = 1.0;
+  }
+  /* ppol is the initial p-polynomial (corresponding to the A-conjugate vector p in CG)
+     rpol is the r-polynomial (corresponding to the residual vector r in CG)
+     cpol is the "corrected" residual polynomial */
+  sappol = 3;
+  sarpol = 3;
+  PetscCall(FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(ppol,sppol,ld,intv,nintv,appol,ld));
+  for (i=0;i<3;i++) for (j=0;j<nintv;j++) arpol[i+j*ld] = appol[i+j*ld];
+  rho00 = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(rpol,srpol,nintv,ld,arpol,sarpol,nintv,ld,intervalWeights);
+  rho = rho00;
+
+  /* corrected CR in vector space */
+  /* we assume x0 is always 0 */
+  PetscCall(MatZeroEntries(C));
+  PetscCall(MatCopy(B,R,SAME_NONZERO_PATTERN));     /* initial residual r = b-A*x0 */
+  PetscCall(MatCopy(R,P,SAME_NONZERO_PATTERN));     /* p = r */
+  PetscCall(MatMatMult(A,P,MAT_REUSE_MATRIX,PETSC_DEFAULT,&AP));  /* ap = A*p */
+
+  for (i=0;i<niter;i++) {
+    /* iteration in the polynomial space */
+    den = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(appol,sappol,nintv,ld,appol,sappol,nintv,ld,intervalWeights);
+    alp0 = rho/den;
+    rho1 = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(baseFilter,nbase,nintv,nbase,appol,sappol,nintv,ld,intervalWeights);
+    alp  = (rho-rho1)/den;
+    srpol++;
+    scpol++;
+    PetscCall(Mat_AXPY_BLAS(srpol,nintv,-alp0,appol,ld,1.0,rpol,ld));
+    PetscCall(Mat_AXPY_BLAS(scpol,nintv,-alp,appol,ld,1.0,cpol,ld));
+    sarpol++;
+    PetscCall(FILTLAN_PiecewisePolynomialInChebyshevBasisMultiplyX(rpol,srpol,ld,intv,nintv,arpol,ld));
+    rho0 = rho;
+    rho = FILTLAN_PiecewisePolynomialInnerProductInChebyshevBasis(rpol,srpol,nintv,ld,arpol,sarpol,nintv,ld,intervalWeights);
+
+    /* update x in the vector space */
+    alpha = alp;
+    PetscCall(MatAXPY(C,alpha,P,SAME_NONZERO_PATTERN));   /* x += alp*p */
+    if (rho < tol*rho00) break;
+
+    /* finish the iteration in the polynomial space */
+    bet = rho / rho0;
+    sppol++;
+    sappol++;
+    PetscCall(Mat_AXPY_BLAS(sppol,nintv,1.0,rpol,ld,bet,ppol,ld));
+    PetscCall(Mat_AXPY_BLAS(sappol,nintv,1.0,arpol,ld,bet,appol,ld));
+
+    /* finish the iteration in the vector space */
+    alpha = -alp0;
+    PetscCall(MatAXPY(R,alpha,AP,SAME_NONZERO_PATTERN));    /* r -= alp0*ap */
+    alpha = bet;
+    PetscCall(MatAYPX(P,alpha,R,SAME_NONZERO_PATTERN));     /* p = r + bet*p */
+    PetscCall(MatMatMult(A,R,MAT_REUSE_MATRIX,PETSC_DEFAULT,&W));         /* ap = A*r + bet*ap */
+    PetscCall(MatAYPX(AP,alpha,W,SAME_NONZERO_PATTERN));
+  }
+  PetscCall(PetscFree5(ppol,rpol,cpol,appol,arpol));
+  PetscFunctionReturn(0);
+}
+
+/*
+   Gateway to FILTLAN for evaluating C=p(A)*B
+*/
+PetscErrorCode MatMatMult_FILTLAN(Mat A,Mat B,Mat C,void *pctx)
+{
+  ST             st;
+  ST_FILTER      *ctx;
+  PetscInt       i,m1,m2,npoints;
+
+  PetscFunctionBegin;
+  PetscCall(MatShellGetContext(A,&st));
+  ctx = (ST_FILTER*)st->data;
+  npoints = (ctx->filterInfo->filterType == 2)? 6: 4;
+  if (ctx->nW) {  /* check if work matrices must be resized */
+    PetscCall(MatGetSize(B,NULL,&m1));
+    PetscCall(MatGetSize(ctx->W[0],NULL,&m2));
+    if (m1!=m2) {
+      PetscCall(MatDestroyMatrices(ctx->nW,&ctx->W));
+      ctx->nW = 0;
+    }
+  }
+  if (!ctx->nW) {  /* allocate work matrices */
+    ctx->nW = 4;
+    PetscCall(PetscMalloc1(ctx->nW,&ctx->W));
+    PetscCall(PetscLogObjectMemory((PetscObject)st,ctx->nW*sizeof(Mat)));
+    for (i=0;i<ctx->nW;i++) {
+      PetscCall(MatDuplicate(B,MAT_DO_NOT_COPY_VALUES,&ctx->W[i]));
+    }
+  }
+  PetscCall(FILTLAN_FilteredConjugateResidualMatrixPolynomialVectorProductBlock(ctx->T,B,ctx->W[0],ctx->baseFilter,2*ctx->baseDegree+2,ctx->intervals,npoints-1,ctx->opts->intervalWeights,ctx->polyDegree,st->work,C,ctx->W[1],ctx->W[2],ctx->W[3]));
+  PetscCall(MatMatMult(ctx->T,ctx->W[0],MAT_REUSE_MATRIX,PETSC_DEFAULT,&C));
   PetscFunctionReturn(0);
 }
 
@@ -986,7 +1088,6 @@ PetscErrorCode MatMult_FILTLAN(Mat A,Vec x,Vec y)
 */
 PetscErrorCode STFilter_FILTLAN_setFilter(ST st,Mat *G)
 {
-  PetscErrorCode ierr;
   ST_FILTER      *ctx = (ST_FILTER*)st->data;
   PetscInt       i,npoints,n,m,N,M;
   PetscReal      frame2[4];
@@ -996,31 +1097,31 @@ PetscErrorCode STFilter_FILTLAN_setFilter(ST st,Mat *G)
   PetscFunctionBegin;
   if (ctx->frame[0] == ctx->frame[1]) {  /* low pass filter, convert it to high pass filter */
     /* T = frame[3]*eye(n) - A */
-    ierr = MatDestroy(&ctx->T);CHKERRQ(ierr);
-    ierr = MatDuplicate(st->A[0],MAT_COPY_VALUES,&ctx->T);CHKERRQ(ierr);
-    ierr = MatScale(ctx->T,-1.0);CHKERRQ(ierr);
+    PetscCall(MatDestroy(&ctx->T));
+    PetscCall(MatDuplicate(st->A[0],MAT_COPY_VALUES,&ctx->T));
+    PetscCall(MatScale(ctx->T,-1.0));
     alpha = ctx->frame[3];
-    ierr = MatShift(ctx->T,alpha);CHKERRQ(ierr);
+    PetscCall(MatShift(ctx->T,alpha));
     for (i=0;i<4;i++) frame2[i] = ctx->frame[3] - ctx->frame[3-i];
   } else {  /* it can be a mid-pass filter or a high-pass filter */
     if (ctx->frame[0] == 0.0) {
-      ierr = PetscObjectReference((PetscObject)st->A[0]);CHKERRQ(ierr);
-      ierr = MatDestroy(&ctx->T);CHKERRQ(ierr);
+      PetscCall(PetscObjectReference((PetscObject)st->A[0]));
+      PetscCall(MatDestroy(&ctx->T));
       ctx->T = st->A[0];
       for (i=0;i<4;i++) frame2[i] = ctx->frame[i];
     } else {
       /* T = A - frame[0]*eye(n) */
-      ierr = MatDestroy(&ctx->T);CHKERRQ(ierr);
-      ierr = MatDuplicate(st->A[0],MAT_COPY_VALUES,&ctx->T);CHKERRQ(ierr);
+      PetscCall(MatDestroy(&ctx->T));
+      PetscCall(MatDuplicate(st->A[0],MAT_COPY_VALUES,&ctx->T));
       alpha = -ctx->frame[0];
-      ierr = MatShift(ctx->T,alpha);CHKERRQ(ierr);
+      PetscCall(MatShift(ctx->T,alpha));
       for (i=0;i<4;i++) frame2[i] = ctx->frame[i] - ctx->frame[0];
     }
   }
 
   /* no need to recompute filter if the parameters did not change */
   if (st->state==ST_STATE_INITIAL || ctx->filtch) {
-    ierr = FILTLAN_GetIntervals(ctx->intervals,frame2,ctx->polyDegree,ctx->baseDegree,ctx->opts,ctx->filterInfo);CHKERRQ(ierr);
+    PetscCall(FILTLAN_GetIntervals(ctx->intervals,frame2,ctx->polyDegree,ctx->baseDegree,ctx->opts,ctx->filterInfo));
     /* translate the intervals back */
     if (ctx->frame[0] == ctx->frame[1]) {  /* low pass filter, convert it to high pass filter */
       for (i=0;i<4;i++) ctx->intervals2[i] = ctx->frame[3] - ctx->intervals[3-i];
@@ -1033,21 +1134,22 @@ PetscErrorCode STFilter_FILTLAN_setFilter(ST st,Mat *G)
     }
 
     npoints = (ctx->filterInfo->filterType == 2)? 6: 4;
-    ierr = PetscFree(ctx->baseFilter);CHKERRQ(ierr);
-    ierr = PetscMalloc1((2*ctx->baseDegree+2)*(npoints-1),&ctx->baseFilter);CHKERRQ(ierr);
-    ierr = FILTLAN_HermiteBaseFilterInChebyshevBasis(ctx->baseFilter,ctx->intervals,npoints,HighLowFlags,ctx->baseDegree);CHKERRQ(ierr);
-    ierr = PetscInfo1(st,"Computed value of yLimit = %g\n",ctx->filterInfo->yLimit);CHKERRQ(ierr);
+    PetscCall(PetscFree(ctx->baseFilter));
+    PetscCall(PetscMalloc1((2*ctx->baseDegree+2)*(npoints-1),&ctx->baseFilter));
+    PetscCall(FILTLAN_HermiteBaseFilterInChebyshevBasis(ctx->baseFilter,ctx->intervals,npoints,HighLowFlags,ctx->baseDegree));
+    PetscCall(PetscInfo(st,"Computed value of yLimit = %g\n",(double)ctx->filterInfo->yLimit));
   }
   ctx->filtch = PETSC_FALSE;
 
   /* create shell matrix*/
   if (!*G) {
-    ierr = MatGetSize(ctx->T,&N,&M);CHKERRQ(ierr);
-    ierr = MatGetLocalSize(ctx->T,&n,&m);CHKERRQ(ierr);
-    ierr = MatCreateShell(PetscObjectComm((PetscObject)st),n,m,N,M,st,G);CHKERRQ(ierr);
-    ierr = MatShellSetOperation(*G,MATOP_MULT,(void(*)(void))MatMult_FILTLAN);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)st,(PetscObject)*G);CHKERRQ(ierr);
+    PetscCall(MatGetSize(ctx->T,&N,&M));
+    PetscCall(MatGetLocalSize(ctx->T,&n,&m));
+    PetscCall(MatCreateShell(PetscObjectComm((PetscObject)st),n,m,N,M,st,G));
+    PetscCall(MatShellSetOperation(*G,MATOP_MULT,(void(*)(void))MatMult_FILTLAN));
+    PetscCall(MatShellSetMatProductOperation(*G,MATPRODUCT_AB,NULL,MatMatMult_FILTLAN,NULL,MATDENSE,MATDENSE));
+    PetscCall(MatShellSetMatProductOperation(*G,MATPRODUCT_AB,NULL,MatMatMult_FILTLAN,NULL,MATDENSECUDA,MATDENSECUDA));
+    PetscCall(PetscLogObjectParent((PetscObject)st,(PetscObject)*G));
   }
   PetscFunctionReturn(0);
 }
-
